#ifndef _LCR_ASSETS_H
#define _LCR_ASSETS_H

/**
  NOTES:
  - All images are 64x64, stored in an indexed mode (8bits pery pixel), the
    image starts with a 256 color palette in RGB565 format (i.e. the palette
    takes 512 bytes, then 64x64 bytes follow).
  - Each sky (background) image is composed of 4 regular images so they have
    higher resolution. The partial images are stored in the above format, one
    sky part consists of interleaved lines, so it effectively stores a
    128x32 strip (instead of 64x64 square) -- this is convenient for drawing
    the sky.
*/

#include <stdint.h>
#include "map.h"

// TODO: images should be 64x64 indexed, sky will be composed of 2x2 normal images

static const uint8_t map1[] =
  {
    LCR_MAP_MAGIC_NUMBER,
    0,
    10,
    10,
    0,
    
    LCR_MAP_BLOCK(LCR_BLOCK_FULL_ACCEL,32,2,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,1,40,1,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL_ACCEL,32,32,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,4,4,4,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,50,20,20,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,5,5,5,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,10,10,10,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,3,3,3,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,2,25,50,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,3,3,3,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,0,0,0,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,40,1,1,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,63,63,0,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,1,1,40,0,0),

/*
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,0,15,0,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_CUBOID_HOLLOW,7,4,8,0,0),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,32,32,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,33,32,32,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_RAMP_CURVED_WALL,34,32,32,LCR_BLOCK_MATERIAL_GRASS,0),
    LCR_MAP_BLOCK(LCR_BLOCK_RAMP_CURVED_WALL,35,32,32,LCR_BLOCK_MATERIAL_CONCRETE,LCR_BLOCK_TRANSFORM_ROT_90),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,63,35,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,0,35,32,LCR_BLOCK_MATERIAL_ICE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,40,35,32,LCR_BLOCK_MATERIAL_DIRT,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,41,35,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,42,35,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,43,35,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,44,35,32,LCR_BLOCK_MATERIAL_CONCRETE,0),

    LCR_MAP_BLOCK(LCR_BLOCK_BOTTOM_LEFT,34,32,33,LCR_BLOCK_MATERIAL_CONCRETE,LCR_BLOCK_TRANSFORM_ROT_90),

    LCR_MAP_BLOCK(LCR_BLOCK_FULL,32,33,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,33,33,32,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,32,33,33,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,33,33,33,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,32,32,33,LCR_BLOCK_MATERIAL_CONCRETE,0),
    LCR_MAP_BLOCK(LCR_BLOCK_FULL,33,32,33,LCR_BLOCK_MATERIAL_CONCRETE,0),
*/
    LCR_MAP_TERMINATOR
  };

#define LCR_IMAGE_SIZE 64 ///< one-dimension resolution of bitmap image
#define LCR_IMAGE_STORE_SIZE (LCR_IMAGE_SIZE * LCR_IMAGE_SIZE + 256 * 2)

// TODO: put this into struct
uint16_t _LCR_currentImagePalette[256];
const uint8_t *_LCR_currentImagePixel;
const uint8_t *_LCR_currentImage;

#define LCR_IMAGE_WALL_CONCRETE   0
#define LCR_IMAGE_WALL_WOOD       1
#define LCR_IMAGE_GROUND_CONCRETE 2
#define LCR_IMAGE_GROUND_GRASS    3
#define LCR_IMAGE_GROUND_DIRT     4
#define LCR_IMAGE_GROUND_ICE      5
#define LCR_IMAGE_GROUND_ACCEL    6
#define LCR_IMAGE_GROUND_FAN      7

static const uint8_t LCR_images[] =
{
  // 0: wall concrete
  0x10,0x84,0x92,0x94,0xf3,0x9c,0x14,0xa5,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,
  0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x6e,0x6b,
  0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,
  0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x75,0xad,
  0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,
  0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,
  0x10,0x22,0x18,0x22,0xf0,0x7b,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,
  0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x72,0x8c,0x10,0x25,0x18,0x25,
  0x51,0x8c,0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,
  0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,
  0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,
  0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0xf4,0x9c,
  0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,
  0x10,0x47,0x18,0x47,0x00,0x60,0x10,0x7c,0x08,0x60,0x10,0x60,0x18,0x60,0x00,0x61,
  0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,
  0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,
  0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,
  0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,
  0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,
  0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,0x18,0x84,0x00,0x85,0x08,0x85,
  0x10,0x85,0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,
  0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,
  0x10,0xa1,0x18,0xa1,0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,
  0x10,0xa3,0xae,0x73,0x18,0xa3,0x00,0xa4,0x08,0xa4,0x10,0xa4,0xef,0x7b,0x18,0xa4,
  0x00,0xa5,0x08,0xa5,0x92,0x8c,0xb6,0xb5,0x10,0xa5,0x18,0xa5,0x14,0x9d,0x00,0xa6,
  0x08,0xa6,0x10,0xa6,0x18,0xa6,0xb4,0x8c,0x00,0xa7,0x08,0xa7,0xcd,0x4a,0x12,0x7c,
  0x10,0xa7,0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x00,0xc1,0x08,0xc1,
  0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0xac,0x42,0x00,0xc3,
  0x08,0xc3,0x10,0xc3,0x18,0xc3,0x96,0xad,0x00,0xc4,0x8b,0x42,0xe8,0x31,0x32,0x8c,
  0x08,0xc4,0xed,0x52,0x10,0xc4,0x6b,0x42,0x13,0x9d,0x18,0xc4,0x00,0xc5,0x09,0x32,
  0x6a,0x3a,0xc7,0x41,0xe8,0x29,0x08,0xc5,0xa7,0x29,0xc7,0x29,0x86,0x49,0xcb,0x72,
  0x76,0xad,0x0b,0x63,0x2a,0x32,0x87,0x39,0xf0,0x83,0x8b,0x72,0xa8,0x49,0x71,0x94,
  0xc2,0xc2,0xc2,0xcb,0xc2,0x3d,0x3d,0xcb,0x3d,0xcb,0x6b,0x32,0x32,0x32,0x32,0x32,
  0x32,0x32,0x32,0x32,0x6b,0x32,0x32,0x6b,0x3d,0x3d,0x3d,0x3d,0x3d,0xc2,0x3d,0x3d,
  0xcb,0xcb,0xcb,0xcb,0x3d,0xcf,0xcf,0x6b,0x7d,0x32,0x32,0x0f,0x0f,0x32,0x7d,0x32,
  0x32,0x7d,0x32,0x0f,0x32,0x32,0x32,0x32,0x6b,0xcf,0x3d,0x3d,0x3d,0x3d,0xcb,0x3d,
  0x32,0x32,0x32,0x00,0x00,0x00,0x00,0xfc,0x32,0x32,0xb9,0xb9,0x32,0x0f,0xb9,0x32,
  0x32,0x32,0x32,0x32,0xb9,0xb9,0x0f,0x32,0xb9,0x0f,0x0f,0x0f,0x0f,0xf9,0x0f,0x3d,
  0xcb,0x32,0x32,0x40,0x00,0x00,0x32,0x32,0x32,0x32,0x32,0xb9,0x32,0xb9,0x32,0x32,
  0xb9,0x32,0x32,0x32,0x0f,0x0f,0x32,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0xf9,0x0f,0x32,
  0xc2,0x3d,0x5f,0xc6,0xc6,0xcb,0x5f,0xc2,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xc2,0xcb,
  0xcb,0xcb,0xc2,0xc2,0x3d,0x3d,0x3d,0x3d,0x3d,0xcf,0x3d,0x3d,0x3d,0x5f,0x6b,0xc2,
  0xcb,0x6b,0xcb,0xcb,0xc2,0xcf,0xcf,0xc2,0x3d,0xc2,0xc2,0x3d,0x3d,0x3d,0xcf,0x3d,
  0x3d,0xcb,0xcb,0xcb,0xcf,0xcf,0xcf,0x6b,0xcf,0x32,0xcf,0x3d,0x3d,0xcb,0x3d,0x5f,
  0x5f,0x32,0x00,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0x3d,0xe7,0xe7,0x5f,0x01,0xc2,0xcb,
  0xc2,0x01,0x01,0x3d,0x3d,0x3d,0x3d,0xe7,0x00,0xcf,0xcf,0x00,0x32,0x00,0x32,0x3d,
  0x3d,0x32,0x3d,0x01,0x01,0x3d,0x3d,0x5f,0x3d,0x01,0x3d,0x3d,0x3d,0x3d,0xe7,0x3d,
  0xe7,0xe7,0x3d,0x3d,0xe7,0x3d,0xe7,0x32,0xe7,0xcf,0x32,0x6b,0x32,0x32,0x32,0x5f,
  0x6b,0x32,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x32,
  0x32,0xb9,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x32,
  0x0f,0xb9,0xe7,0x3d,0x3d,0x5f,0xcb,0xcb,0xcb,0x5f,0xcb,0xcb,0x5f,0x5f,0x5f,0x5f,
  0x5f,0x5f,0xcb,0xcb,0xcb,0xcb,0x01,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0x3d,0x0f,0x32,
  0x32,0x32,0x3d,0x3d,0x5f,0xcb,0xcb,0xcb,0xcb,0x5f,0xcb,0xcb,0x5f,0x5f,0xc6,0xc6,
  0x5f,0xc6,0xcb,0xcb,0x5f,0xcb,0xc2,0xcb,0xc6,0x5f,0x01,0xcb,0xc2,0x6b,0x0f,0x32,
  0x3d,0x32,0xcf,0x6b,0xcf,0x6b,0xcf,0x6b,0x6b,0x6b,0x32,0x6b,0x32,0x6b,0x6b,0x6b,
  0x32,0x32,0x32,0x32,0x32,0x32,0x32,0xb9,0x32,0x32,0x32,0xcf,0x32,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x3d,0x00,0x6b,0xe7,0xcf,0x6b,0x6b,0x6b,0x32,0x32,0x6b,0x32,0x32,0x6b,
  0x6b,0x32,0x32,0x3d,0x3d,0x6b,0x32,0x32,0x32,0x32,0x6b,0x6b,0x7d,0x0f,0x0f,0x5f,
  0x3d,0x0f,0xcf,0x32,0xcf,0x3d,0xcf,0xcf,0xcf,0x6b,0x6b,0x6b,0x6b,0xcf,0xcf,0x6b,
  0x6b,0x6b,0xcf,0x6b,0x6b,0x3d,0x00,0xe7,0x3d,0x32,0x32,0x32,0xcf,0x32,0x0f,0x0f,
  0x0f,0x0f,0x3d,0x6b,0xcf,0xe7,0x00,0x6b,0xe7,0x6b,0x6b,0x32,0x6b,0x32,0x6b,0xc2,
  0x3d,0x6b,0x6b,0x32,0x00,0x40,0x6b,0x6b,0x6b,0x32,0x32,0x00,0x32,0x7d,0x0f,0x00,
  0xe9,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0xf9,0xf9,0x0f,0x0f,0x0f,0xf9,0xf9,
  0xe9,0xf9,0xe9,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0xf9,0x0f,0x0f,0x0f,0xe9,0xf9,0x0f,
  0x0f,0xe9,0xe9,0x0f,0xe9,0xe9,0x0f,0xf9,0xe9,0xf9,0x0f,0x0f,0x0f,0x0f,0xf9,0x0f,
  0x40,0x00,0xfc,0x00,0xfc,0x40,0x40,0x00,0x00,0x00,0x00,0x00,0xb9,0x00,0x3d,0x32,
  0x0f,0xbe,0xb9,0x40,0xfc,0xfc,0x32,0x32,0x32,0x00,0x32,0x32,0xb9,0xb9,0x32,0x00,
  0x00,0x00,0x00,0x00,0x40,0x00,0x40,0x40,0x40,0x00,0x00,0x40,0xbe,0x00,0x3d,0x32,
  0x0f,0x32,0xb9,0x40,0x3d,0x00,0x40,0x01,0xff,0xff,0x40,0x00,0x01,0x40,0x00,0x00,
  0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x01,0x02,0x02,0x01,0xb9,0x3d,0x40,0x00,
  0x32,0xbe,0xe7,0x3d,0x40,0x02,0x01,0x01,0x02,0x01,0x01,0x01,0x3d,0x3d,0x01,0x01,
  0x02,0x02,0x01,0x5f,0x02,0x02,0x5f,0x02,0x03,0x01,0x01,0x01,0xb9,0x32,0x01,0x00,
  0xb9,0x32,0x32,0x3d,0x40,0x5f,0x03,0x1f,0x5f,0x03,0x03,0x02,0x5f,0x5f,0x01,0x01,
  0x5f,0x3d,0x3d,0x3d,0x3d,0x01,0x01,0x01,0x01,0x40,0x40,0x3d,0x01,0x01,0xbe,0x40,
  0x00,0x40,0x5f,0x00,0xfc,0x3d,0x40,0x3d,0x3d,0x01,0x01,0x02,0x01,0x01,0xff,0x01,
  0x02,0x5f,0x03,0x5f,0x01,0x5f,0x01,0x5f,0x5f,0x5f,0x5f,0x01,0x40,0x5f,0xbe,0x00,
  0xbe,0x00,0x01,0x01,0x32,0x5f,0x5f,0x02,0x03,0x1f,0x03,0x5f,0x5f,0x03,0x5f,0x5f,
  0x32,0xb9,0xb9,0xb9,0xf9,0xf9,0xe5,0xf9,0xb9,0xb9,0xb9,0xb9,0xff,0xbe,0xb9,0x0f,
  0xf7,0x0f,0x40,0xf9,0x0f,0x0f,0x0f,0xbe,0xb9,0xbe,0xbe,0xbe,0xbe,0xbe,0xb9,0xb9,
  0xbe,0xb9,0x32,0x32,0x32,0x32,0x7d,0x7d,0x0f,0x32,0x32,0xbe,0x40,0x40,0xb9,0xb9,
  0x0f,0xbe,0x01,0xbe,0x0f,0x0f,0x0f,0xbe,0xb9,0x0f,0xb9,0xb9,0xbe,0x32,0x32,0x0f,
  0x01,0x00,0xbe,0x01,0x01,0x00,0x01,0x00,0xe9,0x0f,0x6b,0x3d,0x40,0x6b,0x00,0xbe,
  0xb9,0xb9,0xb9,0xb9,0x00,0xbe,0x00,0xb9,0xb9,0xbe,0x40,0x02,0xb9,0xbe,0xb9,0xb9,
  0xb9,0xb9,0xbe,0x00,0x01,0x01,0xbe,0x00,0x40,0x01,0x01,0x40,0x32,0x32,0x6b,0x40,
  0x40,0x40,0x00,0x32,0x03,0xcb,0xb9,0xe9,0xb9,0x5f,0x00,0xff,0x00,0xb9,0x0f,0x01,
  0x02,0x01,0x01,0x1f,0x03,0x01,0xbe,0x0f,0x6b,0x01,0x5f,0xc3,0x03,0x1f,0xc3,0xc3,
  0x03,0x03,0x1f,0x5f,0xc3,0x03,0x02,0x40,0x01,0x02,0x02,0x03,0x02,0x1f,0x03,0xc3,
  0x1f,0x1f,0x1f,0x02,0xc3,0x1f,0x03,0x03,0x1f,0x1f,0x1f,0x03,0x02,0x03,0xc3,0xc3,
  0xe3,0xe3,0xc3,0x02,0xc3,0x03,0x5f,0x3d,0xe9,0xbe,0x01,0x03,0x1f,0x40,0x6b,0x5f,
  0x5f,0x3d,0xc2,0x3d,0x01,0xb9,0x0f,0xe7,0x40,0x02,0x1f,0x1f,0x03,0x1f,0x1f,0x1f,
  0x03,0x03,0x03,0x02,0x01,0x02,0x02,0xff,0xff,0x03,0x5f,0x1f,0xc6,0x03,0x1f,0xf8,
  0x1f,0x1f,0x1f,0x01,0x1f,0x1f,0x1f,0x03,0x03,0xe3,0xe3,0x1f,0x02,0xc6,0xc3,0xc3,
  0xc3,0xe3,0xf8,0x5f,0xc3,0x1f,0x01,0xff,0x3d,0xb9,0xb9,0x01,0x01,0x40,0x6b,0x5f,
  0x5f,0x01,0x3d,0x3d,0x0f,0xe9,0x00,0x40,0x02,0x01,0x02,0x03,0x03,0x03,0x1f,0xf8,
  0x1f,0x1f,0x03,0x02,0x02,0x5f,0x40,0x40,0x40,0x02,0x01,0x03,0x5f,0x03,0x1f,0xe3,
  0x1f,0x03,0x03,0xc2,0xc3,0x03,0x03,0x02,0x03,0x1f,0x1f,0x1f,0x5f,0xc6,0xf8,0xe3,
  0xc3,0xc3,0xc3,0xc2,0xc3,0x1f,0xc6,0x5f,0x03,0xc2,0xb9,0xf9,0x3d,0x32,0x00,0x02,
  0x5f,0xc2,0xcb,0x40,0xe9,0x00,0x02,0x02,0x02,0x02,0x02,0x03,0x5f,0x03,0xc3,0x1f,
  0x03,0x03,0x03,0x5f,0xc3,0x03,0x02,0x01,0x01,0x02,0x02,0x1f,0x03,0xf8,0x5f,0x03,
  0x1f,0x1f,0x03,0x01,0xe3,0x1f,0x1f,0x03,0x03,0x1f,0x1f,0xe3,0x5f,0xc6,0xf8,0xc3,
  0xe3,0xc3,0xc3,0x03,0xc3,0x03,0x5f,0x03,0x02,0x01,0x3d,0x0f,0x32,0xcb,0x6b,0x5f,
  0x5f,0x3d,0x01,0x0f,0x32,0x03,0x1f,0x03,0x02,0x1f,0x1f,0x03,0x03,0x03,0xc3,0x1f,
  0x03,0xe3,0x1f,0x02,0xc3,0x02,0x02,0x01,0x02,0x02,0x01,0x5f,0x5f,0x03,0x03,0xec,
  0x1f,0x03,0x03,0x02,0xe3,0x1f,0x1f,0x03,0x1f,0x03,0x03,0x1f,0x5f,0xc6,0xc3,0xc3,
  0xe3,0x1f,0x1f,0x01,0x1f,0x02,0x01,0x01,0x02,0x02,0x02,0x32,0xf9,0xe7,0x40,0x03,
  0x5f,0x01,0x32,0x0f,0x03,0x1f,0x02,0x01,0x01,0x03,0x02,0x1f,0x03,0x02,0x02,0x02,
  0x02,0x02,0x01,0x02,0x03,0x02,0x01,0x40,0x01,0x01,0x01,0x03,0x02,0x02,0x02,0x02,
  0x01,0x01,0x02,0xc2,0x03,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0xe3,0x02,0x02,0x5f,0xec,
  0xc6,0x02,0xc6,0x02,0x03,0x1f,0x03,0x03,0x02,0xc2,0xc6,0xc2,0x32,0x0f,0x3d,0x5f,
  0x5f,0xc2,0x32,0x3d,0x03,0x1f,0x03,0x02,0x01,0x40,0x01,0xc6,0x03,0x1f,0x1f,0x03,
  0x02,0x03,0x03,0x03,0x00,0x40,0x02,0x01,0x02,0x02,0x40,0x02,0x01,0x03,0x03,0x03,
  0x02,0x03,0x03,0x03,0x02,0x02,0xc6,0xc2,0x02,0xc6,0x03,0x02,0x03,0x03,0x1f,0x1f,
  0x03,0x03,0x1f,0x03,0x01,0x02,0xc6,0x03,0x02,0xc2,0xc6,0x02,0x00,0x32,0x3d,0x5f,
  0x5f,0x3d,0x32,0x01,0x01,0x1f,0xff,0x01,0x01,0x01,0x02,0x02,0x1f,0x1f,0x03,0x03,
  0x02,0x02,0x1f,0x03,0x40,0x01,0x02,0xff,0x40,0x40,0x01,0x3d,0xf8,0x03,0x02,0x02,
  0x02,0x01,0x02,0xc6,0x02,0x02,0x02,0x01,0x02,0x02,0x1f,0xc2,0x1f,0x03,0x03,0x02,
  0x03,0x03,0x03,0x03,0x01,0x01,0xff,0x40,0x00,0x40,0x02,0x01,0x01,0x32,0x40,0x5f,
  0x5f,0x3d,0x32,0x5f,0x01,0x02,0x02,0x01,0x01,0x03,0x02,0x02,0x1f,0xec,0x02,0x1f,
  0x03,0x03,0x03,0x1f,0xc2,0x01,0x01,0x40,0x40,0x02,0x01,0xc2,0x03,0x02,0x02,0x03,
  0x03,0xec,0x02,0x03,0x02,0xec,0x03,0xc6,0x02,0x1f,0x1f,0x00,0x1f,0x02,0x03,0x1f,
  0x03,0x03,0x03,0x03,0x02,0x01,0x02,0xc2,0x40,0x02,0x03,0x01,0x03,0x32,0xc2,0x5f,
  0x03,0x40,0xbe,0x02,0x01,0x02,0x03,0x02,0x03,0x02,0x03,0x5f,0xc3,0x03,0x5f,0x03,
  0x03,0x03,0x02,0xc3,0xc2,0x02,0x01,0xff,0x40,0x01,0x02,0x40,0x1f,0xec,0x02,0x03,
  0xec,0x03,0x03,0xf8,0x5f,0x1f,0x1f,0xf8,0x1f,0x1f,0xc3,0x01,0x1f,0x03,0x03,0xc3,
  0xc3,0x03,0x1f,0x03,0x5f,0x02,0x01,0x40,0x01,0x03,0x03,0x40,0x03,0xbe,0xc2,0x5f,
  0x5f,0x40,0x00,0x01,0x40,0x40,0x02,0x01,0x01,0x03,0x02,0x02,0x1f,0x03,0x02,0x03,
  0x03,0x03,0x01,0x1f,0x40,0x01,0x01,0x40,0xff,0x01,0x40,0x40,0x03,0x03,0x02,0xc6,
  0x02,0x02,0x02,0x1f,0x02,0x03,0x03,0xc6,0x03,0x03,0x03,0x3d,0x1f,0x03,0x02,0x03,
  0x03,0x03,0x02,0x03,0x02,0x02,0x03,0x02,0x02,0x03,0x1f,0xc2,0x1f,0x6b,0x3d,0x5f,
  0x5f,0x3d,0x6b,0x02,0xff,0x01,0x01,0x40,0x01,0x01,0x02,0x5f,0x03,0x02,0x02,0x02,
  0x02,0x02,0x03,0x03,0x40,0x02,0x01,0x40,0xff,0x40,0x40,0x40,0x1f,0x03,0x5f,0x02,
  0x02,0x02,0x03,0x03,0x01,0x03,0x1f,0x02,0x03,0x03,0x03,0x00,0x03,0xec,0x02,0x02,
  0x02,0x02,0x02,0x1f,0x02,0x03,0x1f,0x03,0x03,0x03,0x1f,0x40,0x03,0x00,0x3d,0x5f,
  0x01,0x00,0x6b,0x02,0x01,0x02,0x02,0x02,0x3d,0x02,0x02,0x5f,0x1f,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x01,0x3d,0x00,0x00,0x00,0x40,0x40,0x40,0x03,0xc6,0x01,0x01,
  0xec,0x03,0x03,0x02,0x01,0x01,0xec,0x01,0x02,0x02,0x02,0xb9,0x02,0xc2,0x02,0x03,
  0x03,0x03,0x1f,0xc6,0x01,0x01,0x01,0xc2,0xc2,0x02,0x03,0x01,0x03,0x00,0x3d,0x02,
  0x5f,0x40,0x32,0x6b,0x6b,0x40,0x40,0x40,0x40,0xc2,0x01,0x5f,0x5f,0xc6,0x03,0x02,
  0x02,0x02,0x02,0x01,0x40,0x01,0xff,0x40,0x40,0x01,0x02,0x02,0x02,0x02,0x02,0x01,
  0x02,0x01,0x02,0x03,0x01,0x40,0xc6,0x02,0x02,0x03,0x02,0x00,0x40,0x03,0xc3,0xc3,
  0xe3,0x1f,0x03,0x5f,0x01,0x02,0x01,0x01,0x01,0x02,0x1f,0x02,0x02,0x40,0xcb,0x5f,
  0x5f,0x3d,0x32,0x3d,0x01,0x03,0x03,0x02,0x02,0x02,0x40,0x02,0x03,0x40,0x01,0x40,
  0x00,0x40,0x40,0xb9,0x00,0x40,0x40,0xff,0x40,0x01,0x01,0x02,0x40,0x6b,0x40,0x00,
  0x6b,0x00,0x00,0x6b,0x01,0x02,0x02,0x40,0x02,0xc6,0x03,0x01,0x00,0x40,0x03,0x1f,
  0x03,0x03,0x5f,0x6b,0x01,0xc2,0x01,0x01,0x03,0x01,0x02,0x02,0x02,0x00,0x3d,0xc6,
  0x5f,0x40,0x32,0x01,0x02,0x1f,0x03,0x01,0x40,0x01,0x01,0x03,0x01,0x02,0x01,0x01,
  0xbe,0x40,0x00,0xb9,0xff,0x01,0x40,0x01,0x40,0x40,0x01,0x01,0x40,0x01,0x00,0x00,
  0x00,0x40,0x01,0x6b,0x03,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x00,0x3d,0x02,0x02,
  0x03,0x1f,0xc6,0x6b,0x03,0x02,0x02,0x02,0x02,0x01,0x01,0x03,0x02,0x6b,0x01,0x03,
  0x5f,0x00,0xbe,0x40,0xec,0x03,0x03,0x02,0x02,0x02,0x02,0x03,0x02,0xc6,0x02,0x02,
  0x40,0x40,0x40,0x00,0x02,0x02,0x40,0x40,0x00,0x40,0x01,0x01,0x00,0x01,0x40,0x01,
  0x40,0x40,0xff,0x6b,0x02,0x00,0x00,0x40,0x40,0x40,0x02,0x01,0x00,0x01,0x03,0x03,
  0x03,0x03,0x03,0x00,0x1f,0x03,0x02,0x01,0x01,0x40,0xc6,0x02,0x02,0xbe,0x02,0x03,
  0x5f,0x40,0xb9,0x00,0x02,0x01,0x01,0x00,0x40,0x00,0x40,0x02,0x3d,0xc2,0x02,0x02,
  0x01,0x01,0x3d,0x00,0x01,0x01,0x40,0x00,0x00,0x40,0x00,0x01,0x3d,0x02,0x40,0x40,
  0x6b,0x6b,0x40,0xbe,0x02,0x40,0x40,0x40,0x40,0x01,0x02,0x02,0x40,0x01,0x03,0x02,
  0x5f,0x5f,0x02,0x00,0x03,0x01,0x40,0x40,0x01,0x40,0x01,0x00,0x00,0x0f,0xc2,0x03,
  0xcb,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0xcb,0xc2,0x3d,0x3d,0x3d,0xcf,0xcf,0xcf,0xcf,
  0xcf,0xcf,0x3d,0xcf,0xcf,0x3d,0xc2,0xcb,0xcb,0xcb,0xcb,0x5f,0xc2,0xcb,0xcb,0x5f,
  0x5f,0x5f,0xcb,0xcb,0xcb,0xcb,0xcb,0xc2,0x3d,0x3d,0x3d,0xcf,0xcf,0xcf,0x6b,0x6b,
  0xcf,0xcf,0xcf,0x6b,0xcf,0xcf,0xcf,0xcf,0xc2,0x3d,0xcb,0xcb,0x5f,0x5f,0x5f,0x5f,
  0xcf,0x7d,0xe9,0x79,0xf9,0xe9,0xf9,0x0f,0x0f,0xb9,0x0f,0x79,0x0f,0x0f,0x0f,0x32,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x32,0x0f,0x0f,0x0f,0xb9,0xb9,0x0f,0x32,0xe7,
  0xe7,0x32,0xe9,0x79,0xf9,0xf9,0x0f,0x0f,0xb9,0x0f,0x0f,0x0f,0xb9,0x0f,0x0f,0x32,
  0x6b,0x32,0x7d,0x0f,0x7d,0x7d,0x32,0x0f,0x32,0x0f,0x0f,0x32,0x32,0x0f,0x32,0xcf,
  0x3d,0xcf,0x5f,0xcb,0xcb,0x3d,0xcf,0xcf,0x3d,0x3d,0xcf,0xcb,0xcb,0xcb,0xcb,0xcb,
  0xcb,0xcf,0xcb,0xcb,0xcf,0xcf,0xcf,0xcf,0xcf,0x3d,0xcf,0xcb,0x5f,0xcb,0xe7,0xcb,
  0xcb,0x6b,0x5f,0xcb,0xcb,0xcb,0xc2,0x3d,0xc2,0xcb,0xc2,0xcb,0xc6,0x5f,0xc6,0xc6,
  0xc6,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xc2,0xc2,0xcf,0xcf,0xcb,0xcb,0x32,0xcb,
  0xe7,0x7d,0x32,0x32,0x32,0x32,0x32,0xcf,0x32,0x6b,0xcf,0xcf,0x3d,0xe7,0x00,0x3d,
  0xcf,0xcf,0xcf,0xe7,0xcf,0x32,0x32,0xcf,0x6b,0x32,0x32,0xe7,0xe7,0x32,0x32,0x3d,
  0xc2,0x32,0x32,0x32,0xcf,0xcf,0xcf,0x32,0xcf,0xcf,0xcf,0xe7,0xe7,0xcf,0x00,0x3d,
  0x3d,0xe7,0x3d,0x3d,0x3d,0x3d,0x3d,0xe7,0xcf,0x32,0x32,0xe7,0xcf,0x32,0x0f,0xcf,
  0x7d,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x7d,0x32,
  0x32,0x7d,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x32,
  0x32,0x0f,0xcf,0x3d,0x3d,0x3d,0xc2,0xc2,0xcb,0x3d,0x01,0xcb,0xc2,0x5f,0x5f,0x5f,
  0xcb,0xcb,0xcb,0xcb,0x3d,0xcb,0xcb,0xcb,0xcb,0x3d,0x3d,0x3d,0xe7,0x32,0x0f,0x32,
  0x32,0xb9,0xe7,0x3d,0x3d,0xcb,0x5f,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0x5f,0xcb,
  0xcb,0x5f,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xc2,0x3d,0xcb,0x3d,0x0f,0x32,
  0x0f,0x0f,0x6b,0x6b,0x32,0x6b,0x32,0x32,0x32,0x32,0x32,0x32,0x6b,0x6b,0x6b,0x6b,
  0xcf,0x6b,0x32,0xcf,0x6b,0x32,0x32,0x32,0x6b,0x6b,0x6b,0x6b,0x32,0x7d,0x0f,0x6b,
  0x32,0x0f,0x6b,0xcf,0xcf,0x6b,0xcf,0x6b,0x32,0x6b,0x32,0x32,0x6b,0x32,0x6b,0x6b,
  0x6b,0x6b,0x32,0x32,0x32,0x32,0x32,0x32,0x32,0x32,0x32,0xcf,0x32,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x3d,0x40,0xcf,0xe7,0xcf,0x6b,0xcf,0x6b,0x6b,0x32,0x6b,0x32,0x6b,0x3d,
  0x3d,0x6b,0x6b,0x32,0x6b,0x00,0x6b,0x6b,0x6b,0x32,0x32,0x00,0x32,0x32,0x0f,0x32,
  0x32,0x0f,0xcf,0x6b,0xcf,0x3d,0xe7,0x6b,0xcf,0x6b,0x6b,0x6b,0x6b,0xcf,0xcf,0xcf,
  0x6b,0x6b,0xcf,0xcf,0x6b,0x3d,0xe7,0x6b,0xe7,0xcf,0x32,0x32,0x6b,0x32,0x0f,0x0f,
  0xf9,0xf9,0xf9,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0xe9,0xf9,0x0f,
  0x0f,0xe9,0xe9,0x0f,0x0f,0xf9,0x0f,0x0f,0xf9,0xf9,0x0f,0x0f,0x0f,0x0f,0x0f,0xf9,
  0xe9,0xf9,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x79,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0xf9,0xf9,
  0x00,0x40,0x01,0x01,0x40,0x40,0x01,0x01,0x01,0x3d,0x3d,0x01,0x0f,0x40,0x3d,0x32,
  0xb9,0xb9,0xbe,0x01,0x40,0x00,0x00,0x3d,0x3d,0x3d,0x00,0x3d,0x40,0x00,0x00,0x00,
  0x00,0x32,0xbe,0x32,0x32,0x32,0x00,0x00,0x32,0x00,0x00,0x00,0xb9,0x00,0x01,0x00,
  0x0f,0x32,0xb9,0x01,0x40,0x40,0x40,0x00,0x00,0x40,0x01,0x00,0x40,0x00,0x00,0x40,
  0x01,0x02,0x5f,0x5f,0x03,0x03,0x03,0x5f,0x1f,0x03,0x5f,0x01,0x6b,0xc2,0x00,0x00,
  0xfc,0xbe,0x3d,0x3d,0x40,0xcb,0x01,0x03,0x02,0x5f,0x02,0x5f,0x5f,0x01,0x5f,0x02,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x01,0x02,0x01,0xb9,0x3d,0x3d,0x00,
  0xbe,0x32,0x00,0x3d,0x40,0x02,0x02,0x01,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,
  0x5f,0xcb,0x5f,0x5f,0x5f,0x03,0x03,0x5f,0x02,0x5f,0x02,0x01,0x5f,0x3d,0x00,0x00,
  0x32,0x01,0x5f,0x32,0x3d,0x5f,0x5f,0xcb,0x5f,0x01,0xcb,0x01,0xcb,0x03,0x5f,0x02,
  0x01,0xff,0x01,0x01,0x01,0x01,0x01,0x00,0xe7,0x00,0x40,0x40,0x01,0x5f,0xbe,0x40,
  0xbe,0x40,0x5f,0x3d,0xfc,0xff,0x40,0xff,0x01,0x3d,0x01,0x40,0x3d,0x3d,0x3d,0x5f,
  0xf9,0xb9,0x0f,0xb9,0x0f,0x0f,0xf9,0x0f,0xb9,0xf9,0x0f,0x0f,0xb9,0xb9,0xbe,0x0f,
  0x0f,0xb9,0x40,0xb9,0xbe,0xb9,0x32,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0xb9,
  0xf7,0x0f,0xb9,0xb9,0xb9,0xb9,0xb9,0x0f,0x0f,0x0f,0xf9,0x0f,0xbe,0x00,0xb9,0xf7,
  0xf9,0xb9,0x40,0x0f,0x0f,0x0f,0xf9,0xf9,0xe5,0xe5,0xf9,0x0f,0xf9,0x0f,0x0f,0x0f,
  0x40,0x00,0x00,0x40,0xc2,0x40,0x40,0x40,0x40,0x40,0x40,0x00,0xb9,0x40,0x02,0x02,
  0x40,0x01,0x40,0x40,0x01,0x01,0x03,0xbe,0x01,0x3d,0xc3,0xe3,0xca,0x1f,0xc3,0xc3,
  0xca,0xc3,0xc3,0xc3,0x01,0x03,0x03,0x02,0xf8,0x1f,0xb9,0x01,0x00,0x40,0x01,0x40,
  0x32,0x00,0x00,0xb9,0x01,0xbe,0xbe,0x0f,0xb9,0xbe,0x00,0x40,0xb9,0x00,0x3d,0x40,
  0x1f,0x1f,0x03,0xc6,0xec,0x1f,0x1f,0x03,0x1f,0xf8,0xe3,0x03,0x01,0x1f,0xe3,0xc3,
  0xe3,0x1f,0x1f,0xe3,0xc3,0x1f,0x03,0x6b,0x5f,0x01,0x1f,0xf8,0xca,0x5f,0x1f,0xc3,
  0x1f,0xf8,0xf8,0xc3,0x5f,0xc3,0x1f,0x5f,0xc3,0xc3,0x40,0x1f,0x02,0x1f,0xc3,0x1f,
  0x03,0x03,0xc6,0x40,0x03,0x02,0x02,0x40,0x01,0x03,0x03,0xf8,0x5f,0x5f,0xc3,0x1f,
  0x1f,0x1f,0x5f,0x03,0x1f,0x03,0x03,0x02,0x03,0x1f,0x1f,0x1f,0x01,0x1f,0xe3,0xe3,
  0xc3,0xc3,0x1f,0x03,0x1f,0x1f,0x1f,0x6b,0x5f,0x03,0xe3,0x5f,0xca,0x03,0x03,0xc3,
  0xc3,0x1f,0xf8,0xc3,0x03,0xc3,0xc3,0x5f,0xf8,0x01,0xbe,0x1f,0x03,0x1f,0xc3,0x1f,
  0x1f,0x1f,0x03,0x40,0x1f,0x01,0x40,0x40,0x40,0x02,0x02,0x1f,0x5f,0x02,0xc3,0xf8,
  0x1f,0x1f,0x02,0x02,0x1f,0x1f,0x1f,0x03,0x1f,0x1f,0xe3,0xf8,0x02,0x1f,0xc3,0xc3,
  0xc3,0xc3,0xc3,0xc3,0xc3,0x1f,0x1f,0x32,0x5f,0xc6,0x1f,0x1f,0xca,0x1f,0x03,0x03,
  0x03,0xf8,0xf8,0xe3,0x03,0xe3,0xf8,0x03,0xf8,0xf8,0xb9,0x1f,0x02,0x1f,0xc3,0x1f,
  0x03,0x03,0xc6,0xc2,0xc3,0x03,0x01,0x01,0x01,0x02,0x5f,0xf8,0x03,0x03,0x1f,0x1f,
  0x1f,0x03,0xec,0x02,0x1f,0x1f,0x1f,0xc6,0x03,0x1f,0x03,0xe3,0x02,0x03,0xc3,0xf8,
  0x1f,0x1f,0xc6,0x03,0x03,0xc6,0x03,0x6b,0x02,0xc6,0x03,0x03,0xc3,0xc6,0x1f,0x1f,
  0x03,0x1f,0xe3,0xf8,0x5f,0xe3,0xf8,0x03,0x03,0xf8,0xbe,0x1f,0x02,0x03,0xc3,0xc6,
  0x03,0xf8,0x1f,0x01,0xc3,0x02,0x02,0x01,0x01,0x01,0x02,0x03,0x01,0x03,0x03,0x02,
  0x02,0x02,0xc6,0xc2,0xc6,0x1f,0xe3,0xe3,0x1f,0x1f,0x1f,0xe3,0x02,0x03,0xc6,0x03,
  0x1f,0xc6,0x1f,0x5f,0x1f,0x03,0x1f,0x32,0x01,0x02,0xc6,0xc6,0xe3,0x1f,0xc6,0x03,
  0x1f,0x5f,0x03,0xf8,0x02,0x5f,0xf8,0x5f,0x5f,0xc6,0xbe,0x03,0x03,0x5f,0x1f,0x03,
  0x03,0x03,0xc6,0x5f,0x40,0x02,0x02,0x01,0x01,0x02,0x01,0x5f,0x03,0x02,0x03,0xc6,
  0x02,0x02,0x03,0xc6,0x02,0xc6,0xec,0xec,0x03,0x03,0x1f,0x03,0x02,0x03,0x1f,0x03,
  0x03,0x03,0x1f,0x02,0x02,0x1f,0x1f,0xb9,0xe3,0x02,0xc6,0x02,0x1f,0x02,0x5f,0x5f,
  0x03,0x01,0x01,0xf8,0xc2,0x5f,0x5f,0xc2,0x01,0x5f,0xbe,0xf8,0x1f,0x03,0x1f,0xc6,
  0x02,0x02,0x02,0x02,0x6b,0x02,0x01,0xc2,0x01,0x02,0x02,0xc6,0x02,0x02,0x02,0x03,
  0xc6,0x01,0x01,0x02,0x02,0x02,0x01,0xff,0x01,0x5f,0xc6,0x03,0x03,0x03,0x03,0xec,
  0x03,0x03,0x1f,0xc6,0xc2,0x02,0x02,0xb9,0xe3,0xc2,0xc6,0xc2,0xc6,0xc2,0x3d,0xc2,
  0xc6,0x40,0x3d,0x5f,0x3d,0x5f,0xc2,0x00,0xc2,0x5f,0xbe,0x02,0x1f,0x1f,0x1f,0x02,
  0x01,0x03,0x1f,0x03,0x40,0x01,0x01,0x40,0x40,0x40,0x01,0xc2,0x1f,0x03,0x5f,0xc6,
  0x03,0x01,0x02,0x01,0x01,0x03,0xc6,0x03,0x5f,0x1f,0x03,0x02,0x03,0x03,0x03,0x03,
  0x03,0x03,0x1f,0x03,0x3d,0x03,0x03,0x0f,0x02,0x6b,0x02,0x6b,0xcb,0x3d,0x3d,0x40,
  0x3d,0x40,0x01,0x01,0x3d,0x02,0x3d,0xbe,0x3d,0x01,0xb9,0xcb,0xc3,0x03,0x02,0x03,
  0x03,0x03,0x03,0x03,0x40,0x01,0x02,0x40,0x40,0x01,0x01,0x40,0x1f,0x03,0x03,0x5f,
  0xff,0x03,0x1f,0x1f,0x02,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x02,0x03,0x03,0x1f,0xc3,
  0x1f,0x1f,0x03,0x1f,0xc2,0x1f,0xc6,0x0f,0xc2,0x6b,0xcb,0x6b,0xc2,0x3d,0xc2,0x3d,
  0x3d,0x40,0xc2,0x01,0x6b,0x01,0xc2,0x6b,0x40,0x00,0x00,0x5f,0xc3,0x03,0x01,0xf8,
  0x03,0x03,0x02,0x1f,0x01,0x02,0x02,0xff,0x40,0x01,0x02,0x3d,0x1f,0xc6,0x03,0xec,
  0xff,0x01,0xc6,0x03,0x01,0x1f,0x02,0x03,0x03,0x03,0x03,0x5f,0x03,0xec,0x03,0x1f,
  0x03,0x03,0x02,0x1f,0xc2,0x1f,0x1f,0x0f,0x3d,0x6b,0x02,0x6b,0xc2,0x3d,0x40,0x3d,
  0x6b,0xb9,0x40,0x3d,0x6b,0xc2,0x40,0x32,0x00,0x01,0xbe,0x01,0xc3,0x03,0x02,0xe3,
  0x1f,0x03,0x02,0x1f,0xc2,0x01,0x02,0x01,0x01,0x02,0xff,0x40,0x1f,0x03,0x02,0x02,
  0x02,0xec,0x03,0x02,0x01,0xf8,0x03,0xc6,0x03,0x03,0x02,0x01,0x02,0x02,0x02,0xec,
  0xec,0x02,0x03,0x1f,0xc2,0xc3,0x1f,0xe9,0x6b,0x6b,0x40,0xb9,0x40,0x40,0x6b,0x32,
  0x40,0xb9,0x6b,0x6b,0x32,0xc2,0x6b,0xb9,0x6b,0x00,0xb9,0x02,0x1f,0x02,0x02,0x02,
  0x03,0xec,0x02,0x1f,0x40,0x01,0x01,0x40,0x00,0x40,0x01,0x40,0x1f,0x03,0x5f,0x02,
  0x02,0x02,0x02,0x02,0xc2,0x03,0x02,0x01,0x02,0x02,0xc2,0x00,0x01,0x01,0x03,0x03,
  0x03,0x03,0x1f,0xc6,0x40,0x03,0xc6,0xe9,0x6b,0x6b,0x40,0x6b,0x3d,0x6b,0x6b,0xb9,
  0xb9,0x6b,0x40,0x40,0x0f,0xb9,0x0f,0x0f,0xb9,0x0f,0x6b,0xc6,0xc3,0x03,0x03,0x03,
  0x03,0x02,0x03,0x03,0x40,0x40,0x40,0x00,0x40,0x40,0xc2,0x40,0x1f,0x03,0xff,0xc2,
  0x01,0xff,0xec,0x03,0x40,0xc2,0x02,0x02,0x01,0x02,0x40,0x00,0x01,0x03,0x1f,0x1f,
  0x1f,0x03,0x03,0x02,0x01,0x03,0xc6,0x0f,0xbe,0x6b,0xc2,0x6b,0x02,0xc2,0x00,0x0f,
  0x6b,0x32,0x6b,0x32,0x0f,0x0f,0x0f,0xf9,0x0f,0xb9,0xb9,0xc6,0xc6,0x03,0x02,0x02,
  0x02,0x02,0x02,0xc2,0x02,0x01,0x00,0x40,0x40,0x01,0x02,0x01,0x02,0x02,0x01,0x01,
  0x0f,0xb9,0xbe,0x0f,0xbe,0x6b,0x00,0xbe,0x00,0x00,0x00,0xb9,0x0f,0x32,0xc2,0x3d,
  0x40,0x40,0x00,0x0f,0xb9,0xbe,0x40,0x0f,0x0f,0xf9,0xf9,0xf9,0xf9,0xf9,0x0f,0xf9,
  0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xb9,0x40,0x40,0xf9,0x00,0xbe,0xb9,0x6b,0xbe,
  0xbe,0xb9,0xbe,0x0f,0x0f,0xb9,0xb9,0xb9,0xb9,0xbe,0xbe,0xfc,0x0f,0x0f,0xb9,0x0f,
  0xe5,0xe5,0xde,0xde,0xe5,0xe5,0xe5,0xe5,0xe5,0xde,0xfd,0xfd,0xfd,0xde,0xe5,0xe5,
  0xe5,0xe5,0xde,0xe5,0xe5,0xde,0xde,0xde,0xe5,0xde,0xe9,0xde,0xe5,0xde,0xce,0xf9,
  0xf9,0xf9,0xde,0xde,0xe5,0xe5,0xce,0xde,0xde,0xde,0xde,0xde,0xde,0xde,0xce,0xde,
  0xce,0xde,0xde,0xde,0xe5,0xfd,0xfd,0xde,0xde,0xde,0xde,0xeb,0xe5,0xde,0xe5,0xde,
  0xe9,0xe9,0xe9,0xce,0xe9,0xce,0xce,0xe9,0xce,0xce,0xf7,0xf7,0xf7,0xce,0xe9,0xe9,
  0xe9,0xce,0xce,0xce,0xe9,0xce,0xe9,0xce,0xce,0xce,0xe9,0xe9,0xe9,0xe9,0xce,0xf7,
  0xf7,0xf7,0xce,0xce,0xce,0xde,0xce,0xce,0xde,0xde,0xce,0xde,0xce,0xce,0xce,0xce,
  0xe9,0xce,0xce,0xce,0xe5,0xfd,0xfd,0xde,0xce,0xce,0xe9,0xce,0xce,0xce,0xce,0xe9,
  0xf0,0xf0,0xf0,0xf0,0xeb,0xeb,0xeb,0xf0,0xeb,0xe5,0xfd,0xfd,0xfd,0xeb,0xeb,0xe5,
  0xe5,0xe5,0xe5,0xe5,0xeb,0xe5,0xeb,0xe5,0xe5,0xeb,0xe5,0xeb,0xeb,0xeb,0xf0,0x74,
  0x74,0xeb,0xf0,0xeb,0xeb,0xe5,0xe5,0xe5,0xeb,0xe5,0xeb,0xeb,0xf0,0xf0,0xeb,0xf0,
  0xe5,0xf0,0xf0,0xf0,0x74,0x74,0x74,0xfa,0xf0,0xf0,0xf0,0xf0,0xfa,0xf0,0xf0,0xf0,
  0xf4,0xf4,0xf4,0xf4,0xf4,0xe6,0xf5,0xf5,0xf5,0xf4,0xf1,0xfe,0xfe,0xef,0xe6,0xef,
  0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xef,0xe6,0xe6,0xe6,0xf2,0xf5,0xf5,0xf1,
  0xfe,0xfe,0xe6,0xef,0xe6,0xe6,0xe6,0xef,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xf2,0xf5,
  0xe6,0xe6,0xf5,0xf4,0xfb,0xf6,0xfb,0xf4,0xf4,0xf4,0xf4,0xe6,0xf4,0xf4,0xf4,0xf5,
  // 1: wall wood
  0x53,0xb5,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,
  0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,0x94,0xc5,0x00,0x03,0x08,0x03,
  0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,
  0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,
  0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,
  0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,
  0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,
  0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,
  0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,
  0xf5,0xcd,0x12,0xb5,0x10,0x41,0x08,0x4a,0x18,0x41,0x00,0x42,0x10,0x42,0x18,0x42,
  0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,
  0x00,0x45,0x08,0x45,0xd0,0xac,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,
  0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,
  0xb0,0xac,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,
  0x10,0x62,0x18,0x62,0x09,0x6b,0x00,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,
  0x10,0x64,0x18,0x64,0xac,0x83,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,
  0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x31,0x84,
  0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,
  0x00,0x82,0x29,0x8b,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x10,0x83,0x18,0x83,
  0x00,0x84,0x08,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,
  0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,
  0x2f,0x94,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,
  0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x11,0xad,0x00,0xa3,0x74,0xbd,0x08,0xa3,
  0x10,0xa3,0x95,0xbd,0x18,0xa3,0x4e,0x9c,0xb2,0xc5,0x00,0xa4,0x08,0xa4,0x10,0xa4,
  0x18,0xa4,0x00,0xa5,0x08,0xa5,0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x51,0x94,
  0x18,0xa6,0xd6,0xcd,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,0xcc,0x93,0x00,0xc0,
  0x08,0xc0,0x10,0xc0,0x8f,0xac,0xf1,0xb4,0x18,0xc0,0x00,0xc1,0xec,0x8b,0x08,0xc1,
  0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x4f,0xa4,0x12,0xad,0x10,0xc2,0x18,0xc2,
  0x73,0xc5,0x00,0xc3,0x08,0xc3,0x0a,0x6b,0x10,0xc3,0x2e,0x84,0x16,0xce,0x18,0xc3,
  0x00,0xc4,0xe6,0x41,0x08,0xc4,0x8f,0x9c,0x10,0xc4,0x2d,0xa4,0x11,0xb5,0xae,0x83,
  0x4a,0x8b,0xd4,0x94,0xab,0x93,0x4c,0xa4,0xad,0xac,0xee,0xc4,0xcd,0xb4,0x4a,0x6b,
  0x4c,0x83,0x6b,0x73,0xcf,0xac,0x8e,0xb4,0xca,0x6a,0x15,0xa5,0x8b,0x83,0xef,0x8b,
  0xef,0xef,0xef,0xef,0xef,0xef,0xef,0x74,0xef,0xff,0xa8,0xef,0xef,0xef,0xa8,0xa8,
  0xef,0xef,0xef,0x74,0x74,0xef,0xef,0x74,0x74,0xf7,0x74,0xf7,0x74,0xf7,0x74,0xe3,
  0x6f,0xf9,0xe3,0xf9,0xf9,0xf9,0xe3,0xe3,0xe3,0xe3,0xe3,0xf9,0xe3,0xe3,0xe3,0xf9,
  0xe3,0xe3,0xe3,0xe3,0xe3,0xfc,0xe3,0xe3,0xe3,0x74,0x74,0x74,0xf9,0xef,0xef,0xef,
  0x91,0x91,0x91,0x91,0x91,0x91,0x7a,0xce,0x7a,0x7a,0x7a,0x7a,0x7a,0x7a,0x7a,0xf9,
  0x7a,0x7a,0xf9,0xf9,0xf7,0xe3,0xe3,0xe3,0xf9,0xf7,0xf7,0xe3,0xe3,0xe3,0xe3,0xe3,
  0x4b,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xf9,0xe3,0xf9,0xf9,
  0xe3,0xe3,0xe3,0xe3,0xf9,0xf9,0xf9,0xf7,0x72,0x72,0x91,0x91,0x91,0x91,0x91,0x91,
  0xf6,0xfb,0xf6,0xf6,0xf6,0xf3,0xd2,0x5a,0xdc,0xa8,0xa8,0xbb,0xbb,0xa8,0xa8,0xbb,
  0xd2,0xd2,0xff,0xff,0xa8,0xed,0xed,0xbb,0xbb,0xa8,0xa8,0xa8,0xbb,0xa8,0xa8,0xed,
  0x6f,0x7a,0x7a,0x7a,0xce,0xd6,0x7a,0xd6,0xce,0xce,0xce,0xed,0xce,0x7a,0xed,0xed,
  0xed,0xed,0x7a,0xd6,0xed,0xed,0xed,0x7a,0xf3,0xf3,0x91,0xf0,0xf2,0xce,0xf3,0xf6,
  0xfb,0xfb,0xfb,0xfb,0xf6,0xf6,0xd2,0xd3,0xd2,0xfa,0xfa,0xdc,0xff,0xa8,0xa8,0xa8,
  0xa8,0xdc,0xef,0xef,0xff,0xbb,0xce,0xe3,0x7a,0xef,0xef,0xef,0xce,0x7a,0xce,0xce,
  0x6f,0xed,0x7a,0x7a,0x7a,0x7a,0x7a,0x7a,0x7a,0x7a,0xd6,0xce,0xd6,0x7a,0x7a,0x7a,
  0x7a,0xd6,0xf9,0xf9,0x7a,0xd6,0x7a,0xf9,0xed,0xfb,0xf3,0xf6,0xf6,0xf6,0xfb,0xfb,
  0xf3,0xf3,0xf3,0xf3,0xf3,0xf2,0xfb,0xd3,0xff,0xd2,0xa8,0xff,0xff,0xff,0xa8,0xff,
  0xa8,0xd2,0xa8,0xa8,0xd2,0xfb,0xdd,0x00,0xdd,0xf6,0xed,0xd2,0xfb,0xf6,0xed,0xed,
  0x6f,0xce,0xed,0xed,0xf3,0xce,0xce,0xce,0xc7,0xfd,0xc7,0xce,0xed,0xed,0xce,0xce,
  0xce,0xce,0x7a,0x7a,0xce,0xf3,0xf3,0xd6,0xd2,0xfb,0x91,0xf2,0xce,0xed,0xed,0xf3,
  0xed,0xed,0xed,0xce,0xf2,0xf2,0xbb,0xee,0xbb,0xd2,0xa8,0xef,0xef,0xef,0xff,0xbb,
  0xd2,0xa8,0xf5,0xd2,0xef,0x87,0xb9,0x87,0x00,0x87,0xed,0xed,0xce,0xed,0xce,0x7a,
  0x6f,0x7a,0xf9,0x7a,0x7a,0x7a,0xd6,0xef,0xc8,0xc7,0xb9,0xff,0x7a,0xed,0xed,0x7a,
  0xce,0xd6,0xce,0xf3,0xf3,0xed,0xed,0xf3,0x5a,0xd2,0x91,0xf2,0xf2,0xf2,0xf3,0xce,
  0xd2,0xfb,0xfb,0xf6,0xf3,0xf3,0xfb,0xee,0x7a,0xf9,0xd2,0xef,0xef,0xef,0x87,0xc7,
  0xdc,0xa8,0xa8,0xbb,0xbb,0x87,0xc7,0xef,0xe3,0xef,0xfb,0xed,0xfb,0xed,0xed,0xf5,
  0x6f,0xed,0xed,0xf3,0xf3,0xf3,0xf3,0x87,0xc7,0xef,0xe3,0xff,0xce,0xed,0xf6,0xf3,
  0xc7,0xce,0xf8,0x7a,0x7a,0x7a,0xf9,0x7a,0xd3,0x5a,0xf3,0xf3,0xed,0xfb,0xfb,0xfb,
  0xed,0xf3,0xf2,0xf2,0xf2,0xf2,0xed,0xbc,0x74,0x7a,0xff,0xef,0xef,0xef,0xeb,0x87,
  0xff,0xef,0xef,0xef,0xef,0xef,0x87,0xfc,0xe3,0xa8,0xa8,0xa8,0xbb,0xa8,0xce,0xce,
  0x4b,0xd6,0xce,0x7a,0x7a,0x7a,0x7a,0x7a,0xc7,0x6f,0x74,0x7a,0xd6,0x7a,0xce,0x7a,
  0xeb,0xce,0xce,0xf3,0xf6,0xf5,0xf3,0xf5,0xbc,0xd2,0x91,0xf2,0xf2,0xf2,0xf3,0xed,
  0xef,0xc7,0xff,0x7a,0x7a,0xf9,0xf7,0x7a,0xbb,0xd2,0xef,0xef,0xef,0xef,0x5a,0xff,
  0xef,0xef,0xef,0xef,0xef,0xff,0xef,0xf9,0xf8,0xef,0xef,0xef,0xef,0xce,0xd6,0xce,
  0xfc,0xce,0xce,0xce,0xed,0xed,0xce,0xce,0x7a,0xe3,0xf9,0xce,0xce,0x7a,0x7a,0xfe,
  0x00,0xd6,0xce,0x7a,0xce,0xce,0xed,0xed,0x7a,0xf9,0xf9,0xf8,0x7a,0xe5,0xa8,0x7a,
  0xe3,0xbb,0xbb,0xa8,0xa8,0xd6,0x7a,0x7a,0xe5,0xa8,0xef,0xef,0xef,0xef,0xdd,0xa8,
  0xef,0xef,0xef,0xef,0xef,0xa8,0xef,0xef,0xef,0xef,0xa8,0xef,0xce,0xed,0xed,0x7a,
  0x72,0x7a,0x7a,0x7a,0xf9,0x7a,0x7a,0xd6,0x7a,0xf9,0xd6,0xd6,0xd6,0xd6,0x7a,0xfe,
  0x5a,0xce,0xce,0xce,0xd6,0xce,0xed,0xed,0x7a,0x7a,0x7a,0xe5,0xa8,0xbb,0xa8,0xe3,
  0x7a,0xf9,0xf9,0xf9,0xf7,0xf9,0xe3,0xf9,0xf7,0xef,0xef,0xef,0x74,0xe3,0xb9,0xd2,
  0x74,0xef,0x7a,0x7a,0x7a,0xce,0x7a,0xf8,0xf9,0xf9,0xf7,0xe3,0xf7,0x74,0x74,0xf9,
  0x6f,0x7a,0xf9,0xf7,0x7a,0x7a,0x7a,0xf8,0xf9,0x7a,0xce,0xfe,0xf8,0xfe,0xf8,0xd6,
  0xb9,0xdd,0xfe,0xf0,0xf2,0xf2,0xce,0xf3,0xfe,0x7a,0xce,0x7a,0x7a,0x7a,0x7a,0x7a,
  0xe3,0xe3,0xe3,0x7a,0xf8,0xf8,0xf8,0xf8,0x7a,0xf9,0xfc,0xe3,0xe3,0xe3,0xdd,0xa8,
  0x4b,0xfc,0xf9,0xf8,0x7a,0xd6,0x7a,0xa8,0xff,0x7a,0xf8,0xf8,0xf9,0xe3,0xe3,0xe3,
  0x4b,0xe3,0xe3,0xe3,0xe3,0xe3,0xfc,0xe3,0xfc,0xe3,0xe3,0xfc,0xe3,0xf9,0xe3,0xe3,
  0x5a,0xd6,0xf0,0xf0,0xfe,0xe3,0xfe,0xce,0xfe,0xce,0xf8,0xf8,0x7a,0xfe,0xf8,0xe3,
  0xe3,0xf9,0xf9,0xe3,0xf9,0xf9,0xf9,0xe3,0xf9,0xf9,0xf8,0xe3,0xf9,0xe3,0xc7,0xef,
  0xe3,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0xe3,0xf9,0xe3,0xf8,0x6f,
  0x72,0xe3,0xe3,0x72,0xf9,0xe3,0xe3,0xe3,0xf7,0xf9,0xf9,0xf9,0xf9,0xf9,0xf9,0x72,
  0xe3,0xf7,0xf9,0xf9,0xf7,0xe3,0xe3,0xe3,0xe3,0xe3,0xe3,0xf9,0xf9,0xf9,0xf7,0xe3,
  0xa8,0xeb,0xeb,0xeb,0xeb,0xeb,0xa8,0xa8,0xeb,0x5a,0xeb,0xeb,0x5a,0xd6,0xf9,0xd6,
  0xbb,0xeb,0xeb,0xeb,0xeb,0xeb,0xeb,0xeb,0xbb,0xeb,0xeb,0xa8,0xa8,0xd6,0xeb,0x7a,
  0xf9,0xbb,0xbb,0xbb,0xeb,0xbb,0xbb,0xed,0xbb,0xbb,0xbb,0xeb,0xeb,0xeb,0xbb,0x7a,
  0xef,0xe5,0xeb,0xbb,0xeb,0xa8,0xa8,0xa8,0xa8,0xa8,0xeb,0xc7,0xeb,0xc7,0xc7,0xa8,
  0xee,0xb6,0x00,0x00,0x49,0xb6,0x5a,0xb4,0x5a,0xb6,0x00,0x00,0x00,0x00,0xe5,0x00,
  0x00,0xb6,0x49,0x49,0xee,0x00,0xb6,0x00,0xee,0x49,0x00,0xb4,0xdd,0xee,0xee,0xed,
  0xa8,0xd3,0xee,0xee,0xb6,0xee,0xee,0xd3,0xd3,0x49,0x49,0x49,0xd3,0x49,0xd3,0xbb,
  0xe5,0xeb,0x49,0xdd,0x00,0xdd,0xb4,0x5a,0x49,0xb4,0x00,0x00,0xdd,0x00,0xdd,0xdd,
  0xb6,0x0d,0x0d,0xe0,0x0d,0xb6,0x49,0x49,0x00,0x48,0x48,0xc9,0x0d,0xb6,0xee,0x5a,
  0xb6,0x0d,0x0d,0x0d,0xe0,0xe0,0x0d,0x00,0x49,0x49,0x48,0x00,0xb6,0xb6,0xd3,0xfa,
  0xbb,0xee,0xbc,0x00,0x0d,0xe0,0xee,0xee,0x00,0xe0,0xb6,0x00,0x00,0xe0,0xe0,0xd3,
  0x5a,0x68,0x49,0x00,0x49,0x49,0xb4,0xdd,0x49,0x49,0xb6,0xb6,0xb6,0xb9,0x00,0x49,
  0xb6,0x48,0x48,0x0d,0x0d,0xe0,0xee,0x00,0x48,0xc9,0xc9,0x0d,0x48,0x48,0xb6,0xb6,
  0xee,0xc9,0x48,0xbc,0x48,0x48,0x0d,0x00,0x00,0xe0,0x0d,0xb6,0x00,0x00,0x00,0xee,
  0xbb,0xee,0xbc,0xe0,0x0d,0xe0,0x00,0xee,0xe0,0xe0,0xe0,0xbc,0xb6,0x0d,0x0d,0xee,
  0x5a,0x5a,0xee,0x00,0xb6,0x00,0x00,0x49,0x00,0x00,0x0d,0x0d,0xb6,0x0d,0x49,0x49,
  0xe0,0x48,0x48,0x48,0x0d,0xe0,0x00,0x49,0x0d,0xe6,0x48,0x0d,0x48,0xb6,0xb6,0xb6,
  0xb6,0xe6,0x48,0xe0,0x48,0x48,0xe0,0x00,0x49,0x00,0xe0,0x00,0xee,0xee,0x00,0xee,
  0xa8,0xee,0xbc,0xe0,0x48,0x0d,0xb6,0x00,0x00,0xe0,0xe0,0xe0,0xe0,0xe0,0xe0,0x00,
  0x00,0x5a,0x49,0x00,0xd3,0x49,0x00,0xb4,0x00,0x00,0x0d,0x0d,0x00,0xb6,0x49,0x5a,
  0x0d,0xe6,0xe6,0xe6,0x48,0xe0,0x49,0xee,0x0d,0xe6,0x00,0xb6,0xc9,0x48,0xb6,0xc9,
  0x49,0x48,0x48,0x0d,0x48,0x0d,0xe0,0xb6,0xb4,0xb6,0x0d,0xb6,0x00,0x00,0xee,0xd2,
  0xce,0xee,0xbc,0xe0,0x0d,0x0d,0xe0,0x00,0xb6,0xe0,0xe0,0xe0,0x00,0xe0,0xe0,0xd3,
  0x49,0xd3,0x49,0x00,0x49,0x49,0x49,0xb4,0x00,0xb6,0x0d,0x0d,0x49,0x00,0x00,0x5a,
  0xb6,0x48,0xe6,0xc9,0x0d,0x0d,0xb6,0xb6,0x0d,0x48,0xb6,0x48,0x48,0x48,0xe0,0x48,
  0x49,0xc9,0x0d,0x0d,0x48,0x0d,0x0d,0x00,0x49,0x0d,0x0d,0xb6,0xb6,0xb6,0x00,0xd2,
  0xbb,0xee,0xbc,0x0d,0x0d,0x0d,0xe0,0xe0,0x0d,0x0d,0x0d,0x0d,0xe0,0x0d,0xe0,0x68,
  0x49,0x49,0x49,0xb6,0x00,0x49,0xb6,0x5a,0x49,0x0d,0x0d,0x0d,0xb4,0x00,0x00,0xb4,
  0x0d,0xe6,0xe6,0xe6,0xe0,0x00,0xb6,0x0d,0xc9,0x48,0xc9,0x48,0x0d,0x48,0x48,0x0d,
  0x68,0xc9,0x0d,0xe0,0xe0,0xbc,0x0d,0xe0,0xee,0xe0,0x0d,0x0d,0xe0,0xe0,0xe0,0xd3,
  0xbb,0xee,0xee,0xe0,0xe0,0x0d,0xe0,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0xe0,0x49,
  0xd3,0x49,0x00,0x00,0x49,0xb6,0xb6,0x00,0xb6,0xb6,0x0d,0xb6,0x00,0x0d,0x00,0x00,
  0xc9,0xc9,0xe6,0x48,0xe0,0x49,0x0d,0x48,0x48,0xe6,0x48,0xc9,0x0d,0x0d,0x48,0xe0,
  0xd3,0x0d,0x48,0x0d,0x0d,0x48,0x0d,0x48,0xbc,0x0d,0x0d,0x0d,0xb6,0x48,0xbc,0xee,
  0xbb,0xf5,0xbc,0xe0,0xb6,0xbc,0xe0,0xe0,0x0d,0x0d,0x0d,0xe0,0x0d,0x0d,0xee,0xd3,
  0x49,0x49,0x49,0x49,0xd3,0x00,0x00,0x49,0xb6,0xb6,0xb6,0x0d,0x48,0x00,0xb6,0x00,
  0xc9,0xe6,0xe6,0x0d,0x00,0xd3,0x0d,0x0d,0x0d,0xe6,0x48,0xc9,0x0d,0x0d,0x0d,0x48,
  0x49,0x0d,0x48,0x0d,0x0d,0x48,0xe0,0x48,0xee,0xe0,0x48,0x48,0xe0,0xc9,0xbc,0xd2,
  0xbb,0xee,0xbc,0xe0,0xbc,0xe0,0x0d,0x48,0x0d,0x48,0x0d,0x0d,0x0d,0x00,0xb6,0xee,
  0xee,0xee,0x49,0xb6,0x00,0x00,0xb6,0x00,0xb6,0xb6,0x48,0x48,0xb6,0xb6,0xb6,0x00,
  0x48,0xe6,0x0d,0x0d,0x00,0x5a,0x0d,0x0d,0x48,0xe6,0x48,0xe6,0xc9,0x0d,0x48,0xc9,
  0xb6,0x00,0x48,0x0d,0x48,0x0d,0x00,0x0d,0xb6,0xee,0x48,0x0d,0x0d,0x48,0xbc,0xfa,
  0xbb,0xbc,0xe0,0xe0,0xe0,0xe0,0xe0,0x48,0x0d,0x0d,0x0d,0x0d,0x0d,0xb6,0xb6,0xee,
  0xdd,0x00,0x00,0xb6,0x49,0xb6,0xb6,0xb6,0xb6,0x00,0x48,0x48,0xb6,0x48,0x0d,0xb6,
  0xc9,0xe6,0x00,0x49,0x5a,0x5a,0x48,0xb6,0xc9,0xe6,0xe6,0xc9,0xc9,0x0d,0x48,0x48,
  0x00,0xb6,0x48,0xb6,0x0d,0x48,0x49,0xe0,0x00,0xb6,0x48,0x48,0x0d,0x0d,0xbc,0xfa,
  0xeb,0xbc,0xbc,0xe0,0xbc,0x0d,0xe0,0xbc,0xe0,0x0d,0x0d,0xb6,0x48,0x0d,0x5a,0xd3,
  0x5a,0xb6,0x49,0x00,0x00,0x00,0x0d,0xb6,0xb6,0xb6,0x48,0xe6,0xb6,0xb9,0xb6,0xb6,
  0xe6,0xe6,0x48,0xe0,0xdd,0xb4,0x00,0x0d,0x0d,0xc9,0x48,0x48,0x0d,0xb6,0xc9,0x48,
  0xb6,0x0d,0x48,0x0d,0xbc,0x48,0xe0,0x00,0x00,0x0d,0xb6,0x00,0x00,0xe0,0xd3,0xee,
  0xbb,0xee,0xe0,0xe0,0xe0,0x48,0xe0,0xe0,0xe0,0x0d,0xe0,0x0d,0x48,0x0d,0x00,0xd3,
  0xee,0x00,0x49,0xb6,0x00,0xb6,0x00,0xb6,0x48,0x0d,0x48,0xc9,0x0d,0x48,0x0d,0xb6,
  0xe6,0xe6,0xe6,0xc9,0x0d,0x00,0x00,0x0d,0x48,0xc9,0x0d,0xc9,0x48,0x0d,0x0d,0x0d,
  0xb6,0x48,0x48,0x0d,0x48,0x0d,0x0d,0xb6,0xe0,0x0d,0x0d,0x00,0xe0,0x0d,0xbc,0xd3,
  0xbb,0xbc,0xbc,0xbc,0xbc,0x0d,0xe0,0xe0,0xe0,0x48,0x0d,0x0d,0x0d,0xe0,0xe0,0xee,
  0xee,0x49,0x49,0x0d,0x00,0x00,0x00,0x49,0xc9,0x0d,0x0d,0xc9,0x0d,0xb6,0xb6,0x00,
  0x0d,0xe6,0xe6,0xe6,0x48,0x0d,0x0d,0x0d,0x48,0xe6,0x0d,0xe6,0xc9,0x48,0x48,0x48,
  0xb6,0xb6,0x48,0x0d,0x00,0x48,0x48,0x0d,0xb6,0xb6,0x0d,0x00,0x0d,0xb6,0xbc,0xfa,
  0xbb,0xbc,0x0d,0x48,0x0d,0xe0,0xe0,0xe0,0x0d,0x48,0x0d,0x0d,0xe0,0x00,0xe0,0x00,
  0xee,0x49,0x49,0x00,0x00,0x49,0x00,0x00,0x0d,0x0d,0x0d,0x48,0x0d,0xb6,0xb6,0xb6,
  0x48,0xe6,0xb9,0xb6,0xb9,0xb6,0x0d,0x0d,0xe6,0xe6,0xb6,0x0d,0x48,0x48,0x48,0x0d,
  0xb6,0x0d,0xb6,0xb6,0xbc,0xc9,0x0d,0x48,0x48,0x48,0x0d,0x48,0x48,0x00,0xb9,0xbb,
  0xce,0xee,0x00,0x00,0x48,0xe0,0xe0,0xe0,0x48,0xe0,0x0d,0x0d,0x0d,0x49,0xb9,0x49,
  0x00,0xdd,0xdd,0xdd,0x00,0x00,0x00,0x00,0xb6,0xc9,0xe6,0xc9,0x48,0xdd,0xfd,0xdd,
  0xe6,0xf1,0xc7,0x00,0xc9,0x0d,0x49,0xe0,0x48,0xc9,0x48,0xb6,0x49,0xf1,0x87,0x00,
  0x49,0x00,0xc7,0xf1,0x00,0x0d,0x0d,0x00,0x0d,0x48,0x48,0x48,0xb6,0xc7,0xc7,0xa8,
  0x68,0x68,0xf1,0xf1,0x48,0xe0,0xbc,0xbc,0xe0,0x48,0xbc,0x00,0x48,0xc7,0xf1,0xc7,
  0xb6,0xc7,0xf1,0xc7,0xb6,0x49,0xb6,0x00,0xb6,0xc9,0xb6,0xc9,0xb6,0xc7,0xf1,0xc7,
  0xe6,0xdd,0x87,0x00,0x48,0x0d,0x0d,0x00,0xe6,0x0d,0xb6,0xe6,0xd3,0xff,0xc7,0x48,
  0xb6,0x00,0xef,0xc7,0xe0,0x48,0x48,0xbc,0x48,0xe6,0x48,0xe6,0x48,0xeb,0xe5,0xbb,
  0xbb,0xfa,0xef,0x5a,0xbc,0xe0,0xbc,0xe0,0x0d,0x48,0x48,0xe0,0x48,0xb4,0xe5,0xb4,
  0x49,0x49,0xe5,0xdd,0x0d,0x00,0x00,0xb6,0xb6,0xc9,0xc9,0x0d,0x48,0xb4,0xe5,0xb4,
  0xe6,0xe6,0xe6,0xb9,0x48,0xe6,0x00,0xb6,0xb6,0xc9,0x00,0xb6,0x49,0x48,0xc9,0xb9,
  0xb6,0x0d,0xb6,0x0d,0xb6,0x0d,0x49,0x00,0x00,0x00,0x00,0x48,0xb6,0xb6,0xb6,0x68,
  0xed,0xee,0x00,0x00,0xe0,0xb6,0x00,0xb6,0xe0,0x48,0x0d,0x0d,0x0d,0xb6,0xb6,0xee,
  0x49,0xb6,0x49,0x0d,0xc9,0x49,0x00,0xb6,0xb6,0xb6,0xb6,0xee,0x48,0xc9,0xc9,0xb6,
  0xff,0xe5,0xc7,0xc7,0xf9,0xf9,0xf9,0x7a,0x7a,0x7a,0xf9,0xa8,0xd6,0x7a,0xe5,0x72,
  0xa8,0xd6,0xa8,0xeb,0xbb,0xa8,0xa8,0xd6,0xd6,0xa8,0xd6,0xff,0xff,0xd6,0xd6,0xf9,
  0xce,0xd6,0x7a,0xd6,0xf9,0x7a,0x7a,0x7a,0x7a,0xf8,0xe3,0xd6,0xfe,0xfe,0xfe,0xe3,
  0x7a,0x7a,0xd6,0x7a,0xf8,0x7a,0x7a,0x7a,0x7a,0x7a,0xf8,0xd6,0xd6,0xd6,0x7a,0xf8,
  0xef,0xdd,0xb9,0x00,0x7a,0x7a,0xbf,0xdd,0xdd,0xbb,0x7a,0xdd,0x5a,0x68,0x0d,0xfc,
  0x00,0x68,0xb4,0xdd,0xb6,0x00,0x49,0x49,0xb4,0x00,0x49,0xd2,0xb4,0x5a,0x49,0xf8,
  0x68,0xbb,0xbb,0xdc,0x7a,0xbb,0xbb,0xbb,0xa8,0xa8,0xf9,0xd2,0xce,0xbb,0xbb,0xe3,
  0x68,0x68,0xd3,0xa8,0xa8,0x68,0x68,0xdc,0xbb,0x5a,0x68,0xeb,0x68,0x68,0x5a,0xeb,
  0xdd,0xdd,0xc7,0xc7,0x7a,0xf8,0x68,0x68,0xb4,0x68,0x7a,0x5a,0x68,0x5a,0xb6,0xfc,
  0x68,0xbb,0x49,0x5a,0x68,0xdd,0x00,0xdd,0x5a,0xb4,0xd3,0x49,0x68,0xdc,0xbb,0xf8,
  0xd2,0xbb,0xbb,0xbb,0x7a,0xa8,0xbb,0xdc,0xbb,0xbb,0xe3,0x68,0xce,0xbb,0xbb,0xe3,
  0xeb,0xeb,0xa8,0xa8,0xbb,0xbb,0xbb,0xbb,0xbb,0xeb,0xbb,0xbb,0xa8,0xdd,0x68,0xeb,
  0xdd,0xdd,0x00,0xc7,0x7a,0xef,0x5a,0xeb,0x5a,0xa8,0x7a,0xb4,0xeb,0x68,0xb6,0xfc,
  0x0d,0xb6,0x0d,0x00,0x49,0xb6,0xdd,0xdd,0x5a,0xeb,0x68,0x5a,0xdd,0xb4,0x68,0xf8,
  0xd2,0xce,0xbb,0xd2,0xf9,0xa8,0x68,0xd2,0xbb,0xdc,0xfc,0xd3,0xd6,0xbb,0xbb,0xf7,
  0x0d,0x49,0x5a,0xb4,0xd3,0x49,0x68,0x68,0xdd,0xb4,0x68,0xb4,0x49,0xb4,0xb4,0xbb,
  0xdd,0xdd,0xdd,0x87,0xf9,0xf8,0xbb,0xeb,0x0d,0xdd,0xef,0x00,0xdd,0x5a,0xb6,0x4b,
  0xe5,0x7a,0x7a,0x7a,0x7a,0xe5,0xef,0x7a,0x7a,0x7a,0xf9,0xf9,0xf9,0x7a,0xe5,0xe3,
  0xbb,0xa8,0xce,0xdc,0xf9,0xa8,0x68,0x5a,0xdc,0xbb,0xfc,0x68,0xbb,0xdc,0xd2,0xf9,
  0x7a,0x7a,0x7a,0x7a,0xf8,0x7a,0x7a,0xd6,0xd6,0x7a,0xd6,0xd6,0x7a,0x7a,0x7a,0xf8,
  0xc7,0xdd,0xdd,0xc7,0x7a,0xef,0xa8,0x5a,0x00,0xb4,0x7a,0x00,0xbb,0xbb,0x00,0xe3,
  0xc7,0xeb,0xa8,0xc7,0xef,0xc7,0xeb,0xeb,0xc7,0xa8,0xff,0xa8,0xa8,0xeb,0xdd,0xf9,
  0xdc,0xbb,0xbb,0xeb,0x7a,0xa8,0xd2,0xb4,0xbb,0xbb,0xfc,0xd3,0xdc,0xdc,0xdc,0x72,
  0x68,0xd2,0x68,0xd3,0x49,0x00,0xd3,0xb4,0x49,0x49,0xd3,0x49,0xd3,0xbf,0xb4,0xbf,
  0x68,0x5a,0xb9,0xdd,0x7a,0x7a,0xa8,0x68,0xb4,0xeb,0x7a,0xdd,0xbb,0xa8,0xb6,0xfc,
  0xef,0x7a,0xbb,0xce,0xeb,0xff,0xff,0xff,0xa8,0xa8,0xce,0x7a,0xf8,0xef,0xa8,0xfc,
  0xbb,0xbb,0xd2,0x68,0x7a,0xa8,0x68,0xbf,0xeb,0xa8,0x6f,0x5a,0xbb,0xd2,0xbb,0x72,
  0xeb,0xdc,0xdc,0x68,0x68,0xb4,0x5a,0xee,0x68,0x5a,0x5a,0xee,0xb4,0xd3,0xd3,0xd2,
  0x5a,0xdd,0x00,0x00,0x7a,0xbb,0xeb,0xdd,0xb4,0xeb,0xef,0x00,0xa8,0xa8,0xb6,0xe3,
  0x68,0x5a,0xb4,0x5a,0xdd,0xdd,0x68,0xb4,0x5a,0x68,0x5a,0xbb,0xce,0xa8,0xbb,0xf9,
  0xdc,0xa8,0xeb,0x68,0x7a,0xa8,0xb4,0x68,0x00,0xbb,0xfc,0x68,0xbb,0x68,0xd2,0xe3,
  0xeb,0xbb,0xa8,0xa8,0x68,0x68,0x68,0x68,0xb4,0x5a,0x68,0x49,0xd3,0x00,0xd3,0x68,
  0xdd,0x5a,0xdd,0xb6,0xe5,0xa8,0xdd,0x49,0xa8,0xa8,0xef,0xb4,0xa8,0xa8,0x00,0xe9,
  0xf9,0xf7,0xf7,0xf9,0xf9,0xe3,0xe3,0xe3,0xe3,0xe3,0xf9,0xe3,0x7a,0xf8,0xe5,0xfc,
  0xdc,0xbb,0xbb,0x68,0x7a,0xeb,0x49,0xeb,0x68,0x68,0xe3,0x5a,0xd2,0xd2,0x68,0xe3,
  0xf8,0x7a,0x7a,0x7a,0xf8,0xf8,0xf8,0x7a,0x7a,0x7a,0xf8,0x7a,0x7a,0xd6,0xd6,0x7a,
  0x00,0xdd,0xdd,0xb9,0xef,0xc7,0x68,0x00,0x68,0xe5,0x7a,0x49,0x68,0xbf,0x49,0xe3,
  0x68,0x68,0x49,0x00,0xb6,0xb6,0x00,0x49,0x49,0x00,0x00,0x00,0x68,0xdd,0xb4,0xf8,
  0x68,0x68,0x68,0x5a,0x7a,0x68,0x5a,0xa8,0x68,0xb4,0xfc,0x68,0xdc,0x5a,0x5a,0x72,
  0xb4,0xee,0x00,0xe0,0xe0,0xe0,0x00,0xbc,0x49,0x49,0xd3,0x68,0xa8,0xa8,0xd3,0xdc,
  0xc7,0x5a,0x68,0xb6,0x7a,0xc7,0x5a,0xb6,0x49,0xff,0xff,0x00,0xd2,0x68,0x5a,0xf9,
  0xb4,0x49,0x00,0x00,0x00,0x0d,0x00,0x00,0x00,0xeb,0xbb,0xa8,0xf8,0xf8,0xbf,0x7a,
  0xdc,0xbb,0x68,0xd2,0x7a,0xa8,0xbb,0xa8,0x68,0xb4,0xfc,0xdc,0xbb,0xd3,0x68,0xe3,
  0x5a,0xb4,0xd3,0x49,0x0d,0xd3,0x49,0x00,0x5a,0x00,0xdc,0x68,0x5a,0x68,0xdc,0xdc,
  0xc7,0xc7,0xeb,0x00,0x7a,0xc7,0x68,0xb6,0x49,0xeb,0xef,0xdd,0xb4,0xbf,0xb4,0xf9,
  0xb4,0x00,0xb6,0x00,0xbf,0xdd,0x00,0x49,0xdd,0xdd,0xeb,0xbf,0xbb,0xef,0x68,0xf8,
  0xdc,0xbb,0xbb,0xa8,0x7a,0x68,0x5a,0x68,0x5a,0x68,0xe3,0xdc,0xdc,0xd2,0xbb,0xe3,
  0xeb,0xdc,0x68,0x5a,0xd3,0xd3,0xb4,0x0d,0x5a,0x49,0xbf,0xbf,0x68,0x68,0x68,0x68,
  0xdd,0x87,0xc7,0xc7,0xf9,0xff,0xc7,0xeb,0x68,0xa8,0x7a,0xa8,0xc7,0xdd,0x5a,0xef,
  0x5a,0xc7,0x00,0xdd,0xef,0xe5,0xeb,0x68,0x68,0xc7,0x68,0x68,0xc7,0xef,0xc7,0x7a,
  0xe5,0x7a,0xa8,0xa8,0x7a,0x87,0xc7,0xc7,0xa8,0xef,0xf9,0xbb,0xbb,0xeb,0xbb,0xf9,
  0xc7,0xa8,0xeb,0xa8,0xeb,0x68,0xbb,0xeb,0x68,0x68,0xbb,0xa8,0xbb,0xe5,0xbb,0xa8,
  0xc7,0xb9,0xb9,0xdd,0xb9,0xc9,0xb9,0x00,0xb6,0xe6,0xb6,0xdd,0xc7,0xb6,0xe6,0xe6,
  0xe6,0xc7,0xb9,0xc9,0xb9,0xdd,0xef,0xc7,0xc7,0xc7,0xc7,0xdd,0xdd,0xb9,0xe5,0xb9,
  0x00,0xe6,0xb9,0xb6,0xe6,0xb9,0xb9,0xc9,0xb6,0xdd,0x00,0xc7,0xdd,0x00,0x00,0x87,
  0x00,0xc7,0x00,0xb6,0xdd,0xc7,0xdd,0xb9,0xf1,0xe5,0xc7,0xb6,0xe6,0x00,0xc7,0x00,
  0xff,0xe6,0xe6,0xb9,0x00,0xc9,0xe6,0xe6,0xe6,0xe6,0xb9,0xb6,0xb9,0xe6,0xb9,0xb9,
  0xb9,0xb9,0xe6,0xb9,0xb6,0xc9,0xc7,0xc7,0x00,0xc7,0xdd,0xc7,0xdd,0xb6,0xb9,0xdd,
  0xb6,0xb6,0xc9,0xb9,0x00,0xb6,0xb6,0xb6,0xb6,0xe6,0x00,0xdd,0x00,0xb6,0xb9,0x00,
  0xb6,0x00,0xdd,0xb6,0xb9,0xc9,0xb9,0xb9,0xc9,0xf1,0x00,0xb9,0xc7,0xdd,0xb6,0xdd,
  0xb9,0xc9,0xe6,0xe6,0xc9,0xe6,0xf1,0xb9,0xb9,0xe6,0xc7,0xb4,0xc7,0xb6,0xe6,0xb9,
  0xb9,0xe6,0xe6,0xb9,0xb9,0x00,0xf1,0xb9,0x87,0xdd,0xdd,0x00,0xb6,0xb6,0x00,0x5a,
  0xe6,0xe6,0xb6,0xb9,0xe6,0xb9,0xb9,0xe6,0xb6,0xb9,0xb9,0xb9,0xb6,0xb6,0x00,0xdd,
  0xb6,0x00,0x00,0x00,0x00,0xb9,0xb6,0xe6,0xb6,0xc7,0xf1,0xb9,0x87,0xb6,0x00,0xfd,
  0xe9,0xe9,0xe9,0x4b,0x4b,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,
  0xe9,0xe9,0xe9,0x72,0xfe,0xfe,0xfe,0xfe,0x72,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,
  0xe9,0xe9,0xe9,0xe9,0xe9,0x4b,0xe9,0xe9,0x4b,0x72,0x72,0x72,0x72,0x72,0xe9,0xe9,
  0xe9,0xe9,0xe9,0xe9,0x4b,0xe9,0x4b,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,
  0xff,0xef,0xc7,0xa8,0xc7,0xa8,0xc7,0xa8,0xa8,0xa8,0xff,0xff,0xff,0xc7,0xe5,0xff,
  0x7a,0x7a,0xce,0xfa,0xbc,0xbc,0xbc,0xfc,0xd6,0xa8,0xa8,0xa8,0x7a,0xa8,0x7a,0x7a,
  0xe3,0xa8,0xa8,0xff,0xa8,0xa8,0xa8,0xc7,0xa8,0x7a,0xce,0x7a,0x7a,0xfa,0xd6,0xff,
  0xff,0x7a,0xa8,0xe5,0xa8,0xc7,0xc7,0x7a,0x7a,0x7a,0xe5,0xe5,0xe5,0xe5,0xa8,0xe5,
  0x68,0xdd,0x00,0x00,0x00,0x00,0xdd,0xb6,0xdd,0xb6,0xdd,0xdd,0x00,0x00,0xdd,0xb4,
  0xff,0xa8,0xfa,0xbc,0xe0,0xee,0x7a,0xdc,0x68,0xb4,0xb4,0x68,0xa8,0x5a,0xd2,0x7a,
  0xf9,0xb4,0xeb,0xb4,0xdd,0xb4,0x00,0x00,0x00,0x68,0xd6,0xed,0xd6,0xd6,0xf4,0xeb,
  0xbb,0x68,0xb4,0xdd,0x00,0x00,0xb9,0x68,0x68,0x68,0xdd,0xdd,0x5a,0xdd,0x5a,0x68,
  0x49,0xb6,0xb9,0x0d,0xc9,0xb6,0x0d,0xb6,0x0d,0xc9,0x00,0x00,0x0d,0xb9,0xb9,0xdd,
  0xa8,0xfa,0xbc,0xbc,0xfa,0xeb,0x49,0x49,0xd3,0x00,0x49,0x49,0x68,0x00,0xee,0x68,
  0xff,0x0d,0xb6,0x49,0xb6,0xb6,0x0d,0x48,0x48,0x0d,0xb6,0xbb,0xf3,0x7a,0xe5,0xf6,
  0xbb,0x5a,0x00,0xb6,0x00,0x0d,0xc9,0x5a,0x49,0x5a,0xd3,0x00,0xdd,0xb6,0x49,0xdd,
  0x00,0xb6,0xb6,0xc9,0x0d,0x00,0x0d,0x48,0x0d,0xe6,0x00,0x0d,0x0d,0xc9,0xb6,0xd2,
  0xfa,0xe0,0xd3,0xee,0xeb,0x00,0x49,0x68,0xd3,0x49,0xb6,0x49,0xd3,0x68,0xd3,0xd2,
  0xa8,0xe0,0x0d,0x00,0x0d,0x48,0x48,0x48,0x0d,0x48,0x48,0x49,0xe5,0xd3,0xce,0x7a,
  0xee,0xdc,0x49,0x0d,0xb6,0x48,0xe6,0x68,0xd3,0x49,0x00,0x00,0x68,0x00,0xb4,0xdd,
  0x00,0x00,0xb6,0x0d,0x0d,0xb9,0x0d,0xb6,0x0d,0xc9,0x00,0x0d,0x48,0xdd,0xa8,0xfa,
  0xee,0xbb,0xbc,0x68,0x49,0x49,0xb4,0xee,0x49,0xee,0x00,0xd3,0xee,0x5a,0x49,0x5a,
  0xeb,0x0d,0x48,0x0d,0x0d,0x48,0x48,0x0d,0xc9,0x0d,0x48,0x48,0xb6,0xd6,0xd3,0xce,
  0xbb,0xbc,0x49,0xb6,0xb6,0x48,0x48,0xb4,0xb4,0x49,0x49,0x49,0x49,0xdd,0x00,0xdd,
  0x00,0x49,0x00,0xc9,0xb6,0xb6,0xb6,0xb6,0x00,0xb6,0x00,0xb6,0xdd,0xce,0xd2,0x7a,
  0xa8,0xee,0xc7,0x49,0xb4,0xb4,0x5a,0x68,0xd3,0xd3,0x49,0xd3,0x5a,0x68,0x49,0x5a,
  0x68,0x0d,0x48,0x0d,0x48,0x48,0x48,0x0d,0xc9,0x0d,0x0d,0x48,0x0d,0xd3,0xbb,0xfa,
  0x5a,0xf3,0xf5,0x5a,0xb6,0xb6,0xb6,0x5a,0xb4,0xb4,0x49,0x00,0xb4,0xb4,0x49,0x68,
  0x68,0x49,0xdd,0xb9,0x00,0xb6,0xb9,0xb9,0xb6,0x0d,0xb6,0xdd,0xce,0xbb,0x7a,0x5a,
  0xbc,0xa8,0x49,0x00,0xb4,0xd3,0x68,0x68,0x5a,0xd3,0xd3,0xbb,0xd2,0x5a,0xee,0x5a,
  0x68,0x0d,0x0d,0xb6,0x48,0x48,0x48,0x0d,0xc9,0x48,0x49,0x0d,0xb6,0x00,0xb4,0x7a,
  0xfa,0xf5,0xf5,0xbc,0xb4,0x0d,0xb6,0x68,0x49,0x49,0x00,0xb6,0x49,0x49,0x49,0xd2,
  0x68,0x68,0x49,0x00,0x00,0x00,0xb9,0x0d,0x0d,0xc9,0xdd,0xd2,0xfa,0x7a,0xbb,0xee,
  0xef,0xdd,0x49,0x68,0xb4,0xbb,0x68,0xd2,0xfa,0x49,0xd3,0xbb,0xd3,0xd3,0x5a,0x5a,
  0x49,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x0d,0x48,0x48,0x00,0xe0,0x49,0x00,0x49,0x5a,
  0xd6,0xd2,0xf5,0xf5,0xbc,0x5a,0x00,0xb4,0xb4,0xb4,0x00,0x00,0x5a,0x49,0x49,0x5a,
  0x68,0xb4,0x00,0xb6,0x00,0x00,0xb9,0xb6,0x0d,0x00,0xbb,0xfa,0xbb,0xd2,0xee,0xa8,
  0x5a,0x5a,0xb4,0x5a,0x68,0x68,0x5a,0xee,0xd3,0xd3,0xee,0xb4,0x5a,0xd3,0xd3,0xd3,
  0xd3,0x00,0xe0,0xe0,0x0d,0x0d,0xb6,0x0d,0x0d,0x0d,0x00,0xb6,0x0d,0x00,0xee,0x00,
  0xa8,0x7a,0xf4,0xf5,0xf5,0xbc,0xb4,0x49,0xb4,0xb4,0xb4,0xb4,0xd2,0x49,0xdd,0x5a,
  0x68,0xb4,0xdd,0x00,0x49,0xb4,0xb6,0x49,0x00,0xbb,0xd2,0xed,0xfb,0xbc,0xeb,0xeb,
  0x5a,0xeb,0xd3,0x68,0x49,0xbf,0x68,0xee,0x5a,0x68,0x5a,0xd2,0xd3,0xd3,0xee,0x5a,
  0xd2,0x49,0xb6,0x00,0x0d,0xb6,0xb6,0xb6,0xb6,0x0d,0xb6,0xb6,0xb6,0x68,0xee,0x00,
  0xf8,0xeb,0xe5,0xed,0xf5,0xf4,0xbb,0xa8,0x68,0x5a,0x5a,0xdd,0xdd,0xdd,0xd3,0xbb,
  0x68,0x5a,0xdd,0x00,0x00,0xb4,0xb6,0xdd,0xd6,0xdc,0x5a,0xbb,0xfa,0xc7,0x00,0xbf,
  0xeb,0xbb,0x68,0x5a,0xb4,0xbb,0x5a,0xd2,0xd2,0xd2,0xd2,0xd2,0x5a,0xd3,0x5a,0x68,
  0x68,0x49,0x49,0x00,0x00,0x00,0xb6,0xb6,0xb6,0x00,0xb6,0xb9,0x00,0x49,0x00,0xb4,
  0xf8,0xbb,0x5a,0x7a,0xce,0xf2,0xd6,0xf7,0xe5,0x00,0xb4,0x00,0x00,0x49,0x49,0xbb,
  0xbf,0x68,0xb4,0x49,0xdd,0x5a,0xdd,0x7a,0xd6,0xe5,0xeb,0xbb,0xff,0x00,0x68,0xbf,
  0xed,0xbb,0xbb,0xd2,0x68,0xdc,0xeb,0xfa,0xbb,0xbb,0xd2,0xbb,0xed,0xd2,0x5a,0xbb,
  0xeb,0xb6,0xb4,0xee,0xee,0xd3,0x00,0xb6,0x49,0xd3,0xb6,0xb6,0x00,0xb4,0x00,0xb4,
  0xe5,0x7a,0x00,0xdd,0xf9,0xf3,0xfe,0xed,0xce,0x68,0xeb,0x00,0x49,0x49,0x49,0xeb,
  0xa8,0xc7,0xf1,0x5a,0x68,0xbf,0x7a,0xa8,0xff,0xe5,0x7a,0xc7,0xa8,0xfd,0xf1,0xbb,
  0xbb,0xd2,0xc7,0xdd,0xd2,0x68,0xed,0xed,0xbb,0xbb,0xed,0xed,0xed,0x68,0xfd,0xa8,
  0xbb,0x68,0xc7,0xdd,0xd3,0x68,0x49,0x49,0xee,0xd3,0x00,0x49,0x00,0xdd,0xf1,0xc7,
  0xbb,0xc7,0x87,0x00,0x5a,0x7a,0xd6,0xce,0xf9,0xf6,0xe5,0xdd,0xb4,0xf1,0xc7,0xef,
  0x5a,0x87,0xf1,0xc7,0xa8,0xa8,0x7a,0xe5,0xf9,0xe5,0x87,0x5a,0xf8,0xf1,0xc7,0xa8,
  0xf2,0xef,0xf1,0xf1,0xce,0xbb,0xbb,0xce,0xed,0xbb,0xed,0xce,0xed,0xff,0xfd,0xef,
  0xbb,0xa8,0xf1,0xc7,0x5a,0xd2,0xd3,0xb4,0x5a,0x68,0x49,0x49,0x49,0x87,0xfd,0xa8,
  0xd2,0xff,0xfd,0xc7,0xdd,0xeb,0xfe,0xfe,0xfe,0xce,0xf5,0xa8,0xb4,0x87,0xf1,0xfe,
  0xeb,0xd6,0xef,0xeb,0xd2,0xfb,0xf9,0x7a,0xe5,0x7a,0x68,0xeb,0xd2,0x7a,0x7a,0xce,
  0xd6,0xa8,0xfc,0xf9,0xce,0xce,0xce,0xf8,0xce,0xce,0xf3,0xf2,0xce,0xf2,0xfc,0xf0,
  0xa8,0xdc,0xf8,0xbb,0x68,0xfa,0xd2,0xd2,0xd2,0x68,0x5a,0x68,0xd3,0xbb,0x7a,0x5a,
  0xce,0xbb,0xd6,0xeb,0xbb,0xeb,0xeb,0xf7,0xf7,0xf3,0xf5,0xf4,0xa8,0xbb,0xd6,0xce,
  // 2: ground concrete
  0x2d,0x63,0x4d,0x6b,0x6e,0x6b,0x8e,0x73,0xcf,0x73,0xf0,0x7b,0x00,0x00,0x08,0x00,
  0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,
  0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,
  0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,
  0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x8e,0x6b,0x18,0x07,0x00,0x20,
  0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,
  0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,
  0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,
  0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,
  0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,
  0x08,0x42,0x10,0x42,0x18,0x42,0x10,0x84,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,
  0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,
  0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,
  0x00,0x60,0x08,0x60,0x10,0x60,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,
  0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,
  0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,
  0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,
  0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,
  0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,
  0x00,0x84,0x08,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,
  0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,
  0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,
  0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x0c,0x5b,
  0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,0x18,0xa5,
  0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,
  0x00,0xc0,0x08,0xc0,0x10,0xc0,0x10,0x7c,0x18,0xc0,0x00,0xc1,0x08,0xc1,0x10,0xc1,
  0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,
  0x18,0xc3,0x00,0xc4,0xcb,0x5a,0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,
  0x10,0xc5,0x18,0xc5,0x00,0xc6,0x08,0xc6,0x10,0xc6,0xec,0x5a,0x18,0xc6,0x00,0xc7,
  0x08,0xc7,0x10,0xc7,0x18,0xc7,0x00,0xe0,0x08,0xe0,0x10,0xe0,0x18,0xe0,0x00,0xe1,
  0x52,0x84,0x08,0xe1,0x18,0xe1,0x6e,0x73,0x08,0xe2,0x10,0xe2,0x18,0xe2,0x10,0xe3,
  0xb0,0x73,0x08,0xe4,0x0b,0x5b,0x18,0xe4,0x08,0xe5,0x00,0xe6,0x10,0xe6,0xaa,0x52,
  0xff,0xda,0xe5,0x00,0x00,0x00,0xb7,0xda,0xb7,0x00,0xb7,0xb7,0xb7,0xb7,0x00,0x00,
  0x00,0x00,0x00,0x00,0xb7,0xda,0xe5,0xb7,0xb7,0xda,0xda,0xb7,0xb7,0xb7,0xb7,0xda,
  0xda,0xe5,0xda,0xb7,0x00,0xb7,0x00,0x00,0x00,0xfa,0xb7,0xb7,0xb7,0xb7,0x00,0x00,
  0x00,0x00,0xb7,0xb7,0x00,0xb7,0x00,0xb7,0xb7,0xb7,0x00,0x00,0x00,0xb7,0xb7,0xda,
  0xda,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0x02,0x00,0x01,
  0x02,0x25,0x01,0x25,0x25,0x01,0x01,0x02,0x00,0x01,0x00,0x00,0xb7,0x00,0x00,0xda,
  0x00,0x00,0x00,0x00,0x01,0x01,0x02,0x03,0x25,0x01,0x01,0x01,0x01,0x02,0x02,0x02,
  0x02,0x01,0x01,0x02,0x00,0x02,0x25,0x01,0x02,0x02,0x01,0x01,0x02,0x02,0x01,0xb7,
  0x00,0x00,0x00,0x01,0x01,0x01,0x02,0x02,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x01,
  0x01,0x02,0x03,0x02,0x02,0x01,0x25,0x25,0x02,0x00,0x02,0x01,0x00,0x00,0x01,0xe5,
  0xb7,0x01,0x01,0x00,0x01,0x01,0x02,0x02,0x02,0x01,0x25,0x03,0x25,0x04,0x03,0x01,
  0x01,0x25,0x25,0x02,0x02,0x02,0x02,0x25,0x02,0x01,0x25,0x02,0x01,0x02,0x01,0xb7,
  0x00,0x00,0x02,0x00,0x01,0x03,0x02,0x04,0x25,0x01,0x02,0x02,0x03,0x04,0x04,0x25,
  0x03,0x03,0x04,0x04,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x02,0x00,0x00,0x01,0xb7,
  0x00,0x01,0x01,0x00,0x01,0x01,0x03,0x03,0x01,0x03,0x03,0x03,0x01,0x02,0x25,0x25,
  0x03,0x25,0x03,0x03,0x25,0x25,0x02,0x25,0x25,0x25,0x01,0x02,0x02,0x00,0x01,0xb7,
  0x00,0x00,0x01,0x00,0x03,0x04,0x04,0x04,0x01,0x00,0x01,0x01,0x25,0x04,0x04,0x03,
  0x04,0x04,0x04,0x05,0x03,0x03,0x05,0x04,0x05,0x03,0x03,0x02,0x00,0x01,0x03,0xb7,
  0xb7,0x00,0x25,0x02,0x04,0x04,0x05,0x03,0x04,0x04,0x04,0x04,0x25,0x03,0x04,0x04,
  0x04,0x04,0x04,0x04,0x01,0x03,0x03,0x04,0x03,0x03,0x25,0x03,0x02,0x02,0x02,0xb7,
  0x00,0x00,0x00,0xb7,0x25,0x25,0x01,0x01,0x01,0x00,0x02,0x02,0x03,0x03,0x03,0x04,
  0x04,0x04,0x03,0x01,0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x02,0x00,0x00,0x00,0xb7,
  0xb7,0x01,0x25,0x02,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x03,0x02,0x25,0x03,0x03,
  0x04,0x05,0x03,0x03,0x04,0x01,0x01,0x25,0x02,0x01,0x04,0x04,0x01,0x02,0x02,0xb7,
  0x00,0x00,0x00,0x00,0x02,0x01,0x00,0x03,0x00,0x00,0x02,0x02,0x02,0x25,0x03,0x03,
  0x03,0x05,0x25,0x03,0x04,0x04,0x04,0x03,0x03,0x02,0x25,0x25,0x00,0x00,0x00,0xda,
  0x01,0x02,0x00,0x02,0x04,0x03,0x04,0x04,0x04,0x03,0x04,0x03,0x25,0x03,0x03,0x03,
  0x03,0x03,0x03,0x05,0x05,0x03,0x02,0x01,0x03,0x02,0x02,0x25,0x02,0x01,0x02,0x00,
  0x00,0x00,0xb7,0x00,0x01,0x01,0x00,0x00,0x00,0x01,0x03,0x01,0x25,0x04,0x04,0x02,
  0x01,0x03,0x03,0x04,0x05,0x04,0x03,0x02,0x02,0x04,0x03,0x02,0x00,0x00,0x01,0xe5,
  0x00,0x02,0x02,0x01,0x03,0x04,0x05,0x04,0x05,0x04,0x03,0x25,0x25,0x02,0x04,0x04,
  0x04,0x03,0x04,0x03,0x02,0x02,0x01,0x02,0x03,0x25,0x25,0x03,0x02,0x02,0x02,0xe5,
  0x00,0x00,0xb7,0xb7,0x25,0x00,0x00,0x02,0x02,0x01,0x01,0x02,0x04,0x04,0x02,0x02,
  0x01,0x03,0x04,0x04,0x05,0x04,0x25,0x03,0x03,0x02,0x02,0x04,0x01,0x01,0x00,0xb7,
  0x00,0x02,0x02,0x01,0x04,0x04,0x05,0xf8,0x05,0x04,0x04,0x02,0x02,0x04,0x04,0x03,
  0x04,0x04,0x04,0x03,0x03,0x01,0x02,0x25,0x03,0x01,0x02,0x01,0x02,0x02,0x01,0xda,
  0x00,0x02,0x00,0x00,0x01,0x01,0x02,0x02,0x01,0x00,0x01,0x02,0x03,0x04,0x04,0x01,
  0x01,0x03,0x04,0x04,0x05,0x03,0x05,0x04,0x05,0x04,0x04,0x03,0x01,0x02,0x00,0xb7,
  0x00,0x02,0x01,0x02,0x04,0x05,0x04,0x04,0x03,0x04,0x04,0x03,0x25,0x03,0x03,0x03,
  0x04,0x03,0x03,0x03,0x01,0x03,0x03,0x02,0x03,0x01,0x01,0x02,0x02,0x00,0x00,0xda,
  0xb7,0x01,0x00,0x02,0x01,0x01,0x02,0x00,0x01,0x02,0x02,0x01,0x00,0x04,0x03,0x25,
  0x03,0x04,0x04,0x04,0x03,0x03,0x05,0x05,0x04,0x03,0x04,0x03,0x02,0x25,0x02,0xb7,
  0x00,0x25,0x25,0x04,0x05,0x04,0x04,0x03,0x04,0x04,0x05,0x03,0x04,0x03,0x03,0x04,
  0x04,0x03,0x04,0x03,0x03,0x03,0x02,0x25,0x02,0x03,0x04,0x03,0x01,0x01,0x01,0xb7,
  0xb7,0x01,0x00,0x00,0x02,0x02,0x01,0x00,0x02,0x03,0x02,0x02,0x03,0x01,0x04,0x04,
  0x03,0x05,0x05,0x03,0x03,0x04,0x05,0x05,0x05,0x04,0x25,0x03,0x02,0x04,0x25,0xe5,
  0x00,0x02,0x01,0x03,0x04,0x03,0x03,0x04,0x04,0x04,0x05,0x03,0x03,0x04,0x03,0x03,
  0x04,0x03,0x03,0x03,0x02,0x02,0x03,0x03,0x02,0x03,0x03,0x03,0x25,0x01,0x01,0xb7,
  0x00,0x01,0x02,0x02,0x02,0x02,0x25,0x01,0x02,0x03,0xf3,0xf3,0x03,0x03,0x04,0x04,
  0x05,0x04,0x03,0x03,0x04,0x03,0x05,0x04,0x04,0x04,0x03,0x02,0x01,0x02,0x00,0xda,
  0x00,0x04,0x02,0x01,0x04,0x03,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x04,0x05,0x05,
  0x04,0x03,0x04,0x04,0x03,0x25,0x04,0x03,0x03,0x03,0x03,0x04,0x04,0x01,0x01,0xb7,
  0xb7,0x00,0x00,0x00,0x01,0x02,0x04,0x04,0x03,0xf3,0x01,0x01,0x03,0x04,0x05,0x03,
  0x04,0x03,0x02,0x04,0x04,0x05,0x03,0x03,0x04,0x04,0x03,0x05,0x25,0x01,0x02,0xe5,
  0x00,0x02,0x25,0x25,0x03,0x05,0x04,0x04,0x04,0x03,0x03,0x04,0x03,0x03,0x05,0x53,
  0x04,0x04,0x04,0x03,0x25,0x25,0x01,0x03,0x25,0x03,0x03,0x03,0x02,0x02,0x01,0xb7,
  0x01,0x02,0x00,0x00,0x02,0x03,0x03,0x04,0x04,0x03,0x01,0x04,0x03,0x04,0x04,0x04,
  0x03,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x25,0x02,0x01,0xb7,
  0x02,0x02,0x25,0x01,0x04,0x03,0x04,0x04,0x04,0x25,0x03,0x03,0x03,0x04,0x04,0x05,
  0x05,0x04,0x04,0x04,0x03,0x03,0x04,0x04,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0xb7,
  0x00,0x00,0x00,0x00,0x25,0x02,0x02,0x03,0x03,0x25,0x02,0x03,0x03,0x04,0x03,0x03,
  0x04,0x04,0x03,0x04,0x04,0x03,0x04,0x04,0x03,0x03,0x04,0x03,0x25,0x25,0x02,0xb7,
  0x00,0x02,0x01,0x04,0x05,0x04,0x04,0x25,0x02,0x02,0x03,0x04,0x03,0x03,0x04,0x05,
  0x05,0x05,0x53,0x04,0x03,0x04,0x03,0x03,0x03,0x04,0x03,0x03,0x02,0x01,0x01,0xb7,
  0x00,0x00,0x00,0x02,0x04,0x02,0x03,0x02,0x03,0x03,0x03,0x03,0x25,0x05,0x04,0x04,
  0x05,0x05,0x04,0x04,0x04,0x04,0x05,0x05,0x04,0x05,0x05,0x05,0x02,0x02,0x25,0xb7,
  0x01,0x01,0x03,0x04,0x04,0x05,0x04,0x02,0x04,0x03,0x04,0x04,0x25,0x04,0x05,0x05,
  0x04,0x05,0x05,0x04,0x02,0x03,0x03,0x03,0x25,0x01,0x25,0x25,0x01,0x01,0x00,0xda,
  0x00,0x01,0x02,0x02,0x03,0x03,0x02,0x03,0x03,0x02,0x02,0x02,0x04,0x04,0x04,0x05,
  0x05,0x05,0x04,0x04,0x04,0x05,0xcb,0x05,0x04,0x04,0x05,0x04,0x02,0x03,0x03,0x00,
  0x01,0x25,0x01,0x04,0x05,0x03,0x03,0x04,0x04,0x04,0x04,0x03,0x01,0x04,0x04,0x05,
  0x05,0x04,0x04,0x05,0x04,0x03,0x03,0x04,0x03,0x04,0x03,0x02,0x25,0x01,0x00,0xb7,
  0xb7,0x00,0x01,0x02,0x02,0x02,0x25,0x01,0x01,0x01,0x25,0x02,0x02,0x04,0x04,0x04,
  0x05,0x04,0x04,0x04,0xcb,0x04,0x04,0x05,0x04,0x05,0xcb,0x05,0x25,0x03,0x25,0x00,
  0x02,0x04,0x04,0x03,0x04,0x03,0x04,0x04,0x04,0x04,0x03,0x03,0x03,0x04,0x05,0x53,
  0x04,0x04,0x04,0x05,0x04,0x03,0x04,0x04,0x04,0x04,0x03,0x02,0x02,0x02,0x01,0xb7,
  0xb7,0x01,0x00,0x00,0x02,0x01,0x02,0x02,0x01,0x01,0x01,0x03,0x04,0x03,0x04,0x04,
  0x04,0x04,0x04,0x04,0x05,0x05,0x04,0x04,0xcb,0x53,0x05,0x05,0x03,0x25,0x04,0x00,
  0x00,0x03,0x04,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x03,0x04,0x03,0x04,0x04,0x05,
  0x03,0x04,0x03,0x04,0x05,0x04,0x03,0x04,0x04,0x03,0x03,0x03,0x01,0x02,0x02,0xb7,
  0x00,0x02,0x01,0x00,0x01,0x02,0x02,0x01,0x02,0x01,0x00,0x01,0x02,0x02,0x03,0x03,
  0x03,0x04,0x04,0x04,0x05,0x05,0x04,0x05,0x04,0x05,0xcb,0x05,0x03,0x03,0x03,0xb7,
  0x01,0x04,0x03,0x03,0x04,0x25,0x03,0x04,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x05,
  0x05,0x04,0x04,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x03,0x03,0x02,0x02,0x02,0xb7,
  0x00,0x02,0x00,0x00,0x01,0x01,0x02,0x02,0x01,0x03,0x04,0x25,0x02,0x03,0x01,0x03,
  0x02,0x04,0x03,0x04,0x04,0x04,0x05,0x05,0x53,0x05,0x05,0x53,0x04,0x04,0x04,0x00,
  0x01,0x03,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,
  0x05,0x04,0x05,0x03,0x03,0x04,0x04,0x05,0x04,0x04,0x03,0x03,0x03,0x02,0x01,0xb7,
  0x00,0x00,0x00,0x02,0x03,0x02,0x01,0x03,0x03,0x04,0x04,0x03,0x03,0x01,0x02,0x25,
  0x25,0x04,0x04,0x03,0x25,0x05,0x53,0x05,0x05,0x05,0x05,0xcb,0x05,0x04,0x04,0x00,
  0x01,0x25,0x02,0x02,0x04,0x04,0x03,0x04,0x03,0x04,0x04,0x05,0x03,0x04,0x04,0x03,
  0x04,0x05,0x04,0x05,0x04,0x03,0x04,0x04,0x04,0x03,0x03,0x04,0x25,0x25,0x02,0xb7,
  0xb7,0x00,0x01,0x02,0x02,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x03,0x25,0x25,0x25,
  0x04,0x03,0x03,0x03,0x04,0x04,0x05,0x53,0x05,0x05,0x05,0x05,0x04,0x04,0x03,0xb7,
  0x02,0x04,0x01,0x02,0x05,0x03,0x04,0x05,0x05,0x04,0x04,0x04,0x04,0x05,0x05,0x04,
  0x04,0x04,0x05,0x05,0x04,0x04,0x03,0x25,0x25,0x03,0x04,0x04,0x25,0x01,0x01,0x00,
  0xb7,0x01,0x01,0x02,0x02,0x01,0x03,0x04,0x01,0x03,0x04,0x03,0x04,0x04,0x03,0x03,
  0x04,0x04,0x03,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x03,0x00,
  0x02,0x04,0x04,0x04,0x05,0x05,0x05,0x04,0x05,0x05,0x05,0x04,0x05,0x05,0x05,0x03,
  0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x03,0x03,0x03,0x04,0x04,0x25,0x01,0x01,0xb7,
  0x01,0x02,0x00,0x01,0x01,0x04,0x02,0x04,0x04,0x03,0x03,0x04,0x04,0x04,0x05,0x05,
  0x03,0x03,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x03,0x03,0x00,
  0x04,0x04,0x04,0x03,0x53,0x53,0x05,0x05,0x53,0x04,0x53,0xcb,0x05,0x05,0x05,0x04,
  0x05,0x05,0x04,0x03,0x04,0x04,0x04,0x25,0x04,0x03,0x04,0x03,0x01,0x02,0x02,0xb7,
  0x00,0x00,0xb7,0x00,0x01,0x04,0x25,0x03,0x02,0x01,0x01,0x25,0x03,0x04,0x04,0x03,
  0x05,0x05,0x03,0x04,0x04,0x05,0x04,0x04,0x04,0x05,0x05,0xcb,0x04,0x05,0x04,0xb7,
  0x02,0x03,0x03,0x04,0x53,0x53,0x05,0x04,0x04,0x05,0x05,0x05,0x05,0x03,0x04,0x05,
  0x05,0x05,0x53,0x04,0x03,0x04,0x05,0x04,0x03,0x04,0x03,0x03,0x02,0x01,0x01,0x00,
  0xb7,0x00,0x00,0x00,0x01,0x01,0x03,0x03,0x25,0x25,0x25,0x01,0x03,0x03,0x02,0x03,
  0x04,0x05,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x53,0xcb,0x05,0xf8,0x05,0x02,0x00,
  0x01,0x03,0x04,0x04,0x05,0x05,0x05,0x04,0x53,0x53,0x05,0x04,0x05,0x04,0x04,0x05,
  0x05,0x05,0x05,0x04,0x03,0x03,0x04,0x04,0x04,0x03,0x03,0x03,0x01,0x01,0x00,0xb7,
  0xda,0x00,0xb7,0x00,0x00,0x01,0x00,0x00,0x02,0xb7,0x02,0x02,0x01,0x01,0x01,0x03,
  0x04,0x03,0x03,0x25,0x25,0x25,0x04,0x02,0x04,0x03,0x04,0x02,0x25,0x03,0x25,0xb7,
  0x02,0x03,0x25,0x25,0x04,0x03,0x03,0x04,0x04,0x04,0x04,0x03,0x01,0x03,0x25,0x02,
  0x03,0x04,0x03,0x03,0x02,0x02,0x01,0x02,0x01,0x02,0x03,0x25,0x01,0x01,0x00,0xb7,
  0xb7,0x00,0x00,0x00,0x00,0x00,0xb7,0x00,0x01,0x01,0x02,0x02,0x00,0x25,0x25,0x25,
  0x02,0x25,0x01,0x25,0x04,0x03,0x02,0x02,0x03,0x04,0x04,0x04,0x25,0x02,0x03,0xb7,
  0x01,0x25,0x02,0x25,0x05,0x03,0x03,0x04,0x04,0x04,0x25,0x02,0x03,0x04,0x02,0x02,
  0x01,0x01,0x02,0x02,0x01,0x02,0x02,0x00,0x01,0x02,0x02,0x02,0x02,0x00,0x00,0xb7,
  0xb7,0x00,0x00,0x01,0x00,0x00,0x00,0x02,0x00,0x01,0x02,0x00,0x02,0x00,0x02,0x01,
  0x25,0x01,0x03,0x03,0x04,0x03,0x03,0x03,0x04,0x05,0x04,0x04,0x03,0x01,0x01,0xb7,
  0x00,0x03,0x04,0x01,0x00,0x03,0x04,0x04,0x25,0x04,0x04,0x03,0x04,0x04,0x03,0x02,
  0x01,0x25,0x01,0x25,0x01,0x02,0x01,0x01,0x00,0x01,0x01,0x00,0x01,0x02,0x00,0xda,
  0xda,0xe5,0xe5,0xfa,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0xb7,0x00,0x00,
  0x01,0x00,0x00,0x01,0x00,0x01,0x00,0x01,0x00,0x25,0x25,0x02,0x01,0x00,0xb7,0xb7,
  0x00,0x02,0x01,0x00,0x02,0x01,0x02,0x02,0x02,0x02,0x03,0x25,0x02,0x01,0x02,0x01,
  0x01,0x01,0x00,0xb7,0x00,0xb7,0x00,0x00,0xb7,0x00,0xb7,0x00,0x00,0x00,0xe5,0xda,
  0xda,0xe5,0xda,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xda,0xda,0xb7,0xda,0xb7,0xb7,0xb7,
  0xb7,0x00,0x00,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0x00,0xb7,0xda,
  0xda,0xb7,0xb7,0x00,0xb7,0x00,0xb7,0xb7,0x00,0x00,0x00,0xb7,0xb7,0x00,0x00,0x00,
  0xb7,0xb7,0xb7,0xb7,0xda,0xb7,0xb7,0xb7,0xb7,0xb7,0xda,0xb7,0xb7,0xb7,0xe5,0xda,
  0xb7,0x00,0x00,0x00,0x25,0x00,0x00,0x01,0x01,0x00,0x01,0x01,0x00,0x25,0x01,0x02,
  0x03,0x03,0x04,0x04,0x01,0x03,0x04,0x04,0x02,0x25,0x01,0x01,0x02,0x03,0x25,0xb7,
  0x01,0x25,0x03,0x03,0x04,0x04,0x04,0x25,0x03,0x04,0x03,0x04,0x03,0x03,0x25,0x25,
  0x03,0x04,0x02,0x03,0x02,0x01,0x02,0x25,0x02,0x01,0x25,0x02,0x00,0x00,0x00,0xda,
  0xda,0x00,0x25,0x01,0x00,0x00,0x00,0x00,0x25,0x02,0x02,0x25,0x00,0x01,0x03,0x03,
  0x04,0x04,0x04,0x25,0x02,0x03,0x05,0xf8,0x04,0x03,0x05,0x04,0x25,0x04,0x02,0xb7,
  0x03,0x04,0x02,0x03,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x03,0x03,0x03,0x04,0x25,0x02,0x01,0x03,0x01,0x02,0x02,0x01,0x00,0x00,0x00,0xb7,
  0xb7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x25,0x03,0x01,0x03,0x04,0x25,0x02,
  0x02,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x25,0x25,0x02,0x00,
  0x25,0x03,0x04,0x03,0x03,0x04,0x04,0x05,0x04,0x04,0x03,0x03,0x04,0x04,0x04,0x04,
  0x04,0x25,0x03,0x04,0x03,0x03,0x01,0x03,0x03,0x02,0x03,0x02,0x02,0x00,0x00,0xe5,
  0xb7,0x00,0x00,0x02,0x01,0x02,0x02,0x02,0x03,0x01,0x02,0x02,0x04,0x04,0x03,0x03,
  0x03,0x03,0x04,0x05,0x05,0x53,0x05,0x04,0x04,0x04,0x05,0x05,0x04,0x25,0x03,0x00,
  0x25,0x03,0x05,0x03,0x04,0x05,0x05,0x53,0x05,0x04,0x05,0x05,0x05,0x53,0x53,0x53,
  0xcb,0x05,0x04,0x04,0x05,0x04,0x03,0x04,0x04,0x04,0x04,0x03,0x02,0x02,0x02,0xb7,
  0xe5,0x00,0x02,0x02,0x02,0x03,0x02,0x03,0x02,0x02,0x02,0x03,0x04,0x25,0x03,0x03,
  0x04,0x04,0x05,0x05,0xcb,0x05,0x05,0x05,0x03,0x04,0x04,0x05,0x04,0x04,0x04,0x00,
  0x03,0x04,0x03,0x04,0x05,0x05,0x04,0x05,0x04,0x04,0x05,0x05,0x04,0x05,0xcb,0x05,
  0xcb,0x05,0x05,0x03,0x03,0x04,0x05,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x00,0xb7,
  0x00,0x01,0x00,0x01,0x04,0x03,0x03,0x03,0x01,0x03,0x04,0x04,0x03,0x04,0x03,0x03,
  0x05,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x04,0x05,0x00,
  0x25,0x03,0x04,0x04,0x05,0x05,0x05,0x04,0x04,0x53,0x53,0x05,0x04,0x05,0x53,0x05,
  0x05,0x05,0x53,0x04,0x03,0x04,0x05,0xf8,0xf8,0x03,0x03,0x04,0x04,0x02,0x02,0xb7,
  0xb7,0x00,0x02,0x02,0x04,0x03,0x01,0x03,0x03,0x04,0x05,0x05,0x04,0x03,0x04,0x04,
  0x05,0x05,0x05,0x04,0x05,0x53,0x05,0x05,0x05,0x05,0x03,0x04,0x04,0x04,0x04,0xb7,
  0x01,0x03,0x04,0x05,0x03,0x05,0x05,0x05,0x05,0x05,0xcb,0x05,0x04,0x04,0x05,0x03,
  0x05,0x05,0x05,0x05,0x04,0x03,0x03,0x04,0x04,0x05,0x05,0x05,0x04,0x01,0x25,0xb7,
  0xb7,0x00,0x04,0x02,0x04,0x04,0x03,0x25,0x04,0x04,0x04,0x05,0x03,0x03,0x04,0x04,
  0x05,0x05,0x04,0x04,0x04,0x05,0x05,0x53,0x05,0x04,0x04,0x05,0x03,0x04,0x04,0xb7,
  0x02,0x04,0x04,0x05,0x05,0x03,0x05,0x53,0x53,0x05,0x05,0x04,0x04,0x05,0x53,0x05,
  0x04,0x04,0x53,0xcb,0xcb,0x05,0x04,0x03,0x04,0x05,0x05,0x04,0x04,0x03,0x01,0xb7,
  0xb7,0x01,0x02,0x03,0x05,0x02,0x04,0x03,0x03,0x05,0x53,0x04,0x04,0x04,0x05,0x03,
  0x04,0x04,0x04,0x05,0x05,0x05,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x03,0x02,0xb7,
  0x03,0x04,0x05,0x04,0x04,0x05,0x05,0x53,0xcb,0x05,0x05,0x03,0x05,0x53,0x05,0x05,
  0x05,0x04,0x03,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x05,0x05,0x04,0x02,0x02,0xb7,
  0xb7,0x01,0x01,0x02,0x25,0x04,0x04,0x03,0x03,0x04,0x03,0x04,0x04,0xcb,0xf0,0x05,
  0x04,0x03,0x05,0x05,0xf0,0xcb,0x05,0x04,0x05,0x05,0x05,0x05,0x04,0x03,0x25,0xb7,
  0x25,0x05,0x04,0x03,0xcb,0x05,0x53,0x53,0x53,0x04,0x05,0x05,0x05,0x05,0x05,0x04,
  0x05,0x05,0x03,0x04,0x05,0x04,0x04,0x05,0x04,0x04,0x04,0x04,0x03,0x02,0x02,0xb7,
  0x00,0x01,0x01,0x01,0x04,0x05,0x04,0x04,0x04,0x04,0x03,0x03,0x04,0xcb,0x05,0x04,
  0x05,0x04,0x04,0x05,0x05,0x04,0x04,0x04,0x04,0x03,0x05,0x04,0x03,0x04,0x03,0xb7,
  0x01,0x04,0x02,0x04,0xcb,0x53,0x05,0x05,0x05,0x04,0x05,0x05,0x53,0x05,0x05,0x04,
  0xcb,0x04,0x04,0x05,0x03,0x04,0x25,0x03,0x03,0x04,0x03,0x03,0x03,0x03,0x02,0xe5,
  0x00,0x02,0x01,0x04,0x05,0x04,0x04,0x04,0x04,0x04,0x04,0x03,0x02,0x04,0x04,0x04,
  0x05,0x53,0x04,0x04,0x04,0x05,0x05,0xf8,0x05,0x04,0x04,0x04,0x05,0x03,0x03,0xb7,
  0x01,0x03,0x04,0x04,0xcb,0xcb,0x53,0x04,0x05,0x05,0x05,0x05,0x05,0xcb,0x05,0x05,
  0x05,0x05,0x04,0x04,0x03,0x03,0x03,0x04,0x02,0x03,0x25,0x03,0x03,0x25,0x02,0xe5,
  0x00,0x01,0x03,0x04,0x05,0x04,0x03,0x03,0x04,0x05,0x04,0x04,0x04,0x03,0x04,0x05,
  0x05,0x05,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x53,0x04,0x03,0x03,0x00,
  0x25,0x04,0x03,0x04,0xf8,0x05,0x05,0x53,0x05,0x05,0x05,0x05,0x04,0x53,0xcb,0x05,
  0x53,0x53,0x04,0x04,0x04,0x05,0x03,0x05,0x03,0x04,0x03,0x03,0x04,0x03,0x02,0xb7,
  0x01,0x25,0x03,0x02,0xcb,0x05,0x05,0x04,0x04,0x04,0x05,0x04,0x04,0x05,0x05,0x05,
  0x53,0x05,0x04,0x05,0x05,0x04,0x05,0x05,0x04,0x04,0xcb,0x05,0x04,0x04,0x25,0x00,
  0x04,0x04,0x03,0x03,0x03,0x03,0x05,0x05,0x05,0x53,0x04,0x05,0x05,0xcb,0x53,0xf0,
  0x53,0x04,0x04,0x04,0x04,0x04,0x04,0x03,0x03,0x03,0x04,0x03,0x04,0x02,0x01,0xb7,
  0x00,0x02,0x01,0x02,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x53,0x53,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x03,0x25,0x00,
  0x02,0x25,0x04,0x03,0x03,0x05,0x05,0xf8,0xcb,0x04,0x05,0xcb,0x04,0xcb,0x53,0x53,
  0x05,0x53,0x04,0x04,0x04,0x04,0x03,0x03,0x04,0x01,0x04,0x03,0x03,0x01,0x01,0xb7,
  0x00,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x53,0xcb,0x05,0x05,
  0x53,0x53,0x05,0x05,0x05,0x05,0x04,0x04,0x03,0x05,0x05,0x05,0x04,0x03,0x02,0xb7,
  0x25,0x04,0x03,0x03,0x04,0x04,0x04,0x05,0x04,0x04,0x05,0xcb,0x04,0x05,0x53,0x05,
  0x05,0x05,0x05,0x04,0x04,0x04,0x03,0x02,0x04,0x03,0x03,0x03,0x03,0x02,0x02,0x00,
  0x00,0x01,0x01,0x01,0x03,0x04,0x04,0x05,0x03,0x04,0x05,0x04,0x53,0x53,0x04,0x04,
  0x53,0x05,0x04,0x04,0x05,0x03,0x04,0x05,0x04,0x04,0x04,0x04,0x03,0x04,0x03,0xb7,
  0x02,0x03,0x03,0x03,0x04,0x04,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x04,0x05,0x53,
  0xcb,0x05,0x05,0x04,0x03,0x03,0x25,0x04,0x04,0x02,0x03,0x03,0x03,0x03,0x02,0x00,
  0x00,0x02,0x01,0x02,0x04,0x03,0x03,0x03,0x03,0x05,0x05,0x05,0x04,0x04,0x05,0x05,
  0x05,0x04,0xcb,0x03,0x03,0x04,0x04,0x05,0x53,0xcb,0x03,0x03,0x03,0x04,0x25,0x00,
  0x01,0x01,0x03,0x25,0x05,0x05,0xcb,0x05,0x05,0x04,0xcb,0x53,0x04,0x04,0x05,0x53,
  0x53,0x05,0x04,0x03,0x04,0x03,0x03,0x04,0x03,0x02,0x03,0x04,0x03,0x25,0x01,0x00,
  0xb7,0x01,0x02,0x02,0x03,0x03,0x01,0x01,0x04,0x03,0x03,0x05,0x04,0x03,0x04,0x04,
  0x03,0x04,0x04,0x03,0x03,0x03,0x04,0x05,0x05,0x04,0x03,0x04,0x03,0x03,0x04,0xb7,
  0x01,0x03,0x01,0x25,0x04,0x04,0x05,0x05,0x05,0x04,0x04,0x05,0x03,0x05,0x05,0xcb,
  0x05,0x04,0x03,0x04,0x03,0x04,0x04,0x03,0x02,0x04,0x04,0x04,0x04,0x25,0x01,0x00,
  0x00,0x00,0x01,0x00,0x02,0x01,0x25,0x02,0x01,0x02,0x02,0x02,0x03,0x03,0x05,0x04,
  0x04,0x04,0x03,0x04,0x04,0x03,0x03,0x05,0x04,0x04,0x05,0x04,0x04,0x01,0x01,0xb7,
  0x25,0x25,0x25,0x02,0x03,0x04,0x05,0x05,0xcb,0x53,0x05,0x04,0x05,0x04,0x05,0x05,
  0x05,0x03,0x04,0x04,0x02,0x03,0x25,0x03,0x02,0x03,0x04,0x04,0x25,0x01,0x02,0xb7,
  0x00,0x02,0x00,0x03,0x25,0x01,0x03,0x03,0x03,0x03,0x02,0x25,0x04,0x03,0x03,0x03,
  0x03,0x02,0x04,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x04,0x01,0x03,0xb7,
  0x02,0x04,0x02,0x02,0x03,0x03,0x05,0xcb,0xcb,0x05,0x05,0x05,0x04,0x05,0x05,0x04,
  0x04,0x04,0x04,0x04,0x03,0x04,0x03,0x03,0x02,0x01,0x25,0x01,0x03,0x25,0x25,0x00,
  0x00,0x02,0x01,0x00,0x02,0x02,0x03,0xf3,0x03,0x02,0x03,0x03,0x04,0x04,0x04,0x04,
  0x03,0x03,0x03,0x04,0x03,0x04,0x03,0x04,0x04,0x04,0x04,0x05,0x02,0x25,0x04,0x00,
  0x00,0x02,0x03,0x02,0x04,0x04,0x03,0x05,0x04,0xcb,0x05,0x05,0xcb,0x04,0x04,0x04,
  0x04,0x04,0x05,0x04,0x03,0x03,0x03,0x03,0x02,0x00,0x01,0x03,0x04,0x02,0x02,0xb7,
  0x00,0x02,0x00,0x01,0x03,0x02,0x03,0x03,0x01,0x02,0x04,0x04,0x03,0x04,0x04,0x03,
  0x04,0x03,0x03,0x03,0x04,0x03,0x04,0x04,0x04,0x04,0x03,0x04,0x05,0x04,0x25,0xb7,
  0x01,0x01,0x03,0x25,0x05,0x04,0x04,0x04,0x05,0x05,0x05,0x04,0x05,0x53,0x05,0x05,
  0x04,0x05,0x04,0x04,0x03,0x01,0x03,0x04,0x03,0x02,0x01,0x04,0x04,0x02,0x01,0xb7,
  0x00,0x00,0x01,0x01,0x03,0x02,0x04,0x00,0x01,0x02,0x02,0x03,0x03,0x03,0x02,0x03,
  0x03,0x03,0x04,0x02,0x03,0x03,0x04,0x04,0x04,0x04,0x03,0x03,0x04,0x25,0x25,0xb7,
  0x01,0x02,0x01,0x25,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x04,0x04,0x05,0x04,
  0x04,0x05,0x04,0x04,0x02,0x02,0x01,0x03,0x03,0x01,0x03,0x04,0x04,0x02,0x01,0xb7,
  0x00,0x00,0x01,0x00,0x03,0x03,0x01,0x01,0x01,0x02,0x02,0x02,0x03,0x04,0x02,0x03,
  0x04,0x04,0x04,0x01,0x03,0x03,0x03,0x05,0x05,0x03,0x04,0x04,0x25,0x25,0x02,0xb7,
  0x04,0x04,0x25,0x25,0x03,0x04,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x05,0x05,0x05,
  0x04,0x04,0x04,0x04,0x03,0x03,0x03,0x03,0x03,0x25,0x03,0x04,0x04,0x02,0x01,0x00,
  0x00,0x00,0x00,0x00,0x25,0x03,0x25,0x02,0x01,0x25,0x02,0x01,0x25,0x04,0x03,0x03,
  0x03,0x25,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x04,0x05,0x04,0x04,0x01,0x01,0x00,
  0x02,0x25,0x02,0x02,0x02,0x04,0xcb,0x05,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x05,
  0x04,0x04,0x04,0x03,0x03,0x03,0x03,0x01,0x03,0x03,0x04,0x04,0x04,0x02,0x02,0xb7,
  0x00,0x01,0x01,0x00,0x01,0x04,0x03,0x03,0x04,0x03,0x01,0x03,0x04,0x04,0x04,0x04,
  0x04,0x03,0x04,0x04,0x04,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x02,0x02,0x01,0x00,
  0x02,0x01,0x03,0x02,0x03,0x04,0x04,0x05,0x04,0x04,0x05,0x04,0x04,0x04,0x05,0xcb,
  0x04,0x04,0x04,0x03,0x03,0x03,0x25,0x03,0x03,0x02,0x03,0x03,0x25,0x25,0x00,0xb7,
  0x00,0x02,0x01,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x02,0x01,0x02,0x01,
  0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x02,0x25,0x01,0x25,0x01,0x01,0x02,0x02,0xb7,
  0x00,0x02,0x01,0x02,0x02,0x04,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x03,0x04,0x04,0x01,0x03,0x02,0x25,0x01,0x00,0x01,0x00,0x01,0x01,0x02,0xe5,
  0xb7,0x00,0x00,0x02,0x01,0x00,0x00,0x00,0x00,0x02,0x00,0x01,0x01,0x01,0x00,0x02,
  0x02,0x25,0x02,0x25,0x25,0x01,0x02,0x02,0x02,0x25,0x01,0x01,0x01,0x01,0x02,0xb7,
  0x01,0x25,0x02,0x02,0x01,0x25,0x03,0x03,0x02,0x04,0x04,0x25,0x25,0x03,0x03,0x03,
  0x25,0x25,0x02,0x01,0x25,0x01,0x01,0x00,0x00,0x01,0x01,0x02,0x01,0x01,0x00,0xda,
  0xe5,0x00,0x01,0x00,0x00,0x01,0x02,0x00,0x00,0x02,0x02,0x02,0x02,0x00,0x02,0x01,
  0x01,0x02,0x02,0x01,0x01,0x01,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x02,0x01,0xda,
  0x00,0x01,0x00,0x01,0x00,0x01,0x01,0x25,0x03,0x01,0x02,0x25,0x25,0x25,0x25,0x01,
  0x01,0x02,0x25,0x02,0x02,0x02,0x01,0x01,0x01,0x25,0x02,0x25,0x02,0x00,0x00,0xb7,
  0xda,0xb7,0xb7,0xe5,0x00,0xb7,0x00,0x00,0xb7,0x00,0x00,0x00,0x00,0xb7,0x00,0xb7,
  0x00,0x01,0x00,0xda,0xb7,0xb7,0x00,0xb7,0xb7,0x00,0xb7,0xb7,0xb7,0x00,0xe5,0xda,
  0xda,0xb7,0xb7,0xb7,0xb7,0x00,0x00,0x00,0x00,0x00,0xb7,0x00,0x00,0x00,0x00,0x00,
  0x00,0xb7,0x00,0x00,0x00,0xb7,0xb7,0xb7,0xb7,0x01,0x00,0x01,0x00,0xb7,0xe5,0xda,
  // 3: ground grass
  0xe5,0x6c,0xa5,0x6c,0x64,0x6c,0x24,0x64,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,
  0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,
  0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,
  0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,
  0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x05,0x6d,0x00,0x20,0x08,0x20,0x10,0x20,
  0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,
  0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,
  0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,
  0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,
  0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,
  0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,
  0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,
  0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,
  0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,
  0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,
  0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,
  0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,
  0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,
  0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,
  0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,0x08,0x86,0x24,0x5c,
  0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x23,0x5c,0x10,0x87,0x18,0x87,0x00,0xa0,
  0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,
  0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x23,0x64,0x10,0xa3,0x63,0x64,
  0x25,0x75,0x18,0xa3,0x00,0xa4,0x44,0x5c,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,
  0x08,0xa5,0x10,0xa5,0x18,0xa5,0xe5,0x74,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,
  0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x26,0x75,
  0x18,0xc0,0xc6,0x6c,0x00,0xc1,0x08,0xc1,0x10,0xc1,0x63,0x54,0x18,0xc1,0x00,0xc2,
  0x08,0xc2,0x10,0xc2,0x18,0xc2,0x83,0x64,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x18,0xc3,
  0x00,0xc4,0x45,0x75,0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,0x45,0x6d,
  0xc7,0x74,0x10,0xc5,0x18,0xc5,0x00,0xc6,0x08,0xc6,0x10,0xc6,0x18,0xc6,0xe3,0x6b,
  0x00,0xc7,0xa4,0x74,0x44,0x74,0xc4,0x5b,0xc3,0x5c,0x08,0xc7,0x26,0x6d,0x86,0x74,
  0x66,0x6d,0x10,0xc7,0x85,0x75,0xe4,0x63,0x66,0x6c,0x86,0x6d,0xa6,0x6c,0x84,0x5c,
  0xbb,0x02,0x24,0xfa,0x00,0xc3,0x24,0xdb,0x00,0x24,0xe1,0x24,0x02,0x01,0xbb,0x9f,
  0x9f,0x03,0x01,0xdb,0x00,0x00,0x00,0xcf,0x01,0x01,0xdb,0x24,0xb8,0x00,0xe7,0xb8,
  0x00,0x02,0x01,0x00,0xe8,0x00,0x01,0x24,0x01,0xd1,0x00,0x00,0x01,0x01,0x24,0x01,
  0x03,0xcf,0x24,0x24,0x00,0x24,0xcf,0x24,0xb8,0x01,0x01,0xf7,0xfe,0x01,0x00,0xdb,
  0x24,0xdb,0x00,0xcf,0x24,0x24,0xdb,0x24,0x00,0x00,0x24,0x24,0xd1,0x00,0x9f,0xa4,
  0x9f,0x9f,0xcf,0x00,0xcf,0xcf,0x24,0x00,0x24,0x01,0x24,0x00,0xb8,0xb8,0xe1,0xdb,
  0x01,0x01,0x01,0x00,0xcf,0x24,0x00,0x24,0x00,0x24,0x01,0x02,0x00,0xe1,0x02,0x03,
  0xf1,0x02,0x24,0x24,0xe1,0x24,0xcf,0x01,0x00,0x01,0x01,0xd1,0xf6,0xe7,0x00,0x01,
  0x00,0x00,0x00,0x00,0x24,0x24,0xdb,0x24,0x00,0xdb,0x03,0x01,0xdb,0xbb,0x9f,0x9f,
  0xf3,0x9f,0x9f,0x00,0x00,0x24,0x24,0xb8,0x00,0xdb,0xdb,0x24,0x24,0xf6,0x01,0x00,
  0x03,0x01,0x24,0x00,0x24,0xdb,0x00,0x24,0xe7,0x24,0x00,0x00,0x00,0xbb,0x01,0xbb,
  0x03,0x00,0x01,0x24,0x24,0xf6,0xe1,0x00,0x01,0xdb,0xc3,0xd1,0xcf,0xe7,0x00,0x00,
  0xf6,0xdb,0xb8,0x24,0x24,0x00,0xdb,0x01,0x00,0x02,0x00,0xff,0x9f,0xf3,0x03,0xbb,
  0x01,0x03,0x9f,0x03,0x24,0x24,0x00,0xdb,0x01,0x24,0x24,0x24,0xe7,0xdb,0x00,0x01,
  0xdb,0xcf,0xd1,0x00,0xf6,0x24,0xdb,0x00,0x24,0x00,0x02,0x01,0x03,0xbb,0x03,0x02,
  0x01,0xf1,0x02,0x01,0x00,0xb8,0x01,0xcf,0xf7,0xdb,0x24,0xf6,0x00,0xcf,0xf6,0xf6,
  0x24,0x00,0xcf,0x24,0x01,0x02,0xdb,0xdb,0x02,0xdb,0xdb,0xff,0x03,0xbb,0x9f,0xb5,
  0x01,0xbb,0xbb,0xa4,0x03,0x24,0x00,0x00,0x00,0xdb,0x24,0x00,0x24,0xdb,0x24,0x02,
  0x00,0xe7,0x00,0x24,0x00,0x00,0xdb,0x00,0x00,0x03,0x24,0x03,0xfb,0xf3,0x02,0x03,
  0x02,0x02,0x02,0x02,0x02,0x01,0x01,0xfe,0x01,0xd1,0xd1,0xcf,0x00,0x00,0xcf,0x24,
  0x24,0x00,0xe1,0x24,0xc3,0x00,0x01,0x01,0x02,0xdb,0x03,0xa4,0x02,0x03,0xa4,0xa4,
  0xa4,0x02,0xbb,0x9f,0xf3,0x02,0x00,0x00,0x00,0x00,0x00,0x24,0xb8,0x24,0x24,0x00,
  0xc3,0xf6,0x00,0x00,0x00,0xdb,0xdb,0x24,0x24,0x24,0x02,0xa4,0x9f,0x03,0x03,0x03,
  0x02,0x02,0x02,0x02,0x03,0x02,0x01,0xcf,0xb8,0xd1,0xf6,0xe8,0xc3,0x02,0xcf,0xe1,
  0x24,0x00,0xcf,0x00,0x00,0x00,0xb8,0x01,0xbb,0x9f,0xb5,0xb7,0x01,0x03,0x03,0xf3,
  0x9f,0x02,0x01,0x01,0x9f,0x03,0x03,0x24,0x24,0x01,0x24,0x00,0xcf,0xcf,0x00,0x24,
  0x24,0xf6,0xd1,0xdb,0x00,0xdb,0x00,0x24,0x24,0x01,0x03,0x02,0x01,0x03,0x02,0x03,
  0x01,0x01,0xf2,0x02,0x01,0x9f,0x01,0xcf,0xcf,0x00,0xcf,0xf6,0x01,0xc3,0xcf,0xe1,
  0x01,0x00,0x00,0x01,0x24,0x24,0xcf,0x00,0xa4,0x9f,0xa4,0x02,0xdb,0x01,0x01,0xbb,
  0x03,0x03,0x01,0x01,0x9f,0xf3,0xa4,0xa4,0xdb,0xff,0xdb,0xd1,0xcf,0x00,0x24,0xdb,
  0xe7,0x24,0x00,0xdb,0xdb,0x00,0x24,0x24,0x24,0x02,0x03,0x01,0x03,0x03,0xf7,0x02,
  0x02,0xf2,0xf2,0xf1,0x02,0xf7,0x03,0x01,0x01,0xd1,0xdb,0x00,0x01,0x01,0x02,0xdb,
  0x02,0xdb,0x00,0x00,0x00,0xe7,0x24,0x02,0xa4,0xbb,0x02,0x01,0xdb,0x01,0xfe,0xa4,
  0xa4,0x9f,0xfe,0x01,0x01,0xbb,0x9f,0xd5,0xbb,0x01,0x24,0x00,0x00,0x24,0x24,0x24,
  0x24,0x00,0xf6,0x24,0xdb,0x00,0x00,0x24,0xdb,0xa4,0xbb,0x03,0xdb,0x01,0x02,0x02,
  0x01,0xf7,0xf7,0x01,0xd1,0x01,0x01,0x01,0x9f,0xe8,0x01,0x01,0x00,0xf6,0xdb,0x02,
  0xdb,0x00,0x01,0xc3,0x00,0x24,0x01,0xbb,0xa4,0xa4,0xbb,0xa4,0x03,0x00,0xf6,0x03,
  0x01,0x01,0x9f,0x01,0xa4,0x02,0x01,0x01,0xa4,0x01,0xdb,0x03,0x00,0x00,0x00,0xdb,
  0x00,0xff,0xdb,0xcf,0xe7,0xe7,0x00,0xa4,0x01,0xa4,0x9f,0x03,0x02,0xb5,0x02,0xb7,
  0x01,0xc3,0x02,0xf1,0xdb,0x03,0x01,0x02,0x03,0x03,0x01,0x00,0x24,0x24,0x00,0xdb,
  0x01,0x01,0x02,0xcf,0xe7,0xbb,0xbb,0x03,0x9f,0xdb,0xdb,0x00,0x01,0x01,0x01,0x01,
  0xbb,0x9f,0xbb,0x9f,0x9f,0x9f,0xfe,0x03,0x01,0x03,0x01,0x00,0x00,0x24,0x00,0xdb,
  0xdb,0x24,0xff,0xff,0xe7,0x00,0x02,0xdb,0xdb,0xdb,0xdb,0x03,0x03,0xdb,0x03,0xdb,
  0xb5,0x02,0xb5,0x9f,0x03,0x03,0x01,0x01,0x03,0x03,0x03,0xcf,0xf6,0x00,0xdb,0xdb,
  0x01,0x01,0xdb,0x24,0x02,0x02,0x9f,0xa4,0x02,0xa4,0x02,0x02,0xbb,0x01,0xa4,0x03,
  0x9f,0x03,0xbb,0x9f,0x03,0xfb,0x01,0xfc,0x01,0x02,0x03,0xdb,0x00,0x00,0xf6,0xdb,
  0x24,0x00,0x01,0xdb,0x9f,0x9f,0x03,0x01,0x02,0xdb,0xdb,0x02,0x01,0x02,0x03,0xa4,
  0x02,0x03,0x03,0x02,0x03,0xb5,0x01,0xf7,0x01,0x03,0x03,0x03,0x01,0x00,0x01,0x02,
  0x01,0x01,0xdb,0x03,0x03,0x02,0xa4,0xa4,0xb5,0xa4,0xbb,0x03,0xbb,0xfe,0x01,0xa4,
  0xb5,0x03,0x9f,0x03,0x01,0xbb,0x03,0x01,0xd1,0x01,0x03,0xa4,0x01,0x24,0x24,0x00,
  0x00,0x02,0xe1,0x02,0x03,0x03,0x03,0x03,0xdb,0xb5,0xdb,0x03,0xb5,0xdb,0x03,0xa4,
  0x02,0x01,0x01,0x01,0x02,0x03,0x03,0x03,0x01,0x03,0x9f,0x03,0x03,0x03,0x02,0x02,
  0x02,0x01,0x01,0x03,0xdb,0xbb,0x02,0xbb,0x02,0xdb,0xbb,0xbb,0x01,0x01,0xd1,0xa4,
  0x03,0x03,0x03,0xf3,0xdb,0x01,0xdb,0x01,0x01,0x03,0x03,0x00,0x01,0x01,0xcf,0xcf,
  0x24,0x24,0x02,0x03,0x01,0xf3,0x03,0xa4,0x03,0xa4,0x03,0xdb,0xb7,0x03,0x02,0xdb,
  0x02,0x02,0x01,0xb7,0x01,0x03,0x02,0x01,0xa4,0xfb,0x03,0xa4,0x9f,0x03,0xd1,0x00,
  0x01,0x03,0x03,0x03,0x02,0x9f,0xbb,0x01,0x03,0x01,0x02,0xfe,0x02,0x02,0x01,0x03,
  0xfc,0x01,0x01,0x03,0x9f,0x01,0x03,0x02,0x01,0x02,0x01,0xdb,0xdb,0x01,0xf6,0xcf,
  0xcf,0xb5,0x03,0x01,0xa4,0xf3,0xa4,0xb7,0xb5,0xa4,0xa4,0xa4,0x02,0x02,0x03,0x03,
  0xb5,0xb5,0x03,0x02,0x02,0x01,0x00,0x01,0x03,0x03,0x03,0xa4,0x03,0x03,0x02,0x00,
  0x03,0xf3,0x9f,0x03,0x03,0x03,0xbb,0x01,0xa4,0xb7,0x01,0xd1,0x01,0x00,0x9f,0xfc,
  0x01,0x01,0x03,0xdb,0x03,0x9f,0x01,0x01,0x01,0xdb,0x01,0x01,0x00,0x00,0xdb,0xb8,
  0xf7,0x02,0x01,0x03,0x9f,0x9f,0x03,0xa4,0xa4,0xa4,0x9f,0xa4,0xb5,0xa4,0x02,0xb5,
  0xdb,0x01,0xb5,0x01,0x01,0x03,0x02,0x03,0xfb,0xfc,0x03,0xbb,0x02,0x03,0x03,0x01,
  0x01,0x01,0x03,0xfb,0x03,0xdb,0xbb,0x00,0xdb,0x00,0xd1,0x00,0xd1,0x03,0x03,0x03,
  0x9f,0x01,0x03,0x01,0x03,0x03,0x01,0xfc,0xdb,0x03,0xbb,0xdb,0x01,0xf6,0x02,0x00,
  0x00,0x02,0xdb,0x02,0x03,0xa4,0xa4,0xa4,0xa4,0xa4,0xdb,0x01,0xa4,0x02,0xbb,0x03,
  0x03,0x01,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0xfc,0x01,0x03,0x01,0x02,0x02,0x01,
  0x01,0x01,0x03,0x03,0xfb,0x9f,0x02,0x01,0x01,0xdb,0xfc,0xd1,0x03,0x03,0x9f,0x01,
  0x01,0x03,0x03,0x03,0x03,0x02,0xdb,0xbb,0xfb,0xa4,0xbb,0x9f,0xb5,0x03,0xdb,0x00,
  0x00,0x02,0x02,0xdb,0x01,0xa4,0x02,0xb7,0xa4,0x03,0xb5,0x01,0x02,0x01,0x02,0x03,
  0x02,0xdb,0xf7,0x02,0x02,0xfb,0x01,0xf3,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x00,
  0xc3,0x01,0xb7,0xf3,0x03,0xf3,0x03,0x01,0xd1,0x01,0x03,0xe8,0x03,0x03,0xdb,0x03,
  0xfc,0x03,0x03,0x01,0x01,0xdb,0x02,0x01,0xdb,0xdb,0x02,0x03,0x02,0x01,0xdb,0x00,
  0x24,0xf6,0x00,0x00,0x02,0xb5,0x02,0x02,0xb5,0x01,0x02,0x03,0x03,0x03,0x02,0xa4,
  0x02,0x02,0x02,0x02,0x03,0xf3,0xfb,0x02,0x01,0x03,0x9f,0x03,0xa4,0x03,0x00,0xd1,
  0xb5,0x02,0xdb,0xbb,0xf3,0x03,0x9f,0x9f,0xa4,0x01,0xa4,0x01,0xb5,0x03,0x01,0x02,
  0x03,0xfe,0x01,0x03,0x03,0x01,0x01,0x01,0x9f,0x03,0x01,0xa4,0x01,0x00,0x00,0x01,
  0x00,0xf6,0x00,0xdb,0xdb,0xb5,0x03,0xb5,0xb5,0x02,0xb5,0x03,0x03,0xb5,0x02,0x02,
  0x02,0x02,0x02,0x02,0x03,0xf3,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x00,0xc3,0x01,
  0x01,0x01,0x00,0x24,0x00,0x03,0xdb,0xa4,0x02,0xf3,0x02,0x01,0x03,0x01,0xb5,0x01,
  0x9f,0x9f,0x01,0x01,0xa4,0x02,0x01,0x03,0xa4,0x9f,0x01,0xa4,0x00,0xf6,0x24,0x00,
  0xb8,0xc3,0xe1,0xe1,0x02,0x03,0xb5,0xa4,0x02,0x02,0x02,0x00,0xb5,0xb5,0xa4,0x03,
  0x01,0xa4,0x02,0x03,0x02,0x03,0x01,0x00,0x03,0x03,0x03,0x03,0xdb,0x00,0xdb,0x02,
  0x02,0x01,0xdb,0x24,0x24,0x02,0xb5,0xdb,0xfc,0xfe,0x00,0x00,0x02,0x9f,0xa4,0x03,
  0x03,0xa4,0x01,0xff,0xa4,0x02,0xb5,0x02,0xdb,0x01,0xdb,0x24,0x24,0xdb,0x00,0x00,
  0x02,0xcf,0xe1,0xe1,0x24,0xdb,0x02,0xa4,0x03,0x01,0x02,0x00,0xb5,0xa4,0xb5,0xa4,
  0x03,0xb5,0xb5,0x02,0x02,0x03,0x01,0x02,0x03,0x01,0x02,0x01,0xdb,0x01,0x01,0x02,
  0x02,0x02,0x00,0xc3,0x00,0xf6,0xdb,0xdb,0x03,0x03,0x00,0x01,0xff,0x03,0xb7,0x01,
  0x00,0x03,0x02,0x02,0xa4,0x02,0xdb,0x02,0x02,0x03,0x24,0x00,0x24,0xe1,0x00,0x00,
  0x02,0xd1,0x01,0xcf,0x24,0x00,0x02,0xb5,0x01,0x02,0x9f,0xb7,0xb5,0x02,0x01,0x9f,
  0x02,0xa4,0xb5,0xa4,0xa4,0x9f,0x01,0x03,0x01,0x03,0xc3,0x24,0xcf,0x24,0x00,0x01,
  0x03,0xb5,0x02,0xa4,0x01,0x01,0x00,0xbb,0xa4,0x01,0x01,0x01,0xd1,0x03,0x03,0x03,
  0x01,0x03,0x02,0x03,0xa4,0x03,0xbb,0x01,0x03,0xdb,0x01,0x24,0xcf,0xfa,0x24,0x00,
  0x02,0xe1,0xb8,0x00,0x01,0x00,0xdb,0x01,0x02,0xb5,0x03,0x03,0x02,0x02,0xb5,0xb5,
  0x03,0x03,0xb5,0x03,0xb5,0xfb,0xf3,0x9f,0x03,0xc3,0xb8,0xcf,0xd1,0x01,0xd1,0x00,
  0x00,0xb7,0xb8,0x02,0xb8,0x24,0x02,0x01,0x01,0x01,0xdb,0xfc,0x03,0x03,0x02,0x03,
  0xbb,0x01,0x03,0xdb,0xa4,0x9f,0x9f,0x01,0x00,0x01,0x24,0x24,0x24,0x24,0xe1,0x00,
  0x00,0xb8,0x02,0x02,0x00,0x24,0x00,0xb8,0x01,0x02,0xb5,0x02,0x02,0x02,0x02,0x03,
  0x03,0x03,0xa4,0x02,0x9f,0x9f,0x9f,0x9f,0xdb,0xd1,0x00,0xcf,0x01,0x01,0xcf,0x00,
  0xdb,0x03,0x01,0xdb,0x00,0x00,0x03,0x03,0x00,0x01,0xa4,0x02,0x02,0x03,0x01,0x01,
  0x01,0x9f,0x03,0xa4,0xa4,0x03,0xbb,0x00,0xdb,0x01,0x00,0x00,0xf6,0xb8,0xfa,0xf6,
  0xf6,0x24,0xdb,0x00,0x24,0xc3,0xb8,0x00,0xcf,0x00,0xef,0x02,0x02,0x00,0x02,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x01,0x01,0x01,0xf6,0x01,0x00,0xc3,0x00,0xa4,
  0x01,0xb7,0x02,0x00,0xcf,0x00,0x03,0x01,0x24,0x01,0xff,0x01,0x01,0x01,0x02,0x03,
  0x01,0xbb,0xa4,0xf3,0xa4,0x9f,0xdb,0xb8,0x24,0xf6,0xf6,0x01,0x00,0xe1,0xe7,0x24,
  0x24,0x00,0x01,0x00,0xf7,0x00,0x01,0x24,0xcf,0xcf,0xd1,0x02,0x02,0x02,0x03,0x02,
  0x03,0x03,0x03,0x01,0x02,0x01,0x02,0xd1,0x01,0x03,0x24,0x00,0x01,0xdb,0xff,0x01,
  0x00,0xdb,0x00,0x02,0x00,0x02,0xfb,0xdb,0x00,0x02,0x00,0x01,0x01,0xc3,0xb5,0xdb,
  0xdb,0x02,0x01,0x03,0x02,0x00,0x00,0xcf,0xcf,0xcf,0xd1,0xe8,0x00,0x00,0x24,0xfd,
  0xcf,0xe7,0xe7,0x00,0xe1,0x24,0xdb,0x01,0xb8,0xc3,0xcf,0x02,0x02,0x02,0x02,0xdb,
  0xdb,0x03,0xb5,0x03,0x02,0x01,0xdb,0x00,0x02,0xfe,0x01,0xcf,0x00,0xc3,0xdb,0xc3,
  0x00,0x00,0x00,0xb7,0x01,0x03,0x00,0xd1,0x00,0x00,0xf6,0x00,0xfc,0xdb,0xb5,0x03,
  0xa4,0x02,0x01,0xbb,0x24,0x00,0x00,0x00,0xf6,0xf6,0x00,0xf6,0xdb,0x00,0xe7,0x24,
  0xcf,0x24,0x00,0x24,0x00,0x00,0x00,0x00,0x00,0xc3,0xcf,0x00,0xb5,0x03,0x02,0x01,
  0x02,0x03,0x02,0x00,0x00,0xc3,0x01,0xb8,0x00,0x00,0x02,0x01,0x00,0x24,0x00,0x00,
  0x00,0xdb,0x01,0xdb,0x02,0x02,0xb8,0xe1,0x00,0xdb,0xdb,0x01,0x01,0x00,0x03,0xdb,
  0xbb,0x02,0x00,0x00,0xdb,0xdb,0xdb,0x00,0xf6,0x01,0xf6,0xf6,0x01,0xdb,0x24,0xf8,
  0x24,0x24,0x00,0xdb,0x00,0x24,0x00,0xb8,0xe1,0x01,0x00,0xdb,0x00,0xb5,0x02,0x02,
  0xa4,0x03,0xdb,0x02,0x00,0x00,0x01,0x01,0xdb,0xc3,0x02,0x02,0x02,0x24,0x00,0x00,
  0x01,0x00,0xc3,0x01,0x00,0x01,0x00,0xe7,0x00,0x01,0xdb,0x00,0x00,0x24,0x00,0x03,
  0x00,0x03,0x00,0x24,0x01,0xdb,0xdb,0xfc,0xd1,0xfe,0xbb,0x01,0x24,0x00,0x24,0x24,
  0xfa,0x00,0xdb,0x00,0x00,0x01,0x01,0x00,0x02,0x02,0x01,0xdb,0x00,0x01,0x03,0xb5,
  0xa4,0x01,0x01,0x00,0x24,0xe1,0xf1,0x01,0x02,0x24,0x00,0xdb,0xf7,0xc3,0x03,0x01,
  0xcf,0x00,0x02,0xdb,0xb5,0xb5,0x01,0xc3,0x01,0xb8,0xcf,0xb8,0x01,0x02,0xdb,0x02,
  0x02,0x24,0x24,0x01,0x24,0x00,0xd1,0xf6,0xd1,0xbb,0x01,0x24,0x24,0x24,0x00,0xe7,
  0xe7,0xe1,0xdb,0xdb,0xdb,0xdb,0x02,0x01,0x00,0x03,0x01,0xf7,0x00,0xdb,0xdb,0x02,
  0x01,0x00,0x01,0x02,0x02,0x00,0x24,0x00,0xc3,0x24,0x00,0xc3,0xc3,0x00,0xdb,0x01,
  0xdb,0xb8,0x00,0x00,0xa4,0xdb,0x24,0x00,0x00,0x01,0x24,0xb8,0x24,0xc3,0x01,0x02,
  0x02,0x01,0x02,0xcf,0x01,0xd1,0xfe,0xd1,0xfe,0xfe,0x01,0x00,0x00,0x01,0x00,0xf6,
  0x24,0x01,0xdb,0xdb,0x02,0xdb,0x01,0x01,0x02,0x03,0x00,0x00,0xdb,0xdb,0xdb,0x01,
  0x02,0x24,0xdb,0xdb,0x00,0x01,0x00,0x00,0xb8,0xe1,0x24,0x01,0xfc,0x01,0x00,0x02,
  0x02,0x02,0x00,0xdb,0x01,0x01,0x02,0x02,0xd1,0x00,0xc3,0xc3,0x24,0x00,0x01,0xb5,
  0x01,0x02,0x01,0x01,0xfe,0x00,0x00,0xcf,0xfe,0xd1,0x01,0x01,0x01,0x00,0x01,0x24,
  0xe7,0xb8,0x24,0x24,0xd1,0xe1,0xdb,0xdb,0xdb,0x01,0x02,0x01,0x02,0x00,0xb5,0x9f,
  0xdb,0x01,0x00,0x00,0xdb,0x00,0x24,0x00,0xe1,0xb8,0xb8,0x00,0x02,0x02,0x03,0x02,
  0x02,0x01,0x01,0x03,0x01,0x02,0x01,0xf6,0xfa,0xb8,0xcf,0x00,0x24,0x03,0x03,0x01,
  0x03,0x03,0xa4,0xcf,0x24,0x00,0x00,0x24,0x00,0xdb,0x01,0xdb,0x24,0xcf,0x00,0x24,
  0x24,0xe1,0x24,0x00,0x24,0x01,0xfc,0x02,0xdb,0xa4,0x00,0x01,0x24,0xdb,0x03,0xb5,
  0x03,0xdb,0x00,0x01,0x00,0x24,0x00,0xe7,0xb8,0xb8,0x00,0x03,0x01,0x01,0xcf,0xd1,
  0xc3,0x00,0x03,0x01,0x02,0xb8,0x00,0xd1,0xb8,0xcf,0xcf,0x00,0x00,0x02,0x03,0x9f,
  0x03,0xa4,0x9f,0xb5,0x00,0x24,0x00,0x00,0x24,0x00,0x00,0xdb,0x00,0x24,0xe1,0x24,
  0xc3,0x24,0x00,0xe7,0x02,0x01,0xfc,0x01,0xbb,0x01,0x03,0x00,0xdb,0x01,0xa4,0xb7,
  0x02,0xa4,0x03,0xfe,0x00,0x00,0x01,0x24,0x24,0x00,0x00,0x02,0x02,0xb8,0xcf,0xf2,
  0x01,0x02,0xc3,0xd1,0x01,0x00,0x01,0xb8,0xcf,0xcf,0xcf,0xdb,0x03,0x9f,0x9f,0x03,
  0x03,0x03,0xf3,0x9f,0x01,0xf6,0xb8,0x24,0x00,0xb8,0x00,0x24,0x24,0x00,0x00,0x00,
  0xc3,0xb8,0x9a,0x00,0x01,0x01,0x00,0x01,0xdb,0xdb,0xdb,0x01,0x03,0x03,0xa4,0x02,
  0x02,0xa4,0x02,0xf7,0xdb,0xd1,0x24,0x00,0x24,0x00,0x24,0x00,0x02,0xc3,0x00,0x00,
  0x02,0xf1,0x02,0x02,0x02,0x01,0xd1,0xcf,0xcf,0xcf,0xc3,0x01,0x9f,0x03,0x03,0xf3,
  0xfb,0xfb,0xf3,0x9f,0xa4,0xbb,0x00,0x24,0x24,0x00,0xb8,0xc3,0x02,0xc3,0x02,0x00,
  0x00,0x00,0x00,0xd1,0x24,0x01,0x00,0xdb,0x00,0x24,0xdb,0x02,0x03,0xa4,0xa4,0xdb,
  0xdb,0xa4,0x9f,0x03,0x00,0x03,0x00,0x00,0x00,0x00,0xa4,0x02,0xc3,0x02,0x00,0xc3,
  0x02,0xf2,0x02,0x01,0xf7,0xd1,0xd1,0xd1,0xc3,0x02,0x03,0x02,0x01,0x9f,0xf3,0xfb,
  0x03,0x9f,0xa4,0xa4,0xbb,0x01,0xc3,0x24,0xdb,0x00,0x24,0x00,0x00,0xf1,0xcf,0xb8,
  0x02,0x01,0x02,0x01,0xc3,0xd1,0xcf,0x00,0x01,0x03,0xb7,0xb5,0xa4,0xa4,0x03,0x00,
  0xdb,0x03,0xbb,0x03,0x01,0x02,0x02,0x00,0xdb,0xff,0xdb,0x02,0x02,0x00,0x02,0xc3,
  0x02,0x00,0x00,0x01,0x02,0xc3,0xd1,0xd1,0x01,0xfb,0x9f,0x01,0xfc,0x01,0x03,0x03,
  0x03,0x03,0xb5,0x03,0x01,0x01,0x01,0xdb,0x00,0x01,0xc3,0xf1,0xb8,0x00,0xcf,0x00,
  0x01,0xdb,0xdb,0xb8,0xcf,0x00,0x00,0x00,0x03,0x03,0xb5,0x03,0xb5,0x02,0xa4,0x03,
  0xdb,0xdb,0x03,0x02,0xdb,0x02,0xbb,0xb5,0x00,0x02,0x01,0x03,0x02,0x00,0xc3,0x00,
  0x02,0x00,0xa4,0xdb,0x01,0x02,0xf1,0x03,0xbb,0x03,0xfb,0x03,0x03,0x03,0x01,0xa4,
  0x01,0x9f,0x03,0x03,0x02,0xb5,0xa4,0xb5,0x02,0x24,0xcf,0xb8,0xcf,0xe1,0xe1,0xf1,
  0x00,0x00,0x00,0x01,0x00,0x24,0xb8,0x01,0x03,0xa4,0x03,0x03,0x03,0x03,0xdb,0xb7,
  0x00,0x03,0xa4,0xa4,0x02,0x01,0x03,0x02,0x01,0x00,0x02,0x02,0x00,0x02,0xcf,0x00,
  0x01,0x01,0x02,0xb5,0x02,0xcf,0x02,0xb5,0x03,0x02,0x03,0x02,0x03,0xdb,0x03,0x9f,
  0xfb,0x02,0x02,0xb5,0x03,0x02,0xa4,0xb7,0x00,0x00,0xb8,0xb8,0xb8,0xb8,0xf1,0x00,
  0xb8,0xe1,0x02,0x01,0x02,0x00,0xdb,0x02,0x03,0x03,0xa4,0x02,0xa4,0xa4,0xb7,0x03,
  0xa4,0x02,0x03,0xa4,0x03,0x01,0x02,0x03,0xa4,0x03,0x02,0x00,0xb8,0xb8,0x00,0x00,
  0x01,0x01,0x00,0x01,0xc3,0x03,0x03,0x02,0x03,0x03,0x02,0xa4,0x01,0x02,0x9f,0xb5,
  0xb5,0x03,0x02,0xdb,0x01,0x02,0x01,0x02,0x00,0x02,0xdb,0xe1,0xb8,0xb8,0x24,0x01,
  0xcf,0xdb,0x01,0x02,0x02,0x02,0xdb,0x03,0xb5,0x03,0x02,0x02,0xb5,0xb5,0xa4,0xb5,
  0xb5,0xb7,0xb5,0x02,0x03,0x02,0xb5,0x03,0x02,0xa4,0xb5,0x02,0xc3,0xb8,0x24,0x01,
  0xc3,0xe1,0xdb,0xcf,0xd1,0x03,0x02,0x02,0x01,0xdb,0x01,0x02,0x02,0x02,0x03,0xf3,
  0xb5,0x01,0x03,0xa4,0x02,0x02,0x01,0xf1,0x01,0xa4,0x03,0x02,0xe7,0xcf,0xcf,0xb8,
  0xe1,0x24,0x00,0x00,0xdb,0xbb,0x03,0xdb,0xa4,0x02,0x02,0x03,0xa4,0xb5,0xb5,0xb5,
  0xdb,0x02,0xdb,0xbb,0xa4,0x03,0x03,0xb5,0x9f,0xb5,0xa4,0xb5,0xf1,0xf1,0xb8,0x01,
  0xb8,0x00,0xdb,0x02,0x02,0x03,0x03,0x03,0x00,0x00,0x01,0x03,0x03,0xdb,0x9f,0x9f,
  0xa4,0xa4,0xdb,0xb5,0x02,0x03,0x01,0x02,0xb7,0x03,0xa4,0x01,0x00,0xb8,0xcf,0xb8,
  0xcf,0x01,0xcf,0xf1,0xb5,0xa4,0xbb,0xa4,0x03,0x03,0x02,0xb5,0xa4,0xa4,0x02,0x01,
  0x02,0xa4,0xa4,0xb5,0xa4,0x02,0xef,0xb5,0xbb,0x03,0x03,0xa4,0x03,0x02,0xf1,0xc3,
  0xdb,0xdb,0x02,0x03,0xb5,0x9f,0x01,0x02,0x03,0x01,0xfc,0x01,0x02,0xb7,0x02,0x02,
  0xdb,0xa4,0x03,0x02,0x01,0xa4,0x01,0x01,0xa4,0x9f,0x03,0x02,0x03,0xf1,0xcf,0xb8,
  0x01,0xb8,0x01,0x03,0x9f,0xf3,0x03,0x02,0xb5,0xb5,0x01,0x01,0x02,0xb5,0x01,0x01,
  0xdb,0xa4,0xb5,0xdb,0x03,0x03,0x02,0x03,0x9f,0xfb,0xbb,0xfb,0xb5,0x02,0xdb,0x00,
  0xe8,0xa4,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xb7,0xdb,0x02,0x01,0x03,
  0x01,0x00,0xa4,0xb5,0xa4,0xdb,0xc3,0xdb,0xbb,0x03,0xa4,0x02,0x02,0x02,0x01,0xcf,
  0xcf,0xf2,0x03,0x03,0x01,0xfb,0xb5,0x03,0xa4,0xa4,0xb5,0xb5,0xb5,0xb5,0xb7,0xb5,
  0xb5,0x02,0x02,0xa4,0x02,0x03,0x02,0x03,0x02,0x02,0x02,0x02,0xb5,0x02,0x02,0xf2,
  0x02,0x02,0x03,0xb5,0x03,0xa4,0x9f,0x03,0xb5,0x03,0x02,0x02,0x01,0x01,0x03,0x00,
  0x02,0x02,0xb7,0xa4,0xa4,0x02,0x01,0xa4,0xdb,0x02,0x03,0x00,0xa4,0x02,0x00,0x01,
  0xc3,0x02,0xdb,0x03,0x03,0x03,0x03,0x03,0xb7,0xa4,0x03,0xb5,0x02,0x02,0xb5,0xb5,
  0x03,0x02,0x02,0x02,0x01,0xa4,0x03,0xbb,0x9f,0x01,0x02,0x02,0x02,0x03,0x03,0x02,
  0x01,0x03,0x02,0x03,0x03,0x03,0xb5,0x03,0x03,0xef,0x02,0xb5,0xb5,0x02,0x02,0x02,
  0xb7,0xb5,0xa4,0xa4,0x03,0x02,0x00,0xdb,0xdb,0xdb,0x01,0x02,0x01,0xdb,0x01,0x00,
  0x02,0x02,0x02,0xdb,0xa4,0x03,0xb5,0x03,0xb5,0xb7,0xa4,0xb5,0x02,0x02,0xa4,0xb5,
  0xf1,0xa4,0x00,0x01,0xb5,0x03,0x03,0x03,0xb5,0x01,0x01,0x01,0x01,0x03,0x03,0xf2,
  0xc3,0x02,0x02,0x01,0x01,0x03,0x03,0x03,0xa4,0xfb,0x03,0x01,0xdb,0xdb,0xb5,0xb5,
  0xb5,0xdb,0x02,0xa4,0x02,0xb7,0xdb,0x00,0xb7,0x02,0x02,0x02,0xb7,0x00,0x01,0xb8,
  0xf7,0x00,0x02,0x03,0xb5,0xb5,0xbb,0xfb,0x03,0x03,0x02,0x02,0x02,0xb5,0xb5,0x02,
  0x03,0xdb,0xf2,0x02,0xa4,0x03,0x01,0x02,0x03,0x01,0x02,0x02,0x03,0x01,0x02,0xb8,
  0xc3,0x02,0x01,0x01,0x01,0x03,0x03,0x03,0xa4,0xb5,0x02,0x01,0x02,0x02,0xdb,0x01,
  0x01,0x01,0x01,0x01,0x01,0x02,0x02,0xdb,0x02,0x03,0x03,0x03,0x00,0xb8,0x02,0x02,
  0x00,0x00,0x00,0xb5,0xb5,0xa4,0xa4,0x02,0xa4,0xb5,0xdb,0x02,0xf7,0x02,0xb5,0x01,
  0x01,0xbb,0x01,0x03,0xf3,0x01,0x02,0x02,0x03,0x02,0x03,0x02,0x02,0x01,0xc3,0xb8,
  0x24,0x01,0xc3,0x01,0x01,0x03,0x03,0x03,0x03,0x03,0xf7,0x02,0x02,0xb5,0x02,0x02,
  0xd1,0xdb,0x00,0x01,0x02,0x02,0xb5,0x01,0x02,0x03,0x01,0x03,0xdb,0xb8,0x02,0x02,
  0x00,0x00,0x00,0x02,0x9f,0xa4,0xa4,0x03,0x03,0x03,0xf2,0xf1,0x02,0xf2,0xf1,0x03,
  0x03,0x03,0x02,0x03,0x03,0xb5,0x01,0xb5,0x03,0xb5,0xbb,0x03,0xb7,0x01,0x01,0x00,
  0xdb,0xbb,0xdb,0xdb,0x02,0xbb,0x03,0x02,0x02,0x02,0xc3,0x02,0x02,0x02,0xdb,0xc3,
  0xc3,0x03,0x00,0xdb,0x02,0x02,0x01,0x01,0x03,0x01,0xb5,0xdb,0x01,0x24,0x24,0x00,
  0x24,0x00,0x00,0xdb,0x02,0x02,0x03,0xdb,0x02,0x02,0x02,0x02,0x02,0xf2,0x02,0x02,
  0xb5,0x03,0x02,0xb5,0xa4,0xb5,0x03,0xb5,0xf3,0x02,0x02,0xdb,0xb8,0xc3,0xd1,0xf6,
  0x01,0x00,0xdb,0xdb,0xdb,0xdb,0x03,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0xc3,0xb8,
  0x02,0xdb,0x02,0x02,0x01,0xc3,0x03,0x01,0x02,0x02,0x01,0x01,0x03,0xdb,0x00,0xdb,
  0xdb,0xe7,0xcf,0x24,0xb8,0xdb,0x03,0x03,0x03,0x02,0x01,0x02,0xdb,0x02,0x02,0x03,
  0xfb,0xdb,0x02,0xb5,0xb5,0xb5,0x03,0x03,0xa4,0xfb,0xf7,0xc3,0xc3,0xcf,0xcf,0xe8,
  0xf6,0xcf,0xdb,0xdb,0xe7,0x24,0xdb,0xb5,0x02,0x03,0x02,0x03,0x02,0x02,0x02,0xc3,
  0x00,0xb5,0x00,0x00,0x02,0x01,0xa4,0x03,0xb5,0x00,0xc3,0xdb,0xa4,0x24,0x24,0xdb,
  0xb8,0xe1,0xe1,0x24,0x00,0x00,0xdb,0x03,0x02,0xef,0xfb,0x02,0x02,0x03,0x02,0x03,
  0xb5,0xdb,0x03,0xb5,0x02,0x03,0x03,0xa4,0x03,0xf7,0xc3,0x00,0xcf,0xe7,0xf6,0xb8,
  0x00,0x00,0x00,0xdb,0x24,0xb8,0xe1,0x02,0xa4,0x01,0xbb,0xdb,0x02,0x03,0xdb,0x01,
  0x01,0xb5,0xb7,0xb5,0xdb,0x01,0x01,0xb5,0xdb,0xc3,0x02,0xdb,0x00,0x00,0x00,0xdb,
  0xe1,0x24,0xe7,0xe7,0x00,0xdb,0x00,0xdb,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0xf2,
  0x03,0x03,0xdb,0x02,0x02,0x02,0x03,0xdb,0x02,0x24,0x01,0xd1,0xd1,0xf6,0xcf,0x01,
  0xf6,0xdb,0x00,0xb7,0xdb,0x00,0x00,0x00,0x03,0xd1,0x01,0xa4,0xb5,0x02,0x02,0x01,
  0x03,0x02,0x01,0x02,0x02,0x03,0xa4,0xdb,0x24,0x01,0x00,0xdb,0x00,0x00,0x24,0xdb,
  0x00,0x24,0xb8,0x00,0xdb,0x00,0x02,0x01,0x02,0xa4,0x03,0x02,0xef,0x01,0x01,0x02,
  0xb5,0xdb,0xb5,0xdb,0x03,0xdb,0x01,0x02,0xc3,0xcf,0x00,0x00,0x24,0xcf,0xf6,0xd1,
  0xdb,0xc3,0xdb,0xdb,0xdb,0xdb,0x24,0x00,0xcf,0xf6,0x01,0xdb,0x01,0x01,0x01,0x01,
  0x01,0xa4,0x01,0x01,0xdb,0xf3,0xb7,0xb8,0xb8,0x24,0xdb,0xdb,0x01,0x24,0x00,0xdb,
  0x01,0x24,0xdb,0x01,0xdb,0xd1,0x00,0x02,0x00,0x03,0xa4,0x02,0x02,0x02,0xdb,0x01,
  0x02,0xb5,0x03,0x02,0x03,0x01,0x02,0x24,0xc3,0xcf,0xf6,0xe7,0x00,0xcf,0xf6,0xd1,
  0xfc,0xc3,0x24,0x24,0x24,0xcf,0xe1,0xcf,0x24,0x01,0x03,0x03,0x03,0x03,0x03,0x01,
  0xbb,0x03,0x02,0x01,0x02,0x02,0x00,0xcf,0x00,0xcf,0x24,0xdb,0xb8,0x24,0xe1,0xdb,
  0xdb,0xc3,0x00,0xd1,0xe8,0x00,0x00,0x00,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x00,
  0xdb,0xb5,0x01,0x00,0x02,0xdb,0xc3,0x00,0x01,0x00,0xcf,0xd1,0xd1,0x24,0xd1,0x24,
  0xb8,0x01,0x24,0xcf,0x00,0x24,0x00,0x00,0x00,0xe1,0xdb,0xa4,0xb5,0xb5,0x02,0x01,
  0x02,0x01,0xa4,0xdb,0x03,0x24,0xc3,0xc3,0x00,0xe1,0x00,0x24,0xdb,0xe7,0x24,0x24,
  0x24,0x01,0x24,0xf6,0xd1,0x00,0x00,0x01,0x01,0xdb,0x01,0x03,0xb5,0x03,0x02,0x01,
  0x01,0x01,0x01,0x01,0xdb,0xe1,0xe7,0x24,0xb8,0x24,0xe1,0x24,0x00,0x01,0x01,0xd1,
  0x00,0xdb,0x00,0xcf,0xcf,0xf6,0xf6,0xf6,0xd1,0x01,0x00,0x00,0x02,0xa4,0x00,0x02,
  0x03,0x03,0xa4,0x03,0x02,0xdb,0x00,0xcf,0xb8,0xb8,0xb8,0xdb,0xf4,0x24,0xb8,0x00,
  0x24,0x24,0x00,0x24,0x00,0x24,0xc3,0x00,0x00,0xdb,0x01,0x00,0x02,0xb5,0x03,0x02,
  0xd1,0x00,0xdb,0x01,0x24,0xb8,0xc3,0x00,0x24,0xb8,0x00,0x00,0x24,0x01,0x00,0x01,
  0xcf,0xdb,0x03,0x00,0xf6,0xf8,0x00,0xd1,0xe8,0xfe,0x00,0xc3,0x00,0xdb,0x01,0x01,
  0xfb,0xa4,0xb5,0xdb,0x24,0xb8,0xc3,0xcf,0xc3,0x24,0x00,0xdb,0xdb,0x02,0x24,0xdb,
  0x24,0xb8,0x24,0x00,0xdb,0x00,0xcf,0x01,0x00,0x00,0x01,0x01,0x02,0x01,0x01,0x02,
  0x03,0x01,0x00,0xb8,0xe1,0x01,0x24,0xb8,0xb8,0xb8,0x00,0x24,0xf6,0x01,0x01,0xf6,
  0x00,0x01,0xc3,0x24,0x24,0xf6,0x24,0x01,0x00,0xf6,0x24,0x24,0xb8,0xe1,0x01,0x01,
  0x03,0x9f,0x03,0x00,0x02,0xcf,0xcf,0xcf,0x01,0x00,0x24,0xdb,0xdb,0xdb,0xa4,0xd5,
  0x00,0x01,0x24,0xdb,0xdb,0x00,0xf6,0xd1,0xf6,0x24,0x00,0x00,0xdb,0x24,0xdb,0x02,
  0xdb,0xcf,0xe1,0xb8,0x24,0xe7,0xe1,0x24,0x00,0x00,0xd1,0x01,0x01,0x24,0x00,0x24,
  0xdb,0x02,0xcf,0xe1,0xfa,0x00,0xe7,0x01,0x02,0x24,0xcf,0x24,0x00,0x01,0x01,0x01,
  0x03,0xdb,0x01,0x00,0x00,0xcf,0xcf,0xf8,0xdb,0x03,0xdb,0x01,0xdb,0xdb,0xb7,0xdb,
  0x00,0x03,0x00,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0x02,0x00,0xf1,0x00,
  0x24,0x24,0xb8,0x00,0x00,0x24,0xfa,0x24,0xb8,0xdb,0x01,0xe8,0x01,0xdb,0xe1,0x00,
  // 4: ground mud
  0xaa,0x9b,0x08,0x8b,0xeb,0xa3,0xca,0x9b,0x49,0x8b,0x00,0x00,0x08,0x00,0x10,0x00,
  0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,
  0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,
  0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,
  0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,
  0x2c,0xa4,0x6a,0x9b,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,
  0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,
  0x08,0x24,0x0b,0xa4,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,
  0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,
  0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,
  0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,
  0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,
  0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,
  0x00,0x60,0x08,0x60,0x10,0x60,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,
  0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,
  0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,
  0x69,0x93,0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,
  0x18,0x67,0x00,0x80,0x6a,0x8b,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,
  0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x10,0x83,
  0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,
  0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,
  0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,
  0x18,0xa1,0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,
  0x18,0xa3,0x89,0x93,0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,
  0x10,0xa5,0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,
  0x10,0xa7,0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x4c,0xa4,0x00,0xc1,
  0xc8,0x7a,0x6c,0xb4,0x08,0xc1,0x10,0xc1,0x18,0xc1,0xec,0xa3,0x00,0xc2,0x08,0xc2,
  0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x0b,0x9c,0x18,0xc3,0x00,0xc4,
  0x08,0xc4,0x10,0xc4,0x18,0xc4,0xe8,0x82,0x00,0xc5,0x29,0x7b,0x08,0xc5,0x2a,0x8b,
  0x10,0xc5,0x18,0xc5,0x00,0xc6,0x08,0xc6,0x10,0xc6,0x8a,0x8b,0x18,0xc6,0x00,0xc7,
  0x08,0xc7,0x89,0x8b,0xac,0xb4,0x6b,0xa4,0xa8,0x72,0xab,0xa3,0x29,0x93,0x4b,0x9c,
  0x2c,0x9c,0xce,0xbc,0xa7,0x82,0xc7,0x7a,0xa7,0x7a,0x89,0xa3,0xca,0x8b,0x08,0xe1,
  0x02,0x28,0x00,0xe5,0xe3,0x80,0xe5,0xe5,0x00,0x03,0x00,0x04,0x00,0x28,0x29,0x04,
  0x02,0x04,0xf3,0xf2,0xdd,0x28,0x28,0x39,0xdd,0xd1,0x02,0x39,0x00,0xf3,0x39,0x00,
  0x00,0x39,0xb9,0xe5,0x00,0xb9,0x00,0x39,0xe5,0x02,0x00,0xe3,0x04,0x02,0x02,0x80,
  0xb9,0xe5,0xe7,0xd0,0xf4,0x8a,0x00,0x01,0x04,0x03,0xce,0x03,0x02,0x04,0x03,0xb9,
  0xb9,0x02,0x04,0x29,0xe3,0xe3,0xf4,0xb9,0x00,0xe5,0x04,0xe5,0x00,0x00,0x29,0x00,
  0xf1,0x01,0x02,0xce,0xdd,0xce,0x39,0x03,0x02,0x02,0xce,0x29,0x03,0x39,0x39,0x29,
  0xb9,0xe3,0xb9,0xd5,0x00,0x29,0x39,0x8a,0xb9,0x03,0xb9,0x29,0x03,0x03,0x00,0x00,
  0x03,0x01,0xf4,0xe5,0x8a,0x03,0xdd,0xd0,0xf1,0xe5,0x28,0xdd,0x03,0x00,0x00,0x8a,
  0x00,0x02,0x04,0x04,0x29,0x80,0x80,0x00,0x02,0x03,0x03,0x39,0x00,0x01,0x8a,0x80,
  0x39,0xdd,0x00,0x02,0x03,0x80,0x00,0xb9,0x00,0x04,0x04,0x01,0xb9,0xb9,0x02,0x04,
  0xe3,0x04,0xe3,0x29,0x29,0x8a,0x29,0x29,0x03,0x00,0x00,0x28,0x00,0x00,0x00,0x29,
  0x80,0x01,0xf4,0x04,0x03,0x00,0x80,0x80,0xdd,0x04,0x80,0x00,0x03,0x03,0x03,0x00,
  0xd5,0xd1,0x00,0xb9,0x04,0xd1,0x02,0x00,0xce,0xd1,0xd1,0x02,0x04,0x00,0x00,0x04,
  0x00,0x80,0x00,0x03,0xe3,0x01,0x04,0x00,0x04,0xb9,0x01,0x04,0x01,0xb9,0x28,0x29,
  0xe7,0x8a,0x04,0x04,0x8a,0xe5,0x04,0x02,0x03,0x80,0x00,0x80,0x04,0x00,0xb9,0x80,
  0x04,0xb9,0x00,0xd0,0xe3,0x8a,0x80,0xe3,0x04,0x03,0x03,0x02,0x03,0xb9,0x00,0x02,
  0xfd,0x00,0x29,0xb9,0xb9,0xb9,0xf5,0x39,0x02,0x28,0xd5,0x39,0x39,0x03,0x00,0x01,
  0x04,0xe3,0x00,0xed,0xb9,0x00,0xdd,0x03,0x00,0xb9,0x00,0x39,0x39,0x39,0x28,0x02,
  0x8a,0xdd,0xdd,0x03,0xe7,0x8a,0xb9,0x03,0x03,0x03,0x02,0x02,0x00,0x00,0x00,0x03,
  0xb9,0x00,0xe3,0xe5,0xdd,0x00,0x8a,0xe3,0xb9,0x02,0x03,0x02,0x02,0x02,0x00,0xb9,
  0x02,0x03,0x00,0xfb,0x01,0xb9,0x00,0x28,0x39,0xb9,0xd1,0xce,0x03,0x03,0x02,0x04,
  0xf3,0xe3,0xe3,0x04,0x00,0x03,0x03,0x04,0x04,0x01,0x03,0x04,0xdd,0x03,0x02,0x03,
  0xf4,0xdd,0xdd,0x04,0xe3,0x03,0x04,0x29,0x28,0x39,0x39,0x00,0x80,0x00,0x80,0x04,
  0x29,0x00,0xf4,0xd0,0x39,0x03,0xb9,0x80,0x80,0x80,0x04,0xdd,0x39,0x39,0x28,0x00,
  0x39,0x00,0x03,0x80,0x01,0xd1,0xd1,0x02,0x03,0x39,0xd1,0xd1,0x00,0xed,0xdd,0xf1,
  0x00,0x03,0x01,0xfb,0xe5,0x00,0xb9,0x29,0x03,0x39,0x03,0xb9,0x01,0x39,0x03,0x03,
  0x8a,0x39,0x39,0x00,0x00,0x03,0x04,0x80,0x28,0x03,0x00,0xb9,0x01,0xb9,0xb9,0x04,
  0x04,0x04,0xd0,0xfb,0x03,0xb9,0x00,0x80,0xb9,0x03,0x28,0xce,0x28,0x39,0x28,0x28,
  0xb9,0xb9,0x00,0x02,0x03,0x39,0xb9,0x39,0x00,0xd1,0xd1,0xdd,0xf1,0x00,0x39,0x04,
  0x03,0x03,0xb9,0x03,0x03,0xdd,0x04,0x03,0xdd,0x00,0x03,0x03,0xf1,0xdd,0x04,0x00,
  0x00,0x02,0x00,0xb9,0xe5,0x04,0xb9,0xb9,0x39,0x03,0x03,0x00,0xb9,0x04,0x04,0x80,
  0xb9,0x8a,0xf4,0xd0,0x00,0xb9,0x03,0xb9,0x39,0x02,0x39,0xce,0xf3,0x02,0x39,0x39,
  0x00,0x03,0x80,0xdd,0x04,0xfd,0xb9,0xf3,0x39,0x03,0x39,0xe3,0xb9,0x04,0x39,0x00,
  0x00,0x03,0x03,0x03,0x00,0x80,0xb9,0x00,0xb9,0x04,0xb9,0x04,0x04,0xb9,0xd0,0xd0,
  0x04,0xf7,0x03,0x04,0x8a,0x29,0x80,0x00,0xdd,0x04,0xd0,0x8a,0x04,0x04,0x00,0xb9,
  0xb9,0x04,0x04,0x04,0xe3,0xdd,0x03,0x03,0x02,0x00,0x80,0x03,0x39,0x02,0xdd,0x02,
  0x00,0x39,0xd1,0x03,0x00,0x39,0x39,0xdd,0x03,0x03,0x03,0xb9,0xe3,0x04,0x02,0x02,
  0x03,0x01,0x04,0x04,0x03,0xb9,0x04,0x39,0x03,0x03,0x04,0xe3,0x03,0xdd,0x00,0x00,
  0x03,0xdd,0xb9,0x04,0x01,0xe3,0xf4,0xe3,0xe5,0xe3,0xe3,0x80,0x00,0x00,0x03,0x00,
  0xb9,0x03,0xb9,0x04,0xe5,0x01,0x03,0xf1,0xe3,0xe3,0x01,0x03,0xf3,0x39,0xd1,0x39,
  0x04,0x39,0x39,0xdd,0xf1,0xdd,0xdd,0xfe,0x03,0x03,0xe5,0x04,0xf4,0xf1,0x03,0xb9,
  0x00,0xed,0xf1,0xfc,0xf4,0x01,0x03,0x39,0x39,0x02,0x03,0x00,0xdd,0xdd,0x04,0x28,
  0x28,0x03,0x00,0xe3,0xe5,0x04,0xb9,0x00,0x00,0x03,0x03,0x00,0xb9,0x03,0x00,0x00,
  0x00,0x04,0x04,0x04,0xb9,0xe3,0xd0,0xf4,0xd0,0x00,0x04,0x04,0xfe,0xdd,0x28,0x03,
  0x00,0x03,0xf3,0xdd,0x03,0x00,0x04,0x03,0xdd,0x03,0x04,0xe3,0x04,0xfb,0x04,0x00,
  0x04,0xf1,0x04,0x00,0x03,0x03,0x00,0xdd,0x01,0x03,0x04,0xe3,0x03,0x00,0x03,0xb9,
  0x04,0x00,0x80,0xf4,0xe5,0x04,0x04,0xdd,0xdd,0x39,0x03,0x04,0xe3,0x04,0x04,0x04,
  0x00,0x00,0x02,0x03,0xb9,0x80,0x04,0x04,0x03,0xdd,0x04,0xb9,0x03,0xb9,0x03,0xdd,
  0x39,0x39,0xdd,0xdd,0x04,0xe3,0x01,0xe3,0x03,0xe3,0xfb,0xb9,0x03,0x01,0xb9,0xb9,
  0x03,0xd0,0x00,0xb9,0x04,0x00,0x00,0x00,0xf1,0x03,0xe5,0xe3,0x03,0xb9,0x03,0xe3,
  0x03,0x03,0x04,0xf4,0x00,0x04,0x80,0x03,0x03,0x00,0x03,0xe3,0xe3,0x00,0xb9,0xb9,
  0xb9,0x00,0xb9,0x00,0xb9,0x39,0xb9,0x00,0x39,0x03,0x80,0x39,0xf3,0xd1,0x00,0x00,
  0x03,0x03,0x00,0x28,0x39,0xb9,0x03,0x01,0x00,0xf3,0xd1,0xf1,0x03,0xe3,0xb9,0xb9,
  0xf4,0xd0,0x04,0xe3,0xe3,0x01,0x29,0xb9,0x00,0x03,0x04,0xe3,0x39,0x03,0x00,0x03,
  0x04,0x04,0xe3,0xb9,0xe3,0x00,0xb9,0x80,0x00,0x28,0x03,0xb9,0x80,0x04,0x04,0x80,
  0x00,0x00,0x04,0x80,0x00,0xd1,0xd1,0x00,0x01,0x00,0x03,0xf3,0x39,0x39,0x03,0xdd,
  0x28,0xdd,0xdd,0x03,0x80,0xb9,0x00,0x00,0x02,0x39,0x39,0xe3,0xe3,0x80,0x04,0x29,
  0xe5,0xb9,0xb9,0x00,0x03,0x00,0xb9,0x00,0x03,0x39,0x03,0xe3,0xdd,0x03,0x03,0x03,
  0x80,0xb9,0x8a,0xe5,0x04,0x8a,0x04,0x02,0x03,0x03,0x04,0xb9,0x04,0xd0,0xf1,0x80,
  0x02,0x80,0x03,0xd1,0xd1,0x02,0xd1,0x02,0x02,0x04,0x80,0x39,0x03,0x00,0xed,0x39,
  0x02,0x03,0x02,0x03,0xd0,0x04,0xf4,0x04,0xdd,0x00,0x04,0xf1,0x39,0xe3,0x04,0x29,
  0xed,0xb9,0x03,0x03,0xb9,0x01,0xb9,0x39,0xd1,0x00,0x80,0xb9,0xdd,0x04,0xf4,0xed,
  0x04,0x00,0xf1,0xe3,0xe5,0x29,0x00,0x28,0xd5,0xdd,0x02,0x00,0xe3,0x00,0x00,0x03,
  0x02,0x00,0x00,0x02,0x39,0xd1,0x28,0xdd,0x02,0xe3,0xfb,0x03,0x39,0x39,0x80,0xf3,
  0x03,0x00,0x02,0x03,0x29,0xf4,0xdd,0x39,0x03,0x01,0xdd,0x03,0x03,0x80,0xce,0x8a,
  0xf1,0xb9,0x04,0x04,0xb9,0x80,0x00,0x28,0xdd,0x03,0xb9,0xb9,0x03,0xe5,0xe3,0x00,
  0x00,0x04,0x04,0xb9,0x01,0x04,0x00,0x00,0x03,0x28,0xdd,0xb9,0xd0,0x80,0xdd,0xdd,
  0x80,0x04,0xd0,0xe3,0x00,0xe3,0xfb,0x04,0x80,0x03,0x03,0xd1,0x28,0x39,0xd5,0x28,
  0x39,0x28,0x02,0x29,0x80,0x02,0x28,0x03,0x39,0xf3,0xd1,0x80,0x80,0xb9,0x39,0x04,
  0x00,0xdd,0x00,0x00,0x03,0x00,0x04,0x03,0x39,0xb9,0x03,0x03,0x8a,0x04,0x00,0x03,
  0xb9,0x29,0x8a,0x29,0xe3,0x04,0xb9,0x29,0xe5,0x04,0x04,0x04,0x04,0x04,0x00,0x03,
  0x80,0xd0,0xf4,0xf4,0xd0,0x80,0xb9,0x00,0x03,0xce,0xdd,0xd1,0x03,0x00,0x00,0x28,
  0x28,0x28,0x03,0x01,0x00,0x04,0x01,0xb9,0x02,0x39,0xb9,0x01,0x00,0x03,0x00,0x02,
  0xd0,0xe3,0xdd,0x02,0x02,0x00,0xdd,0x39,0x00,0x80,0x39,0x03,0x02,0xd0,0x04,0x00,
  0x04,0x00,0x8a,0x29,0xb9,0x29,0xe3,0xb9,0x04,0x04,0xe5,0x04,0xe3,0xb9,0x03,0x00,
  0xe3,0xe3,0xb9,0xce,0x03,0x39,0x39,0x39,0x28,0x28,0xb9,0xe3,0x04,0x29,0x03,0xdd,
  0xd5,0x02,0x00,0x03,0x00,0xfb,0x04,0x02,0x03,0xe3,0x03,0xb9,0x39,0xb9,0x39,0xb9,
  0xb9,0x00,0x00,0xe3,0xd0,0xfb,0xb9,0x00,0x00,0x00,0xed,0xb9,0x00,0xd0,0x00,0x00,
  0xe3,0xe3,0x04,0xe3,0x04,0xd0,0x04,0xed,0xe5,0x00,0xf8,0x00,0xf4,0xe5,0xe7,0xd0,
  0x8a,0xb9,0x03,0x03,0x04,0x04,0xd1,0x28,0x29,0x80,0xb9,0xb9,0x03,0x80,0xe5,0x03,
  0x28,0xd1,0xf3,0xf3,0x39,0x03,0x02,0x00,0x03,0x03,0x03,0x03,0xce,0x04,0x00,0x00,
  0x04,0x00,0xe3,0xe3,0xf4,0xe3,0x02,0x02,0x03,0xb9,0xe5,0x04,0x8a,0x04,0xb9,0x00,
  0x29,0x29,0x04,0x01,0xd0,0xd0,0xf4,0xe5,0xe7,0x39,0xce,0xd5,0x04,0x29,0xe3,0x29,
  0xe3,0xe5,0xf4,0x8a,0x00,0x8a,0x8a,0x39,0x00,0xce,0x03,0xf1,0x04,0x04,0x03,0x28,
  0x28,0x39,0x03,0x39,0x39,0x01,0xd0,0x03,0x00,0x00,0x00,0x04,0x29,0x29,0x29,0x80,
  0x29,0x80,0xd0,0x04,0x04,0xb9,0x80,0x03,0xb9,0x00,0xd0,0x04,0xb9,0xf4,0xe3,0x03,
  0xb9,0xb9,0x29,0x8a,0xf4,0xf4,0x29,0x00,0x00,0x00,0x8a,0x8a,0xb9,0xe5,0x03,0x04,
  0x04,0xb9,0x04,0x03,0x03,0xed,0xed,0xe5,0x00,0xdd,0x04,0x71,0xfb,0x04,0x03,0x39,
  0x03,0x39,0xf3,0xd1,0x39,0x02,0x00,0x28,0xce,0x28,0x04,0x04,0x00,0xe3,0xe5,0xe3,
  0xe3,0xe3,0xfc,0x04,0x01,0xdd,0x03,0x00,0x03,0x04,0xd0,0x00,0xe3,0x29,0xdd,0x04,
  0xd0,0xd0,0x29,0xe3,0xd0,0x80,0x80,0xb9,0x28,0x00,0x28,0x02,0xb9,0x04,0x8a,0x8a,
  0x04,0xb9,0x00,0x29,0x04,0x8a,0x04,0x8a,0xe5,0xfb,0xe3,0x04,0x02,0x02,0x00,0xdd,
  0x39,0x39,0xf3,0x39,0x00,0x00,0xfc,0xd0,0x00,0x39,0x39,0x03,0x04,0xfc,0x04,0xd0,
  0xd0,0x04,0xf4,0x04,0xe3,0x80,0x28,0x00,0x39,0x29,0x04,0x03,0x00,0xb9,0xb9,0x29,
  0x29,0x00,0x01,0x04,0x01,0x02,0x00,0x8a,0x02,0x00,0xd1,0xd1,0x03,0x04,0x00,0x8a,
  0xe5,0x04,0x04,0x00,0x02,0xdd,0xdd,0xdd,0xb9,0xb9,0xd1,0x39,0xd5,0x00,0x03,0x03,
  0xb9,0x00,0x03,0x03,0x00,0xf4,0xe3,0xb9,0xdd,0x00,0xdd,0x01,0x04,0x02,0x00,0x80,
  0x01,0xe3,0x03,0x39,0x03,0x03,0xb9,0x29,0x00,0x03,0x03,0x39,0x00,0xe3,0x29,0x00,
  0x00,0x29,0xd0,0x80,0x02,0x00,0x00,0xb9,0x29,0x00,0x00,0xf5,0x29,0x8a,0x8a,0xe5,
  0xd0,0xf6,0x29,0x03,0x03,0xce,0x39,0x28,0xd1,0x03,0xd1,0x28,0x02,0x28,0xce,0xd5,
  0x28,0xce,0xdd,0x00,0x03,0x00,0x28,0x03,0xdd,0x8a,0xed,0x04,0x00,0xb9,0x04,0x04,
  0xf4,0x01,0x80,0x01,0x03,0x03,0x01,0xe5,0x04,0x03,0x29,0x28,0x39,0x03,0x00,0x29,
  0x01,0x04,0x00,0x03,0x80,0x29,0x02,0x29,0x00,0xb9,0xb9,0x00,0xb9,0x00,0x8a,0xe3,
  0x01,0x29,0x04,0xb9,0x80,0x28,0xce,0x03,0xd1,0x39,0x00,0x00,0x03,0xf2,0xd1,0xf9,
  0xce,0xf9,0x02,0x28,0x00,0x00,0x28,0x8a,0xb9,0xb9,0x00,0x00,0xd0,0x71,0xd0,0xd0,
  0x80,0x39,0xb9,0xf1,0x80,0xe3,0xb9,0xe3,0x39,0x00,0x00,0x28,0x28,0xb9,0x01,0xd0,
  0xb9,0x03,0x03,0xb9,0x29,0x00,0x00,0xb9,0xb9,0x29,0x28,0x28,0x00,0x02,0x00,0xf6,
  0x29,0x29,0x00,0x01,0xb9,0x03,0xf3,0x28,0x28,0x80,0xe7,0x03,0x28,0xf9,0xd1,0xd1,
  0xd1,0x28,0x39,0xd1,0x02,0x8a,0x03,0xdd,0x28,0x03,0x02,0xd1,0xdd,0x04,0xb9,0x80,
  0x00,0xb9,0x80,0x03,0x03,0x02,0x00,0x04,0x03,0x03,0xb9,0xd1,0x00,0x01,0x29,0x29,
  0x29,0xe3,0xb9,0x01,0x80,0x00,0x00,0x29,0x00,0x04,0xdd,0x03,0x00,0x02,0x80,0x29,
  0x00,0x80,0x00,0x04,0xb9,0xb9,0xb9,0x00,0x04,0xb9,0x29,0x02,0xd1,0xd1,0x02,0x28,
  0x28,0x28,0xd5,0xd1,0x28,0x39,0xd1,0xce,0xf9,0xf9,0xce,0xd1,0x28,0x39,0x02,0x02,
  0xed,0x80,0x29,0x01,0x04,0x04,0xe3,0x04,0x03,0xb9,0x39,0x02,0x00,0x03,0x02,0x80,
  0xfb,0x01,0x29,0x00,0x01,0x01,0xfd,0x29,0x03,0x04,0x00,0xb9,0x01,0xb9,0x00,0x00,
  0x80,0xe3,0x00,0x80,0x8a,0x04,0x00,0x00,0x02,0x00,0x00,0x03,0x00,0x00,0xd1,0xf2,
  0xd1,0x00,0x28,0x28,0xce,0xd1,0xce,0xd1,0xd1,0xce,0x28,0xf3,0x39,0xb9,0x39,0x28,
  0x29,0x00,0x01,0xd0,0xe3,0x80,0x00,0x03,0x00,0xd0,0x8a,0xe3,0x29,0xb9,0x04,0xd0,
  0xf4,0xd0,0x00,0x29,0xfd,0x00,0x00,0x00,0x00,0xe3,0x03,0x39,0x04,0x80,0xb9,0xb9,
  0x04,0xfc,0x02,0x04,0xe3,0x04,0x02,0x03,0xb9,0xe7,0xdd,0x03,0xce,0x03,0x03,0x28,
  0x02,0x8a,0x28,0xce,0xf2,0x39,0xd1,0xdd,0xdd,0x03,0x39,0x03,0x03,0xdd,0x03,0x28,
  0x00,0xb9,0x04,0x00,0x04,0x80,0xb9,0xf4,0x04,0x03,0x00,0xf4,0xe3,0xd0,0x04,0xe3,
  0xe5,0x00,0x29,0x39,0x02,0x00,0x02,0xf5,0x02,0xe3,0x00,0x39,0x02,0x00,0x80,0xb9,
  0x00,0x04,0x29,0xe3,0x04,0xb9,0x39,0xe7,0x03,0x04,0x03,0x28,0xce,0x28,0x02,0x00,
  0x00,0xd5,0xd1,0xd1,0xd1,0x00,0x01,0xb9,0x8a,0x03,0x28,0x02,0x00,0x02,0x29,0xd5,
  0x29,0x00,0x04,0x29,0x04,0x04,0xb9,0xb9,0x03,0xb9,0x28,0x00,0xb9,0x29,0x00,0x04,
  0x00,0xb9,0x29,0x03,0x03,0x80,0xd1,0x02,0xf5,0xd0,0x29,0x02,0x02,0x03,0x80,0x80,
  0x01,0x01,0x80,0xe3,0x04,0x02,0xdd,0x29,0xb9,0xb9,0xd1,0xf9,0xce,0x28,0xce,0x02,
  0x03,0xd1,0x28,0x02,0x00,0x29,0x03,0x00,0x02,0x28,0x28,0xce,0x03,0x00,0x00,0x03,
  0x8a,0x03,0xb9,0x29,0xe3,0xe5,0x8a,0x8a,0x04,0xd0,0xd5,0x01,0x04,0xf1,0x04,0x29,
  0x03,0x04,0x80,0x00,0x29,0x00,0x00,0x29,0xf4,0x29,0x39,0x02,0x02,0xb9,0xb9,0x01,
  0x04,0x04,0x01,0xd0,0xfb,0x80,0xe3,0xb9,0x03,0xd1,0xd1,0xd1,0x00,0x00,0x28,0xb9,
  0x03,0x02,0x28,0xf9,0x00,0xb9,0x39,0x02,0x39,0x28,0x39,0xd5,0x29,0x04,0x00,0x00,
  0x29,0x29,0x80,0x80,0xb9,0xb9,0xe3,0x03,0xe7,0xe3,0x29,0x04,0x80,0xf1,0xd0,0xe3,
  0xe3,0x04,0xb9,0xb9,0x04,0xd0,0x04,0xe5,0x8a,0x02,0x02,0xd5,0x00,0x80,0x00,0x04,
  0x01,0x04,0x80,0xb9,0x04,0xe3,0x04,0x80,0x03,0xce,0x00,0xd5,0x00,0xce,0xf9,0xce,
  0x28,0xd1,0x28,0x02,0x28,0xb9,0x03,0x28,0x39,0xce,0xf8,0x00,0xb9,0x04,0x03,0x03,
  0x29,0x00,0x04,0xe3,0x8a,0x03,0x00,0x00,0xf4,0xe5,0x28,0x39,0x28,0xdd,0xe5,0x8a,
  0x03,0x03,0x00,0x04,0xd0,0xe3,0x04,0x04,0x29,0x00,0x03,0x29,0x00,0x03,0x29,0x39,
  0x80,0x04,0xe5,0x04,0x00,0x8a,0x8a,0x04,0xf3,0xdd,0x28,0x28,0x00,0x28,0xd1,0x28,
  0x00,0x39,0x02,0xb9,0x00,0x00,0xb9,0x04,0x29,0x8a,0xb9,0x29,0xe5,0x00,0x03,0x03,
  0x03,0x29,0x8a,0x29,0x04,0x8a,0x03,0x04,0xe5,0xf1,0x03,0xb9,0x00,0xe7,0x00,0x39,
  0x39,0x00,0x29,0x04,0x00,0x03,0x03,0x03,0x00,0x00,0x00,0x02,0x29,0xe5,0x00,0x04,
  0x04,0xe3,0x04,0xf1,0x00,0x03,0x00,0x00,0x39,0xce,0x28,0xd5,0x00,0x28,0x02,0x00,
  0x03,0x02,0x03,0x00,0x02,0x00,0x80,0x39,0x00,0x28,0x02,0x29,0xb9,0x29,0x00,0xdd,
  0x03,0x00,0xb9,0x00,0x03,0x00,0x29,0xe5,0xf4,0xe5,0x03,0xb9,0x03,0xdd,0x04,0x8a,
  0x8a,0xfe,0x04,0x8a,0x8a,0xed,0xd5,0x28,0x00,0xb9,0x00,0x02,0x02,0x00,0x29,0x80,
  0x04,0x8a,0x29,0x01,0x00,0xb9,0x02,0x39,0x02,0xdd,0xb9,0x04,0xb9,0xd5,0x00,0x02,
  0x28,0x02,0x02,0x29,0x03,0xb9,0xb9,0x28,0x02,0xb9,0x00,0x02,0x00,0x28,0x8a,0x8a,
  0xd5,0xb9,0x29,0x00,0x03,0x29,0x8a,0x29,0x04,0x00,0x03,0x00,0x03,0x29,0x80,0x29,
  0x29,0x00,0x8a,0xdd,0x00,0x00,0xb9,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x28,0x03,
  0x04,0x29,0x04,0xe3,0x00,0x00,0x28,0x02,0x00,0x00,0xb9,0x04,0x28,0x28,0xdd,0x28,
  0x00,0x28,0x39,0x29,0xe3,0x03,0x03,0x00,0xdd,0xf5,0x00,0x02,0x29,0x02,0x00,0xed,
  0xe7,0x8a,0x8a,0x00,0x29,0x8a,0x04,0xe3,0x04,0xb9,0x00,0xed,0x00,0x03,0x00,0xb9,
  0x03,0x00,0xed,0x02,0xce,0xdd,0x03,0x02,0x00,0xd5,0x00,0x80,0x02,0x29,0x02,0x04,
  0x00,0x03,0xe3,0xd0,0x00,0xdd,0x28,0xdd,0x29,0xb9,0xe5,0xd1,0x02,0x28,0x03,0x02,
  0x03,0x02,0x02,0x02,0x28,0xd1,0x39,0x00,0x02,0x02,0x00,0xe3,0x04,0x00,0x00,0x04,
  0x29,0xe3,0xe3,0x8a,0x04,0xe5,0x29,0xb9,0xe5,0xed,0x8a,0x8a,0x29,0xdd,0x03,0x02,
  0x00,0x8a,0xb9,0x00,0x04,0x00,0xd5,0x00,0x03,0xd1,0x02,0x29,0x00,0x29,0x00,0x04,
  0xdd,0xb9,0x04,0x03,0x03,0xdd,0x02,0x00,0xb9,0x00,0x39,0x28,0x02,0xd1,0x03,0xd5,
  0xd5,0x02,0x02,0x39,0x00,0x02,0xb9,0x03,0x02,0xb9,0x29,0x03,0xb9,0x03,0x29,0xe3,
  0xb9,0x04,0x29,0xe5,0xe5,0xe3,0xdd,0x8a,0x01,0xd0,0xe5,0xe3,0x8a,0xb9,0x00,0x03,
  0x8a,0x04,0xe3,0x29,0x03,0xd5,0x02,0x00,0xb9,0x02,0x28,0xd5,0x29,0x80,0x00,0x00,
  0x28,0x00,0x00,0x00,0x8a,0x03,0xd5,0x02,0x28,0x03,0xd1,0x02,0x28,0xf8,0x03,0x00,
  0xd1,0x28,0x03,0xe3,0xd0,0xe5,0xb9,0x29,0x00,0x00,0xb9,0x00,0x8a,0xb9,0x29,0x8a,
  0xb9,0x29,0xb9,0xe7,0x8a,0xdd,0x00,0x00,0xe3,0xe3,0x28,0x39,0x39,0x03,0x39,0x03,
  0xb9,0x8a,0x03,0x03,0x03,0xd5,0x28,0x02,0x03,0x28,0x28,0x28,0x02,0x03,0x04,0x04,
  0x00,0x04,0x03,0x00,0x04,0xed,0x03,0x00,0x00,0x02,0xd5,0x02,0xd5,0x02,0x02,0x00,
  0x39,0x03,0xe3,0xb9,0xe5,0x8a,0x8a,0xe3,0x00,0xd5,0x00,0xb9,0x8a,0x00,0x29,0x8a,
  0x03,0x29,0xd0,0x04,0xb9,0xb9,0x03,0x28,0x29,0xe3,0x00,0x29,0x8a,0x00,0xdd,0x8a,
  0x8a,0xe7,0x00,0x00,0x39,0x28,0xce,0x28,0xd5,0x28,0xd5,0x02,0x03,0x02,0x00,0x00,
  0xfc,0x04,0x80,0x8a,0xe3,0x04,0xf1,0xdd,0x8a,0xb9,0xd5,0xd5,0x00,0x03,0xce,0x02,
  0x04,0xe5,0x04,0x80,0x03,0xb9,0xe3,0xe3,0x00,0xb9,0x8a,0x00,0x03,0xb9,0x03,0x00,
  0xb9,0x8a,0x8a,0x28,0x28,0xf3,0xe7,0xe5,0x04,0x00,0x00,0xdd,0x00,0xdd,0x03,0x8a,
  0x00,0x03,0x00,0x28,0x28,0x02,0x28,0xf5,0xd1,0xd5,0xd5,0x29,0x00,0xb9,0xb9,0x28,
  0x04,0xf6,0x00,0x29,0xe5,0x04,0xb9,0x00,0x28,0xb9,0x03,0x02,0x03,0x39,0xdd,0x29,
  0x28,0xe3,0xe5,0x29,0xf6,0xfc,0x04,0xe5,0xb9,0x00,0x00,0x03,0x8a,0xdd,0x28,0xe5,
  0x8a,0xe5,0x03,0xb9,0x8a,0x00,0x29,0x00,0x8a,0xe5,0xe5,0x01,0xb9,0xb9,0x8a,0xb9,
  0x00,0x00,0x00,0x28,0x28,0x00,0x03,0x02,0x02,0xd5,0xd5,0xd5,0x29,0x29,0x29,0x00,
  0xe5,0xd0,0xe3,0xe5,0x04,0xb9,0xb9,0x00,0x8a,0xe7,0x00,0xf5,0x39,0xb9,0x02,0x03,
  0x00,0xe5,0xe5,0xfb,0x04,0x00,0xe7,0x04,0x39,0xd5,0x28,0xb9,0xdd,0x03,0xd5,0xe5,
  0xd5,0x8a,0xdd,0xe3,0xd0,0xb9,0x00,0x39,0x8a,0x29,0x03,0x03,0xed,0xe3,0x00,0x80,
  0xb9,0x28,0x00,0x02,0x03,0x00,0x02,0x02,0x29,0x28,0x28,0x28,0xd5,0x29,0x04,0xb9,
  0x8a,0x01,0x01,0xe3,0x29,0x04,0xed,0x04,0x03,0x29,0x8a,0x8a,0x03,0xd1,0x00,0x03,
  0x03,0x04,0xe5,0x80,0x02,0x39,0xb9,0x00,0x03,0x39,0xdd,0xd5,0xb9,0x00,0xd1,0xb9,
  0x00,0x00,0xb9,0xce,0xd5,0x03,0xdd,0x00,0xdd,0x00,0x00,0xb9,0x04,0x00,0x00,0xb9,
  0x00,0x04,0xe7,0xb9,0xb9,0x29,0x00,0xd5,0x00,0x00,0x29,0xd5,0x00,0x00,0x04,0x03,
  0x04,0x8a,0x8a,0xe5,0x00,0x04,0x00,0x00,0xe5,0x00,0x8a,0xd0,0x8a,0x8a,0x29,0x04,
  0x39,0x02,0x04,0x00,0x80,0x04,0x04,0xd5,0x03,0xb9,0xce,0x28,0x00,0xd1,0x28,0x29,
  0xe3,0x00,0xdd,0xb9,0x00,0x00,0x39,0x39,0xdd,0x01,0x00,0x29,0xb9,0x03,0xdd,0x03,
  0x29,0xe3,0x00,0x00,0xd5,0x00,0x00,0xd5,0x03,0x28,0xd1,0x39,0xb9,0x00,0xe3,0x04,
  0xe5,0xb9,0x04,0x04,0x80,0x04,0x00,0x04,0x29,0x8a,0xe3,0xe3,0x04,0x04,0x00,0x28,
  0x28,0x02,0x03,0x04,0xb9,0x03,0x02,0x00,0xd5,0xb9,0x03,0x28,0x03,0xd1,0x28,0x04,
  0xe3,0x04,0x00,0x00,0xdd,0xdd,0x28,0x00,0x00,0x28,0x39,0xb9,0x02,0x02,0x00,0x39,
  0x80,0x80,0x02,0x8a,0x29,0x29,0x29,0xd1,0x28,0xce,0xce,0xce,0xce,0x02,0x00,0xe7,
  0xb9,0x04,0xe3,0x00,0x00,0xb9,0x00,0x00,0x04,0x8a,0x00,0x29,0x29,0x29,0x28,0x28,
  0x03,0x03,0x00,0x04,0x02,0x02,0x03,0x02,0x28,0xd5,0xdd,0xb9,0x03,0x28,0x00,0x01,
  0x00,0x03,0x03,0xe3,0xd0,0xe5,0xb9,0x29,0x80,0x00,0x02,0x29,0x00,0xe3,0x29,0x03,
  0x00,0xdd,0x00,0x29,0xdd,0xd5,0x39,0xce,0x39,0x02,0x28,0xd1,0x28,0x02,0x03,0x03,
  0x00,0x04,0x00,0x29,0x01,0x00,0xdd,0xb9,0xb9,0x29,0x00,0x29,0x00,0x03,0x02,0x03,
  0xd1,0x00,0x29,0x00,0x02,0x28,0x00,0x29,0x00,0x8a,0x00,0xb9,0x8a,0xb9,0xd5,0x29,
  0x8a,0x29,0x03,0x8a,0xe5,0x04,0xed,0x00,0x03,0xb9,0x04,0x29,0x04,0x29,0x02,0x02,
  0x00,0x8a,0x03,0x02,0xd5,0x03,0x00,0x39,0x28,0x28,0x28,0x03,0x02,0x28,0x03,0xdd,
  0x03,0xe3,0xb9,0xb9,0x03,0x00,0x00,0x03,0x00,0x03,0x00,0x80,0x29,0x28,0x03,0x02,
  0x28,0x03,0x03,0xd1,0x00,0xe3,0x04,0xf5,0x03,0x00,0x28,0x00,0x02,0xdd,0x00,0x04,
  0x00,0xe5,0x29,0xb9,0x29,0x00,0x04,0x04,0x80,0x80,0x80,0xe3,0x29,0x02,0xd1,0x28,
  0x00,0x28,0x80,0x00,0x03,0x28,0x03,0xce,0x28,0x02,0x28,0x00,0xdd,0xb9,0x28,0x39,
  0x00,0xb9,0x03,0xed,0x04,0xb9,0x00,0xb9,0x80,0x03,0x00,0x00,0x00,0xf5,0x03,0x00,
  0x28,0x02,0x00,0xd5,0xe5,0x04,0xe3,0x02,0xd5,0xd1,0x28,0x28,0x39,0xd5,0x02,0x80,
  0xdd,0xdd,0x8a,0xe5,0x8a,0x03,0xb9,0x00,0xe3,0xe3,0x00,0x29,0xd1,0x28,0xf5,0x00,
  0x00,0x00,0xdd,0x28,0xdd,0x02,0x00,0x28,0xd5,0x04,0xb9,0x28,0xce,0x28,0x03,0x00,
  0x8a,0x28,0xf8,0x03,0x04,0xb9,0xe3,0x00,0x00,0xce,0x39,0x02,0x02,0x00,0xce,0x28,
  0x29,0xe3,0xb9,0x00,0x00,0x00,0x00,0xb9,0xd5,0x02,0xdd,0x02,0x39,0x28,0x28,0x03,
  0xd1,0xb9,0x29,0x29,0xb9,0x8a,0x00,0x00,0xf4,0xe5,0x00,0xb9,0x00,0x28,0x39,0x00,
  0xb9,0x03,0x29,0xb9,0xb9,0x00,0x29,0x03,0x28,0x03,0x28,0xd1,0xce,0xb9,0x8a,0xb9,
  0x03,0x03,0xdd,0x04,0xb9,0x8a,0x04,0x00,0x28,0x00,0x39,0x28,0xce,0x28,0x00,0xb9,
  0x03,0xb9,0x01,0x03,0x00,0x29,0x80,0x02,0xdd,0x29,0x28,0x28,0x00,0x28,0x02,0x80,
  0x02,0x04,0x00,0xce,0x02,0xb9,0x8a,0xe5,0x00,0x39,0xd1,0x29,0x00,0x28,0x28,0x04,
  0xf5,0x02,0x00,0x03,0x03,0x00,0x01,0xdd,0x03,0x29,0x00,0x80,0x00,0x01,0x04,0x39,
  0x29,0x04,0xb9,0x04,0x03,0x8a,0xe5,0xe5,0xf1,0x39,0x03,0x00,0x00,0x29,0x29,0x02,
  0x02,0xd5,0x39,0x02,0x80,0x02,0x02,0x00,0x39,0xd1,0xd1,0x28,0x03,0x03,0xdd,0x39,
  0x03,0x29,0x29,0x03,0x39,0xdd,0x29,0xb9,0xd1,0xd1,0xd1,0x28,0x03,0xd1,0x8a,0x04,
  0x29,0x39,0x00,0xb9,0x29,0x00,0x00,0x29,0x00,0x03,0x39,0x04,0x04,0x00,0x03,0x03,
  0x29,0x03,0x03,0x03,0x80,0xe5,0xb9,0xe3,0x00,0xdd,0x39,0x00,0x29,0xe7,0x02,0x02,
  0x02,0xd1,0x02,0x39,0x39,0x39,0x04,0x00,0x28,0xd1,0xd1,0x00,0x03,0xdd,0xdd,0x03,
  0xdd,0x03,0x00,0xb9,0x8a,0xe3,0xb9,0xdd,0x39,0x39,0x02,0x03,0x29,0x8a,0x00,0x02,
  0xd1,0x02,0x02,0x00,0x04,0x29,0xf6,0x29,0x00,0x04,0x04,0xb9,0xed,0x04,0x29,0x04,
  0xb9,0x03,0x00,0x00,0xb9,0xe3,0x02,0xdd,0xb9,0x00,0xce,0x00,0xe7,0xe7,0x8a,0x02,
  0x02,0x39,0x03,0x00,0x39,0x29,0x29,0xdd,0x28,0xce,0xd1,0x28,0x28,0xd1,0xdd,0xdd,
  0x00,0x03,0x28,0xdd,0x29,0x04,0xdd,0x03,0x8a,0x28,0x28,0x02,0x80,0x00,0xd1,0x03,
  0xe3,0xe5,0x29,0x29,0xb9,0xb9,0x29,0x04,0x29,0x29,0x03,0x00,0x00,0x04,0xb9,0x03,
  0x80,0x00,0x00,0x00,0x03,0x04,0x80,0xdd,0x00,0x39,0x00,0x39,0xe7,0x03,0x00,0x02,
  0x29,0x39,0x39,0x03,0x03,0xfa,0x80,0x28,0x28,0x00,0x39,0xce,0xb9,0x29,0xb9,0x28,
  0x03,0x00,0xdd,0x02,0x00,0xed,0xb9,0x04,0x04,0x39,0x39,0x29,0x39,0x00,0x04,0x00,
  0xe7,0xed,0x8a,0x29,0x00,0x00,0x00,0x29,0x00,0x02,0x00,0x80,0x00,0x04,0x04,0x00,
  0x00,0x04,0x04,0x04,0xfc,0xfb,0xe3,0x00,0x28,0x00,0xf7,0x8a,0x04,0x28,0x03,0xf5,
  0x02,0x39,0x00,0x80,0xe3,0x02,0x39,0xd1,0xd1,0x03,0x03,0x39,0x03,0x28,0x39,0x39,
  0x03,0x29,0x80,0xb9,0xe3,0x00,0xdd,0xb9,0x00,0x39,0xce,0xdd,0xd0,0x8a,0xe5,0xe5,
  0xe5,0x03,0x29,0x04,0x8a,0xb9,0x29,0x00,0x29,0x00,0x00,0xf5,0x00,0x80,0x01,0x04,
  0x00,0x29,0x29,0xe3,0x04,0xb9,0x04,0x03,0xdd,0x03,0x00,0xb9,0x29,0x29,0xb9,0x03,
  0x03,0x00,0x00,0xf6,0xb9,0x00,0x39,0x03,0xce,0xd1,0x00,0x04,0xdd,0xd1,0x39,0xdd,
  0x28,0x03,0x02,0xe5,0x39,0x00,0x00,0x29,0x28,0xce,0xb9,0xf4,0x04,0x04,0x8a,0xb9,
  0xdd,0xce,0x8a,0xe3,0xe3,0x29,0xd5,0x00,0x03,0x00,0x29,0xe7,0xe3,0x04,0x29,0xb9,
  0xf6,0x02,0x00,0x00,0x39,0xb9,0x00,0x04,0xe3,0x29,0x03,0xed,0xb9,0x04,0xe3,0xdd,
  0x39,0xce,0xb9,0x00,0x03,0xb9,0xb9,0x03,0x28,0xd5,0xe3,0x29,0x8a,0xd5,0x03,0x01,
  0x03,0x02,0x8a,0x39,0x03,0x00,0x00,0x03,0x28,0x28,0xb9,0x04,0xe5,0x00,0x39,0x03,
  0xdd,0x02,0x04,0x00,0x03,0x03,0x00,0x8a,0xdd,0x03,0x00,0x80,0x80,0xd0,0xe5,0x28,
  0x02,0x02,0x29,0x29,0x02,0x00,0x80,0x80,0x03,0xdd,0x02,0x04,0x80,0x00,0x04,0x02,
  0x00,0x28,0x00,0x00,0x80,0x03,0x00,0x00,0x8a,0x29,0x80,0x02,0x04,0x29,0x29,0xdd,
  0x29,0xe3,0x03,0xdd,0x03,0xdd,0x00,0x04,0xb9,0x00,0xdd,0x39,0x03,0xdd,0x28,0x39,
  0x00,0x03,0xdd,0x00,0x03,0xb9,0xb9,0xdd,0x00,0x8a,0x29,0xf6,0x00,0xb9,0xd0,0xd0,
  0x28,0x28,0x28,0xdd,0x00,0x04,0xe3,0xe3,0x02,0xed,0xb9,0xdd,0x00,0x39,0x02,0x00,
  0x00,0x03,0x03,0xb9,0x04,0xdd,0x80,0x03,0x01,0x39,0xb9,0xdd,0xe3,0x28,0xe3,0xdd,
  0xdd,0x03,0xd0,0xb9,0xb9,0xce,0x28,0x02,0x02,0xd1,0xf2,0x00,0x00,0x39,0x02,0x00,
  0x03,0x29,0x00,0x02,0xe5,0x03,0x29,0x03,0x00,0x02,0x00,0x01,0xb9,0x80,0x00,0x00,
  0x04,0x80,0x29,0x00,0xb9,0x80,0x01,0x80,0xb9,0xe3,0xdd,0xdd,0x03,0x80,0x00,0x02,
  // 5: ground ice
  0x1d,0x7e,0xfd,0x7d,0x5d,0x86,0xfc,0x85,0x1d,0x86,0xbc,0x7d,0x00,0x00,0x08,0x00,
  0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,
  0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,
  0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,
  0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0xbb,0x75,0x10,0x07,0x9b,0x6d,0x18,0x07,
  0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,
  0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,
  0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,
  0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,
  0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,
  0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,
  0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,
  0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,
  0x00,0x60,0x08,0x60,0x10,0x60,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,
  0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,
  0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,
  0x00,0x66,0x08,0x66,0x10,0x66,0x9e,0x8e,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,
  0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,
  0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,
  0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,
  0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,
  0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,
  0x18,0xa1,0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,
  0x18,0xa3,0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,
  0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,
  0x18,0xa7,0x7e,0x86,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x00,0xc1,0x08,0xc1,
  0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,
  0x10,0xc3,0xdc,0x6d,0x18,0xc3,0x00,0xc4,0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,
  0x08,0xc5,0x10,0xc5,0x9d,0x8e,0x18,0xc5,0x00,0xc6,0x08,0xc6,0x10,0xc6,0x18,0xc6,
  0x00,0xc7,0x08,0xc7,0x7a,0x6d,0x10,0xc7,0x5d,0x7e,0x18,0xc7,0x00,0xe0,0x08,0xe0,
  0xbc,0x6d,0xde,0x96,0x10,0xe0,0x18,0xe0,0xbe,0x96,0x00,0xe1,0x08,0xe1,0x18,0xe1,
  0xde,0x8e,0xdb,0x75,0x7e,0x7e,0x5d,0x96,0x10,0xe5,0x18,0xe5,0x18,0xe6,0x10,0xe7,
  0xd9,0x05,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x01,0x05,0x03,0x04,0x03,0x05,0x05,
  0x05,0x05,0x05,0x04,0x04,0x00,0xd9,0x01,0x01,0x00,0x00,0xd9,0x02,0x00,0xd9,0xd9,
  0xd9,0x01,0x01,0x01,0x01,0x02,0x00,0x02,0x02,0x02,0xc9,0x02,0x02,0x02,0x02,0xc9,
  0xc9,0xc9,0x00,0x02,0x01,0x05,0x01,0xd9,0x00,0x04,0x00,0x04,0x04,0x01,0x01,0x05,
  0x05,0x01,0x04,0x04,0x04,0x01,0x05,0x05,0x05,0x05,0x01,0x01,0x04,0x01,0x05,0x05,
  0xd9,0x05,0x01,0x01,0x00,0x00,0x01,0xd9,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,
  0x00,0x04,0x01,0x00,0x04,0x02,0x02,0x02,0x02,0xe2,0x02,0x02,0x02,0x02,0xc9,0xc9,
  0x02,0xc9,0xec,0x00,0x00,0x02,0x02,0x02,0x02,0x04,0x02,0x02,0x04,0x01,0x01,0x05,
  0x01,0x01,0x02,0x04,0x04,0x04,0x05,0x05,0x04,0x04,0x05,0x04,0x04,0x01,0x04,0x05,
  0x05,0x05,0x05,0x02,0x00,0x01,0x02,0x00,0x02,0x00,0xec,0xd9,0x00,0xec,0x02,0x00,
  0x00,0x00,0x00,0x02,0x02,0x02,0xec,0x02,0xe2,0x02,0x83,0x02,0x02,0x02,0x02,0x01,
  0x00,0x00,0x02,0x02,0x02,0x83,0x83,0x02,0x02,0x02,0x83,0x83,0x04,0x01,0xd9,0x01,
  0x01,0x01,0x00,0x04,0x01,0x03,0x03,0x05,0x05,0x03,0x01,0x04,0x04,0x04,0x04,0x05,
  0x01,0x01,0x03,0x01,0x01,0x00,0x00,0x00,0x02,0x00,0x02,0x00,0x02,0xec,0xec,0x00,
  0x02,0x02,0x00,0x02,0x00,0x02,0xc9,0x83,0x83,0xc9,0x02,0x04,0x04,0x02,0x83,0x00,
  0x00,0x83,0x01,0x02,0x00,0x02,0xf8,0x02,0x83,0xf1,0xf1,0x83,0x02,0x04,0x01,0x04,
  0x01,0x01,0x01,0x01,0x01,0x04,0x01,0x05,0x01,0x05,0x05,0x01,0x03,0x03,0x05,0x03,
  0x04,0x04,0x04,0x01,0x04,0x04,0x00,0x02,0x00,0xec,0xec,0xec,0xec,0xec,0x02,0x02,
  0x02,0x02,0xc9,0x02,0x02,0x02,0x83,0x83,0x83,0x83,0x02,0x02,0xe2,0x02,0x83,0x02,
  0x02,0xe2,0x00,0x05,0x02,0x83,0x83,0x83,0x83,0xf1,0x83,0x83,0x02,0x02,0x00,0x01,
  0xd9,0x05,0x01,0x01,0x05,0x04,0x04,0x05,0x03,0x04,0x03,0x00,0x04,0x04,0x01,0x00,
  0x02,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc9,0x02,0x02,0x02,0xc9,
  0xc9,0xc9,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x83,0x83,
  0x00,0x02,0xf4,0x00,0x02,0x02,0x02,0xc9,0xf8,0xf4,0xe2,0x83,0x83,0x02,0x00,0x01,
  0x01,0xd9,0x01,0x05,0x03,0x05,0x05,0x05,0x05,0x04,0x05,0x05,0x01,0x04,0x04,0x03,
  0x02,0x02,0x02,0x01,0x00,0x02,0x02,0x02,0x02,0xec,0xc9,0x83,0x83,0x83,0x83,0x83,
  0xc9,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc9,0x00,0x02,0x02,0x83,0x83,0x02,0x02,
  0x00,0x02,0x02,0x83,0xf4,0x00,0x02,0x83,0xf1,0xf1,0xf1,0x83,0x83,0x02,0x00,0x01,
  0x00,0x05,0x04,0x01,0x05,0x05,0x03,0x05,0x05,0x05,0x05,0x04,0x04,0x01,0x04,0x04,
  0x04,0x04,0x04,0x00,0x02,0x00,0x02,0x02,0xc9,0xc9,0xf8,0xf8,0x83,0xf8,0xc9,0xc9,
  0x83,0xc9,0x02,0x02,0x02,0x02,0x00,0x02,0xe2,0x02,0x02,0xe2,0x83,0x83,0x83,0x83,
  0x02,0x83,0x02,0x00,0x02,0x83,0x02,0x02,0xf1,0xf1,0xf1,0xc9,0x02,0x02,0x02,0x01,
  0x00,0x05,0x01,0x01,0x05,0x01,0x05,0x05,0x05,0x05,0x03,0x03,0x03,0x04,0x03,0x04,
  0x04,0x04,0x04,0x02,0x02,0xec,0xec,0x83,0xf8,0xf1,0xf8,0x83,0xf1,0xf8,0xc9,0xc9,
  0x83,0x83,0x02,0x02,0x02,0x00,0x00,0x00,0x83,0xf1,0x02,0x02,0x02,0x02,0x02,0xe2,
  0x02,0x02,0x02,0x00,0x01,0x83,0x83,0x04,0xc9,0xf1,0xf1,0xc9,0xc9,0x02,0x02,0x00,
  0xd9,0x05,0x04,0x04,0x04,0x04,0x01,0x05,0x03,0x04,0x03,0x03,0x03,0x04,0x04,0x04,
  0x04,0x04,0x03,0x01,0x00,0x00,0x02,0x83,0x83,0x83,0xf1,0x02,0x83,0xc9,0xc9,0xc9,
  0xf8,0xf8,0x00,0x02,0x01,0x01,0x04,0x04,0x02,0x83,0xf4,0xe2,0xe2,0x02,0x04,0x04,
  0x00,0x04,0x01,0x04,0x02,0x00,0x83,0x02,0x83,0xf1,0xf1,0xc9,0x02,0x02,0x02,0x04,
  0x01,0x04,0x02,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0x03,0x03,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x00,0xec,0x01,0x00,0x83,0xc9,0xc9,0x83,0x02,0x02,0xc9,0xc9,0xc9,
  0x83,0x83,0x02,0x00,0x01,0x01,0x04,0x01,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x04,
  0x04,0x04,0x03,0x04,0x02,0x02,0x04,0xe2,0x04,0x83,0xc9,0x02,0x02,0x83,0x02,0x04,
  0x04,0x00,0x02,0x01,0x04,0x01,0x01,0x04,0x01,0x03,0x04,0x05,0x04,0x04,0x04,0x04,
  0x02,0x00,0xc9,0x00,0x00,0x00,0x00,0x01,0x02,0x83,0x02,0x02,0xc9,0x02,0xc9,0x83,
  0x83,0x02,0x01,0x01,0x01,0x01,0xd9,0x04,0x04,0x03,0x04,0x04,0x01,0x04,0x03,0x03,
  0x01,0x01,0x04,0x01,0x02,0x02,0x02,0x83,0x83,0x04,0x83,0x02,0x83,0x83,0x83,0x04,
  0x04,0x00,0x00,0x05,0x04,0x00,0x00,0x04,0x04,0x04,0x04,0x01,0x04,0x04,0x04,0x00,
  0x00,0x00,0x01,0x01,0xd9,0xd9,0x00,0x00,0xd9,0x00,0xec,0x02,0x02,0xec,0x83,0xc9,
  0xc9,0xc9,0x01,0x01,0x01,0x01,0x05,0x01,0x03,0x04,0x05,0x04,0x04,0x05,0x05,0x01,
  0x04,0x05,0x04,0x04,0x04,0x02,0x02,0x83,0x83,0x02,0xe2,0x02,0x83,0x02,0x02,0x00,
  0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x02,0x04,0x02,0x02,0x02,0x00,0x00,
  0x01,0x01,0x01,0x01,0xf0,0xd9,0xd9,0xd9,0xd9,0x00,0x00,0x00,0x02,0x02,0xc9,0x02,
  0x02,0x00,0x01,0x00,0x01,0x01,0x01,0x05,0x05,0x03,0x01,0x04,0x04,0x03,0x04,0x05,
  0x03,0x03,0x00,0x04,0x04,0x04,0xe2,0x02,0xc9,0x02,0x02,0x83,0xc9,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0x03,0x01,0x04,
  0x05,0x05,0x05,0x01,0xd9,0x24,0xd9,0x05,0x01,0x01,0x01,0x00,0x02,0x02,0xc9,0x02,
  0x02,0x00,0x01,0x00,0xd9,0x01,0x01,0x05,0x05,0x01,0x05,0x04,0x01,0x04,0x01,0x01,
  0x04,0x04,0x04,0x04,0x04,0x04,0x02,0xe2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,
  0x02,0x83,0x83,0x83,0xe2,0xe2,0x02,0x02,0x00,0x00,0x04,0x04,0x04,0x03,0x05,0x05,
  0x05,0x05,0x05,0xd9,0x05,0xd9,0x05,0xd9,0xd9,0x00,0x00,0x00,0x02,0x02,0x02,0x00,
  0x02,0x04,0x00,0x01,0xd9,0x01,0x01,0xd9,0x05,0x01,0x03,0x03,0x05,0x04,0x01,0x05,
  0x05,0x04,0x04,0x04,0x02,0x04,0x02,0xc9,0x02,0xec,0x00,0x00,0x00,0x00,0x00,0x00,
  0x83,0xe2,0x83,0x83,0xe2,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x03,0x05,
  0x05,0xd9,0xd9,0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0x00,0x00,0x02,0x02,0xc9,0x02,
  0x02,0x00,0x00,0x01,0x00,0x04,0x05,0x01,0x03,0x04,0x04,0x05,0x05,0x04,0x05,0x04,
  0x04,0x02,0x04,0x04,0x04,0x04,0x04,0xe2,0x02,0x02,0x01,0x01,0x01,0x02,0x00,0x00,
  0x02,0x02,0xe2,0x83,0xe2,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x05,0x05,
  0xd9,0x01,0x01,0x05,0x05,0x05,0xf0,0x05,0x05,0xd9,0x00,0x00,0xfa,0x02,0x02,0x02,
  0xd9,0x00,0xd9,0xd9,0x01,0x01,0x01,0x03,0x03,0x04,0x01,0x04,0x05,0x05,0x01,0x05,
  0x05,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x00,0x00,0x05,0x01,0x00,0x02,0x00,0x02,
  0x04,0x04,0x02,0x83,0x83,0x83,0x02,0xe2,0x02,0x04,0x04,0x05,0x05,0x05,0x05,0x05,
  0xd9,0xd9,0x05,0x05,0x05,0xf0,0x05,0x05,0x05,0x00,0x02,0x01,0xec,0x83,0x02,0x00,
  0x00,0x01,0xd9,0x00,0x00,0x04,0x01,0x01,0x04,0x03,0x01,0x05,0x05,0x05,0x01,0x05,
  0x05,0x01,0x01,0x01,0x04,0x04,0x04,0x02,0x00,0x02,0xd9,0x04,0x04,0x01,0x01,0x00,
  0x04,0x01,0x04,0x02,0x02,0x02,0x04,0x02,0x02,0x04,0x03,0x05,0x05,0x05,0xd9,0xf0,
  0xd9,0x05,0x05,0x05,0x26,0x26,0x01,0x05,0x05,0x00,0x00,0x00,0x00,0x83,0x02,0x00,
  0x00,0xd9,0xd9,0xd9,0x01,0x01,0x05,0xd9,0x05,0x05,0x05,0x05,0x05,0x03,0x01,0x05,
  0x05,0x05,0x03,0x01,0x01,0x01,0x02,0x00,0x00,0x00,0x01,0x04,0x04,0x04,0x01,0x01,
  0xd9,0x01,0x01,0x01,0x02,0x02,0x04,0x02,0x02,0x04,0x04,0x05,0x05,0x05,0x24,0xd9,
  0x05,0x24,0xd9,0x05,0x05,0x05,0x05,0x01,0x01,0x01,0xd9,0x00,0x02,0x02,0x02,0x02,
  0x00,0xd9,0xd9,0xd9,0xd9,0xd9,0x01,0x05,0x05,0x05,0x04,0x05,0x05,0x05,0x01,0x03,
  0x05,0x03,0x05,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x04,0x04,0x01,0x04,0x04,0x01,
  0x03,0x05,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x04,0x01,0x05,0x00,0x05,0x05,0x24,
  0x05,0x05,0xf0,0x05,0x05,0x04,0x05,0x01,0x01,0x00,0x01,0x00,0xc9,0x02,0xe2,0xc9,
  0x00,0xd9,0x01,0x05,0x05,0x05,0x01,0x01,0x01,0x05,0x01,0x05,0x03,0x05,0x05,0x05,
  0x05,0x05,0x04,0x05,0x00,0x00,0x00,0xec,0x00,0x04,0x04,0x04,0x04,0x04,0x04,0x05,
  0x05,0x05,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x04,0x04,0x01,0x01,0xd9,0xd9,0x05,
  0x05,0x24,0x26,0x05,0x01,0x01,0x05,0x05,0x01,0x01,0xd9,0x01,0x00,0x02,0x02,0x00,
  0x04,0x01,0xd9,0xd9,0x01,0x03,0x01,0x01,0x05,0x05,0x01,0x05,0x04,0x01,0x05,0x04,
  0x05,0x05,0x04,0x01,0x04,0x02,0xec,0x02,0x04,0x01,0x01,0x01,0x04,0x05,0x01,0x05,
  0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x01,0x00,0x83,0x83,0x02,0x00,0x00,0x00,0xd9,
  0x24,0x24,0x24,0x05,0x05,0x05,0x01,0x05,0x00,0x01,0x01,0x00,0x02,0x00,0x02,0x00,
  0x04,0x01,0x01,0x01,0x04,0x01,0x05,0x01,0x03,0x01,0x01,0x05,0x04,0x04,0x05,0x01,
  0x05,0x04,0x04,0x04,0x02,0x00,0x02,0x02,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x03,
  0x03,0x05,0x03,0x01,0x04,0x04,0x01,0x00,0x01,0x02,0x02,0x03,0x01,0x05,0x05,0x00,
  0xd9,0x05,0x24,0x24,0x05,0x01,0x05,0x05,0x05,0xd9,0x00,0x01,0x02,0xec,0x02,0x00,
  0x01,0xd9,0x05,0x03,0x05,0x05,0x01,0x01,0x04,0x04,0x04,0x02,0x02,0x04,0x04,0x04,
  0x05,0x04,0x04,0x04,0x02,0xc9,0x02,0x02,0x02,0x04,0x04,0x05,0x05,0x05,0x05,0x05,
  0x05,0x01,0x04,0x04,0x01,0x04,0x01,0x00,0x00,0x05,0x01,0x00,0x01,0x01,0x05,0x01,
  0xec,0x00,0x05,0x24,0x03,0x01,0x05,0x26,0xd9,0xd9,0x01,0x00,0x00,0x02,0x04,0x03,
  0x05,0x05,0xd9,0x05,0x05,0x05,0x04,0x04,0x83,0x04,0x04,0x04,0x04,0x04,0x02,0x02,
  0x02,0x04,0x04,0x04,0x04,0xc9,0x02,0x02,0x04,0x04,0x01,0x05,0x01,0x05,0xd9,0x05,
  0x03,0x05,0x05,0x04,0x05,0x01,0x01,0x00,0x01,0x00,0x05,0x00,0x01,0xd9,0x05,0x05,
  0xd9,0x01,0x01,0x05,0x05,0x05,0x05,0x05,0xd9,0x01,0xd9,0x00,0xd9,0x00,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x01,0x03,0x04,0x04,0x01,0x04,0x04,0x02,0x04,0x04,
  0x04,0x04,0x02,0xc9,0x02,0xec,0x02,0x02,0x01,0x04,0x04,0x04,0x01,0x04,0x04,0x05,
  0x05,0x05,0x05,0x01,0x01,0x04,0x01,0x01,0x04,0x04,0x01,0x00,0x00,0xec,0x00,0x24,
  0x05,0xd9,0x00,0x01,0x05,0x05,0x05,0x01,0x01,0xd9,0xd9,0x00,0xd9,0x24,0x24,0x05,
  0xd9,0x03,0x05,0x05,0x05,0x01,0x04,0x01,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x01,0x04,0x00,0x00,0x02,0x02,0x04,0x01,0x04,0x04,0x04,0x04,0x01,0x03,0x04,
  0x05,0x05,0x05,0x01,0x04,0x04,0x04,0x00,0x01,0x00,0x02,0x02,0x00,0x00,0x02,0x02,
  0x02,0x00,0x00,0x00,0xd9,0xd9,0x01,0xd9,0x01,0x00,0x00,0x00,0x00,0x01,0xd9,0xd9,
  0x05,0x05,0x05,0x05,0x05,0x01,0x04,0x04,0x04,0x01,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x02,0x04,0x00,0x00,0x02,0x00,0x02,0x04,0x04,0x01,0x04,0x04,0x05,0x05,0x05,
  0x05,0x05,0x05,0x01,0x01,0x04,0x01,0x04,0x00,0x00,0x02,0x02,0x02,0xe2,0x83,0x02,
  0x02,0x02,0x01,0xd9,0xf0,0x01,0xd9,0x01,0x00,0x00,0x00,0x00,0xd9,0xd9,0x01,0x00,
  0x01,0x01,0x05,0x01,0x03,0x03,0x03,0x05,0x01,0x04,0x04,0x04,0x04,0x04,0x01,0x05,
  0x00,0x00,0x01,0x00,0x02,0xe2,0x00,0x00,0x02,0x04,0x04,0x04,0x03,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x01,0x04,0x01,0x02,0x02,0x02,0xe2,0xe2,0x02,0x83,0x83,0x83,
  0x00,0x00,0x00,0x00,0xd9,0x05,0xd9,0xd9,0x00,0x00,0x00,0x00,0xd9,0x01,0x01,0x01,
  0x01,0x01,0x05,0x01,0x04,0x04,0x01,0x05,0x05,0x01,0x01,0x04,0x04,0x01,0x01,0x01,
  0x04,0x02,0x01,0x02,0x83,0xf8,0x02,0x00,0x04,0x00,0x04,0x01,0x05,0x05,0x05,0x05,
  0x04,0x01,0x01,0x01,0x01,0x01,0x00,0x02,0x02,0x83,0x83,0x83,0x83,0x83,0xf1,0xf8,
  0x02,0x02,0x00,0xd9,0x00,0xd9,0xd9,0xd9,0x00,0x00,0x00,0xd9,0xd9,0xd9,0x01,0x04,
  0x01,0x04,0x00,0x03,0x04,0x05,0x01,0x01,0x00,0x04,0x01,0x01,0x05,0x05,0x01,0x05,
  0x01,0x00,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x04,0xd9,0x05,0xd9,0x01,
  0x02,0x00,0x01,0xd9,0xd9,0x01,0x01,0x04,0x02,0xf4,0x83,0x83,0x83,0xc9,0xe2,0xf4,
  0x02,0xe2,0x02,0x05,0xd9,0xd9,0x01,0x00,0x00,0x00,0x00,0xd9,0xd9,0xd9,0x00,0x00,
  0x04,0x00,0x01,0x02,0x01,0x01,0x04,0x05,0x01,0x05,0x01,0x01,0x01,0x03,0x01,0x05,
  0x01,0xd9,0x05,0x00,0x83,0x83,0x02,0x02,0x02,0x00,0x00,0x02,0x02,0x00,0x01,0x00,
  0xe2,0x01,0x05,0x00,0x00,0x01,0xec,0x02,0x02,0x02,0xe2,0x02,0x02,0x02,0xe2,0xe2,
  0xe2,0x04,0x02,0x00,0xd9,0xd9,0x00,0x02,0x00,0x00,0xd9,0xd9,0xd9,0xd9,0x04,0x00,
  0x02,0x01,0x01,0x00,0x01,0x05,0x01,0x03,0x05,0x05,0x05,0x01,0x02,0x01,0x01,0x05,
  0x01,0x00,0x02,0xf4,0x83,0xf1,0xf4,0x83,0x83,0x83,0xe2,0x83,0x02,0x01,0x02,0x00,
  0xd9,0x01,0x01,0x01,0x01,0x00,0x02,0x02,0xe2,0x83,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x00,0x00,0x00,0xd9,0x01,0x00,0x00,0xd9,0xd9,0xd9,0x02,0x00,0x02,
  0x02,0x00,0x00,0xec,0x00,0x01,0x01,0x01,0x00,0x00,0x02,0x02,0x04,0x02,0xec,0xec,
  0xc9,0x02,0x02,0xe2,0xf4,0xe2,0xe2,0x02,0x83,0x02,0x83,0xf4,0x83,0x00,0x02,0x02,
  0x05,0x01,0x01,0x04,0x05,0x05,0x01,0x00,0x04,0x02,0xe2,0x02,0x02,0x02,0x02,0x02,
  0x00,0x02,0x02,0x02,0x00,0x00,0xd9,0xd9,0x00,0xd9,0xd9,0x00,0xd9,0x02,0x02,0x02,
  0x04,0x00,0x04,0x02,0x02,0xec,0x00,0x01,0x01,0x02,0xe2,0xe2,0xec,0xec,0x02,0x00,
  0x02,0x00,0x00,0x02,0xe2,0x02,0x02,0x02,0x02,0x00,0x02,0x00,0x01,0x01,0x01,0xd9,
  0x05,0x01,0x01,0x04,0x05,0x01,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0xe2,0x02,0x02,
  0x00,0x00,0x02,0x02,0x00,0x02,0x02,0x00,0x00,0x01,0x01,0xd9,0xf0,0x01,0x00,0x00,
  0x00,0x00,0x04,0x00,0x00,0x00,0x02,0x00,0x01,0x02,0x83,0x02,0x02,0x00,0x02,0x01,
  0x00,0x01,0x01,0x00,0x83,0x02,0x02,0x00,0x00,0x02,0x00,0x04,0x05,0x01,0x01,0x05,
  0x04,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x02,
  0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x01,0xd9,0x24,0x05,0xd9,0xd9,0xd9,0x00,0x04,
  0x00,0xd9,0x00,0xd9,0x01,0x00,0x00,0x01,0x00,0x02,0x02,0x02,0x02,0x00,0x00,0xd9,
  0x02,0x02,0xd9,0x00,0x02,0xe2,0x00,0x00,0x04,0x00,0x01,0x00,0x04,0x05,0x05,0x04,
  0x05,0x03,0x05,0x01,0x05,0x05,0x04,0x04,0x03,0x03,0x04,0x04,0x01,0x01,0x02,0x02,
  0x00,0x02,0x00,0x00,0x00,0xe2,0x02,0x05,0x26,0x26,0x05,0xd9,0xd9,0x26,0x05,0x05,
  0xd9,0x05,0x05,0x05,0xd9,0xd9,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0xd9,
  0xd9,0x01,0x02,0x01,0x00,0x02,0x02,0x00,0x00,0xd9,0x05,0x01,0x05,0xd9,0x04,0x01,
  0x03,0x04,0x05,0x05,0x01,0x05,0x03,0x03,0x03,0x03,0x03,0x05,0x01,0x01,0x01,0x00,
  0x02,0x02,0x00,0x00,0xd9,0x00,0x05,0x05,0x03,0x24,0x24,0x24,0x26,0x26,0x05,0x05,
  0xd9,0x05,0x05,0x05,0x05,0x05,0xd9,0xd9,0xd9,0xd9,0xd9,0x00,0x02,0x02,0x00,0x00,
  0xd9,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x05,0x05,0x05,0x05,0x05,0x05,
  0x01,0x04,0x01,0x01,0x05,0x05,0x05,0x05,0x03,0x04,0x05,0x05,0x01,0x05,0x01,0x01,
  0x02,0x00,0xd9,0xd9,0x01,0x05,0x24,0x03,0x02,0x05,0x24,0x24,0x26,0x05,0x26,0x05,
  0x05,0x05,0xd9,0x05,0x26,0x24,0xd9,0x26,0xd9,0xd9,0xd9,0x01,0x00,0x02,0x00,0x00,
  0xd9,0xd9,0xd9,0x01,0xd9,0x00,0x00,0x00,0x00,0x00,0x05,0x05,0x05,0x05,0x05,0x01,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x01,0x01,0x01,
  0x00,0x00,0x00,0x01,0x01,0x05,0x24,0x02,0x03,0x05,0x05,0x26,0x05,0x05,0x24,0x24,
  0x26,0x05,0x05,0x26,0x24,0xf0,0xd9,0xd9,0xd9,0xd9,0x00,0xd9,0x00,0x02,0x00,0x00,
  0xd9,0xd9,0xd9,0xd9,0x05,0xd9,0x01,0x00,0x01,0x01,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x24,0x05,0x05,0x05,0x01,0x01,0xd9,
  0x01,0x00,0x00,0x01,0x01,0x05,0x24,0x01,0x03,0x05,0x26,0x24,0x26,0x05,0x05,0x24,
  0x26,0xd9,0x26,0x26,0x26,0x24,0xd9,0xd9,0xd9,0x01,0x01,0xd9,0x00,0x00,0xd9,0xd9,
  0xd9,0xd9,0xd9,0x01,0x01,0x05,0xd9,0xd9,0x00,0x00,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0xd9,0xd9,
  0x01,0x01,0xd9,0x00,0x03,0x05,0x24,0xd9,0x03,0x05,0x24,0x24,0x26,0x26,0x05,0x05,
  0xd9,0x26,0x26,0x24,0x26,0x24,0x26,0x26,0xd9,0x26,0x01,0x01,0xd9,0xd9,0xd9,0x05,
  0x26,0x26,0x05,0x05,0x01,0xd9,0xd9,0x05,0xd9,0x01,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x03,0x05,0x05,0x05,0x05,0x05,0x05,0x24,0x24,0x05,0x01,0x05,0x24,0xd9,
  0xd9,0x00,0x01,0x02,0x02,0x24,0x26,0x26,0xd9,0x24,0x24,0x05,0x24,0x24,0x26,0x26,
  0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,0x26,0xd9,0x00,0xd9,0xd9,0xd9,0xd9,0xd9,
  0xd9,0x26,0x26,0x05,0x05,0xf0,0x26,0x05,0xd9,0x00,0x05,0x05,0x05,0x24,0x05,0x05,
  0x05,0x05,0x05,0x03,0x03,0x04,0x05,0x01,0x05,0x05,0x24,0x24,0xd9,0xd9,0x05,0x26,
  0xd9,0x02,0x00,0x02,0x02,0x01,0xf9,0x24,0xf9,0x24,0x26,0x24,0x24,0x24,0x26,0x24,
  0x26,0x26,0x24,0xea,0x26,0xea,0x26,0x24,0xd9,0x01,0x00,0xd9,0xd9,0xd9,0x26,0x26,
  0x05,0x26,0x26,0x05,0xf0,0x05,0xd9,0x05,0xd9,0x00,0x00,0x05,0x24,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x03,0x05,0x05,0x24,0x05,0x05,0x05,0x05,0x05,0x05,0x26,
  0x01,0x04,0x03,0x02,0xfb,0x01,0x05,0x24,0x26,0x26,0x26,0xd9,0xd9,0x24,0x26,0x26,
  0x24,0x26,0x26,0x26,0x26,0x26,0x26,0x05,0xd9,0xd9,0xd9,0xd9,0x01,0xd9,0x05,0x26,
  0x05,0x26,0x26,0xd9,0xd9,0xf0,0x05,0xd9,0x05,0x05,0x00,0x05,0x05,0x05,0x24,0x05,
  0x05,0x05,0x05,0x03,0x05,0x05,0x05,0x24,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x26,
  0x26,0x00,0x02,0x02,0x00,0x01,0xd9,0xea,0xea,0xea,0x26,0x26,0x24,0xd9,0xd9,0x24,
  0x26,0x24,0x24,0x24,0x26,0x24,0x26,0x26,0x26,0xd9,0x05,0x01,0x01,0xd9,0x05,0x26,
  0x26,0x26,0x26,0xd9,0xd9,0xd9,0xd9,0x05,0x05,0x05,0x01,0x01,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x03,0x05,0x05,0x05,0x24,0x05,0x01,0x05,0x05,0x24,0x05,0x24,0x26,
  0xd9,0x01,0x02,0x01,0x01,0x04,0x00,0xd9,0x26,0x26,0x24,0x26,0x24,0x26,0x05,0x26,
  0x24,0x26,0x26,0x26,0x26,0x24,0x05,0x24,0x26,0xd9,0xd9,0x01,0x05,0x26,0x05,0x26,
  0x26,0x05,0x05,0x01,0xd9,0x01,0x05,0x05,0x05,0xd9,0x01,0x05,0x05,0x01,0x01,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0x24,0xea,0x24,
  0x00,0x02,0x02,0xd9,0xd9,0x01,0x01,0x00,0x01,0xd9,0x24,0x26,0x24,0x24,0x24,0xea,
  0xea,0x26,0x26,0x26,0x26,0x24,0x24,0x26,0x05,0x05,0x01,0xd9,0x26,0x26,0x26,0x05,
  0x26,0x26,0x05,0x01,0xd9,0x01,0x05,0x24,0x26,0x24,0xd9,0x05,0x01,0x05,0x24,0x05,
  0x05,0x05,0x03,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x26,0xea,0xea,
  0xd9,0x02,0x02,0x01,0xd9,0xd9,0x01,0xd9,0x01,0x05,0x24,0x24,0x24,0x24,0x24,0xea,
  0x26,0x24,0x26,0x24,0x26,0x05,0x24,0x24,0x26,0x26,0x01,0xf0,0x26,0x05,0xf0,0x26,
  0x24,0x05,0x05,0xd9,0x01,0x05,0x05,0x26,0x24,0x24,0x05,0x05,0xd9,0x01,0xd9,0x24,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0x26,0x24,0x24,0x24,
  0x24,0x04,0x04,0xd9,0xd9,0xd9,0xd9,0x01,0x05,0x05,0x05,0xf9,0x24,0xea,0xea,0x26,
  0x26,0x24,0x24,0x26,0x24,0x24,0x05,0x24,0x26,0x26,0x00,0xd9,0x00,0xf0,0x26,0x26,
  0x26,0x26,0x05,0x05,0x05,0x05,0x26,0x24,0x24,0x26,0x24,0x26,0x24,0xd9,0xd9,0x05,
  0x05,0x05,0x05,0x24,0x24,0x24,0x24,0x05,0x05,0xd9,0x24,0x05,0x24,0x05,0x24,0x01,
  0xd9,0x04,0x03,0x01,0x01,0x01,0xd9,0x05,0x24,0x24,0x24,0x24,0x26,0xea,0xea,0x26,
  0x24,0x26,0x26,0x26,0x05,0x05,0x05,0x24,0x26,0x26,0xd9,0xf0,0x01,0xd9,0x26,0x26,
  0x26,0x26,0x05,0x05,0x24,0x26,0x26,0x26,0x26,0x24,0x26,0x05,0x24,0x26,0xd9,0x05,
  0x05,0x05,0x05,0x05,0x24,0x24,0x24,0x00,0x05,0x05,0x05,0x24,0x24,0x24,0x03,0xd9,
  0x01,0x01,0x03,0x04,0x03,0x03,0xd9,0x26,0x26,0x24,0x26,0x26,0x24,0xea,0x26,0x26,
  0x26,0x24,0x24,0x24,0x24,0x24,0x24,0x05,0x26,0x26,0xd9,0xd9,0xd9,0xf0,0x26,0x26,
  0x26,0x26,0x24,0x05,0x24,0x05,0x26,0x24,0x26,0x24,0x26,0x24,0xd9,0x05,0x24,0x05,
  0x24,0x05,0x05,0x24,0x24,0x05,0x05,0x01,0x05,0x05,0x05,0x05,0x05,0x05,0x02,0x01,
  0x01,0x05,0xd9,0x01,0x04,0x01,0xd9,0xd9,0x26,0x26,0x26,0x26,0x26,0x24,0x26,0x05,
  0x05,0x24,0x24,0x24,0x24,0x05,0x24,0x24,0x26,0xd9,0xd9,0xd9,0xd9,0x26,0x26,0xd9,
  0x26,0x26,0x24,0x26,0x05,0xd9,0x26,0x05,0x24,0xea,0x24,0x24,0x24,0x26,0x24,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0x02,0x04,
  0x00,0x00,0x01,0x01,0x04,0x01,0xd9,0xd9,0xd9,0x26,0x26,0x26,0x26,0x24,0x24,0x05,
  0x05,0x24,0x05,0x26,0x24,0x24,0x24,0x26,0x26,0xd9,0xd9,0xd9,0xd9,0xf0,0x26,0x26,
  0xd9,0xd9,0xd9,0x24,0x05,0x05,0x05,0x05,0x05,0x24,0x24,0x26,0x24,0xea,0x26,0xd9,
  0x01,0xd9,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x01,0x00,0x01,0xd9,0xd9,0x00,0x00,
  0x04,0x02,0x01,0x01,0x04,0x00,0x02,0xd9,0xd9,0x26,0x05,0x24,0x26,0x24,0x26,0x26,
  0x24,0x24,0x24,0x24,0x26,0x26,0x05,0x05,0xd9,0x00,0x00,0xd9,0xd9,0xd9,0xf0,0xf0,
  0x26,0xd9,0xd9,0xd9,0x05,0x24,0x05,0x05,0x24,0x24,0x05,0x24,0x26,0x05,0x26,0x26,
  0x05,0x26,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0x05,0x05,0x01,0x00,0x01,0x00,
  0x01,0x02,0x02,0x00,0x00,0x00,0x02,0x02,0xd9,0xd9,0xd9,0x24,0x24,0x24,0x26,0x26,
  0x24,0x05,0x05,0x05,0x05,0x24,0x26,0xd9,0xd9,0x00,0xd9,0xd9,0xd9,0xd9,0x26,0xf0,
  0x26,0xd9,0x01,0xd9,0xd9,0x05,0x05,0x24,0x24,0x26,0x05,0x24,0x26,0x26,0x05,0x05,
  0x05,0x05,0x26,0x26,0x24,0x24,0x24,0x05,0x24,0x05,0x05,0x05,0x05,0x01,0x05,0x01,
  0x02,0x00,0x02,0x00,0x00,0x02,0x02,0x00,0x00,0xd9,0xd9,0x26,0x26,0x26,0x26,0x26,
  0x24,0x05,0x05,0x05,0x05,0x05,0xd9,0x05,0xd9,0x01,0xd9,0xd9,0xd9,0x00,0xd9,0xd9,
  0x01,0xf0,0x01,0x00,0xd9,0x01,0x05,0x05,0x24,0x24,0x26,0x26,0x26,0x26,0x05,0x05,
  0xd9,0x05,0x26,0x24,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x24,
  0x05,0x05,0x05,0x05,0x00,0x00,0x02,0x02,0x00,0x01,0xd9,0xd9,0xd9,0x26,0x26,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x01,0xd9,0xd9,0x00,0x00,0xd9,0xd9,0x01,
  0x00,0xd9,0x01,0x01,0x01,0x01,0xd9,0x05,0x05,0x05,0x26,0x26,0x05,0x05,0x05,0x04,
  0xd9,0xd9,0xd9,0x05,0x26,0x05,0x05,0x05,0x24,0x05,0x05,0x05,0x04,0x05,0x05,0x05,
  0x05,0x05,0x05,0xd9,0xd9,0x01,0x00,0x02,0x00,0x00,0x00,0xd9,0xd9,0xd9,0x26,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0xd9,0xd9,0x00,0xd9,0xd9,0x00,0x00,0x00,0xd9,0x00,
  0x00,0x00,0x02,0x00,0xd9,0xd9,0xd9,0x05,0x05,0x05,0x05,0x05,0xd9,0xd9,0xd9,0xd9,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x01,0x05,0x05,0x01,0x24,0xd9,0xd9,0x00,0x00,0x00,0x00,0x00,0x00,0xd9,0xd9,0x24,
  0x05,0x24,0x24,0x05,0x24,0x05,0x05,0x01,0x01,0x00,0x01,0x02,0x00,0x00,0xd9,0xd9,
  0x00,0x00,0x02,0x04,0x01,0xd9,0x05,0xd9,0x05,0xd9,0x05,0x01,0x01,0x01,0xd9,0xd9,
  0x05,0x01,0xd9,0x05,0x01,0x05,0x05,0x05,0x05,0x05,0x05,0x03,0x05,0x05,0x05,0x01,
  0x05,0x05,0x05,0x05,0x01,0x01,0x01,0x05,0x00,0x00,0x00,0xec,0x00,0xd9,0xd9,0xd9,
  0x05,0x05,0x05,0xd9,0x05,0x01,0x01,0x01,0x00,0x02,0x00,0x00,0x02,0x00,0x00,0x02,
  0xec,0x02,0x02,0x01,0x01,0xd9,0x05,0x01,0x01,0xd9,0x00,0x01,0x04,0x01,0x00,0x01,
  0x05,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x03,0x05,0x05,0x03,0x01,0x05,0x05,
  0x05,0xd9,0x05,0x05,0x05,0x01,0xd9,0x01,0x00,0x00,0x00,0x00,0xec,0xd9,0xd9,0xd9,
  0x05,0x05,0x01,0x05,0x01,0x01,0x00,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x83,0x83,
  0xc9,0x02,0x02,0x00,0x01,0xd9,0x01,0xd9,0xd9,0x01,0x01,0x04,0x00,0x01,0x00,0x00,
  // 6: ground accel
  0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,
  0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,
  0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,
  0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,
  0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,
  0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,
  0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,
  0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,
  0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x28,0x42,0x00,0x41,0x08,0x41,0x10,0x41,
  0x18,0x41,0x00,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,
  0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,
  0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,
  0x00,0x60,0x08,0x60,0x10,0x60,0x01,0x69,0x18,0x60,0x08,0x61,0x10,0x61,0x18,0x61,
  0x00,0x62,0x08,0x62,0x10,0x62,0x28,0x6b,0x18,0x62,0x00,0x63,0x10,0x63,0x18,0x63,
  0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,
  0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,
  0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x28,0x8b,0x10,0x81,
  0x18,0x81,0x00,0x82,0x31,0x8c,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x10,0x83,
  0x18,0x83,0x09,0x84,0x00,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,
  0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,
  0x00,0xa0,0x01,0xaa,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0xcf,0x7b,
  0x10,0xa1,0x18,0xa1,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x09,0xa3,0x00,0xa3,0x10,0xa3,
  0x08,0xac,0x18,0xa3,0x00,0xa4,0x10,0xa4,0x18,0xa4,0xae,0x73,0x29,0xa5,0x00,0xa5,
  0x10,0xa5,0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,
  0x10,0xa7,0x18,0xa7,0x0a,0x73,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x00,0xc1,
  0x08,0xc1,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0x08,0xc3,
  0x00,0xc3,0xea,0x7a,0x28,0xc5,0x28,0xcc,0x10,0xc3,0x18,0xc3,0x00,0xc4,0x10,0xc4,
  0x18,0xc4,0x00,0xc5,0x10,0xc5,0x18,0xc5,0x09,0xce,0x00,0xc6,0x10,0xc6,0x18,0xc6,
  0x00,0xc7,0x08,0xc7,0x10,0xc7,0xcf,0x83,0x18,0xc7,0x00,0xe0,0xc4,0xe2,0x04,0xeb,
  0x44,0xec,0x8a,0x52,0x08,0xe0,0x62,0xb2,0x10,0xe0,0x84,0xeb,0x6c,0x73,0x18,0xe0,
  0x45,0xb2,0x07,0xdd,0x26,0xe5,0x2b,0xef,0xc6,0xdc,0x22,0xa2,0x62,0x89,0xc3,0x82,
  0x66,0xbc,0xcb,0x5a,0xa1,0x91,0x84,0xe2,0xaa,0xd5,0x88,0xe5,0xc9,0xe5,0xe3,0xeb,
  0xf0,0xe6,0xf4,0xf3,0xf1,0xeb,0xf6,0xeb,0xf8,0xfd,0xff,0xe6,0xed,0xf2,0xfe,0xff,
  0xe6,0xff,0xf1,0xfc,0xd0,0xfa,0xfa,0xf7,0x6b,0xf9,0xf9,0xee,0xa7,0xa7,0xa7,0xa7,
  0xa7,0xa7,0xa7,0xa7,0xee,0xee,0xee,0xc2,0xf7,0xf6,0xfa,0xd0,0xfc,0xd2,0xff,0xe6,
  0xff,0xfe,0xfd,0xed,0xe6,0xed,0xfd,0xf1,0xeb,0xf6,0xf5,0xf4,0xf3,0xf2,0xed,0xf0,
  0xe6,0xe8,0xfd,0xf3,0xf4,0xa1,0x68,0xf7,0x86,0xb2,0xe6,0xfb,0xed,0xfd,0xfe,0xff,
  0xe6,0xed,0xf8,0xb0,0xae,0xf7,0xfa,0xf8,0xb6,0x69,0xee,0xf9,0xb5,0xa7,0xa7,0xa7,
  0xb5,0xb5,0xb5,0xb5,0xf9,0xf9,0xe9,0x6b,0xf7,0xf6,0x69,0xf7,0xb2,0xf8,0xed,0xe6,
  0xff,0xfd,0xfd,0xff,0xfb,0xe6,0xd6,0xb2,0xf7,0x68,0xf5,0xff,0xf3,0xfe,0xff,0xe6,
  0xf4,0xfd,0xf3,0xfd,0xd0,0xfa,0x69,0x6b,0x6b,0xf7,0xa1,0xeb,0xff,0xfd,0xf1,0xed,
  0xeb,0xa1,0x86,0x86,0xd1,0x8b,0xa1,0xd2,0xdc,0xf7,0xc2,0xc2,0xb5,0xa7,0xa7,0xb5,
  0xb5,0xb5,0xb5,0xb5,0xf9,0xf9,0xf7,0xf7,0xb0,0x89,0x69,0x6b,0xd1,0x86,0xa1,0xeb,
  0xe6,0xf1,0xfe,0xff,0xcb,0xeb,0xf5,0x69,0x69,0x69,0x89,0xd0,0xf1,0xf3,0xfd,0xf4,
  0xf3,0xf3,0xfd,0xff,0xf5,0xd1,0xd1,0xd1,0xd1,0xd1,0x8b,0xf5,0xd0,0xfd,0xf8,0xeb,
  0xf6,0xb3,0xd1,0xee,0xd1,0xf6,0xd0,0xdc,0xdc,0xf7,0xc2,0xc2,0xb5,0xa7,0xa7,0xa7,
  0xa7,0xa7,0xb5,0xb5,0xf9,0xe9,0xf7,0xf7,0xb6,0xeb,0xf6,0xc2,0x6b,0x6b,0x69,0xf6,
  0xa1,0xf8,0xfe,0xd0,0xa1,0x8b,0x69,0x69,0x6b,0xd1,0xee,0xf5,0xd0,0xfd,0xf3,0xf3,
  0xf1,0xe8,0xd0,0xf5,0xfa,0xf5,0xf5,0x86,0xad,0xaa,0x8b,0x86,0xae,0xb0,0xf5,0x8b,
  0x69,0xb8,0xee,0xee,0x8b,0xfa,0xd6,0xfc,0xdc,0xf7,0x6b,0xee,0xf9,0x44,0x44,0x44,
  0xf9,0xe9,0xf9,0xf9,0xe9,0xe9,0x86,0xf7,0xd2,0xd0,0xfa,0x6b,0x6b,0xc2,0xc2,0x6b,
  0x63,0xf0,0xb2,0xae,0xf5,0x8b,0xfa,0xf7,0x86,0xad,0xf0,0xfa,0xf5,0xd0,0xe8,0xf1,
  0xeb,0xf5,0xaa,0x6b,0xf5,0xd0,0xf1,0xd3,0xd0,0xeb,0xeb,0xd0,0xf8,0xf7,0x45,0x46,
  0xf9,0x8a,0x69,0xc2,0xf6,0xeb,0xe8,0xf3,0xd2,0xf7,0xd1,0x6b,0x69,0xf9,0xee,0xe9,
  0xee,0xee,0xee,0xe9,0x69,0x69,0xf7,0xf7,0xb6,0xd3,0xa1,0xf6,0x6b,0xc2,0x69,0xc2,
  0x69,0x6b,0x86,0xb2,0xd0,0xeb,0xeb,0xd0,0xf8,0xd2,0xed,0xa1,0xee,0xf6,0xf5,0xeb,
  0xf5,0xf7,0xee,0xd1,0xad,0xd3,0xfd,0xe8,0xe7,0xfb,0xed,0xf4,0xfe,0xae,0xc2,0x69,
  0xe3,0x8a,0xc2,0x6b,0xfa,0xd0,0xf4,0xfc,0xf8,0xf5,0xf7,0x89,0x89,0x8b,0xc2,0x69,
  0xee,0xee,0x6b,0x86,0x86,0x86,0x86,0xb2,0xb0,0xf8,0xd0,0xfa,0x8b,0xc2,0xc2,0xc2,
  0x6b,0x69,0xad,0xf1,0xf4,0xed,0xfb,0xe7,0xe8,0xfd,0xf1,0xae,0xee,0x69,0x6b,0xf7,
  0xeb,0xf7,0xe3,0xd1,0x86,0xf8,0xe8,0xed,0xfb,0xe7,0xe8,0xfe,0xfd,0xeb,0x8b,0xc2,
  0x8a,0xee,0xc2,0x8b,0xa1,0xff,0xf4,0xd2,0xd6,0xd6,0xd6,0xf8,0xf8,0xb0,0x6b,0xc2,
  0xee,0x69,0xf7,0xf7,0x8e,0xb2,0xae,0xae,0xb2,0xf8,0xff,0xa1,0x8b,0xc2,0xc2,0xc2,
  0xc2,0x69,0xa1,0xf1,0xfe,0xe8,0xe7,0xfb,0xed,0xe8,0xf1,0xb0,0xd1,0x69,0x8b,0xeb,
  0xf1,0xae,0xee,0x69,0x86,0xd0,0xe7,0xfb,0xe7,0xff,0xfd,0xfe,0xe8,0xeb,0x8b,0xe3,
  0xe3,0xb5,0xee,0xf5,0xd0,0xff,0xf1,0xf1,0xf2,0xfc,0xfc,0xf3,0xfe,0xfc,0xf8,0x6b,
  0x6b,0xf7,0xb6,0xdc,0xfc,0xfc,0xd2,0xf8,0xf8,0xf4,0xe8,0xd0,0xfa,0xd1,0xd1,0xc2,
  0xc2,0xd1,0xa1,0xe8,0xfe,0xfd,0xe8,0xe7,0xfb,0xe7,0xe6,0xad,0xd1,0x69,0x86,0xf8,
  0xfd,0xf8,0xad,0x69,0x8b,0xeb,0xfb,0xe7,0xff,0xfd,0xf3,0xf1,0xed,0xf0,0xf5,0x8a,
  0xc2,0xe3,0x86,0xeb,0xe8,0xd2,0xf3,0xf3,0xfe,0xfc,0xdc,0xf1,0xf8,0xeb,0xf5,0xf5,
  0xf5,0xf5,0xae,0xf8,0xf8,0xd2,0xdc,0xdc,0xf3,0xf3,0xfd,0xff,0xeb,0x86,0x6b,0xf9,
  0xee,0x8b,0xa1,0xed,0xf1,0xfe,0xfd,0xe8,0xe7,0xfb,0xcb,0xaa,0xd1,0xf7,0xae,0xfd,
  0xff,0xe7,0xeb,0xf6,0x8b,0xeb,0xe7,0xe8,0xfd,0xfe,0xfd,0xff,0xe6,0xe6,0xeb,0xd1,
  0xe3,0xe3,0xf0,0xf8,0xfc,0xf3,0xf3,0xfd,0xf4,0xff,0xed,0xeb,0xf5,0xf0,0x69,0xf6,
  0xf6,0x69,0x69,0xfa,0xf5,0xd0,0xd6,0xe8,0xd2,0xfc,0xf3,0xfc,0xf8,0xf5,0xc2,0xb5,
  0xee,0xf0,0xf0,0xe6,0xed,0xf1,0xfe,0xfd,0xe8,0xed,0xeb,0x8b,0x8b,0xa1,0xe6,0xff,
  0xe6,0xfb,0xf0,0xa1,0x8b,0xcb,0xf4,0xfd,0xfe,0xf1,0xff,0xe6,0xe6,0xed,0xae,0xd1,
  0xc2,0xee,0xb0,0xfc,0xfc,0xfc,0xf1,0xff,0xd0,0xa1,0xfa,0xf5,0x8b,0x6b,0x8b,0xeb,
  0xd0,0xf0,0xd1,0x6b,0x68,0xf6,0xfa,0xeb,0xd0,0xe8,0xf1,0xfc,0xde,0xb0,0x86,0xb5,
  0xee,0x86,0xed,0xcf,0xe6,0xed,0xf1,0xfe,0xfe,0xf4,0xd0,0xf0,0xf5,0xeb,0xfb,0xe6,
  0xed,0xed,0xff,0xd0,0xae,0xb0,0xfd,0xf1,0xe8,0xff,0xe6,0xe6,0xed,0xd3,0xb0,0x86,
  0xee,0x86,0xd2,0xd2,0xf1,0xd3,0xf0,0xf5,0xf5,0xf0,0x69,0x69,0x69,0xc2,0xae,0xf8,
  0xd2,0xb2,0x6b,0x69,0xc2,0x69,0x69,0x89,0xfa,0xf5,0xcf,0xf4,0xd2,0xfc,0x86,0xee,
  0xee,0xb0,0xd3,0xff,0xcf,0xe6,0xed,0xe8,0xf1,0xfd,0xf8,0xae,0xd0,0xff,0xed,0xed,
  0xf2,0xfd,0xfd,0xfe,0xb0,0x86,0xae,0xeb,0xeb,0xeb,0xfb,0xed,0xd3,0xd2,0xd3,0xf0,
  0xd1,0xae,0xf8,0xed,0xf0,0xf5,0xf5,0x86,0xd1,0xd1,0xc2,0x69,0x6b,0x91,0xb0,0xfc,
  0xfc,0xf8,0x86,0xc2,0xc2,0xc2,0x69,0x6b,0x6b,0x8b,0xf6,0xaa,0xcf,0xd3,0x86,0x86,
  0x86,0xf8,0xd2,0xd3,0xed,0xcf,0xf0,0xeb,0xeb,0xeb,0xf7,0xae,0xf1,0xfd,0xfd,0xf2,
  0xfe,0xfe,0xf1,0xf8,0xf5,0x69,0x69,0x63,0x8b,0x8b,0xf0,0xae,0xb0,0xd3,0xd3,0xad,
  0xf0,0xf0,0xeb,0xf5,0x86,0x86,0xee,0xb5,0xc2,0xe9,0xf9,0xee,0xee,0xad,0xd0,0xff,
  0xff,0xff,0xeb,0xd1,0xb5,0xee,0xf9,0xe9,0xc2,0xf9,0xb5,0xd1,0x8b,0xf5,0xf0,0xf0,
  0xf7,0xb0,0xd3,0xb0,0xad,0xf0,0x8b,0x8b,0x69,0x69,0x69,0xf7,0xf8,0xf1,0xfe,0xfe,
  0xff,0xff,0xed,0xeb,0x89,0x69,0x6b,0xc2,0xe3,0xe3,0xee,0xd1,0xad,0xae,0xb6,0xb0,
  0xeb,0xf5,0x86,0xe3,0xb5,0xee,0xee,0xc2,0xf9,0xf9,0xe9,0x69,0x89,0xa1,0xfb,0xe6,
  0xe6,0xe6,0xeb,0x8b,0x69,0xe9,0xf9,0xa7,0xb5,0xee,0xa7,0xa7,0xb5,0xc2,0x8b,0xf5,
  0xad,0xb6,0xb0,0xad,0xd1,0xee,0xc2,0xee,0xc2,0x69,0x69,0x68,0xa1,0xe6,0xff,0xff,
  0xe6,0xe6,0xeb,0xfa,0x6b,0xee,0xa7,0xee,0xee,0xee,0xa7,0xa7,0xb5,0xd1,0xf7,0xed,
  0xcf,0xf7,0x91,0x86,0xe3,0xb5,0xc2,0xe9,0x69,0x44,0x69,0x69,0xfa,0xeb,0xe6,0xed,
  0xed,0xed,0xd0,0xa5,0x69,0x69,0x44,0x44,0xb5,0xe3,0xb5,0xee,0xee,0x86,0x86,0xaa,
  0xf5,0x86,0xee,0xf9,0xee,0xb5,0xee,0xe9,0xc2,0xc2,0xee,0xc2,0x8b,0xa1,0xe6,0xe6,
  0xff,0xed,0xa1,0x86,0xe3,0xee,0xe9,0xee,0xee,0xee,0xb5,0xd1,0x8b,0xfa,0xf0,0xf5,
  0xf5,0xed,0xf8,0xb0,0x86,0x86,0x8b,0x69,0x69,0xe9,0xe9,0xf7,0xeb,0xe8,0xf4,0xf1,
  0xf2,0xf2,0xf4,0xd0,0xfa,0x45,0x44,0x69,0x45,0x69,0x86,0x86,0x91,0xb0,0xcf,0xaa,
  0xfa,0xf5,0x8b,0xd1,0xc2,0xee,0xf9,0xee,0xc2,0xc2,0xc2,0xee,0xd1,0xa1,0xe6,0xff,
  0xf1,0xf8,0xf7,0x6b,0x6b,0x69,0xc2,0xc2,0xc2,0xd1,0x8b,0xf5,0xb0,0xd3,0xcf,0x8b,
  0xf0,0xd3,0xfc,0xfc,0xd6,0xeb,0xf5,0xf5,0xf5,0xf5,0xf7,0xae,0xf1,0xfe,0xfe,0xfe,
  0xfe,0xfe,0xfe,0xfd,0xae,0x68,0x89,0xfa,0xf6,0xf5,0xeb,0xed,0xd2,0xfe,0xd2,0xf5,
  0x8b,0xaa,0xf8,0xad,0xf0,0x8b,0xc2,0xb5,0xc2,0xc2,0xc2,0xc2,0xc2,0xf7,0xf8,0xf1,
  0xfc,0xf8,0x86,0x69,0x6b,0xd1,0xc2,0x8b,0xfa,0xf5,0xf8,0xfc,0xd2,0xed,0xfa,0x86,
  0x91,0xb6,0xfc,0xe8,0xed,0xfb,0xe6,0xe6,0xd6,0xf8,0xb0,0xae,0xf4,0xe8,0xe8,0xe8,
  0xe8,0xe8,0xe8,0xf4,0xae,0xb2,0xf8,0xd6,0xed,0xe6,0xfb,0xe6,0xe8,0xfc,0xd2,0x86,
  0xc2,0xfa,0xcf,0xd3,0xfc,0xf8,0xf5,0x8b,0xd1,0x6b,0xc2,0xe9,0x6b,0x6b,0xb2,0xfc,
  0xd0,0xf5,0x6b,0xc2,0xd1,0x8b,0xfa,0xf5,0xcf,0xe8,0xfc,0xfc,0xd3,0xcf,0xf0,0xb5,
  0x8a,0xb0,0xe8,0xed,0xe6,0xe7,0xff,0xf1,0xfe,0xfd,0xf8,0xeb,0xeb,0xe6,0xe7,0xe6,
  0xe6,0xe6,0xe6,0xeb,0xf5,0xd0,0xf1,0xfe,0xfd,0xe8,0xed,0xe6,0xe6,0xff,0xb0,0x6b,
  0xee,0x8b,0xaa,0xff,0xfc,0xfc,0xd3,0xeb,0xfa,0xf6,0x69,0x6b,0xc2,0xd1,0xf7,0xd0,
  0xfa,0x8b,0x69,0x8b,0xfa,0xa1,0xd0,0xff,0xe8,0xfc,0xf3,0xf1,0xd3,0xaa,0x86,0xe3,
  0xee,0xae,0xe6,0xe6,0xed,0xe8,0xfd,0xfe,0xf1,0xff,0xeb,0xfa,0xf6,0xeb,0xe6,0xe7,
  0xe7,0xe7,0xeb,0xf6,0x89,0xeb,0xff,0xe8,0xfe,0xfe,0xe8,0xed,0xe6,0xe6,0xeb,0xb5,
  0xc2,0xc2,0xf5,0xcf,0xf1,0xf3,0xfc,0xe8,0xff,0xd0,0xa1,0xf6,0x8b,0x6b,0x69,0xfa,
  0xfa,0xf7,0xf5,0xae,0xd6,0xe8,0xf4,0xf1,0xd2,0xf3,0xfc,0xd3,0xcf,0xf5,0xee,0xb5,
  0xc2,0xa1,0xeb,0xe6,0xe8,0xfe,0xfe,0xf4,0xff,0xe7,0xeb,0xf6,0x63,0xeb,0xd0,0xe8,
  0xe8,0xff,0xeb,0x8b,0x8b,0xa1,0xfb,0xed,0xe8,0xfe,0xfe,0xe8,0xed,0xcb,0xf5,0xe3,
  0xee,0xc2,0x8b,0xf0,0xe8,0xfc,0xf3,0xdc,0xd2,0xf4,0xf8,0xd0,0xae,0xf5,0xf7,0x89,
  0xf7,0xf7,0xd2,0xdc,0xfc,0xf3,0xf3,0xd2,0xdc,0xf3,0xf1,0xed,0xf5,0x86,0xc2,0xc2,
  0x69,0xf6,0xa1,0xff,0xfd,0xfe,0xe8,0xed,0xe6,0xe6,0xfb,0xeb,0x45,0x86,0xb0,0xfd,
  0xfd,0xf8,0xee,0xd1,0xf0,0xeb,0xe6,0xfb,0xed,0xe8,0xfe,0xfd,0xff,0xeb,0x8b,0xee,
  0xa7,0xc2,0xc2,0xf5,0xe6,0xf4,0xf3,0xdc,0xd2,0xfc,0xf3,0xfc,0xdc,0xd2,0xf7,0xf7,
  0xc2,0x91,0xb6,0xfc,0xd2,0xd2,0xb0,0xd6,0xfc,0xf3,0xe8,0xcf,0xf5,0xd1,0xf9,0xe9,
  0x45,0xd1,0xa1,0xf4,0xfe,0xf4,0xed,0xe6,0xe7,0xff,0xd3,0xb0,0x45,0x6b,0xae,0xd0,
  0xd0,0xf7,0xee,0xc2,0x86,0xd6,0xff,0xe7,0xfb,0xed,0xe8,0xfe,0xf1,0xeb,0x8b,0xc2,
  0xb5,0xee,0xc2,0xf7,0xeb,0xff,0xfc,0xfc,0xf8,0xf8,0xd2,0xd2,0xfc,0xb6,0x91,0xee,
  0xf9,0x6b,0xf7,0xf7,0xf7,0xf7,0xae,0xae,0xfc,0xfc,0xff,0xa1,0x8b,0xc2,0xf9,0x44,
  0x24,0xe3,0xa1,0xf1,0xfd,0xff,0xe6,0xe6,0xff,0xf1,0xfd,0xb2,0x45,0x69,0x63,0xf5,
  0xf6,0x68,0xe3,0xc2,0x86,0xf1,0xf1,0xff,0xe7,0xfb,0xff,0xf1,0xfd,0xeb,0x45,0xc2,
  0xc2,0xa7,0xc2,0xd1,0xfa,0xed,0xd2,0xf3,0xb0,0xae,0x86,0xae,0xf7,0xf7,0x6b,0xf9,
  0xe9,0xf9,0xc2,0x69,0x6b,0x6b,0x86,0xb0,0xfc,0xf1,0xe6,0xfa,0xc2,0xee,0xf9,0x45,
  0xf9,0xee,0xf7,0xf1,0xe8,0xe6,0xfb,0xed,0xf4,0xfe,0xf4,0xeb,0x63,0x89,0xfa,0xf5,
  0xfa,0xfa,0xad,0x69,0xf0,0xf8,0xfe,0xf1,0xed,0xfb,0xfb,0xff,0xf1,0xf7,0x45,0xe9,
  0xc2,0xe3,0xc2,0x6b,0x89,0xeb,0xf4,0xf3,0xb0,0x86,0x6b,0x6b,0x6b,0xc2,0xf9,0xe9,
  0xe9,0x44,0xf9,0x44,0xee,0xd1,0x86,0xf8,0xfe,0xf4,0xf0,0x8b,0xb5,0xee,0xe9,0x45,
  0xe3,0x68,0xb0,0xae,0xeb,0xa1,0xeb,0xd0,0xf8,0xf8,0xd0,0xf5,0xfa,0xeb,0xd0,0xff,
  0xff,0xed,0xeb,0xfa,0xf6,0xeb,0xeb,0xf8,0xeb,0xa1,0xa5,0xa1,0xae,0xae,0x68,0x69,
  0x69,0xb5,0xee,0x6b,0xd1,0xfa,0xd6,0xf3,0xb6,0xf7,0x6b,0x6b,0xc2,0xf9,0xe9,0xe9,
  0xee,0xee,0xee,0xb5,0xc2,0x6b,0xf7,0xd2,0xfc,0xed,0xf5,0xd1,0xb5,0xee,0x69,0x63,
  0xaa,0xeb,0xd2,0xeb,0xf5,0x89,0x8b,0xf7,0x86,0xf7,0xf6,0xfa,0xeb,0xe8,0xfd,0xfe,
  0xf3,0xfe,0xf4,0xd0,0xf5,0x89,0x86,0xd1,0xd1,0xd1,0xd1,0xf6,0xeb,0xf8,0xae,0xfa,
  0x69,0xee,0xe3,0xd1,0xc2,0x89,0xd0,0xdc,0xdc,0xf7,0x6b,0xf9,0xee,0xee,0xf9,0xee,
  0xb5,0xb5,0xa7,0xa7,0x44,0x69,0xf7,0xd2,0xfc,0xeb,0x8b,0xee,0xc2,0xc2,0xf0,0xf6,
  0xeb,0xf4,0xfe,0xff,0xeb,0xa5,0xf6,0x69,0x69,0x69,0xf6,0xeb,0xe8,0xfe,0xf3,0xfe,
  0xfd,0xfe,0xf3,0xf2,0xeb,0xf6,0x69,0x69,0x69,0x63,0xf6,0xa1,0xe6,0xfd,0xf1,0xd0,
  0xa5,0xf5,0xe3,0xc2,0xc2,0x8b,0xa1,0xd2,0xfc,0xf7,0x69,0xe9,0xa7,0xb5,0xee,0xb5,
  0xb5,0xb5,0xb5,0xa7,0x44,0xe9,0xd1,0xb0,0xd2,0xf5,0xd1,0xee,0x86,0x86,0xeb,0xeb,
  0xff,0xfd,0xfe,0xff,0xfb,0xcb,0xeb,0x86,0x63,0x68,0xf5,0xed,0xfe,0xfe,0xf4,0xe8,
  0xff,0xe8,0xfe,0xfe,0xff,0xf5,0x63,0x45,0xf7,0xae,0xcb,0xf0,0xed,0xfd,0xfd,0xff,
  0xeb,0xcb,0xae,0x86,0x86,0xc2,0xf5,0xb0,0xb6,0xf7,0xc2,0xe9,0xb5,0xb5,0xb5,0xb5,
  0xee,0xa7,0xa7,0xa7,0xe9,0xf9,0xe9,0x91,0xf7,0xf0,0x8b,0xeb,0xd2,0xd2,0xed,0xe6,
  0xff,0xfe,0xfd,0xed,0xfb,0xed,0xf4,0xf8,0xf5,0xf6,0xa1,0xf4,0xf3,0xfd,0xed,0xe6,
  0xe6,0xed,0xf2,0xf3,0xf2,0xeb,0x63,0xfa,0xb0,0xf8,0xed,0xfb,0xed,0xf2,0xfe,0xff,
  0xe6,0xed,0xd3,0xd2,0xeb,0x8b,0xf7,0xf7,0x91,0xf9,0xf9,0xe9,0xa7,0xb5,0xb5,0xb5,
  0xb5,0xa7,0xa7,0xe3,0x44,0xe9,0xf9,0xc2,0xf7,0xf6,0xf5,0xed,0xfc,0xf1,0xed,0xe6,
  0xff,0xfe,0xfd,0xed,0xfb,0xed,0xf1,0xf1,0xeb,0xf6,0xf5,0xf2,0xf3,0xf2,0xed,0xf0,
  0xf0,0xe6,0xe8,0xf3,0xf2,0xeb,0xfa,0xeb,0xf1,0xf1,0xff,0xfb,0xed,0xf2,0xfe,0xff,
  0xe6,0xed,0xf1,0xfc,0xed,0xf5,0xf6,0xf7,0x6b,0xc2,0xc2,0xe9,0xe3,0xa7,0xa7,0xa7,
  0xb5,0xb5,0xb5,0xe3,0x44,0xf9,0xf9,0xc2,0xf7,0xf7,0x8b,0xeb,0xb0,0xd2,0xed,0xe6,
  0xff,0xfd,0xfd,0xed,0xfb,0xe7,0xd3,0xb0,0xf5,0x63,0xf5,0xe8,0xf3,0xfd,0xff,0xe6,
  0xe6,0xed,0xf2,0xf3,0xf4,0xeb,0x63,0xf7,0xae,0xf8,0xe7,0xfb,0xed,0xf2,0xfe,0xff,
  0xe6,0xed,0xf8,0xb6,0xeb,0x8b,0xf6,0x86,0x6b,0xe9,0xf9,0xe9,0xa7,0xa7,0xb5,0xa7,
  0xb5,0xb5,0xb5,0xa7,0xe9,0xf9,0x6b,0x91,0xb0,0xf5,0x6b,0x6b,0x86,0xf7,0xeb,0xf0,
  0xed,0xfd,0xfe,0xff,0xfb,0xcb,0xeb,0xf7,0x69,0x69,0xfa,0xd0,0xdc,0xf3,0xf2,0xe8,
  0xe8,0xf2,0xfe,0xfe,0xed,0xf5,0x69,0xd1,0x86,0xad,0xeb,0xcb,0xff,0xfd,0xfd,0xff,
  0xeb,0xeb,0xae,0x86,0xd1,0xc2,0xf5,0xb0,0x91,0xf7,0xe9,0x44,0xa7,0xa7,0xa7,0xb5,
  0xa7,0xa7,0xb5,0xa7,0xe9,0xf9,0x86,0x86,0xb6,0xeb,0x8b,0xc2,0xee,0xc2,0x8b,0xf6,
  0xeb,0xf8,0xfe,0xff,0xa1,0xfa,0x8b,0x69,0x69,0xc2,0xd1,0xa1,0xff,0xfe,0xf3,0xfe,
  0xfe,0xf3,0xfe,0xe8,0xeb,0x8b,0x69,0x69,0x69,0x69,0xf6,0xa5,0xd0,0xfe,0xf1,0xeb,
  0xf6,0xad,0xc2,0xee,0xee,0x8b,0xeb,0xb0,0x91,0xf7,0x44,0xe9,0xa7,0xa7,0xa7,0xb5,
  0xe9,0xe9,0xf9,0xe9,0xf9,0xc2,0x86,0xf7,0xd2,0xd0,0xf6,0xc2,0xc2,0xc2,0xc2,0x6b,
  0xf6,0xeb,0xf8,0xae,0xfa,0x69,0x8b,0x8b,0x86,0x86,0x86,0xf5,0xeb,0xff,0xf1,0xfe,
  0xfe,0xf1,0xff,0xeb,0xfa,0x8b,0xad,0x86,0x86,0x86,0x8b,0x8b,0xf5,0xf8,0xeb,0xf6,
  0xc2,0x8a,0xee,0xe3,0x6b,0xf6,0xd0,0xd2,0x91,0xf7,0x44,0xf9,0xf9,0xf9,0xf9,0xf9,
  0xe9,0xf9,0xf9,0xc2,0x69,0x6b,0x86,0xf7,0xd2,0xe8,0xf5,0x8b,0xd1,0xc2,0x69,0xc2,
  0x44,0xc2,0x86,0xae,0xeb,0xa5,0xa1,0xeb,0xf8,0xf8,0xd0,0xf5,0x86,0xf5,0xeb,0xd0,
  0xd0,0xeb,0xf5,0x8b,0xfa,0xeb,0xf8,0xf8,0xd0,0xeb,0xa1,0xa1,0xae,0xf7,0x69,0x46,
  0xe3,0xe3,0xc2,0x69,0x8b,0xf5,0xff,0xb6,0x86,0xf7,0x6b,0x69,0x69,0xf9,0xf9,0xe9,
  0xee,0xf9,0xd1,0x86,0x86,0x86,0xad,0xb2,0xd2,0xf8,0xd0,0xf6,0xd1,0xb5,0xe9,0xf9,
  0x69,0x69,0x86,0xd2,0xe8,0xe6,0xfb,0xe7,0xe8,0xfe,0xf1,0xeb,0xee,0x68,0x8b,0xf6,
  0xfa,0x8b,0xd1,0xd1,0xf0,0xf8,0xfe,0xf4,0xed,0xfb,0xe6,0xe8,0xfd,0xae,0x69,0xe9,
  0x8a,0xc2,0xee,0xc2,0xfa,0xd0,0xe8,0xb6,0xb2,0xf7,0xf7,0x86,0xf7,0xf7,0xe9,0xf9,
  0xf9,0x6b,0x91,0x91,0x91,0xb2,0xae,0xae,0xd2,0xd2,0xff,0xf5,0x8b,0x69,0xe9,0xc2,
  0xc2,0x69,0xeb,0xf1,0xfd,0xff,0xe7,0xe6,0xff,0xf4,0xfd,0xae,0xd1,0x69,0x69,0xf5,
  0xf5,0x8b,0xee,0xd1,0x86,0xf1,0xf4,0xff,0xe6,0xe6,0xff,0xfd,0xfd,0xeb,0x8b,0xe9,
  0x8a,0xc2,0xc2,0x8b,0xf5,0xed,0xd2,0xd2,0xae,0xae,0xb2,0x91,0xb0,0x91,0x6b,0xf9,
  0xf7,0xf7,0xb6,0xfc,0xfc,0xfc,0xd2,0xf8,0xdc,0xfc,0xe8,0xe6,0xfa,0x69,0xb5,0xc2,
  0xc2,0x69,0xa1,0xf4,0xfe,0xf1,0xff,0xe6,0xe6,0xed,0xff,0x86,0xd1,0x69,0xf7,0xd0,
  0xd6,0x86,0xd1,0xd1,0x86,0xd0,0xed,0xe6,0xe6,0xed,0xf1,0xfe,0xf1,0xeb,0x8b,0x8a,
  0xb5,0xee,0xc2,0xfa,0xeb,0xff,0xfc,0xfc,0xf8,0xd2,0xfc,0xfc,0xfc,0xb6,0xf7,0xf7,
  0xf6,0x89,0xf5,0xeb,0xd6,0xd3,0xf1,0xd2,0xdc,0xf3,0xf1,0xed,0xf5,0x8b,0xc2,0xc2,
  0xc2,0x8b,0xa1,0xff,0xf1,0xfe,0xf4,0xed,0xe6,0xfb,0xfb,0xaa,0xd1,0x8b,0xae,0xfd,
  0xfd,0xf8,0xf7,0x8b,0x8b,0xeb,0xfb,0xe6,0xed,0xf4,0xfe,0xfd,0xff,0xcc,0xaa,0x8a,
  0xee,0xc2,0x8b,0xf5,0xed,0xf1,0xf3,0xfc,0xd2,0xf1,0xf8,0xff,0xae,0xf5,0xf6,0x89,
  0xfa,0x69,0x6b,0x8b,0xfa,0xa1,0xd0,0xff,0xe8,0xfc,0xfc,0xd3,0xcf,0xf5,0xc2,0xd1,
  0xc2,0xf5,0xeb,0xe6,0xff,0xfd,0xfe,0xf4,0xff,0xe7,0xeb,0x8b,0xd1,0xa1,0xd0,0xff,
  0xff,0xed,0xa1,0x8b,0x86,0xa1,0xe6,0xff,0xf4,0xfe,0xfd,0xe8,0xe6,0xcc,0xf0,0xd1,
  0xc2,0xc2,0xf5,0xf0,0xe8,0xfc,0xfc,0xf4,0xff,0xe6,0xeb,0xfa,0x89,0x6b,0x6b,0xf6,
  0xd0,0xf5,0xc2,0x6b,0xc2,0x8b,0xf6,0xf5,0xd0,0xd3,0xfc,0xd2,0xd3,0xf0,0x86,0xc2,
  0xc2,0xf7,0xd0,0xfb,0xe7,0xff,0xfd,0xfe,0xf2,0xe8,0xeb,0x8b,0x8b,0xa1,0xfb,0xe6,
  0xe6,0xfb,0xeb,0xf5,0x8b,0xeb,0xff,0xf2,0xfe,0xfd,0xe8,0xe7,0xe6,0xed,0xae,0xee,
  0xee,0xd1,0xf5,0xed,0xf1,0xfc,0xf1,0xe6,0xf5,0xf6,0x8b,0x6b,0xd1,0xd1,0x86,0xae,
  0xfc,0xb0,0xd1,0xd1,0x6b,0xc2,0xd1,0x69,0xf6,0xeb,0xf8,0xfc,0xd3,0xcf,0xf0,0xc2,
  0x6b,0xb2,0xe8,0xed,0xe6,0xe7,0xff,0xf4,0xfe,0xfe,0xf8,0xf5,0xa1,0xe6,0xe7,0xe7,
  0xe7,0xe7,0xe7,0xcb,0xf5,0xae,0xfd,0xfe,0xf4,0xff,0xe6,0xe6,0xed,0xf1,0xb0,0x86,
  0xc2,0x86,0xcf,0xd3,0xfc,0xf8,0xeb,0xfa,0x8b,0xd1,0xc2,0xc2,0xc2,0xd1,0xf7,0xd2,
  0xfd,0xf8,0x86,0xd1,0xee,0xc2,0xc2,0xf9,0xee,0xd1,0xad,0xb0,0xb0,0xd3,0xaa,0x6b,
  0x86,0xf8,0xf1,0xe8,0xed,0xfb,0xe6,0xe6,0xd0,0xf8,0xb0,0xb2,0xf1,0xf1,0xe8,0xe8,
  0xe8,0xf4,0xf1,0xf1,0xb2,0xb0,0xae,0xd0,0xe6,0xe6,0xcc,0xed,0xf1,0xfc,0xd2,0xae,
  0xc2,0xaa,0xd3,0xf8,0xb0,0xae,0x8b,0xc2,0xee,0xc2,0xc2,0xd1,0xc2,0xf7,0xb0,0xd2,
  0xff,0xed,0xeb,0xd1,0xb5,0xee,0xc2,0xc2,0xee,0xa7,0xb5,0xee,0x91,0xae,0xf0,0x8b,
  0xf5,0xf8,0xd2,0xd2,0xd0,0xeb,0xa5,0xf6,0xfa,0x86,0x69,0xf7,0xd2,0xfe,0xfe,0xfe,
  0xfe,0xfe,0xfe,0xf1,0xae,0x69,0x8b,0x8b,0xf6,0xf6,0xeb,0xf8,0xd2,0xd2,0xd3,0xf0,
  0x8b,0xf0,0xf8,0x91,0x91,0xc2,0xc2,0xb5,0xc2,0xc2,0xc2,0xc2,0xd1,0xeb,0xd0,0xff,
  0xe6,0xe6,0xeb,0xf6,0x69,0xf9,0xb5,0xa7,0xee,0xb5,0xe3,0xa7,0xee,0xc2,0x86,0xfa,
  0xaa,0xcf,0xf0,0xf7,0xf7,0xf7,0xd1,0xc2,0xc2,0x69,0x69,0x8b,0xeb,0xff,0xf4,0xf4,
  0xe8,0xe8,0xff,0xeb,0xf7,0x69,0x69,0x69,0xee,0xe3,0xd1,0x86,0xf7,0xb0,0xad,0xaa,
  0xf0,0xf5,0x86,0xee,0xc2,0xb5,0xc2,0xc2,0xee,0xf9,0xe9,0x69,0xf6,0xa1,0xfb,0xe6,
  0xff,0xff,0xed,0xa1,0x8b,0x69,0x44,0xf9,0xe3,0xe3,0xee,0xc2,0x6b,0x86,0xf7,0xf5,
  0xaa,0x8b,0xd1,0x69,0x69,0xee,0xd1,0xc2,0xee,0xee,0xee,0x6b,0xf6,0xf0,0xe6,0xe6,
  0xe6,0xe6,0xf0,0xf6,0x69,0xc2,0xee,0xa7,0xe3,0xb5,0xb5,0xee,0xee,0xc2,0x86,0xaa,
  0xf8,0x86,0x86,0x6b,0xee,0xb5,0xee,0xc2,0xe9,0x44,0x44,0x69,0xf5,0xe6,0xff,0xff,
  0xfe,0xfd,0xf1,0xd6,0xf5,0x69,0x69,0x69,0x63,0xd1,0xad,0x86,0xb0,0xb0,0xb6,0xb0,
  0xf5,0xf5,0x86,0x6b,0xe9,0xee,0xee,0xc2,0xee,0xee,0xee,0xb5,0xd1,0xa1,0xe6,0xe6,
  0xe6,0xe6,0xeb,0x86,0xb5,0xa7,0xb5,0xf9,0xb5,0xee,0xc2,0xee,0xd1,0xfa,0xa1,0xf0,
  0x86,0xb6,0xb0,0xb0,0x86,0xf0,0x8b,0x69,0x69,0x69,0x69,0xf7,0xeb,0xf1,0xfd,0xfe,
  0xfd,0xfd,0xfe,0xfe,0xb2,0xf7,0xf7,0xf5,0xa1,0xa1,0xf0,0xed,0xd3,0xd2,0xb0,0x86,
  0xd1,0xf0,0xf0,0xf5,0xf7,0x69,0xc2,0xb5,0xee,0xf9,0xee,0xb5,0xee,0xad,0xd0,0xe8,
  0xe8,0xd3,0xae,0xd1,0xd1,0xb5,0xa7,0xb5,0x69,0x6b,0x86,0xfa,0xaa,0xcf,0xf8,0x86,
  0x86,0xb0,0xd2,0xd3,0xed,0xf0,0xa1,0xa1,0xf5,0xf7,0xf7,0xae,0xfd,0xfe,0xfd,0xfd,
  0xed,0xed,0xff,0xff,0xae,0xae,0xd2,0xf1,0xe8,0xed,0xe6,0xe6,0xed,0xd3,0xb0,0x86,
  0xe9,0x91,0xb0,0xed,0xcf,0xf5,0xf6,0x69,0xe9,0xee,0xa7,0xb5,0xb5,0xd1,0xb0,0xfe,
  0xfe,0xb0,0x86,0xd1,0xc2,0xee,0xee,0x6b,0x8b,0xfa,0xf0,0xcf,0xd3,0xf8,0x86,0xc2,
  0xd1,0xb0,0xd3,0xed,0xe6,0xe6,0xed,0xe8,0xf1,0xd2,0xb0,0xae,0xd0,0xff,0xff,0xed,
  0xe6,0xfb,0xfb,0xa1,0xf5,0xeb,0xf4,0xfe,0xfe,0xf1,0xed,0xe6,0xe6,0xed,0xae,0x6b,
  0xb5,0x91,0xb6,0xfc,0xd3,0xf4,0xf0,0xf5,0xf6,0x86,0xd1,0xb5,0xa7,0xee,0x86,0xf8,
  0xd3,0xae,0x6b,0xc2,0xc2,0x6b,0xf6,0xf6,0xa1,0xe6,0xd3,0xfc,0xfc,0xb6,0x86,0xee,
  0xee,0xf7,0xed,0xe6,0xe6,0xed,0xe8,0xfe,0xfe,0xf1,0xd0,0xf5,0xa1,0xcb,0xfb,0xfb,
  0xed,0xe7,0xeb,0xfa,0x8b,0xeb,0xed,0xe8,0xfd,0xfe,0xf1,0xed,0xe6,0xcf,0xf0,0xc2,
  0xc2,0xc2,0xae,0xd2,0xfc,0xfc,0xf4,0xff,0xd0,0xf5,0xfa,0x8b,0xc2,0xee,0x8b,0xeb,
  0xeb,0x8b,0xc2,0x69,0xf6,0xfa,0xeb,0xe6,0xff,0xf4,0xfc,0xfc,0xd2,0xae,0xd1,0xa7,
  0xee,0xf0,0xf0,0xe6,0xed,0xf4,0xfe,0xfe,0xe8,0xed,0xcb,0xf6,0xf6,0xa1,0xe6,0xed,
  0xfd,0xd6,0xad,0x8b,0x8b,0xeb,0xfb,0xe7,0xe8,0xfd,0xfe,0xf1,0xed,0xf0,0xaa,0xe3,
  0xee,0xc2,0x86,0xeb,0xd2,0xf3,0xf3,0xdc,0xf1,0xe8,0xd6,0xeb,0xa1,0xf7,0x6b,0xf6,
  0xf6,0x69,0xf6,0xfa,0xd0,0xd0,0xd6,0xf4,0xf1,0xf3,0xfe,0xf1,0xd0,0xf5,0xee,0xa7,
  0xee,0x8b,0xa1,0xed,0xe8,0xfe,0xfe,0xe8,0xed,0xfb,0xeb,0xf5,0x63,0xf5,0xd0,0xfd,
  0xd2,0xb0,0xe3,0xd1,0xf0,0xd0,0xe7,0xfb,0xe7,0xe8,0xfd,0xfe,0xe8,0xeb,0x86,0xa7,
  0xa7,0xb5,0x6b,0xf5,0xe6,0xf4,0xfc,0xf3,0xf3,0xf3,0xdc,0xd2,0xf8,0xb2,0xae,0x89,
  0x89,0x8e,0xae,0xf8,0xd2,0xd2,0xfc,0xdc,0xfc,0xfc,0xf4,0xed,0xf5,0xd1,0xc2,0xee,
  0xc2,0x63,0xa1,0xe8,0xfe,0xfe,0xe8,0xed,0xfb,0xe7,0xe6,0xf5,0x68,0x68,0xb0,0xf1,
  0xeb,0xf5,0xe3,0x69,0x86,0xf8,0xff,0xed,0xfb,0xe7,0xe8,0xfe,0xf1,0xeb,0xd1,0xee,
  0xe3,0xee,0xc2,0x8b,0xf5,0xff,0xd2,0xd2,0xd2,0xdc,0xf3,0xf3,0xf3,0xfc,0xb0,0x6b,
  0xc2,0x91,0xb6,0xb6,0xb6,0xb0,0xb0,0xb2,0xf8,0xd2,0xff,0xeb,0x8b,0xee,0xb5,0xc2,
  0x69,0x45,0xf5,0xf1,0xfe,0xe8,0xed,0xfb,0xe7,0xff,0xf8,0x86,0x69,0x45,0x86,0xeb,
  0xf5,0x68,0xe3,0x69,0x86,0xf1,0xfd,0xe8,0xe7,0xfb,0xed,0xf4,0xfe,0xae,0x69,0xd1,
  0xa7,0xf9,0xc2,0xd1,0xfa,0xd0,0xf4,0xdc,0xf4,0xae,0xae,0xae,0xb2,0x92,0x6b,0xee,
  0xf9,0x6b,0xf7,0xf7,0xf7,0x8e,0xae,0xae,0xd2,0xf8,0xed,0xf6,0xd1,0xee,0xc2,0xe9,
  0x45,0x69,0xad,0xd2,0xf1,0xed,0xfb,0xe7,0xff,0xfd,0xf1,0xae,0x69,0x45,0xf7,0xf6,
  0xeb,0xf5,0x86,0x8b,0xf0,0xff,0xfd,0xf1,0xed,0xcb,0xcb,0xd0,0xf8,0xf7,0x69,0xc2,
  0xee,0xe3,0xee,0xc2,0x8b,0xeb,0xe8,0xf3,0xd2,0xf7,0x86,0xd1,0xf7,0x6b,0xc2,0xe9,
  0xe9,0xe9,0xc2,0xc2,0x6b,0xd1,0x86,0xae,0xd2,0xe8,0xeb,0xf6,0xc2,0xc2,0xc2,0xe9,
  0x45,0xe3,0xf7,0xb0,0xd0,0xcb,0xcb,0xe6,0xf1,0xfd,0xe8,0xa1,0x6b,0xf6,0xf5,0xeb,
  0xf2,0xe8,0xd0,0xf5,0xfa,0xa1,0xeb,0xb0,0xae,0xf5,0xaa,0xf0,0xae,0xae,0x89,0x69,
  0xc2,0x8a,0xd1,0xd1,0xd1,0xfa,0xd6,0xfc,0xdc,0xf7,0xd1,0xee,0xee,0xc2,0xf9,0xe9,
  0xe9,0xe9,0x44,0xf9,0xee,0xc2,0xf7,0xf7,0xd2,0xd6,0xfa,0x69,0xc2,0xd1,0xc2,0x69,
  0x45,0x86,0xf7,0xae,0xf5,0xfa,0xf5,0xf5,0xb0,0xae,0xeb,0xf6,0xf5,0xeb,0xff,0xf4,
  0xf3,0xf3,0xf1,0xed,0xf5,0x8b,0x6b,0xc2,0xd1,0xd1,0x8b,0xf5,0xeb,0xd2,0xf8,0xa1,
  0x89,0xb3,0xd1,0xd1,0xd1,0x8b,0xd0,0xdc,0xdc,0x8e,0xd1,0xee,0xb5,0xa7,0xee,0xb5,
  0xb5,0xb5,0xa7,0xa7,0xf9,0x69,0xf7,0x91,0xd2,0xd0,0xf6,0xc2,0x69,0x6b,0x69,0xf6,
  0xf6,0xeb,0xd2,0xd0,0xf5,0x8b,0x69,0x69,0x69,0x69,0x69,0xf5,0xd0,0xf1,0xf3,0xf3,
  0xf2,0xfe,0xf3,0xfd,0xd0,0xfa,0x69,0x69,0x69,0xf7,0xf5,0xeb,0xed,0xfd,0xf1,0xe6,
  0xa1,0xf0,0x86,0xd1,0xc2,0x69,0xf5,0xd2,0xdc,0xf7,0x69,0xc2,0xee,0xa7,0xb5,0xb5,
  0xb5,0xa7,0xb5,0xa7,0xe9,0xe9,0xf7,0x91,0xd2,0xf5,0x8b,0xee,0xc2,0xf7,0xf5,0xf5,
  0xd0,0xf1,0xfe,0xff,0xeb,0xa1,0xf6,0x69,0x69,0x69,0xf6,0xeb,0xf1,0xf3,0xfe,0xf2,
  0xed,0xff,0xfd,0xf3,0xe8,0xf5,0x63,0x63,0xf7,0xae,0xe6,0xfb,0xed,0xfd,0xfe,0xff,
  0xe6,0xd0,0xb2,0x86,0xf7,0x6b,0xf7,0xb0,0xb6,0x6b,0xee,0xe9,0xee,0xb5,0xb5,0xb5,
  0xee,0xb5,0xb5,0xa7,0xf9,0xe9,0x44,0x6b,0x86,0xf6,0xc2,0xf7,0xf7,0xb2,0xd0,0xfb,
  0xff,0xfd,0xfd,0xed,0xfb,0xe6,0xeb,0xf7,0x63,0x45,0xf5,0xff,0xf3,0xfe,0xe8,0xed,
  0xf0,0xe6,0xf4,0xf3,0xf1,0xeb,0x89,0xf5,0xf8,0xf1,0xff,0xfb,0xed,0xf2,0xfe,0xff,
  0xe6,0xed,0xf1,0xfc,0xae,0xfa,0x89,0xf7,0x6b,0xc2,0xc2,0xe9,0xee,0xa7,0xa7,0xb5,
  0xb5,0xb5,0xa7,0xa7,0xf9,0xee,0xf9,0xe9,0xf7,0x89,0xf6,0xae,0xd2,0xf1,0xff,0xe6,
  0xff,0xfe,0xfd,0xed,0xfb,0xed,0xf1,0xf8,0xeb,0xf6,0xf5,0xf4,0xf3,0xf2,0xed,0xf0,
  // 7: ground sticker
  0x2c,0x63,0x4d,0x6b,0x08,0x42,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,0x00,0x01,
  0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,0x00,0x03,
  0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,0x00,0x05,
  0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,
  0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,
  0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,
  0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,
  0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,
  0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,
  0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,
  0x10,0x43,0x18,0x43,0x8c,0x73,0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,
  0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0xaa,0x5a,0x08,0x46,0x10,0x46,0x18,0x46,
  0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0xae,0x7b,0x10,0x60,
  0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x11,0x84,
  0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,
  0x10,0x64,0x6d,0x73,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x69,0x52,0x18,0x65,
  0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,
  0x00,0x80,0x08,0x80,0x10,0x80,0x2b,0x6b,0x18,0x80,0xc7,0x41,0x00,0x81,0x08,0x81,
  0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,
  0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,0x6e,0x6b,0x18,0x84,0x00,0x85,0x08,0x85,
  0x10,0x85,0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,
  0x10,0x87,0xeb,0x62,0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,
  0xcb,0x5a,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,0x08,0xa2,0xec,0x62,0x11,0xa5,
  0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x00,0xa4,0x08,0xa4,
  0x10,0xa4,0x6e,0x73,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x18,0xa5,0x48,0x4a,0x00,0xa6,
  0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,0x00,0xc0,
  0x08,0xc0,0x10,0xc0,0x18,0xc0,0xed,0x83,0x00,0xc1,0xef,0x7b,0x08,0xc1,0x07,0x4a,
  0x10,0xc1,0x6f,0x94,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0x2d,0x73,
  0x00,0xc3,0xed,0x7b,0xa6,0x39,0x08,0xc3,0x10,0xc3,0xeb,0x72,0x18,0xc3,0x00,0xc4,
  0x8a,0x62,0xec,0x5a,0x8f,0x94,0xc7,0x39,0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,
  0xb0,0x9c,0x0d,0x5b,0x08,0xc5,0x10,0xc5,0xd4,0x8c,0x16,0x95,0xb4,0x8c,0x56,0x95,
  0x08,0x52,0xc8,0x39,0xc7,0x51,0x6a,0x42,0x97,0x9d,0x89,0x52,0xa7,0x41,0x2e,0x8c,
  0xf1,0xf1,0xf1,0x01,0x00,0x00,0x00,0x00,0xf1,0xf1,0x00,0x01,0x01,0x00,0x01,0x01,
  0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x00,0x00,
  0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x00,0xb6,0xb6,0x01,0xb6,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xb6,0xb6,0xb6,0xb6,0x00,0xb6,0x00,0x00,0x00,0xb6,0x00,
  0xc1,0x6f,0x7e,0x01,0x01,0x01,0x01,0x01,0x9c,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0xc1,0xc1,0xc1,0x9c,0x66,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0x66,0xc1,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xc1,0x01,0xc1,0x01,0x01,0x01,0x01,0x01,
  0x00,0x01,0x01,0x01,0x00,0x01,0x00,0x00,0xb0,0xb6,0x00,0x00,0x00,0x6f,0x7e,0x00,
  0x01,0x66,0xb0,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0xc1,0x01,0xc1,0x01,0xc1,0x01,0x01,0xc1,0xc1,0x01,0xc1,0x01,
  0xc1,0x01,0x01,0x00,0x00,0x01,0x01,0x00,0x01,0x00,0x00,0x00,0xb6,0x00,0x00,0x01,
  0x01,0x00,0xb6,0xb6,0x00,0x00,0x00,0x00,0x00,0xb0,0xb0,0x00,0x00,0x66,0xfd,0x00,
  0x01,0x01,0x01,0x5c,0xc6,0xfb,0x7e,0xfd,0xfb,0x7e,0xfb,0x7e,0x7e,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xfd,0xfd,0x5c,0x7e,0x7e,0x7e,0xfd,0xfb,0xfd,0x7e,0xfd,0xb0,0x7e,
  0x7e,0xfb,0xfb,0xc6,0xc6,0x02,0x02,0xfb,0x7e,0xfb,0x7e,0xc6,0x7e,0xc6,0x02,0x02,
  0x02,0xc6,0x02,0xc6,0xc6,0x7e,0xfb,0x7e,0x7e,0x02,0x7e,0xfd,0xb6,0x00,0x00,0x00,
  0x01,0x01,0x00,0x02,0x6f,0x6f,0xd9,0xea,0xf4,0x6f,0xff,0xd9,0xf0,0xb7,0xf4,0xf0,
  0xea,0xd9,0xd9,0x6f,0xd9,0x6f,0xf6,0xea,0xd9,0xd5,0x00,0xa9,0xb6,0x00,0x00,0xe9,
  0xe9,0x00,0x00,0x00,0xdf,0x66,0x6f,0xd9,0xd9,0xd9,0xd9,0x6f,0xd9,0x6f,0xd9,0xd9,
  0xd9,0xd9,0xd9,0xd9,0xea,0xea,0xf0,0xea,0x6f,0x6f,0x6f,0x02,0x5c,0x00,0x01,0x00,
  0x01,0x9c,0x00,0x02,0x6f,0xf7,0xf7,0xf7,0xf7,0xb7,0xb7,0xb7,0xb7,0xf7,0xf7,0xc5,
  0xb7,0xc5,0xf6,0xf0,0xf7,0xc5,0xf4,0x6f,0x01,0x00,0x01,0x01,0xe9,0xb0,0xb0,0xe9,
  0xb0,0xb0,0xb0,0x00,0x9c,0x66,0x9c,0xd5,0xf6,0xc5,0xfc,0xfc,0xb7,0xf4,0xb7,0xb7,
  0xf0,0xf6,0xf4,0xf0,0xf0,0xf4,0xb7,0xc5,0xf7,0xc5,0xf6,0x02,0x5c,0x00,0x00,0x00,
  0x01,0xc1,0x01,0x02,0x6f,0xf4,0xb7,0xc5,0xf7,0xc5,0xb7,0xb7,0xc5,0xb7,0xb7,0xf0,
  0xb7,0xb7,0x6f,0xd9,0x6f,0x00,0x9c,0x01,0x00,0xfd,0x02,0x8d,0x02,0x8d,0xe2,0xeb,
  0x8d,0x02,0x8d,0xe2,0x02,0xfd,0xb0,0x00,0x9c,0x9c,0x6f,0xb7,0xc5,0xb7,0xb7,0xb7,
  0xb7,0xb7,0xb7,0xb7,0xf0,0xb7,0xb7,0xc5,0xc5,0xf6,0xd5,0x8d,0x5c,0x00,0x00,0x01,
  0x9c,0xc1,0x01,0x02,0x6f,0x6f,0x6f,0xb7,0xf7,0xb7,0xb7,0xb7,0xb7,0xb7,0xb7,0xf0,
  0xb7,0xf0,0x66,0x00,0x9c,0x00,0x5c,0xc6,0xc6,0xe2,0xe2,0xe2,0xb0,0xb0,0x02,0xeb,
  0xe2,0x02,0xeb,0x8d,0xeb,0x02,0x7e,0xb0,0x00,0xe9,0x9c,0xd5,0x6f,0xea,0xf0,0xf0,
  0xb7,0xb7,0xb7,0xf0,0xf0,0xf4,0xb7,0xf0,0xd9,0xf6,0xc1,0x8d,0xb0,0x01,0x9c,0x01,
  0x9c,0x01,0x01,0x02,0x6f,0x6f,0x6f,0x6f,0xb7,0xf0,0xf0,0xf0,0xf0,0xf7,0xb7,0xf0,
  0xd9,0x01,0x01,0x00,0xfd,0x8d,0xe2,0x8d,0xeb,0xfb,0xf1,0x6f,0x6f,0x6f,0xd5,0x6f,
  0xd5,0xd5,0xd5,0x66,0x00,0xe9,0xb0,0xb0,0xb0,0xfd,0xe9,0x01,0x9c,0xd5,0x6f,0xf4,
  0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0x6f,0xd5,0x6f,0x79,0x8d,0xb0,0x01,0x9c,0x01,
  0xc1,0x9c,0x01,0x02,0x66,0x66,0x66,0xd5,0x6f,0x6f,0x6f,0xc0,0xf0,0xb7,0xc5,0xd5,
  0x00,0x01,0x5c,0xc6,0xe2,0xe2,0x02,0x66,0xf4,0xf4,0xd5,0x52,0x52,0x8b,0x79,0x8b,
  0x8b,0x79,0x8b,0x66,0x66,0xd5,0x66,0xd5,0xf1,0xfb,0xb0,0x00,0x00,0x9c,0xd5,0xd5,
  0xd9,0x6f,0xd9,0x6f,0xd9,0x6f,0x6f,0xd5,0x6f,0x6f,0xdf,0x8d,0xe9,0x00,0x9c,0x01,
  0xc1,0xc1,0x01,0x02,0xd5,0xd5,0xd5,0x66,0xd5,0x66,0xd9,0xf4,0xc5,0xf0,0x9c,0x01,
  0xa9,0xc6,0xc6,0xc6,0xfb,0xf6,0xf7,0x6f,0x52,0xa9,0x52,0x8b,0x8b,0x66,0x52,0x52,
  0x52,0x52,0x79,0x8b,0xa9,0x8b,0x73,0x66,0x66,0x6f,0x9c,0xb0,0x00,0xb0,0x00,0x9c,
  0xd5,0x6f,0xb7,0xf7,0xf0,0x66,0x66,0xd5,0x66,0x6f,0xdf,0xfe,0xe9,0x01,0x9c,0x01,
  0xc1,0xc1,0x01,0x02,0xd5,0x6f,0x6f,0x66,0x66,0x6f,0xf7,0x66,0xd9,0xc5,0x00,0xb0,
  0x8d,0x8d,0xd7,0x78,0xf7,0x66,0x52,0x52,0x8b,0x66,0xd3,0xff,0xd9,0xea,0xea,0xf0,
  0xf0,0xf0,0xb7,0xf0,0xf0,0xff,0x66,0x73,0x01,0x00,0xd5,0xc1,0xb0,0xfd,0x7e,0x01,
  0x01,0xf7,0x6f,0x79,0xb7,0xd5,0x66,0xd5,0x6f,0xd5,0xc1,0x02,0xe9,0x9c,0x01,0x01,
  0xc1,0x01,0x01,0x02,0x66,0x6f,0xc0,0x66,0xd5,0xd9,0xd5,0xf8,0xb6,0xdf,0x5c,0x8d,
  0x8d,0xc6,0xfb,0xf4,0x8b,0x52,0x52,0xd3,0xd3,0xff,0xd3,0xff,0xd9,0xd3,0xea,0xff,
  0xd9,0xf0,0xd9,0xb7,0xb7,0xb7,0xb7,0xd9,0xd3,0x8b,0x01,0x52,0xa9,0x5c,0x02,0xd7,
  0x00,0xf0,0xb6,0x01,0xe8,0x79,0x66,0x66,0xf6,0x6f,0x79,0xc6,0xe9,0x01,0x9c,0x9c,
  0xc1,0xc1,0x9c,0xc6,0x66,0x6f,0xc0,0xd9,0x66,0xd5,0x52,0xfd,0xe2,0xfd,0xf8,0x8d,
  0xe2,0xe2,0x9c,0x8b,0x52,0x52,0x52,0xe1,0xd3,0x52,0xd3,0x52,0x8b,0x8b,0x8b,0x8b,
  0x8b,0x8b,0x8b,0x8b,0x52,0xea,0xea,0xf3,0xb7,0xb7,0x52,0x8b,0xb0,0x02,0xc6,0x8d,
  0x02,0xd5,0xf8,0xe2,0xb0,0xc1,0x66,0xd5,0x6f,0x6f,0xdf,0xeb,0xe9,0x01,0x9c,0x01,
  0xc1,0xc1,0xc1,0x02,0x66,0xd5,0x6f,0xd5,0x66,0x66,0x79,0xfd,0xc6,0xfd,0xf8,0xf8,
  0xe2,0x8d,0x00,0x52,0xd3,0x52,0xd3,0xe1,0x8b,0x66,0x8b,0xe1,0xd3,0xd3,0xd9,0xd9,
  0xf0,0xd9,0xf0,0xff,0x52,0x8b,0x66,0x52,0xb7,0xf3,0xb7,0xd9,0xfd,0xe2,0x8d,0x8d,
  0x8d,0x7e,0xe8,0xc6,0xb0,0x66,0x66,0xd5,0xd5,0xd5,0x00,0xeb,0xe9,0x9c,0x9c,0xc1,
  0xc1,0xc1,0x01,0x02,0x66,0xd5,0xd5,0xd5,0x66,0x79,0xc1,0x00,0x7e,0x7e,0xf8,0x5c,
  0x02,0xd7,0xfd,0x79,0x52,0xe1,0x8b,0x8b,0xe1,0xe1,0xd3,0xd3,0xff,0xd9,0xea,0xea,
  0xf0,0xb7,0xb7,0xf3,0xb7,0xf0,0xd9,0x79,0xa9,0xd3,0xf0,0x01,0x8d,0x8d,0xd7,0x8d,
  0xe2,0x8d,0xfd,0xa9,0xc1,0x6f,0x66,0x66,0x66,0x66,0x00,0xeb,0xe9,0x9c,0x01,0xc1,
  0xc1,0x9c,0x01,0x02,0x66,0xd5,0xd5,0xd5,0x66,0x79,0x79,0x5c,0xf8,0x7e,0x7e,0x7e,
  0xfd,0x7e,0xc6,0x5c,0x66,0x8b,0xe1,0xd3,0x52,0xd3,0xd3,0xe1,0x66,0x8b,0x66,0x8b,
  0x8b,0xe1,0x52,0xd9,0xf0,0xf3,0xb7,0xf3,0xd3,0x01,0x01,0x7e,0x8d,0x8d,0xd7,0xd7,
  0x8d,0xc6,0xc6,0xa9,0xa9,0xd5,0x6f,0x66,0x66,0x66,0x00,0x8d,0xe9,0x01,0x01,0xc1,
  0xc1,0xc1,0xc1,0x02,0x66,0xd5,0xd5,0xd5,0x79,0x79,0xe8,0xc6,0x8d,0x02,0xc6,0x7e,
  0x7e,0x7e,0x7e,0xd7,0xfd,0x79,0x66,0x66,0x79,0xe1,0x01,0x52,0x66,0x66,0xd3,0xff,
  0xff,0xd3,0xe1,0x52,0x8b,0x52,0xb7,0xea,0xb7,0x79,0xa9,0xc6,0xc6,0x8d,0xd7,0xd7,
  0xc6,0x02,0x5c,0xb0,0x00,0x66,0x6f,0x6f,0x66,0x66,0xdf,0xeb,0xe9,0x00,0x01,0xc1,
  0xc1,0xc1,0x9c,0x02,0x66,0xd5,0xd5,0x66,0x79,0xb0,0x7e,0x8d,0x8d,0xc6,0xc6,0xfb,
  0x7e,0x7e,0xc6,0xc6,0xd7,0xc6,0x01,0x52,0x8b,0x8b,0x52,0xd3,0xd3,0xd9,0xff,0xff,
  0xd9,0xd9,0xd9,0xff,0xd9,0x52,0xe8,0x8b,0x66,0x01,0xb0,0xc6,0x8d,0x8d,0xc6,0x5c,
  0xc6,0x7e,0xb0,0xb0,0x00,0x01,0xd5,0x6f,0x66,0x66,0x79,0x02,0xe9,0xf1,0x01,0xc1,
  0xc1,0xc1,0x9c,0x02,0x66,0xd5,0xd5,0x01,0x00,0x7e,0xfd,0xc6,0xb0,0xb0,0xc6,0xc6,
  0xc6,0xc6,0xfd,0xc6,0x8d,0xe2,0xc6,0x66,0xd3,0xd9,0xd3,0xd9,0xd9,0xff,0xd9,0xd9,
  0xd9,0xea,0xd9,0xf0,0xd9,0xea,0xd9,0x01,0x00,0x5c,0xfd,0x02,0x02,0x02,0x7e,0xfd,
  0xf1,0xf6,0x66,0x66,0x00,0xa9,0xe9,0x6f,0x6f,0x66,0xdf,0x8d,0xe9,0x01,0x01,0xc1,
  0xc1,0xc1,0x9c,0x02,0x66,0xd5,0xc1,0x66,0x5c,0xc6,0xa9,0x6f,0x00,0x52,0x8b,0xfd,
  0xc6,0x02,0xd7,0xc6,0xe2,0xe2,0xe2,0x8d,0xd5,0xd3,0xff,0xd3,0xd3,0xff,0xd3,0xff,
  0xd3,0xff,0xff,0xd9,0xff,0xd9,0x79,0x00,0xfd,0xfd,0x7e,0x02,0x02,0xc6,0x7e,0x9c,
  0xb7,0xd3,0xa9,0xa9,0xd5,0x00,0xfb,0x9c,0x6f,0x66,0xdf,0x8d,0xe9,0x01,0x01,0xc1,
  0xc1,0xc1,0x9c,0x02,0x66,0xd5,0x01,0x01,0xfd,0x7e,0xd5,0x73,0x66,0x52,0xd3,0x00,
  0x5c,0x02,0x8d,0x02,0xeb,0x8d,0xe2,0xe2,0x02,0xd5,0xd3,0xff,0xd3,0xd3,0xd3,0x52,
  0xd3,0xff,0xd3,0xff,0xd9,0x66,0x01,0xb0,0x5c,0xfd,0xc6,0x02,0x02,0xc6,0x78,0xf6,
  0xd9,0xf0,0x52,0x00,0xe1,0x01,0xb0,0xe9,0x6f,0x6f,0x00,0x8d,0xe9,0x01,0x01,0xc1,
  0xc1,0xc1,0x9c,0x02,0x66,0x66,0xc1,0x00,0x5c,0xb6,0x6f,0x79,0x52,0x52,0x66,0x66,
  0x00,0x5c,0xc6,0x02,0x02,0x02,0xeb,0xe2,0xe2,0xc6,0x66,0x52,0x52,0x52,0x8b,0x52,
  0x8b,0x8b,0x79,0x66,0x01,0xb6,0x5c,0x5c,0xfd,0xfb,0x7e,0xfd,0xfb,0x9c,0xf7,0x00,
  0xd3,0xea,0xd9,0x8b,0x01,0x66,0xb0,0xfd,0x78,0xf6,0x00,0x02,0xb6,0x01,0x9c,0xc1,
  0xc1,0xc1,0xc1,0x02,0x66,0x01,0x01,0xb0,0x5c,0x6f,0x01,0x52,0x8b,0x52,0x52,0x52,
  0x52,0x8b,0x5c,0x8d,0xe2,0x02,0x02,0x8d,0xe2,0xe2,0xc6,0x01,0x52,0x52,0x52,0xd3,
  0x52,0x66,0x8b,0xa9,0x7e,0x7e,0xb0,0x7e,0xfd,0x7e,0xfd,0xfd,0x78,0xf5,0xea,0x52,
  0xe8,0xea,0xea,0x52,0x5c,0x01,0x00,0xb0,0x9c,0xf4,0x00,0x02,0xf1,0x9c,0x01,0xc1,
  0xc1,0xc1,0x01,0x02,0x66,0x01,0x01,0x5c,0x5c,0x66,0x52,0x8b,0x79,0x8b,0x52,0x66,
  0x8b,0x66,0xa9,0xb6,0x02,0x8d,0x8d,0x8d,0x02,0x8d,0xd7,0xc6,0x66,0x52,0x52,0x8b,
  0x8b,0xd3,0x00,0xc6,0xd7,0x8d,0xc6,0x02,0xc6,0x5c,0xb0,0x9c,0xf4,0x52,0xf0,0xd9,
  0x79,0x52,0xf0,0xff,0x5c,0x79,0x00,0xfb,0x9c,0xf4,0xd5,0x02,0x00,0x9c,0x01,0xc1,
  0xc1,0xc1,0xc1,0x02,0x79,0x66,0x00,0x5c,0x00,0xd5,0x52,0x79,0xa9,0x52,0x52,0x8b,
  0x79,0x8b,0x8b,0x00,0xb0,0x02,0x02,0xeb,0xc6,0xc6,0x8d,0xe2,0x8d,0xd7,0x8d,0xe2,
  0xe2,0xe2,0xd7,0x8d,0xc6,0x8d,0xd7,0x7e,0xfd,0xb0,0x78,0xf4,0xff,0xa9,0xd3,0xf0,
  0x52,0x00,0xd9,0xd9,0x79,0xfd,0x79,0x02,0xfd,0x6f,0x6f,0x02,0xf1,0x9c,0x9c,0xc1,
  0xc1,0xc1,0x01,0x02,0x01,0xd5,0x5c,0xb0,0x9c,0x52,0x66,0xe8,0x79,0x52,0x01,0x79,
  0x8b,0x66,0x8b,0x8b,0x01,0xb0,0xc6,0x7e,0x7e,0xc6,0xd7,0xd7,0xd7,0xd7,0xd7,0x8d,
  0x8d,0x8d,0x8d,0xc6,0xd7,0x02,0x7e,0xfd,0xa9,0x9c,0xf4,0xd9,0xff,0x52,0x79,0xd9,
  0xff,0x5c,0xff,0xd9,0x52,0x01,0x8b,0xc6,0x02,0x6f,0xf4,0x8d,0xf1,0x9c,0x01,0xc1,
  0xc1,0xc1,0x01,0x8d,0x01,0xc1,0x5c,0xb0,0x66,0x66,0x52,0x01,0x01,0x66,0x52,0x8b,
  0x66,0x52,0x8b,0x79,0x8b,0x00,0x5c,0xc6,0xfd,0xc6,0xd7,0xd7,0x8d,0xd7,0xd7,0x8d,
  0xd7,0xd7,0xd7,0xd7,0xd7,0xc6,0x7e,0x5c,0x9c,0xf4,0xd3,0xd9,0xff,0xff,0x5c,0xd3,
  0xea,0x01,0x52,0xd9,0xd3,0x5c,0x52,0xf1,0x8d,0x6f,0xf4,0xfe,0x00,0x9c,0xc1,0xc1,
  0xc1,0xc1,0x9c,0x02,0x79,0xc1,0x00,0xe9,0x66,0x52,0x52,0xa9,0x52,0xd3,0x8b,0xa9,
  0xe1,0x52,0x00,0x00,0x01,0x8b,0x01,0x5c,0xc6,0xc6,0xd7,0xc6,0x8d,0xe2,0xe2,0xe2,
  0xe2,0xd7,0xd7,0x8d,0xc6,0xc6,0x7e,0x00,0xf4,0x8b,0xd3,0xd3,0xff,0xd3,0x00,0xd3,
  0xd3,0x52,0x52,0xff,0xff,0x00,0x00,0x00,0x5c,0x9c,0xf4,0xfd,0xf1,0x9c,0x9c,0xc1,
  0xc1,0x66,0xc1,0x02,0xc1,0xc1,0x00,0xe9,0x66,0x52,0x52,0xa9,0xe1,0x52,0xa9,0x01,
  0x52,0x01,0xa9,0xa9,0x52,0x52,0xd3,0xa9,0xc6,0x8d,0xd7,0x8d,0xe2,0xeb,0x00,0x01,
  0x5c,0xe2,0x8d,0xd7,0x8d,0xd7,0x7e,0xf6,0x52,0xa9,0x52,0xff,0xd3,0xff,0x52,0x8b,
  0xff,0x52,0xa9,0xff,0xd3,0x01,0xb0,0x01,0xfd,0x5c,0x6f,0xe9,0x00,0x01,0x01,0xc1,
  0xc1,0xc1,0xc1,0x8d,0xc1,0x01,0xb0,0x00,0xe1,0xe1,0xa9,0x79,0x52,0x52,0x8b,0x8b,
  0xd3,0xa9,0x8b,0x52,0x8b,0xd3,0xe1,0x52,0x7e,0x8d,0xd7,0x8d,0x8d,0x00,0xe8,0xd7,
  0x6e,0x00,0xe2,0x8d,0xd7,0x8d,0xc6,0xe1,0x52,0xa9,0xd3,0xd3,0xff,0xd3,0x52,0x79,
  0xd3,0xd3,0xa9,0xd3,0xff,0x8b,0xa9,0x01,0x5c,0xfd,0x6f,0x9c,0xe9,0x9c,0x9c,0xc1,
  0xc1,0xc1,0xc1,0x02,0xc1,0x01,0x00,0x01,0x66,0xd3,0x79,0x5c,0xd3,0xe1,0xa9,0x79,
  0xd3,0x79,0xa9,0xa9,0x66,0xe1,0xd3,0x00,0x7e,0xd7,0xd7,0xe2,0x00,0x7e,0xc6,0xd7,
  0x7e,0xd7,0xb0,0xe2,0xd7,0xd7,0x02,0x79,0xe1,0xa9,0x52,0xd9,0xd3,0xd3,0xe1,0xe8,
  0xd9,0xd3,0xa9,0xd3,0xd3,0x66,0x5c,0x52,0x5c,0xb0,0x6f,0x9c,0xe9,0x01,0x9c,0xc1,
  0xc1,0xc1,0xc1,0x8d,0xc1,0x01,0xb0,0x00,0xd3,0xd3,0xe8,0x79,0xd3,0x52,0x00,0x52,
  0xd3,0xa9,0x00,0x52,0x8b,0xff,0xd3,0x01,0xc6,0xd7,0xd7,0xe2,0x01,0xc6,0x7e,0x8b,
  0x7e,0xd7,0x01,0xe2,0xd7,0xc6,0xc6,0x66,0x52,0xa9,0x79,0xd3,0xff,0xd3,0x8b,0x66,
  0xff,0xd3,0xa9,0xd3,0xd3,0x8b,0x00,0x8b,0x5c,0xa9,0x6f,0x9c,0x00,0x9c,0x9c,0xc1,
  0xc1,0x66,0x9c,0x02,0xc1,0x00,0xa9,0x00,0xe1,0xd3,0x79,0x5c,0x66,0xe1,0xa9,0x01,
  0xd3,0x01,0xa9,0xa9,0x79,0xff,0xd3,0x8b,0x7e,0xc6,0xd7,0xe2,0x01,0xe8,0xe8,0x00,
  0xfd,0xd7,0xd5,0xe2,0xd7,0x8d,0xc6,0x66,0x79,0xa9,0x52,0x66,0xd3,0xd3,0xd3,0xa9,
  0xd3,0xd3,0xa9,0xe1,0xd3,0x01,0xb0,0x01,0x5c,0x01,0x6f,0x9c,0xf1,0x9c,0xc1,0xc1,
  0xc1,0xc1,0x01,0x02,0xc1,0x01,0xb0,0x00,0xe1,0xff,0x01,0xa9,0xd3,0x52,0xa9,0x52,
  0xd3,0x00,0x00,0x01,0x52,0xd3,0xff,0x79,0x00,0x7e,0xc6,0xd7,0xc6,0xc1,0xfd,0x8b,
  0x7e,0xf5,0x00,0x8d,0xd7,0xd7,0xc6,0x79,0x52,0x00,0x79,0xd3,0xe1,0xd3,0x8b,0x01,
  0xff,0x52,0x00,0xd3,0xd3,0x52,0xa9,0x01,0x00,0x9c,0x6f,0x00,0xf1,0x9c,0x9c,0xc1,
  0xc1,0xc1,0x01,0x02,0x9c,0x01,0xb0,0xb6,0x66,0xff,0x8b,0x01,0x01,0xd3,0x00,0x5c,
  0xd9,0x8b,0xa9,0x01,0xa9,0x66,0xd9,0x79,0x5c,0x7e,0x7e,0x7e,0x7e,0xc6,0xc1,0x6f,
  0xca,0x9c,0x02,0x8d,0xd7,0x02,0xc6,0x8b,0xdf,0x8b,0x52,0x52,0xd3,0xe1,0x8b,0x66,
  0xe1,0xe1,0xa9,0xd3,0xff,0xa9,0x01,0x01,0x01,0x66,0xf6,0xe9,0x00,0x9c,0x9c,0xc1,
  0xc1,0xc1,0x01,0x02,0x9c,0x9c,0x00,0xb0,0x66,0xff,0x66,0x5c,0x66,0xff,0xa9,0x79,
  0xd3,0x79,0xa9,0xb0,0x66,0xa9,0x66,0xb0,0x5c,0x5c,0x7e,0x7e,0xd7,0x8d,0x8d,0xe2,
  0xe2,0xe2,0xc6,0xd7,0xd7,0xd7,0xd7,0x7e,0x00,0x52,0x8b,0x66,0x52,0xe1,0x01,0x52,
  0xff,0x8b,0x79,0xd3,0xe1,0x79,0xa9,0x01,0x01,0xd5,0xf4,0x02,0x01,0x9c,0x9c,0x9c,
  0xc1,0xc1,0x9c,0x02,0x01,0xc1,0xb0,0xfd,0x01,0xd9,0x52,0x79,0xa9,0xff,0xe1,0xe8,
  0xd5,0xd9,0xa9,0x01,0x5c,0x9c,0xa9,0x5c,0xfd,0x5c,0x5c,0x5c,0xd7,0x8d,0x02,0x8d,
  0xd7,0xe2,0xd7,0xc6,0xf8,0xf8,0xf8,0xf8,0xb0,0x00,0x79,0x52,0x66,0x52,0xa9,0x66,
  0xd3,0x79,0x52,0xd3,0xd3,0xe8,0x79,0x01,0x66,0xd5,0xf6,0x8d,0x00,0x9c,0x9c,0x9c,
  0xc1,0xc1,0xc1,0x02,0xdf,0x66,0x01,0x00,0x01,0xff,0xd9,0x5c,0x00,0xd3,0xd3,0x00,
  0xa9,0xea,0x01,0xa9,0x01,0x00,0x5c,0x5c,0x5c,0xb0,0x00,0x5c,0x5c,0x7e,0xc6,0x02,
  0xc6,0x7e,0xfd,0x7e,0x7e,0x7e,0xc6,0xd7,0xe2,0x7e,0x00,0x66,0x52,0x01,0x52,0x52,
  0x66,0x8b,0xd3,0xd3,0x8b,0x00,0x00,0xc1,0xd5,0x6f,0x6f,0xd7,0x01,0xc1,0xc1,0xc1,
  0xc1,0xc1,0x9c,0x02,0x79,0x66,0x9c,0xb0,0xe9,0xd3,0xea,0x52,0x01,0x52,0xf0,0x52,
  0xa9,0xd9,0xd3,0x01,0x5c,0x7e,0x5c,0xfd,0x5c,0xa9,0x00,0x00,0xb0,0x5c,0x5c,0x5c,
  0x7e,0xb0,0x5c,0x5c,0x7e,0x7e,0x7e,0xd7,0xe2,0xf8,0xc6,0x01,0x79,0x01,0xe1,0xe1,
  0x66,0x01,0xd3,0xd3,0x8b,0x00,0x01,0x66,0x9c,0x6f,0x6f,0x02,0xf1,0x9c,0xc1,0x9c,
  0xc1,0xc1,0x9c,0x02,0x66,0x66,0x00,0xe9,0x00,0x66,0xff,0xd3,0x5c,0xa9,0xd9,0x52,
  0x8b,0x52,0xff,0xb0,0x02,0xd7,0xfd,0xfd,0xfd,0xa9,0xb0,0x00,0xf4,0x66,0xc1,0xd5,
  0x6f,0x6f,0x79,0x5c,0x5c,0xd7,0xc6,0xc6,0xc6,0xc6,0x8d,0x7e,0x00,0xa9,0x52,0x52,
  0x73,0x52,0x52,0x8b,0xa9,0x8b,0x01,0x01,0x66,0xf6,0xc1,0x02,0xf1,0xc1,0xc1,0x9c,
  0xc1,0xc1,0xc1,0x02,0x79,0x66,0x66,0x00,0xe9,0x01,0xff,0xd9,0xc1,0xa9,0xe1,0xf0,
  0x8b,0xc1,0x00,0xc6,0xf8,0xc6,0xc6,0x7e,0x7e,0xb0,0x00,0xf7,0xb7,0xb7,0xf0,0xd9,
  0xff,0xd9,0xd3,0x79,0x5c,0x5c,0xfd,0xc6,0xd7,0x5c,0x7e,0xc6,0xb0,0x79,0xd3,0x01,
  0x79,0x52,0xe1,0x79,0xa9,0x66,0xc1,0xc1,0xd5,0xf4,0xb6,0x02,0x00,0x9c,0xc1,0xc1,
  0xc1,0xc1,0xc1,0xc6,0x66,0x66,0xd5,0x00,0xb0,0x00,0x52,0xea,0xff,0xa9,0x01,0xf0,
  0xd5,0x00,0xc6,0xc6,0xfd,0x5c,0x5c,0xc6,0x7e,0x00,0xf4,0x8b,0x66,0xd9,0xd9,0xea,
  0xff,0xd3,0xff,0x8b,0x01,0x00,0xb0,0x7e,0xc6,0xa9,0xa9,0xfd,0xb0,0xa9,0x01,0x79,
  0x52,0xe1,0x52,0xa9,0x66,0x00,0x01,0x66,0x6f,0xf6,0xb6,0x02,0xf1,0x9c,0xc1,0x9c,
  0xc1,0xc1,0xc1,0xc6,0x79,0xd5,0x66,0xc1,0xb0,0x5c,0x00,0xd9,0xd9,0x52,0x00,0x79,
  0x5c,0xf8,0x5c,0x02,0xd7,0xc6,0x7e,0x5c,0xc1,0xf5,0x8b,0x52,0x79,0x8b,0x79,0x01,
  0x01,0x79,0xa9,0x01,0x8b,0xd5,0x9c,0x5c,0x00,0x00,0x5c,0xb0,0xb0,0xb0,0x00,0x01,
  0x66,0x52,0x01,0x01,0xa9,0x9c,0xc1,0xc1,0xf4,0x66,0xe5,0x02,0x00,0x9c,0x9c,0xc1,
  0xc1,0xc1,0xc1,0xc6,0x79,0x66,0x66,0xd5,0x01,0x7e,0x02,0x01,0xb7,0xd3,0x01,0x5c,
  0xd7,0xd7,0xc6,0x8d,0x8d,0xd7,0xd7,0x00,0xf4,0x00,0x01,0x52,0x52,0x52,0x52,0x52,
  0x52,0x52,0x01,0x52,0x00,0x8b,0xd5,0x66,0xc1,0x01,0x7e,0xfb,0xfb,0xfb,0xfd,0x01,
  0x79,0x52,0x01,0x00,0xb0,0x00,0x01,0xd5,0xf4,0x79,0xe5,0x02,0x00,0x9c,0xc1,0x9c,
  0xc1,0xc1,0xc1,0xc6,0x66,0x66,0x66,0xd5,0x66,0x5c,0xc6,0xd7,0xfd,0x00,0x5c,0xc6,
  0xd7,0xd7,0xe2,0xe2,0xf8,0x8d,0xb0,0xca,0xea,0x8b,0x00,0x8b,0x8b,0x52,0x8b,0x66,
  0x52,0x8b,0x66,0xa9,0x8b,0x52,0x8b,0xd5,0xd5,0x01,0xb0,0xb0,0xfb,0xfb,0x02,0xc6,
  0xb0,0x00,0xb0,0x5c,0x02,0x00,0x66,0xf5,0x66,0xdf,0x8b,0x02,0x00,0xc1,0x66,0xc1,
  0xc1,0x66,0xc1,0xc6,0x79,0x66,0x66,0x66,0xd5,0x01,0xa9,0xf8,0xd7,0xc6,0xc6,0xf8,
  0xd7,0xe2,0xe2,0xe2,0xd7,0xb0,0xf4,0xff,0xb7,0xb7,0xb7,0x66,0x52,0x8b,0x66,0x8b,
  0x8b,0x66,0xa9,0x66,0x52,0x79,0x66,0x52,0x66,0x66,0xb0,0xb0,0xfb,0xfb,0xc6,0x02,
  0xc6,0x7e,0xb0,0xfd,0xf1,0x01,0x6f,0xf4,0x79,0xdf,0xe5,0xd7,0xf1,0x9c,0xc1,0x9c,
  0x9c,0xc1,0x9c,0xc6,0x79,0x66,0x66,0x66,0xd5,0xd5,0x00,0x7e,0xd7,0xd7,0xc6,0xf8,
  0xe2,0xe2,0xfd,0x7e,0x00,0xf7,0x8b,0x8b,0x79,0x52,0xb7,0xf3,0xc9,0xb7,0xf3,0xf0,
  0xea,0xea,0xd5,0xd3,0x66,0xe1,0x52,0x66,0x52,0xd5,0x9c,0xb0,0xb0,0xb0,0xc6,0x7e,
  0xfd,0x5c,0xb0,0x5c,0xa9,0xd5,0xf7,0x79,0x79,0xdf,0xb6,0x02,0x00,0x9c,0x9c,0x9c,
  0xc1,0xc1,0x9c,0xc6,0x79,0x66,0xd5,0xd5,0x66,0xd5,0xd5,0xb6,0xf8,0xc6,0xf8,0xd7,
  0xc6,0xe2,0xd7,0xb6,0xf4,0xb7,0xb7,0xd9,0x00,0x52,0x8b,0x52,0x79,0xd3,0xd3,0xd9,
  0xd3,0xd3,0x52,0x79,0x52,0x52,0x52,0x52,0x66,0x01,0xd5,0x9c,0xb0,0xfd,0x7e,0xfd,
  0x00,0x00,0x00,0xb0,0x79,0xf5,0x6f,0x79,0x79,0x79,0xb6,0x02,0x00,0x9c,0xc1,0x9c,
  0xc1,0x66,0xc1,0xc6,0x66,0x66,0x66,0xd5,0xd5,0xd5,0x6f,0x9c,0x5c,0x7e,0xc6,0xd7,
  0xd7,0xd7,0xf8,0x6f,0x8b,0x01,0xd9,0xf3,0xb7,0xd9,0xd3,0x8b,0x52,0x52,0x52,0x52,
  0x79,0x52,0x52,0x66,0x52,0x66,0xe1,0x79,0x52,0x66,0x52,0x66,0xe9,0xc6,0xc6,0x7e,
  0xfd,0x5c,0x00,0xc1,0xf6,0xf5,0x79,0x79,0xdf,0x79,0xb6,0x02,0x00,0x9c,0x9c,0x9c,
  0x9c,0xc1,0xc1,0x02,0x79,0x66,0x66,0xd5,0xd5,0x66,0xf0,0xb7,0xfc,0x6f,0xc6,0xc6,
  0xc6,0x8d,0x5c,0xea,0x8b,0x00,0xa9,0xa9,0xea,0xd9,0xf3,0xb7,0xb7,0xff,0xf0,0xd3,
  0xd3,0xff,0xd3,0xe1,0xe1,0xe1,0x8b,0x52,0x8b,0x8b,0x52,0x8b,0x00,0xfd,0xc6,0xa9,
  0xb0,0x6f,0xb7,0xf7,0xfc,0xdf,0xdf,0x66,0x79,0x79,0x8b,0xd7,0x00,0x9c,0x01,0xc1,
  0x01,0xc1,0xc1,0xc6,0x66,0x66,0xd5,0x6f,0xd5,0x6f,0xb7,0xdf,0x79,0xb7,0xa9,0x5c,
  0xfd,0xc6,0x00,0xca,0xfc,0xf0,0x52,0x52,0x8b,0x79,0x52,0xff,0xf0,0xd9,0xb7,0xf0,
  0xea,0xea,0x52,0xff,0x52,0x52,0x66,0x52,0x66,0x52,0x52,0x66,0x00,0xa9,0xa9,0xa9,
  0x01,0xb7,0x79,0xb6,0xf0,0x66,0xdf,0x79,0x79,0x79,0xe5,0xd7,0x00,0x9c,0x9c,0xc1,
  0x01,0xc1,0xc1,0xc6,0x66,0xff,0x6f,0x6f,0xd9,0xd9,0x66,0x00,0xa9,0xe8,0x00,0xa9,
  0xa9,0xb0,0xf8,0xff,0xca,0xca,0xca,0xd9,0x66,0x8b,0x79,0x8b,0x8b,0x66,0x8b,0x66,
  0x52,0x8b,0xd5,0x8b,0x52,0x66,0x8b,0x66,0x52,0x52,0x79,0x00,0xa9,0x5c,0x5c,0x00,
  0xc1,0xd9,0xf8,0xb6,0x7e,0xdf,0x79,0x79,0x66,0x66,0x8b,0xfe,0xf1,0x9c,0x9c,0x9c,
  0x01,0xc1,0xc1,0xc6,0x66,0x6f,0x6f,0xd5,0xd5,0x66,0xd3,0xd7,0xe2,0xf1,0x6f,0x66,
  0xa9,0x7e,0x7e,0xfd,0x00,0xea,0xca,0xca,0xca,0xb7,0xea,0xd3,0x52,0x8b,0x79,0x8b,
  0x8b,0x66,0x8b,0x66,0x52,0xe1,0x66,0x52,0x66,0x52,0x8b,0xa9,0x5c,0xfd,0x7e,0x01,
  0xf4,0xd9,0xf8,0xe2,0xd7,0xdf,0x79,0x79,0x66,0xdf,0xb6,0x8d,0x00,0x01,0x01,0x01,
  0x01,0xc1,0xc1,0xc6,0x66,0xd5,0xd5,0xd5,0x66,0x66,0x00,0x5c,0x5c,0xe5,0x79,0xf6,
  0xd5,0x01,0xe8,0xe8,0x5c,0xa9,0xb0,0xd9,0xb7,0xce,0xca,0xca,0xca,0xb7,0xb7,0xb7,
  0xf0,0xf0,0xd5,0xea,0xff,0xd5,0xd5,0x79,0x01,0x7e,0xc6,0xd7,0x7e,0x7e,0x66,0xfc,
  0xf5,0x8b,0x8b,0x5c,0x5c,0xe5,0xdf,0x79,0x66,0xdf,0xdf,0xd7,0xb6,0x9c,0x01,0x9c,
  0x01,0x9c,0x9c,0xc6,0x66,0xd5,0xd5,0xd5,0x66,0xdf,0xdf,0xe5,0xe5,0xe5,0xe5,0xdf,
  0x6f,0x6f,0x79,0x00,0xb0,0xa9,0xfd,0x5c,0x00,0x01,0xd3,0xf0,0xd9,0xc9,0xf0,0xb7,
  0xf0,0xff,0xd9,0x52,0x52,0x8b,0x00,0x66,0x00,0xeb,0xb0,0xe9,0x66,0xf4,0xca,0x6f,
  0x79,0xdf,0xe5,0xe5,0xe5,0xe5,0xdf,0x79,0x79,0x79,0xdf,0x02,0x00,0x01,0x9c,0x01,
  0x01,0x9c,0x01,0xc6,0x66,0xd5,0x6f,0x66,0xdf,0xb6,0xe5,0xe5,0x8b,0xe5,0xdf,0xdf,
  0xe5,0xc1,0x6f,0x6f,0xd5,0x79,0x7e,0x5c,0x7e,0x5c,0xa9,0x6f,0x9c,0x66,0x01,0x01,
  0x00,0x79,0x00,0xfd,0xe9,0xe9,0x00,0x00,0xb0,0xb0,0x66,0x6f,0xca,0xfc,0x79,0xdf,
  0x79,0x79,0xdf,0xe5,0xdf,0xe5,0xe5,0xdf,0x66,0x66,0x00,0x02,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0x02,0x79,0xd5,0x66,0xdf,0xb6,0x8b,0x8b,0x8b,0x8b,0xe5,0xe5,0xdf,
  0xdf,0xdf,0x8b,0x79,0x6f,0x6f,0xd5,0xc1,0x01,0x5c,0x5c,0x01,0x01,0x5c,0xc6,0x02,
  0xc6,0xb0,0x5c,0x9c,0xa9,0xa9,0xa9,0x6f,0xfb,0x6f,0xca,0xfc,0x79,0xdf,0xdf,0xdf,
  0x00,0xdf,0xdf,0xe5,0xe5,0xdf,0x8b,0x8b,0xdf,0x79,0xdf,0x02,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0x02,0xdf,0xdf,0xe8,0xe8,0xe8,0x5c,0xe8,0x5c,0xe8,0x8b,0x8b,0x8b,
  0xe5,0x8b,0xe8,0xe8,0xe8,0x00,0xd5,0xf6,0xf6,0x6f,0xd5,0xd5,0xd5,0x79,0xa9,0xb0,
  0xf1,0x00,0x9c,0x6f,0x6f,0x6f,0xf5,0xca,0xfc,0xf5,0xdf,0x8b,0x8b,0x8b,0xe5,0xe5,
  0xa9,0xe8,0x00,0xa9,0xe8,0xe8,0xe8,0xe8,0xa9,0xb6,0x00,0x02,0x00,0x01,0x01,0x01,
  0x01,0x00,0xb6,0xc6,0xa9,0xfa,0xe8,0x02,0x7e,0x7e,0x7e,0x7e,0x7e,0x7e,0x7e,0xe8,
  0x7e,0x7e,0x7e,0x7e,0x7e,0x7e,0x7e,0x7e,0xf1,0xd5,0x6f,0xf6,0xf5,0xf5,0xf5,0xf4,
  0xf5,0xf7,0xca,0xca,0xfc,0xf7,0xf4,0x01,0x7e,0x5c,0xe8,0xa9,0xe8,0xe8,0xe8,0x7e,
  0x7e,0x7e,0x5c,0x7e,0x5c,0x7e,0x6e,0x6e,0xf8,0xf8,0x7e,0x02,0x00,0x01,0x01,0x01,
  0x01,0x01,0x00,0xfb,0x02,0x02,0xf9,0xeb,0xeb,0xf9,0xf9,0xeb,0xf9,0xeb,0xeb,0x02,
  0xe2,0xfe,0x8d,0xe2,0xe2,0xe2,0xe2,0xe2,0xeb,0xe2,0xe2,0xeb,0xe2,0xfb,0xf1,0xf1,
  0xf1,0xf1,0xfb,0xe2,0xe2,0xe2,0xeb,0xe2,0xeb,0xeb,0xeb,0xeb,0xeb,0xe2,0xe2,0xeb,
  0x02,0xeb,0x02,0x02,0xeb,0xeb,0xeb,0xeb,0xeb,0xeb,0xeb,0x02,0xb0,0x00,0x01,0x01,
  0x66,0x9c,0xb0,0xf1,0xf1,0xf1,0xf1,0x00,0xb6,0x00,0x00,0xb0,0xb0,0xb6,0xb0,0xb6,
  0xa9,0xb6,0xb6,0x00,0xb6,0xa9,0xb0,0xb0,0xb0,0xb6,0xb6,0xb6,0xb6,0x00,0xb6,0xb6,
  0xb0,0xe9,0xb6,0xb6,0xb6,0xb6,0xb0,0xb0,0xb6,0xb0,0xb6,0xb6,0x00,0x00,0xb6,0x00,
  0x00,0x00,0xb6,0x00,0xb6,0x00,0x00,0x00,0x00,0x00,0x00,0xb6,0xb0,0xd5,0x5c,0x01,
  0xc1,0xa9,0xfd,0xf1,0x01,0xf1,0xf1,0x00,0xf1,0x00,0xb6,0x00,0xb6,0x00,0xb6,0xb6,
  0xb6,0xb6,0x00,0x00,0xb6,0xe9,0xb0,0xb0,0xb0,0xb6,0x00,0x00,0x00,0xe9,0xb6,0xb6,
  0x00,0xe9,0x00,0x00,0xb6,0xb0,0xb6,0xb6,0xb6,0x00,0xb6,0xb6,0x00,0x00,0xb6,0x00,
  0x00,0xb6,0xb6,0x00,0x00,0x00,0xb6,0x00,0x00,0x00,0x00,0x00,0xb6,0x66,0xc6,0x01,
  0x01,0x00,0x00,0x01,0x00,0xb6,0x00,0x00,0x00,0x00,0xb6,0xe9,0xb6,0xe9,0xb6,0xb6,
  0xb6,0xb6,0xb6,0xb6,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0xb0,0x00,0xb6,0xe9,0xb0,0xb0,
  0xb0,0xe9,0xe9,0x00,0xb6,0xb6,0xb6,0xb0,0xb0,0xb0,0xe9,0xb6,0xb6,0xb6,0xb6,0x00,
  0xb6,0xb6,0xb6,0xb6,0xb6,0x00,0x00,0x00,0x00,0x00,0x00,0xf1,0xf1,0xf1,0x00,0x00,
  // 8: sky 1, part A
  0xde,0xc6,0xba,0x5c,0x7a,0x54,0x7a,0x4c,0x00,0x00,0x3f,0xd7,0x08,0x00,0x10,0x00,
  0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0xbe,0xbe,
  0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,
  0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,
  0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,
  0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,
  0x10,0x22,0xfb,0x6c,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,
  0x08,0x24,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,
  0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,
  0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,
  0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,
  0x08,0x44,0xbc,0x8d,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x9d,0xb6,
  0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,0xfd,0x9d,0x18,0x46,0x7b,0x7d,0x00,0x47,
  0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,0x5f,0xd7,0x18,0x60,
  0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,
  0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,
  0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,
  0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,
  0x00,0x81,0x08,0x81,0x5c,0xae,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,
  0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,
  0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,
  0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x3b,0x6d,0x18,0x87,0x00,0xa0,0x08,0xa0,
  0x10,0xa0,0x18,0xa0,0x9c,0x7d,0x3f,0xcf,0x00,0xa1,0x3c,0xa6,0x08,0xa1,0x10,0xa1,
  0x18,0xa1,0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,
  0x18,0xa3,0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,
  0x9f,0xe7,0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,
  0x10,0xa7,0xbe,0xc6,0x18,0xa7,0x1b,0x75,0x00,0xc0,0x7d,0xb6,0x08,0xc0,0x10,0xc0,
  0x18,0xc0,0x7e,0xb6,0x00,0xc1,0x08,0xc1,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,
  0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x18,0xc3,0x00,0xc4,0x08,0xc4,
  0x10,0xc4,0xfe,0xd6,0x7f,0xe7,0x18,0xc4,0x00,0xc5,0x08,0xc5,0x10,0xc5,0x18,0xc5,
  0x00,0xc6,0x08,0xc6,0x10,0xc6,0x18,0xc6,0x00,0xc7,0x08,0xc7,0x1d,0x96,0x10,0xc7,
  0x18,0xc7,0x08,0xe0,0x10,0xe0,0xdd,0xc6,0xdc,0x8d,0x00,0xe1,0x08,0xe1,0x00,0xe7,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,
  0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x02,0x03,0x03,0x03,0x02,
  0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x03,0x03,0x02,
  0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x03,0x02,0x03,0x03,
  0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x03,0x03,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x31,0x01,0x01,0x01,0x01,0x01,0x31,0x31,0x31,0x31,0x31,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x03,0x03,0x03,0x02,0x03,0x03,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x02,0x01,
  0x01,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x31,0xac,
  0xd3,0x31,0x31,0xac,0xac,0xac,0x31,0x31,0x31,0x31,0xac,0xac,0xd3,0xac,0xac,0x31,
  0x31,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x31,0x31,0x31,0x31,0x31,0xac,0xd3,0xd3,0x66,
  0x66,0xd3,0xac,0xd3,0xd3,0xac,0xac,0xd3,0xd3,0xd3,0xd3,0x66,0xb2,0x66,0xd3,0xac,
  0xac,0xd3,0xac,0x31,0x31,0x31,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x31,0x31,0x31,0x01,0x31,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x31,0x31,0x31,0xd3,0xac,0xac,0xac,0xac,0xd3,0xd3,0x66,0xb2,0x59,0x59,
  0xb2,0xb2,0x66,0x66,0x66,0x66,0xb2,0x59,0x59,0x59,0xb2,0x59,0x66,0xb2,0xb2,0xb2,
  0x66,0x66,0x66,0x66,0x66,0xd3,0x31,0x31,0x31,0x01,0x31,0x31,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x31,0x01,0x31,0x31,0x31,0x31,0xac,0xac,0xd3,0x01,0xd3,0xac,0xac,0xac,
  0x31,0x31,0x31,0x31,0x31,0x31,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x31,0x31,0xac,0xac,0xd3,0xac,0xd3,0xac,0xd3,0x66,0x66,0xb2,0x59,0x59,0xfc,0x59,
  0x59,0xfc,0xfc,0x59,0x59,0x59,0x64,0x64,0xfc,0xfc,0x59,0xb2,0xb2,0x59,0xfc,0xfc,
  0xfc,0xfc,0x59,0x59,0x59,0xb2,0x66,0xd3,0xd3,0xac,0xd3,0xac,0xd3,0x31,0x31,0xd3,
  0xd3,0xac,0xd3,0x31,0x01,0x01,0x01,0x31,0x31,0x01,0x01,0x31,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x31,0xac,0xac,0xac,0xac,0x31,0x31,0x31,
  0xd3,0xd3,0xd3,0xac,0xac,0xd3,0xac,0xd3,0x66,0x66,0xd3,0xd3,0x66,0xd3,0xd3,0xd3,
  0xd3,0xd3,0xd3,0xac,0xd3,0xd3,0xac,0xac,0x31,0x31,0x01,0x31,0x01,0x01,0x01,0x01,
  0x31,0xd3,0xd3,0xac,0xac,0xd3,0xd3,0xac,0xd3,0xb2,0x59,0xfc,0x59,0xb2,0xfc,0x64,
  0xf6,0x64,0xb5,0xf6,0xb5,0x92,0x92,0x92,0x92,0xf6,0xf6,0xfc,0xfc,0x64,0xf6,0xb5,
  0xb5,0x64,0x64,0x64,0x64,0xfc,0x59,0xb2,0xb2,0x66,0xd3,0x66,0xd3,0xac,0xd3,0xac,
  0xd3,0xd3,0xd3,0xac,0xd3,0xac,0xac,0xd3,0xd3,0xac,0xac,0xd3,0xac,0xac,0xd3,0xac,
  0x01,0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x31,0x01,0x31,0x31,0x31,0x31,
  0x31,0x31,0x31,0x01,0x31,0xac,0xac,0xac,0xd3,0xd3,0xd3,0xd3,0xd3,0xac,0xac,0xd3,
  0xd3,0x66,0x66,0x66,0x66,0x66,0xb2,0x59,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0x66,0xb2,
  0xb2,0xb2,0x66,0xb2,0x66,0x66,0x66,0xd3,0xd3,0xac,0xac,0xd3,0xac,0x01,0xac,0xac,
  0xd3,0x66,0xac,0xd3,0x66,0xd3,0xd3,0xb2,0x59,0x59,0x59,0x59,0xfc,0xfc,0xf6,0x92,
  0xb5,0x92,0xd5,0x92,0xd5,0x5f,0x5f,0x5f,0xd5,0x92,0xd5,0x92,0xb5,0xb5,0x92,0x5f,
  0xd5,0xb5,0xb5,0xb5,0xb5,0xb5,0xf6,0x64,0xfc,0x59,0x66,0x66,0x66,0x66,0x66,0x66,
  0xd3,0xd3,0x66,0x66,0xd3,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0xd3,0x66,0xd3,
  0xd3,0xd3,0xac,0xd3,0xd3,0xac,0xd3,0xac,0xd3,0xac,0x31,0xd3,0xac,0x31,0x01,0x31,
  0xac,0xd3,0xd3,0xd3,0xac,0xd3,0xd3,0x66,0x66,0x66,0x66,0x66,0x66,0xb2,0xb2,0x59,
  0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0x59,0x59,0xb2,0x59,0xfc,0x59,0xfc,0xfc,0x59,0x59,
  0x59,0xfc,0x59,0xfc,0xfc,0xfc,0x59,0x59,0xb2,0xb2,0xb2,0x66,0x66,0x66,0x66,0xd3,
  0xac,0xd3,0xd3,0xac,0x66,0x66,0xb2,0x59,0xb2,0x59,0x59,0x59,0xfc,0x64,0xb5,0xb5,
  0x92,0xd5,0x5f,0x5f,0x5f,0x0f,0xb3,0xb3,0x00,0xb3,0x00,0x00,0xd5,0xd5,0x5f,0x0f,
  0x5f,0xd9,0x92,0xd9,0xd9,0xd9,0x92,0xb5,0xb5,0xfc,0xb2,0x66,0x66,0xb2,0xb2,0xb2,
  0x66,0x66,0x66,0x66,0x66,0x66,0xb2,0xb2,0xb2,0xb2,0x66,0x66,0xb2,0x66,0x66,0xb2,
  0x66,0x66,0xd3,0x66,0xd3,0xd3,0xac,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0xac,0xac,0xd3,
  0x66,0xd3,0xac,0xd3,0x66,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0x59,0xfc,0xfc,0xfc,0x64,
  0x64,0x64,0xfc,0x59,0x64,0x64,0xfc,0xfc,0x64,0x64,0x64,0x64,0xb5,0xb5,0xb5,0xb5,
  0xb5,0xb5,0xf6,0xf6,0x64,0x64,0xfc,0x59,0xfc,0xfc,0xfc,0x59,0xb2,0xb2,0xb2,0xd3,
  0xb2,0x66,0x66,0xb2,0x59,0x59,0x59,0x59,0x59,0x59,0xfc,0x59,0x64,0x64,0xb5,0x92,
  0x92,0xd9,0x5f,0x0f,0xd1,0xb3,0x05,0x6e,0x6e,0x6e,0x05,0xb3,0x00,0x00,0x00,0x00,
  0x0f,0x0f,0x5f,0x5f,0xd5,0xd5,0xd5,0x92,0x92,0xb5,0xb5,0x64,0xb2,0xb2,0xb2,0xb2,
  0xb2,0x66,0x66,0xb2,0x66,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,
  0x66,0x66,0x66,0x66,0x66,0x66,0x66,0xd3,0xd3,0xd3,0x66,0x66,0x66,0x66,0x66,0x66,
  0xb2,0xb2,0x66,0xb2,0x59,0xfc,0xfc,0x59,0x59,0xfc,0xfc,0xfc,0xfc,0x64,0x64,0x64,
  0x64,0x64,0x64,0xb5,0xb5,0xfc,0x64,0xf6,0xb5,0xb5,0x92,0x92,0x92,0xd9,0x92,0x92,
  0x92,0x92,0x92,0xb5,0x92,0xb5,0xb5,0xb5,0xb5,0xb5,0xf6,0xb5,0x64,0xfc,0x59,0x59,
  0xfc,0xfc,0x59,0x59,0x59,0x59,0x59,0xfc,0x59,0x59,0xfc,0xfc,0xf6,0x64,0xb5,0x92,
  0x92,0xd5,0x0f,0x00,0xb3,0x05,0x6e,0x6e,0xea,0xea,0xea,0xe9,0xea,0x05,0xb3,0x00,
  0x00,0x00,0x00,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,0xd9,0xf6,0xb2,0x59,0xb2,0xb2,
  0xfc,0xb2,0xb2,0xb2,0x66,0xb2,0xb2,0xb2,0x59,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,
  0xb2,0xb2,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0x66,0xb2,0xb2,
  0x59,0x59,0x59,0x59,0x59,0x59,0x59,0x64,0x64,0x64,0x64,0x64,0x64,0xb5,0xb5,0xb5,
  0xb5,0x92,0x92,0x92,0x64,0xf6,0x92,0x92,0x92,0xd9,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,
  0x5f,0x5f,0x5f,0xd9,0x5f,0xd5,0x92,0xd9,0x92,0x92,0x92,0x92,0xb5,0xb5,0xfc,0xfc,
  0x64,0xfc,0xf6,0xf6,0x64,0xfc,0xf6,0xf6,0xf6,0xf6,0xfc,0x64,0xb5,0x92,0x5f,0x0f,
  0x0f,0xd1,0x00,0x00,0xb3,0x05,0x6e,0x6e,0xe9,0xea,0xea,0xc8,0xea,0x6e,0x05,0xb3,
  0xb3,0x00,0x00,0x00,0x00,0xd1,0x0f,0x0f,0xd1,0x0f,0x5f,0x64,0xfc,0x59,0xfc,0xfc,
  0x59,0xb2,0xb2,0xb2,0x59,0xb2,0x59,0x59,0x59,0x59,0x59,0x59,0x59,0xfc,0xfc,0xfc,
  0xfc,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,0x66,0xb2,0x66,0xb2,0xb2,0xb2,0x59,
  0xfc,0xfc,0xfc,0xfc,0x64,0x64,0xf6,0xb5,0xb5,0xb5,0xb5,0xb5,0x92,0x92,0x92,0x92,
  0x92,0x92,0x92,0xb5,0xb5,0x92,0xd9,0xd5,0x5f,0x5f,0x0f,0xd1,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x5f,0x5f,0x5f,0xd9,0xd5,0xd9,0xd9,0xd5,0xd9,0xd9,0x92,0x92,0xb5,0xb5,
  0xb5,0xb5,0x92,0x92,0xb5,0xb5,0xb5,0x92,0x92,0x92,0xd5,0xd5,0x0f,0x0f,0x00,0xb3,
  0x00,0x00,0xb3,0xb3,0xb3,0x05,0x05,0x6e,0xe9,0xc8,0xc8,0xc8,0xe9,0x6e,0x05,0x05,
  0xb3,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x00,0x00,0x0f,0x5f,0x92,0x64,0x59,0xf6,0xfc,
  0x59,0x64,0x59,0xfc,0x59,0x59,0x59,0x59,0x59,0x59,0x64,0xfc,0x59,0xfc,0xfc,0x59,
  0x59,0x59,0x59,0xfc,0x59,0x59,0x59,0xb2,0xb2,0x59,0x59,0xb2,0xfc,0x59,0x59,0xfc,
  0x64,0x64,0x64,0x64,0x64,0xb5,0xb5,0xb5,0x92,0x92,0x92,0x92,0xd9,0xd5,0x92,0x92,
  0xd9,0x92,0x92,0x92,0x92,0xd9,0x5f,0xd5,0x5f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x00,
  0x0f,0x0f,0x5f,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,0x0f,0x5f,0xd9,0xd9,0x92,0x92,
  0xd5,0xd5,0xd9,0xd5,0xd9,0x92,0x92,0x5f,0x0f,0x0f,0x00,0x00,0x00,0x00,0xb3,0xb3,
  0xb3,0xb3,0xb3,0x05,0x05,0x05,0x05,0x05,0x6e,0x6e,0xe9,0xe9,0x6e,0xe9,0x6e,0x6e,
  0x05,0x05,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x00,0x00,0xd1,0x5f,0x92,0xb5,0x92,0xb5,
  0x64,0x64,0x64,0xf6,0x64,0xfc,0xfc,0x64,0x64,0xb5,0xf6,0xb5,0xb5,0xb5,0x64,0xf6,
  0x64,0xfc,0x59,0x59,0xfc,0xfc,0x59,0x59,0xfc,0x59,0xfc,0x64,0x64,0x64,0x64,0xb5,
  0xb5,0xb5,0x92,0xb5,0x92,0x92,0x92,0x92,0x92,0xd9,0xd5,0xd5,0x5f,0x5f,0xd9,0x5f,
  0x5f,0x5f,0x5f,0x0f,0x0f,0x0f,0x5f,0x0f,0xd1,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x00,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0xd5,0xd5,0xd5,
  0x5f,0x5f,0x0f,0x0f,0x0f,0x5f,0x0f,0x0f,0x00,0xb3,0x05,0x05,0xb3,0xe9,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x6e,0x6e,0x6e,0x05,0x6e,0x6e,0x6e,0x6e,
  0x6e,0x05,0x05,0x05,0xb3,0xb3,0xb3,0xe9,0xe9,0xb3,0x00,0x0f,0x5f,0xd9,0xd5,0xd9,
  0x92,0xb5,0xb5,0x92,0x92,0x92,0xb5,0x64,0xb5,0x92,0x92,0x92,0x92,0xb5,0xb5,0xb5,
  0xf6,0xf6,0xf6,0xf6,0x64,0x59,0x59,0x64,0x59,0xfc,0xf6,0xb5,0xb5,0x92,0x92,0x92,
  0x92,0xd9,0xd5,0x5f,0x0f,0x5f,0x5f,0x5f,0x0f,0x0f,0x0f,0x0f,0x0f,0xd1,0x0f,0x0f,
  0x0f,0x0f,0x00,0x0f,0xd1,0xd1,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x00,0x0f,0x0f,0x0f,
  0x00,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,
  0x5f,0x0f,0x00,0x00,0x00,0xb3,0xb3,0x05,0xb3,0xb3,0xb3,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0xb3,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x6e,0x6e,0x6e,0x6e,0x05,
  0x05,0x05,0x05,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x0f,0x0f,0x0f,
  0x5f,0xd5,0xd5,0xd9,0xd5,0x92,0x92,0x92,0xd5,0x5f,0x0f,0x0f,0x5f,0xd5,0xd9,0x92,
  0x92,0xb5,0xb5,0xb5,0x64,0xfc,0x64,0x64,0x64,0x64,0x64,0xb5,0x92,0xd9,0xd5,0x5f,
  0x0f,0xd1,0x0f,0x0f,0x00,0x00,0xd1,0xd1,0xd1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0xb3,0x00,0x0f,0x0f,0x00,0x0f,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x00,0x0f,0x0f,0x0f,0x5f,0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0x0f,0x5f,
  0x0f,0x00,0x00,0xb3,0xb3,0xb3,0x05,0x05,0x05,0xb3,0x05,0x05,0x6e,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xe9,0x05,0xe9,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0xb3,0xb3,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,0x00,
  0x0f,0x0f,0x5f,0x0f,0x0f,0x5f,0xd5,0x5f,0x0f,0x00,0x0f,0x00,0x00,0x0f,0x0f,0x0f,
  0x0f,0xd9,0x92,0x92,0x92,0x92,0xb5,0x92,0xb5,0xb5,0xb5,0x92,0xd9,0x5f,0x0f,0x0f,
  0x0f,0xd1,0x00,0x00,0xb3,0x00,0x00,0x00,0x00,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x00,
  0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x00,0xb3,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x00,0x0f,
  0x00,0xd1,0x0f,0x00,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0xd9,0xd9,0x5f,0x5f,
  0x0f,0x00,0x00,0xb3,0xb3,0xb3,0x05,0x6e,0x6e,0x6e,0xea,0xe9,0x6e,0x6e,0x05,0x05,
  0x6e,0x05,0x05,0x6e,0x05,0x05,0x05,0x05,0x05,0xe9,0xe9,0xe9,0x05,0x05,0x05,0x05,
  0xe9,0xb3,0xe9,0x05,0x05,0xb3,0xb3,0xb3,0x00,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x0f,
  0x0f,0x0f,0x0f,0xd1,0x0f,0x0f,0x0f,0x00,0x00,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,
  0x00,0x00,0x0f,0x0f,0x0f,0x5f,0xd5,0xd5,0xd5,0xd5,0xd5,0xd9,0x5f,0x5f,0x0f,0x0f,
  0x0f,0x0f,0x00,0xb3,0x00,0xb3,0x00,0x00,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x0f,0x0f,0x00,0xb3,0xb3,0xb3,0xb3,0x00,0x0f,0xd1,0x0f,0x0f,0x0f,0x00,
  0x00,0x00,0xb3,0xb3,0x00,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0xd9,0xd5,0x5f,0x5f,0x0f,
  0x0f,0x0f,0x00,0x00,0x00,0xb3,0x05,0x6e,0xe9,0xea,0xea,0x6e,0x05,0x6e,0x6e,0x6e,
  0x6e,0x05,0x6e,0x6e,0x6e,0x05,0x6e,0x6e,0x05,0x05,0x05,0xe9,0xe9,0xb3,0xe9,0xe9,
  0x00,0x00,0x00,0xe9,0xe9,0x05,0x05,0xb3,0x00,0x00,0xb3,0xb3,0xb3,0xb3,0x00,0x00,
  0xd1,0x00,0x00,0x00,0x00,0x00,0xb3,0x05,0xb3,0xb3,0x05,0x05,0x05,0x05,0x05,0xb3,
  0xb3,0xb3,0xb3,0xb3,0xb3,0x00,0xd1,0x0f,0xd1,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x00,0x00,0xb3,0xb3,0xb3,0xb3,0xb3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xd1,0xd1,0x0f,0x00,0xb3,0xb3,0xb3,0xb3,0xb3,0x00,0x0f,0x0f,0x0f,0x00,0xb3,0x00,
  0x00,0xb3,0xb3,0xb3,0xb3,0x00,0xd1,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0x0f,
  0x0f,0x0f,0x0f,0x00,0x00,0xb3,0xb3,0x6e,0x6e,0x6e,0x6e,0xe9,0x6e,0xea,0x6e,0x6e,
  0x05,0x6e,0x05,0x05,0x6e,0x05,0x05,0x05,0x05,0x05,0x05,0xe9,0xe9,0xb3,0xb3,0xe9,
  0xe9,0xe9,0xb3,0xe9,0x05,0x05,0x05,0xe9,0xb3,0x00,0x00,0x00,0xb3,0x00,0x00,0x00,
  0xd1,0x00,0x00,0x00,0x00,0xb3,0x05,0x05,0xb3,0x05,0x6e,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0xb3,0x05,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0xd1,0xd1,
  0x0f,0xd1,0x00,0xb3,0xb3,0xb3,0x00,0xd1,0xb3,0x00,0xb3,0x00,0x00,0xb3,0xb3,0xb3,
  0x00,0x00,0xb3,0x00,0x00,0xb3,0xb3,0xb3,0xb3,0x00,0x00,0xd1,0xd1,0x00,0x00,0x00,
  0xb3,0xb3,0xb3,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0x0f,
  0x0f,0x0f,0x0f,0xd1,0xd1,0x00,0xb3,0x05,0x05,0x05,0x6e,0x05,0x05,0xe9,0xe9,0x6e,
  0x05,0xe9,0x05,0x05,0x05,0x05,0x05,0x05,0xe9,0x05,0x05,0xe9,0xe9,0xe9,0xe9,0xe9,
  0xe9,0xe9,0x05,0x05,0xe9,0xe9,0xe9,0x05,0x05,0xe9,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0xb3,0x05,0x05,0x05,0x05,0x6e,0x6e,0x6e,0x6e,0x6e,0x05,0x05,
  0x6e,0x6e,0x05,0x05,0x05,0xb3,0x00,0xd1,0x0f,0x0f,0x5f,0x5f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x00,0xb3,0xb3,0x00,0x00,0x00,0x00,0x00,0xb3,0x00,0xb3,0xb3,0xb3,0xb3,
  0xb3,0xb3,0xb3,0xb3,0xb3,0x05,0xb3,0xb3,0xb3,0x00,0xb3,0x0f,0x00,0x00,0x00,0x00,
  0x00,0xb3,0x0f,0x0f,0x0f,0x0f,0x5f,0xd5,0xd5,0xd9,0xd5,0x5f,0x5f,0xd5,0xd5,0x5f,
  0x0f,0x0f,0x0f,0x5f,0x0f,0x0f,0x00,0xb3,0x05,0xb3,0x05,0xb3,0x05,0xb3,0x05,0x05,
  0xb3,0xe9,0xb3,0xe9,0xb3,0xb3,0xe9,0xe9,0x05,0xe9,0xe9,0x05,0x05,0x05,0xe9,0xe9,
  0x05,0x05,0x05,0x6e,0x6e,0x05,0x05,0x05,0x05,0x05,0xe9,0xb3,0x00,0x00,0x00,0xb3,
  0x00,0x00,0x00,0x00,0xe9,0x05,0xea,0xc8,0xc8,0xea,0xea,0xea,0xea,0x6e,0x05,0x6e,
  0x05,0x05,0xe9,0x00,0x00,0x00,0xd1,0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x0f,0x0f,
  0x0f,0xd1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,
  0xb3,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb3,0x00,0xb3,0xb3,0xb3,0x00,0x00,0x00,
  0x00,0x00,0x0f,0xd1,0x0f,0x5f,0x5f,0xd5,0xd5,0x5f,0x5f,0xd5,0xd9,0x5f,0xd5,0xd5,
  0xd5,0xd5,0x5f,0x0f,0x00,0x0f,0x00,0x00,0xb3,0xb3,0xb3,0x00,0x00,0xb3,0xb3,0xe9,
  0x00,0x00,0xb3,0x05,0x05,0xe9,0xc8,0xe9,0xe9,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xea,
  0xe9,0xe9,0x05,0x05,0x05,0x6e,0x05,0x05,0x05,0x05,0xe9,0x05,0x05,0x05,0xe9,0xe9,
  0x00,0x00,0x00,0x6e,0xc8,0xea,0xc8,0xc8,0xc8,0xc8,0xc8,0xea,0xea,0xea,0x6e,0x6e,
  0x6e,0x05,0xe9,0x00,0x00,0xd1,0xd1,0x00,0x00,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,
  0x5f,0x5f,0x0f,0xd1,0x00,0xb3,0xb3,0xb3,0xb3,0xb3,0x05,0x05,0xb3,0xb3,0x05,0x6e,
  0x05,0x05,0x05,0x6e,0x05,0x6e,0x05,0x6e,0xb3,0xd1,0xb3,0xb3,0x00,0xd1,0x0f,0xd1,
  0x0f,0xd1,0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0xd5,0xd9,0xd9,0xd5,0x92,0x92,0xd9,0xd5,
  0xd5,0x92,0xb5,0x92,0x0f,0xe9,0x05,0x05,0xe9,0xea,0xea,0xe9,0xea,0xe9,0xe9,0xb3,
  0xb3,0xe9,0xe9,0xea,0xc8,0xea,0xc8,0xc8,0xc8,0xc8,0xc8,0xe9,0xe9,0xea,0xe9,0xe9,
  0x05,0x05,0xe9,0xe9,0x05,0x05,0x05,0x05,0xe9,0xe9,0xea,0x05,0xe9,0xe9,0x00,0xb3,
  0x00,0x05,0xc8,0xea,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xea,0xc8,0xea,0x6e,0x6e,
  0x6e,0x05,0x05,0x05,0xb3,0x00,0xd1,0xd1,0x5f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,0x5f,
  0x5f,0x5f,0x0f,0x00,0xb3,0xe9,0x05,0x05,0x05,0x05,0x6e,0x05,0xb3,0x05,0x6e,0x05,
  0x6e,0x05,0x05,0xb3,0xb3,0x05,0x05,0xb3,0xb3,0x00,0x0f,0x0f,0x0f,0x00,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x5f,0x5f,0x5f,0x5f,0x5f,0xd5,0xd9,0x92,0xd5,0xd5,0x92,0xd5,0x92,
  0xb5,0x92,0x92,0x92,0xd1,0x05,0xe9,0xc8,0xc8,0xc8,0xc8,0xc8,0xea,0xea,0xea,0x05,
  0x00,0xe9,0xc8,0xc8,0xc8,0xea,0xc8,0xc8,0xc8,0xc8,0xe9,0x05,0x05,0x6e,0xe9,0xe9,
  0xe9,0xe9,0xe9,0xe9,0xe9,0xea,0xe9,0xe9,0xe9,0x05,0xe9,0x05,0xe9,0xe9,0xe9,0xe9,
  0xe9,0xea,0xea,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xea,0x6e,0xea,0xe9,0x6e,
  0x05,0x05,0xe9,0xe9,0xfb,0x00,0x00,0x00,0xd1,0xd1,0x0f,0x5f,0x5f,0x5f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x00,0xb3,0x05,0x05,0x05,0x05,0x05,0x05,0xb3,0x05,0x6e,0x6e,
  0x6e,0x05,0xb3,0x05,0xb3,0x05,0x6e,0x05,0xb3,0xb3,0x00,0xd1,0x0f,0xd1,0xd1,0xd1,
  0x0f,0x5f,0x5f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x5f,0x5f,0xd5,0x92,0x92,0xd5,0x92,
  // 9: sky 1, part B
  0x3a,0xae,0x5c,0xb6,0x9c,0xbe,0xba,0x9d,0xfe,0xce,0x00,0x00,0x08,0x00,0x10,0x00,
  0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x5e,0xdf,0x00,0x02,0x08,0x02,
  0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,
  0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0xbf,0xe7,0x10,0x05,0x18,0x05,0x00,0x06,
  0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x1e,0xdf,0x10,0x07,0x18,0x07,
  0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,
  0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,
  0x00,0x24,0x08,0x24,0xfa,0xa5,0x3c,0xae,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,
  0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,
  0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,
  0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,
  0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,
  0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,
  0x7c,0xae,0x10,0x47,0x18,0x47,0xbd,0xce,0x00,0x60,0x08,0x60,0x10,0x60,0x18,0x60,
  0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0xfd,0xce,
  0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,
  0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,
  0xda,0x95,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,
  0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,
  0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x39,0x8d,0x18,0x83,0x00,0x84,
  0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x00,0x86,0x08,0x86,
  0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,
  0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0xbe,0xce,0x10,0xa1,0x18,0xa1,0x00,0xa2,
  0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x00,0xa4,
  0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,0x18,0xa5,0x00,0xa6,
  0x08,0xa6,0x10,0xa6,0x99,0x95,0x18,0xa6,0xfe,0xd6,0x00,0xa7,0x08,0xa7,0x10,0xa7,
  0x79,0x8d,0x18,0xa7,0x00,0xc0,0xbe,0xbe,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x00,0xc1,
  0x08,0xc1,0xfc,0x9d,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x7b,0xbe,0x08,0xc2,0x10,0xc2,
  0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x18,0xc3,0x00,0xc4,0x08,0xc4,0x10,0xc4,
  0x18,0xc4,0x7f,0xe7,0x00,0xc5,0xb9,0xa5,0x08,0xc5,0x10,0xc5,0x3a,0xb6,0x18,0xc5,
  0x7e,0xb6,0x00,0xc6,0x7c,0xc6,0x08,0xc6,0x3f,0xcf,0x10,0xc6,0x18,0xc6,0xfd,0xd6,
  0x9c,0x85,0x5a,0x8d,0x18,0xe1,0x99,0x8d,0x1d,0x96,0x08,0xe2,0x18,0xe2,0x18,0xe4,
  0x3b,0xf0,0xb4,0x0d,0xe9,0xf4,0xe9,0xe9,0xe9,0xe9,0x0d,0x0d,0x0d,0x1c,0x1c,0xe9,
  0x0d,0x1c,0x1c,0x1c,0x1c,0xe9,0x0d,0x0d,0x1c,0x1c,0x0d,0x0d,0x1c,0x1c,0x1c,0x1c,
  0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,0x1c,0x1c,0xe9,0x0d,0x0d,0xf4,0x04,0xcc,0x1c,
  0xe9,0x1c,0xe9,0xe9,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,0xe9,0xe9,0x1c,0xe9,0x0d,
  0xe9,0xcc,0x04,0x04,0x04,0x04,0xb4,0xd3,0xd3,0xd3,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
  0xf0,0xf0,0xd3,0xd3,0xb4,0xf4,0xf4,0xf4,0xf4,0xf4,0xe9,0xf4,0xf4,0xf4,0xf4,0xe9,
  0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0x04,0x04,0x04,0xf4,0x04,
  0x04,0xb4,0xd3,0xf0,0xb4,0xb4,0xb4,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0x3b,0x3b,
  0xb4,0xf4,0x0d,0xe9,0x1c,0xe9,0x1c,0x1c,0x1c,0xe9,0x0d,0x1c,0x1c,0x1c,0x1c,0x1c,
  0xe9,0x1c,0x1c,0x1c,0x1c,0x0d,0x0d,0xf4,0x0d,0xe9,0x0d,0x1c,0x1c,0x1c,0x1c,0x1c,
  0x1c,0x1c,0x1c,0x1c,0x1c,0x0d,0x0d,0xcc,0x0d,0xe9,0x1c,0x0d,0xf4,0xcc,0x0d,0x1c,
  0x1c,0x1c,0x0d,0x0d,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,
  0xf4,0xf4,0xb4,0xb4,0xb4,0x04,0xd3,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
  0xf0,0xf0,0xf0,0xf0,0xd3,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xe9,0xf4,0xe9,0xf4,0xe9,0xf4,0xf4,0x04,0xb4,0x04,
  0xb4,0xd3,0xd3,0xd3,0xd3,0x04,0x04,0xd3,0xf0,0xf0,0xf0,0xd3,0xf0,0xf0,0xf0,0xf0,
  0xe9,0xe9,0xe9,0x1c,0x1c,0xe9,0xe9,0x1c,0x1c,0xe9,0xe9,0xe9,0x1c,0x1c,0x1c,0x1c,
  0x1c,0x1c,0x1c,0x1c,0x1c,0x0d,0xf4,0xf4,0x0d,0x0d,0xe9,0x1c,0x1c,0x1c,0xe9,0x1c,
  0x1c,0x1c,0x1c,0x1c,0x0d,0x0d,0x04,0x6b,0x6b,0xcc,0xf4,0x0d,0xf4,0x04,0x0d,0x1c,
  0x1c,0x1c,0x0d,0xf4,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x0d,
  0xf4,0xf4,0xb4,0xb4,0xd3,0xb4,0xb4,0xd3,0xd3,0xd3,0xf0,0xf0,0xf0,0x3b,0x3b,0x3b,
  0xf0,0x3b,0xf0,0xf0,0xf0,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xe9,0xf4,0xe9,0xf4,0xf4,0xf4,0xe9,0x0d,0xf4,0xf4,0xf4,0xf4,0xf4,0xd3,0xf0,
  0xf0,0xd3,0x04,0x04,0xb4,0x04,0x04,0xb4,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xd3,0x04,
  0xe9,0xe9,0x1c,0x1c,0x1c,0xe9,0x1c,0x1c,0x1c,0x1c,0xe9,0x1c,0x1c,0x1c,0x1c,0x1c,
  0x1c,0x1c,0xe9,0x1c,0x0d,0xf4,0x04,0x04,0xf4,0x0d,0x1c,0x1c,0x1c,0x1c,0x0d,0x0d,
  0xe9,0x1c,0x0d,0x0d,0x0d,0x0d,0xcc,0x04,0xb4,0x04,0x6b,0x04,0x04,0xf4,0x1c,0x1c,
  0x1c,0x1c,0x0d,0xcc,0x0d,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,0x0d,0xe9,
  0xf4,0xcc,0x04,0x04,0x04,0x04,0x6b,0x04,0x04,0x04,0xd3,0xf0,0xf0,0x3b,0x3b,0x3b,
  0x3b,0x3b,0x3b,0xf0,0xf0,0xd3,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0x0d,0xe9,0xf4,0xe9,0xf4,0xe9,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0x04,0x04,0x04,
  0xd3,0xd3,0xb4,0x04,0x04,0xb4,0xd3,0xd3,0xd3,0xd3,0xf0,0xf0,0x3b,0x04,0xf4,0xe9,
  0xe9,0x1c,0x1c,0x1c,0xe9,0x0d,0xe9,0x1c,0x1c,0xe9,0xcc,0x0d,0x1c,0x1c,0xe9,0x0d,
  0xf4,0xf4,0xf4,0x0d,0x0d,0xf4,0xf4,0xcc,0xcc,0xcc,0xe9,0xe9,0xe9,0x0d,0x0d,0x0d,
  0xcc,0xf4,0xf4,0x04,0x04,0x04,0x6b,0x04,0xcc,0x6b,0x6b,0x6b,0x6b,0xe9,0x1c,0x1c,
  0x1c,0x1c,0xe9,0xf4,0x0d,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,0xe9,0x0d,0x0d,
  0xe9,0x0d,0xf4,0xcc,0x04,0x04,0x04,0x6b,0x6b,0x04,0xd3,0xf0,0x3b,0x3b,0x3b,0x3b,
  0x3b,0x3b,0x3b,0x3b,0x3b,0xf0,0xd3,0x04,0xb4,0x04,0xf4,0xf4,0xe9,0xf4,0xe9,0xf4,
  0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xe9,0xf4,0xf4,0xf4,0x04,0x04,
  0x04,0xd3,0xb4,0x04,0x04,0x04,0xd3,0xd3,0xd3,0xd3,0xd3,0x3b,0xf0,0xf4,0x0d,0x1c,
  0x1c,0x1c,0x1c,0x0d,0xcc,0xf4,0x0d,0x0d,0x1c,0xe9,0x0d,0x0d,0x1c,0xe9,0x25,0x6b,
  0x04,0x6b,0x6b,0x04,0xf4,0xcc,0x04,0x04,0xf4,0xf4,0x0d,0x0d,0xf4,0x04,0x04,0x04,
  0x04,0x04,0xcc,0x6b,0x6b,0x6b,0xb4,0x6b,0x04,0x04,0xb4,0xd3,0xd3,0x1c,0x0d,0xf4,
  0x0d,0x0d,0x0d,0x0d,0xe9,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,0xf4,0x0d,0xe9,0xe9,
  0xe9,0x0d,0xf4,0x04,0x04,0xcc,0x04,0x04,0x04,0x04,0xd3,0x02,0x01,0x3b,0x3b,0x3b,
  0x3b,0x3b,0xf0,0xf0,0x3b,0xf0,0xd3,0xb4,0x04,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0xf4,0xe9,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,0xd3,0xf4,0xd3,0xd3,0x04,0x0d,0xe9,0x1c,
  0x1c,0x1c,0x0d,0xf4,0x6b,0x6b,0x6b,0x6b,0x04,0x0d,0xf4,0xf4,0xcc,0x04,0xb4,0xd3,
  0x01,0x01,0x02,0xb4,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0xf4,0x25,0x04,0xb4,0xd3,0xd3,
  0x6b,0x6b,0x6b,0x6b,0x6b,0xf4,0x04,0xcc,0x04,0xb4,0x02,0xd3,0x6b,0xe9,0x6b,0x6b,
  0x04,0xf4,0x0d,0x0d,0xe9,0x1c,0xe9,0x1c,0x1c,0x1c,0x1c,0xe9,0xf4,0xe9,0xe9,0x0d,
  0x0d,0x0d,0xf4,0x6b,0x6b,0x04,0xd3,0xd3,0xd3,0xd3,0xd3,0x02,0x01,0xf0,0x3b,0x3b,
  0xf0,0xf0,0x01,0xf0,0x3b,0xf0,0xf0,0xd3,0xb4,0x04,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xe9,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0x04,0x04,0xd3,0x04,0xf4,0xf4,0xf4,0x04,0xf4,0xf4,0xf4,0x0d,0x0d,0x1c,0x1c,
  0x0d,0x0d,0xcc,0xf4,0x04,0xd3,0xd3,0xd3,0xd3,0xb4,0xd3,0xf0,0x02,0x01,0x01,0x01,
  0xf0,0x3b,0x3b,0x3b,0x02,0x02,0xd3,0xd3,0xd3,0x6b,0x6b,0x6b,0xd3,0x01,0x01,0x01,
  0x01,0x01,0x6b,0xb4,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0xd3,0xd3,0x6b,0xf4,0x6b,0xd3,
  0xb4,0x6b,0xf4,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0xe9,0xcc,0x25,0x0d,0x0d,0x0d,
  0x0d,0x0d,0x25,0xcc,0xd3,0x01,0xf0,0x01,0x02,0x02,0x02,0xd3,0x01,0x02,0x01,0x01,
  0x01,0x01,0x3b,0xf0,0xf0,0xf0,0xf0,0xf0,0xd3,0xb4,0xb4,0x04,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0x04,0xf4,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0x04,0x04,0x04,0xf4,0xe9,0xf4,0xf4,0xf4,0xe9,0x1c,0x0d,0x0d,0x1c,0x1c,
  0xf4,0x04,0x04,0x04,0xb4,0x01,0x68,0x01,0xf0,0x3b,0x3b,0x3a,0x3a,0x3b,0x3b,0x3a,
  0x3a,0x3b,0x3b,0x3a,0x3a,0x3a,0x3b,0x3b,0x3b,0x01,0x3b,0x3b,0x3b,0x3b,0x01,0x3b,
  0x3b,0x01,0xd3,0xd3,0x02,0x02,0xd3,0x02,0x6b,0xd3,0xd3,0xd3,0xd3,0xcc,0xcc,0x6b,
  0xd3,0xd3,0x6b,0x0d,0x0d,0xe9,0x0d,0xe9,0x0d,0x0d,0xf4,0x25,0xf4,0x0d,0xf4,0x25,
  0x0d,0x0d,0xf4,0xcc,0xd3,0x01,0x01,0x01,0x01,0xf0,0x01,0x01,0x02,0x01,0x01,0x01,
  0x01,0xf0,0x3b,0xf0,0xf0,0xf0,0xf0,0xf0,0xd3,0x04,0x04,0x04,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0x04,0x04,0x04,0xf4,0x04,0x04,0xf4,0xf4,0x04,0x04,0xf4,0xf4,0xf4,0xf4,
  0xf4,0xf4,0xf4,0xf4,0xf4,0xf4,0x0d,0xe9,0x0d,0xe9,0x1c,0xe9,0x1c,0x1c,0x1c,0x0d,
  0x6b,0x04,0x04,0x6b,0x04,0xd3,0x3b,0x3b,0x3b,0x3b,0xd9,0x03,0xd9,0x3a,0xd9,0x03,
  0x03,0x3a,0x3a,0x3b,0x3b,0x3b,0x3b,0x01,0x01,0x01,0x3b,0x3a,0xd9,0x3a,0x3b,0x3b,
  0x3b,0x01,0x01,0x02,0xd3,0xd3,0xd3,0xd3,0x02,0x02,0xd3,0x6b,0x6b,0x25,0x04,0x6b,
  0x02,0x01,0xd3,0x04,0xcc,0x04,0x6b,0x04,0x04,0xcc,0xcc,0x04,0xcc,0xf4,0x0d,0x1c,
  0x0d,0xcc,0x6b,0x6b,0x6b,0xd3,0x6b,0x3b,0x3b,0x3b,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0xf0,0x01,0x01,0x6b,0x6b,0xcc,0xcc,0xcc,0xcc,0xf4,0xf4,0xf4,0xf4,
  0x04,0xf4,0x04,0xf0,0xf0,0xb4,0x04,0xd3,0xd3,0xd3,0xd3,0xb4,0xb4,0x04,0x04,0xd3,
  0xd3,0x04,0x04,0x04,0xf4,0xf4,0xf4,0x0d,0x1c,0x1c,0x1c,0x0d,0x1c,0x1c,0x1c,0xcc,
  0x25,0x0d,0xe9,0x6b,0xb4,0xd3,0x01,0xd9,0x3b,0x3b,0xd9,0x03,0x03,0xd9,0xd9,0xd9,
  0x3b,0x3a,0x3b,0x3b,0x3b,0x3b,0x3b,0x01,0x02,0xd3,0x01,0x3b,0xd9,0x3b,0x3b,0x01,
  0x01,0x01,0x01,0xd3,0x6b,0x02,0x01,0x02,0xd3,0xd3,0xd3,0xb4,0x6b,0x25,0x25,0x6b,
  0x6b,0xd3,0xd3,0x6b,0x6b,0x6b,0x04,0x6b,0xb4,0x6b,0x6b,0x6b,0x04,0x25,0x0d,0xe9,
  0x0d,0x6b,0x6b,0x6b,0xd3,0xd3,0x6b,0x01,0x3b,0x3b,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x02,0x02,0x01,0xf0,0x01,0xd3,0x6b,0xcc,0x25,0xf4,0xf4,0xcc,0x04,0xf4,0x04,
  0x04,0xd3,0xd3,0xb4,0xf0,0xd3,0x04,0x04,0xd3,0x04,0xd3,0xd3,0xd3,0xf0,0xf0,0xd3,
  0xd3,0xd3,0xd3,0xf0,0xf4,0xf4,0xcc,0x0d,0x1c,0x1c,0xe9,0x1c,0x1c,0x1c,0x0d,0x0d,
  0xcc,0x04,0xf4,0x04,0x04,0xd3,0x3b,0x3b,0xd9,0xd9,0x03,0x3a,0x3a,0x3b,0x3b,0x3b,
  0x3b,0x3b,0x3b,0x3a,0x3b,0x3b,0x3b,0x01,0x02,0x02,0x01,0x3b,0x3a,0x3a,0x3b,0x3b,
  0x3b,0x01,0x3b,0x01,0x02,0x01,0x01,0x01,0xd3,0x6b,0x04,0x6b,0x04,0xf4,0xf4,0x6b,
  0x6b,0x02,0x6b,0x6b,0x6b,0xb4,0x04,0xb4,0x6b,0xcc,0x6b,0x6b,0x6b,0xcc,0x0d,0x25,
  0x04,0x6b,0x6b,0x6b,0xd3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x3b,0xee,0x3b,0x01,
  0x01,0x02,0xd3,0xd3,0x01,0x01,0xd3,0x6b,0x6b,0xcc,0x04,0xf4,0x04,0x04,0xf4,0x04,
  0x04,0x04,0xd3,0xd3,0xf0,0xf0,0xf0,0xd3,0xd3,0xd3,0xd3,0xd3,0xf0,0xf0,0x3b,0x3b,
  0x3b,0xf0,0x3b,0x3b,0x04,0x0d,0x0d,0x0d,0xe9,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x0d,
  0xcc,0x6b,0x6b,0x6b,0x04,0xd3,0x3b,0x03,0x88,0xd9,0x03,0x3a,0x3b,0x3b,0xd9,0xd9,
  0xd9,0xd9,0x3a,0x03,0x3a,0xd9,0xd9,0x3b,0x3b,0x3b,0x3b,0xd9,0xd9,0x03,0x3a,0x3a,
  0x3a,0x3b,0x3b,0x3b,0x3b,0x01,0x01,0x01,0x01,0x6b,0x6b,0x6b,0x6b,0x6b,0x04,0x6b,
  0x6b,0x6b,0x6b,0x6b,0xd3,0xd3,0xd3,0xd3,0x6b,0x6b,0x6b,0x6b,0x04,0xf4,0x6b,0x6b,
  0x6b,0x02,0x6b,0xd3,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x3b,0x3b,0x3b,0x3b,0x01,
  0x01,0x01,0x01,0x01,0x01,0x3b,0x02,0xb4,0x6b,0x6b,0xcc,0xcc,0x04,0xf4,0xcc,0x04,
  0x04,0xcc,0xcc,0xb4,0xd3,0xf0,0xf0,0xd3,0xd3,0xd3,0xd3,0xd3,0xf0,0x3b,0x3b,0x3b,
  0xfc,0xfc,0x3b,0xf0,0x04,0xe9,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x0d,
  0x6b,0x6b,0x04,0x6b,0x04,0xd3,0x3b,0xd9,0x88,0x88,0xd9,0x3b,0x3b,0x03,0x03,0x88,
  0x88,0x03,0x03,0xd9,0xd9,0xd9,0x88,0x03,0x3a,0x3a,0xd9,0x03,0x03,0x03,0xd9,0xd9,
  0x03,0x03,0x3a,0x3a,0x3a,0x3b,0x3b,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x6b,0x6b,
  0x02,0x02,0x6b,0xd3,0xd3,0x02,0x01,0x01,0x01,0xd3,0xd3,0x02,0x6b,0x6b,0x02,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x3b,0x3b,0x00,0x3b,0x3a,0x3a,0x3a,0x3b,0x3b,
  0x01,0x01,0x3b,0x01,0x3b,0x3b,0x01,0xd3,0xb4,0x6b,0xb4,0x6b,0x04,0x04,0x04,0xb4,
  0xb4,0x6b,0xb4,0x04,0xb4,0xf0,0xf0,0xf0,0xd3,0xf0,0xf0,0x3b,0x3b,0x3b,0xfc,0xfc,
  0x3b,0xf0,0xf4,0xe9,0x0d,0x0d,0x0d,0x1c,0xe9,0x1c,0x0d,0xe9,0xe9,0x0d,0xcc,0xcc,
  0x04,0x04,0x6b,0x6b,0x02,0x02,0x02,0x01,0x3b,0x3a,0x3b,0x3b,0x3a,0x03,0x88,0x88,
  0x88,0xf9,0xfb,0x88,0x88,0x03,0x03,0x03,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x03,0x03,
  0x03,0x3a,0x01,0x02,0xd3,0xd3,0x01,0x01,0x01,0x01,0x02,0x02,0x6b,0xd3,0xcc,0x6b,
  0x02,0xd3,0x6b,0xd3,0x6b,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x3b,0x3a,0x3a,0x3b,0x01,0x3b,0x3a,0x3a,0x3a,0x3a,0x3a,0xd9,0xd9,0x3a,0x3b,
  0x01,0x01,0x3b,0x3b,0x3b,0x01,0x01,0x01,0x02,0xb4,0xd3,0x6b,0xcc,0xcc,0x6b,0xd3,
  0x01,0x02,0xb4,0x04,0xd3,0x02,0x3b,0x3b,0xf0,0xd3,0xd3,0x3b,0x3b,0x3b,0xf0,0xf0,
  0x3b,0xd3,0xf4,0x0d,0x0d,0x0d,0x0d,0xcc,0xe9,0x1c,0x0d,0xf4,0xcc,0xcc,0xf4,0xcc,
  0xf4,0x04,0x6b,0x6b,0xd3,0x01,0x02,0x01,0x01,0x3b,0x3b,0xf0,0x3b,0x03,0x88,0x88,
  0xf9,0xd0,0x88,0x03,0x03,0xd9,0x3a,0x3a,0x3a,0x3a,0x03,0x3a,0x3b,0x3b,0x3a,0x88,
  0x3a,0x01,0x6b,0x6b,0xd3,0x6b,0x6b,0x6b,0x01,0x01,0x01,0x02,0x6b,0x02,0xcc,0xcc,
  0x02,0x6b,0x6b,0x6b,0x02,0x02,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x01,0x01,0x01,
  0x3b,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x03,0x03,0x03,0x03,0x3a,0x3a,0x3a,
  0x3b,0x3b,0x01,0x3b,0x01,0x01,0x01,0x01,0x01,0x01,0xd3,0x6b,0x6b,0x04,0xb4,0x04,
  0x01,0x01,0x02,0xb4,0xd3,0xd3,0xf0,0x6b,0xd3,0xf0,0xf0,0xf0,0xf0,0xd3,0xd3,0xd3,
  0xd3,0x04,0xf4,0xf4,0xf4,0xf4,0xe9,0x0d,0x1c,0x1c,0x1c,0x0d,0xcc,0x04,0xf4,0xcc,
  0x04,0x6b,0x6b,0x6b,0xb4,0x02,0x01,0x68,0x01,0x3b,0x3a,0x3a,0x3a,0x88,0x88,0xd9,
  0x88,0xfb,0x03,0x03,0x03,0x3a,0x3b,0x3b,0x3b,0x3a,0x03,0x03,0x3a,0x3a,0x3a,0x3a,
  0x3a,0x01,0x01,0xd3,0xd3,0x02,0x6b,0x6b,0x02,0x02,0x02,0x6b,0x04,0x77,0x25,0xcc,
  0x6b,0x6b,0x6b,0xd3,0x02,0x01,0x01,0x01,0x3b,0x3b,0x3a,0x3a,0x3b,0x3b,0x3b,0x3a,
  0x3a,0x03,0x3a,0x03,0x03,0x03,0x88,0x03,0x03,0x03,0x03,0x03,0x03,0x3a,0x3a,0x3a,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x04,0x6b,0x04,0x04,
  0x01,0x00,0x01,0x01,0x01,0x3b,0x3b,0xf0,0xf0,0x3b,0x3b,0xf0,0xf0,0xf0,0xf0,0xf0,
  0xd3,0x04,0xd3,0x04,0xf4,0x04,0xf4,0x0d,0x1c,0xe9,0x1c,0x0d,0xcc,0x04,0x6b,0x04,
  0x6b,0x6b,0x6b,0x6b,0x02,0x01,0x3b,0x3b,0x3a,0x3b,0x3a,0x03,0x88,0x88,0xd0,0x88,
  0x88,0xd0,0x88,0x03,0x03,0x03,0x3a,0x3a,0x3a,0x3a,0x03,0x03,0x03,0x3a,0x3b,0x3b,
  0x01,0x01,0x01,0x02,0x6b,0x02,0x01,0x01,0x01,0x02,0x6b,0x6b,0x6b,0x04,0xcc,0xcc,
  0x6b,0x02,0x01,0x01,0x01,0x01,0x01,0x00,0x3a,0x3a,0x3a,0x3a,0x3b,0x3a,0x3a,0x03,
  0x03,0x03,0x88,0x03,0x03,0x03,0x88,0x88,0x88,0x88,0x88,0x03,0x03,0x3a,0xd9,0x3a,
  0x3a,0x3b,0x01,0x3b,0x01,0x01,0x01,0x01,0x01,0x01,0xd3,0x02,0x04,0xf4,0x04,0xcc,
  0x6b,0x01,0x00,0x01,0xd3,0xf0,0x3b,0x3b,0xd9,0xd9,0xd9,0x3b,0x3b,0x3b,0x3b,0x3b,
  0x3b,0xf0,0xf0,0xf4,0xf4,0xf4,0xf4,0x0d,0xe9,0xe9,0x0d,0xcc,0xf4,0x04,0x04,0x6b,
  0x6b,0x01,0x01,0x01,0x01,0x00,0x3a,0x03,0x03,0xd9,0x3a,0x3a,0x88,0x88,0x88,0x03,
  0x03,0x03,0x03,0x3a,0x3a,0xd9,0x03,0x3a,0x03,0x88,0x88,0x88,0x3a,0x3b,0x01,0x02,
  0x02,0x6b,0x02,0x6b,0xcc,0xd3,0x01,0x01,0x02,0x02,0x02,0x6b,0x6b,0xd3,0x02,0x6b,
  0x01,0x00,0x00,0x00,0x00,0x3b,0x3b,0x01,0x3b,0x3a,0x03,0x3a,0x3a,0x3a,0x88,0x88,
  0x88,0xd0,0xd0,0xd0,0x88,0x88,0x88,0x88,0x88,0xd0,0xd0,0x03,0xd9,0x3a,0x3a,0x03,
  0x3a,0x3a,0x3a,0x3a,0x01,0x01,0x01,0x01,0x01,0x02,0xb4,0xcc,0xcc,0xb4,0x6b,0x04,
  0xf4,0xf4,0xb4,0xb4,0xd3,0x3b,0xd9,0xf9,0xf8,0xd9,0xd9,0xd9,0xd9,0xd9,0x3b,0xf0,
  0x3b,0x01,0xb4,0x04,0xf4,0xcc,0xcc,0x04,0x0d,0xe9,0xcc,0xcc,0x25,0x04,0x6b,0x6b,
  0x01,0x01,0x3b,0x00,0x3b,0x3a,0x03,0x03,0x3a,0x01,0x3b,0x3a,0x3a,0x3a,0x3b,0x01,
  0x3b,0xd9,0xd9,0x03,0x88,0x03,0x88,0x03,0xd9,0x03,0x88,0x88,0x03,0x3a,0x00,0x01,
  0x01,0x02,0x6b,0x6b,0xcc,0xf4,0x02,0x00,0x01,0x01,0x00,0x01,0x01,0xd3,0x04,0xd3,
  0x3a,0x03,0x3a,0x3a,0x00,0x3b,0x3b,0x3b,0x3b,0x3a,0x3a,0x3a,0x3b,0xd9,0x88,0x88,
  0xd0,0xf9,0xf9,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x3a,0x03,0x03,0x03,
  0x03,0x03,0x03,0x3a,0x3b,0x01,0x01,0x02,0x01,0x3b,0x01,0xb4,0xd3,0xd3,0xb4,0xb4,
  0xb4,0x04,0xf4,0x04,0xf0,0xd9,0xf8,0xd9,0xfc,0xfc,0x3b,0x3b,0xf0,0x01,0xd3,0xd3,
  0x6b,0xb4,0xcc,0x6b,0x04,0x04,0x04,0xcc,0x0d,0xe9,0x0d,0xcc,0x0d,0xf4,0x6b,0x02,
  0x01,0x02,0x01,0x3a,0x03,0x3a,0x00,0x3b,0x3b,0x00,0x3a,0xd9,0x3a,0x00,0x01,0x01,
  0x3b,0x3b,0x3b,0x3b,0x3a,0x03,0x03,0x88,0x88,0x03,0x88,0x03,0x3a,0x3b,0x01,0x01,
  0x02,0x6b,0x04,0x04,0x6b,0x6b,0x6b,0x01,0x01,0x01,0x01,0x02,0x02,0x04,0x04,0x3b,
  0x03,0x03,0x03,0x3a,0x3b,0x01,0x3a,0x3a,0x3a,0x3a,0x03,0x3a,0x3a,0x03,0x88,0x88,
  0x88,0xd0,0xd0,0x88,0x88,0x88,0x03,0x88,0x88,0xd0,0x88,0xca,0x03,0x03,0x3a,0x3a,
  0x03,0x03,0x3a,0x00,0x01,0x02,0x01,0x01,0x00,0x00,0x01,0x01,0x6b,0x04,0x6b,0x6b,
  0xb4,0xd3,0xb4,0xb4,0x3b,0xf8,0xfc,0x3b,0xfc,0xf0,0x04,0xb4,0x04,0x6b,0xb4,0xd3,
  0x02,0xd3,0x6b,0xcc,0xcc,0x04,0x6b,0x6b,0x04,0xcc,0xcc,0xcc,0x77,0x6b,0x02,0x01,
  0x01,0x01,0x00,0x3a,0x03,0x00,0x01,0x00,0x3a,0x3a,0x03,0x3a,0x3a,0x3a,0x3b,0x01,
  0x01,0x3b,0x3b,0x3b,0x3a,0x88,0x3a,0x3a,0x03,0x88,0xd9,0x3b,0x3b,0x00,0x01,0x01,
  0x02,0x02,0x6b,0x6b,0x02,0x02,0x01,0x02,0x6b,0x6b,0x6b,0xd3,0x02,0xd3,0x01,0x03,
  0x03,0x88,0x03,0x3a,0x3b,0x00,0x3a,0x3a,0x88,0x3a,0x03,0x88,0x88,0x88,0xd0,0xf9,
  0xd0,0xd0,0xd0,0x88,0x88,0x88,0x88,0x88,0xd0,0xd0,0xd0,0x03,0x03,0x3a,0x01,0x00,
  0x00,0x00,0x01,0x02,0x6b,0x02,0xdd,0x01,0x01,0x00,0x3a,0x3a,0x00,0xd3,0x04,0x04,
  0xcc,0xd3,0x6b,0x6b,0xf0,0x3b,0x3b,0x3b,0x3b,0xd3,0x04,0xf4,0xcc,0x6b,0x6b,0xd3,
  0x6b,0xb4,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x02,0x6b,0x6b,0x77,0x04,0x6b,0x02,0x01,
  0x00,0x00,0x01,0x01,0x01,0x01,0x00,0x3a,0x3a,0x3a,0x3a,0x03,0x3a,0xd9,0x3a,0x01,
  0x01,0x00,0x01,0x01,0x01,0xd9,0x03,0x03,0x3a,0x3a,0x01,0x3b,0x3b,0x3b,0x01,0x02,
  0x02,0x02,0x6b,0x04,0x6b,0x6b,0x6b,0x6b,0x6b,0x04,0x6b,0xd3,0x02,0x02,0x01,0x3a,
  0x03,0x03,0x03,0x3a,0x3a,0x3a,0x88,0x03,0x88,0x88,0x88,0x03,0x88,0xd0,0xd0,0xd0,
  0x88,0x88,0xd0,0x88,0x88,0x03,0x88,0xd0,0xd0,0xd0,0x88,0x03,0x3a,0x00,0x01,0x01,
  0x02,0x02,0x02,0x6b,0x6b,0xf2,0x01,0x00,0x00,0x03,0x03,0x03,0x3a,0x01,0x6b,0x6b,
  0x6b,0x6b,0xd3,0x01,0x3b,0xf0,0x3b,0x3b,0x3b,0xd3,0xf4,0xcc,0x6b,0xd3,0xd3,0x6b,
  0xcc,0xf4,0x04,0x6b,0x6b,0x02,0xd3,0x02,0x02,0x02,0x02,0x02,0x6b,0x01,0x68,0x01,
  0x3a,0xd9,0x00,0x3a,0x00,0x00,0x00,0x3a,0x3a,0xd9,0x03,0x3a,0x00,0x01,0x01,0xd3,
  0x02,0x01,0x01,0x3b,0x01,0xd9,0x3a,0x3b,0x3b,0x3b,0x3a,0x3b,0x3b,0x3b,0x01,0x01,
  0x02,0x6b,0xcc,0xcc,0x04,0x04,0x77,0x04,0x04,0x6b,0x6b,0xd3,0x01,0x00,0x3a,0x03,
  0x03,0x03,0x03,0x03,0x3a,0x3a,0x03,0x88,0xd0,0x03,0x3a,0x3a,0x03,0xca,0x88,0x3a,
  0x3a,0x03,0xca,0x88,0x88,0x88,0x88,0xd0,0xd0,0x88,0x03,0x3a,0x00,0x01,0x02,0x6b,
  0x6b,0x02,0x02,0xdd,0x01,0x00,0x00,0x3a,0x03,0x03,0x03,0x03,0x3a,0x00,0x01,0x02,
  0x02,0x6b,0xb4,0x3b,0xd9,0xd9,0x3b,0x3b,0xfc,0x04,0xf4,0xcc,0xb4,0xb4,0xd3,0xd3,
  0x6b,0xcc,0x6b,0x02,0x01,0x01,0x01,0x02,0x02,0x02,0xdd,0x01,0x3b,0x00,0x3a,0x3a,
  0x03,0x03,0x3a,0xd9,0x00,0x01,0x01,0x00,0x00,0x01,0x3a,0x01,0x01,0x02,0x02,0xd3,
  0xd3,0x01,0x01,0x00,0x3b,0x3b,0x03,0x3a,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x01,0xd3,
  0x6b,0x04,0xcc,0xcc,0x6b,0x6b,0x6b,0x6b,0xb4,0xd3,0xd3,0xd3,0x01,0x03,0x03,0x03,
  0x03,0x3a,0x3a,0x03,0x03,0xca,0x03,0x03,0x3a,0x3a,0x3a,0x88,0x03,0x03,0x88,0x3a,
  0x3a,0x03,0x88,0x3a,0x3a,0x88,0x88,0xd0,0x88,0x03,0x3a,0x00,0x01,0x01,0x02,0x02,
  0xf2,0xdd,0x01,0x00,0x00,0x3a,0x3a,0x03,0x03,0xca,0xca,0x03,0x3a,0x00,0x00,0x01,
  0x02,0x6b,0xcc,0xf0,0x3b,0x3b,0x3b,0x3b,0x3b,0x04,0xcc,0x6b,0xd3,0xd3,0xb4,0xb4,
  0x6b,0xd3,0xd3,0x01,0x01,0x02,0x02,0x02,0xb4,0xd3,0x01,0x3a,0x3a,0x3a,0x3a,0x3a,
  0x3a,0x03,0xd9,0x3b,0x01,0x01,0x01,0xd3,0x02,0xdd,0x01,0x01,0x00,0x3b,0x00,0x01,
  0x01,0x01,0x00,0x3a,0x3a,0x3b,0x3b,0xd9,0x00,0x3b,0x3b,0x3b,0x3b,0x01,0x01,0x01,
  0x02,0xd3,0x02,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0x02,0x68,0x3a,0x03,0x03,
  0x03,0x03,0x3a,0x03,0xca,0xca,0x03,0x3a,0x88,0x03,0x03,0x88,0x88,0x03,0x03,0x3a,
  0x00,0x3a,0x3a,0x3a,0x3a,0x03,0x88,0xca,0x03,0x00,0x00,0x01,0x01,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0x00,0xee,0x00,0x3a,0x3a,0x3a,0x3a,0x03,0xeb,0x3a,0x3a,0x01,
  0x01,0x02,0x6b,0xd3,0xf0,0xd3,0xd3,0xd3,0x04,0xf4,0x25,0xcc,0xd3,0xd3,0xd3,0xd3,
  0xb4,0x6b,0x01,0x02,0x02,0x6b,0x77,0xcc,0x6b,0x02,0x3a,0x03,0x88,0x3a,0x3b,0x3a,
  0x3a,0x88,0x88,0x03,0xd9,0x00,0x00,0xdd,0x02,0x01,0x02,0x6b,0x01,0x3a,0x3b,0x00,
  0x01,0x00,0x3b,0x00,0x00,0x3b,0x00,0x3a,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0xf0,
  0x01,0x01,0x01,0x01,0xf0,0x01,0x02,0xd3,0x68,0x01,0x3b,0x01,0xd3,0x3a,0x03,0x03,
  0x03,0x03,0x3a,0x3a,0x03,0x03,0xd9,0x88,0x03,0x88,0x03,0x88,0x03,0xca,0x88,0x3a,
  0x3a,0x3a,0x03,0x3a,0x3b,0x3a,0x03,0x3a,0x3a,0x00,0x00,0x01,0x01,0x00,0x00,0x00,
  0x00,0x01,0x01,0x01,0x01,0xf2,0x01,0xee,0x00,0x3a,0x00,0x3a,0x03,0x3a,0x00,0x01,
  0x02,0x6b,0xd3,0x04,0xf4,0xd3,0xd3,0xf4,0x0d,0x0d,0xf7,0x6b,0xd3,0xb4,0xb4,0x6b,
  0x02,0x01,0x02,0x6b,0x6b,0x25,0xe9,0x0d,0x04,0x04,0x01,0x3a,0x3a,0x3a,0x01,0x00,
  0x3b,0x88,0x88,0x3a,0x3a,0x3a,0x00,0x02,0x02,0x01,0x01,0x01,0x01,0x3a,0x3a,0x3a,
  0xd9,0xd9,0x3a,0x3a,0x3b,0x3a,0xee,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0xf0,0x01,
  0xf0,0xf0,0x68,0x01,0x02,0xf0,0xf0,0xf0,0x3b,0x01,0x3b,0xf0,0x68,0x88,0x03,0x03,
  0x03,0x03,0x03,0x03,0x88,0x03,0x03,0x03,0x88,0x03,0xfb,0xca,0xca,0x03,0x3a,0x3a,
  0x00,0x3b,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x00,0x00,0x01,0x00,0x00,0x00,0x00,
  0x00,0x00,0xee,0x00,0xee,0x00,0x3a,0x03,0x03,0x03,0x03,0x03,0x88,0x00,0xee,0x01,
  0x02,0x02,0xd3,0xb4,0x04,0xb4,0xb4,0xf4,0x0d,0x0d,0xcc,0xd3,0x6b,0x6b,0x02,0x01,
  0x01,0xd3,0x6b,0x04,0xcc,0x0d,0x0d,0xf7,0xcc,0x77,0x6b,0xdd,0x00,0x00,0x01,0x00,
  0xd9,0x88,0xfb,0xca,0xca,0x3a,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x3a,0x3a,0x3b,
  0x3a,0x3b,0x00,0x3b,0x00,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x00,0x3b,0x3b,0x3b,0x01,
  0x01,0xf0,0xf0,0x68,0xf0,0x3b,0x3b,0x3b,0x3a,0x3b,0x3b,0x3b,0x3a,0x03,0x3a,0x03,
  0x03,0xca,0xca,0xca,0xca,0x03,0x3a,0x03,0xca,0x03,0x03,0x03,0x3a,0x00,0x00,0x00,
  0x01,0x00,0x00,0x00,0x3a,0x00,0x3a,0x3a,0x3a,0x00,0x00,0x00,0x00,0x3a,0x00,0x00,
  0x00,0xee,0x01,0xdd,0xee,0x03,0xca,0xca,0xd0,0xca,0x03,0x03,0x3a,0x00,0x01,0xdd,
  0xf2,0xf2,0x02,0x6b,0x6b,0x6b,0x25,0x0d,0xe9,0x0d,0xcc,0x6b,0x6b,0x02,0x02,0x01,
  0x01,0xd3,0x6b,0x6b,0x77,0x77,0x77,0x77,0x77,0x6b,0x6b,0x6b,0x02,0x01,0xdd,0x68,
  0x68,0x3a,0xca,0x9d,0xca,0x00,0x01,0x01,0x00,0x3a,0x00,0x00,0x3a,0x3a,0x3a,0x00,
  0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3a,0xd9,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,
  0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0xf0,0x3b,0x88,0x03,0x03,0x03,
  0xca,0xca,0xd0,0x03,0x3a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3a,0x3a,0x00,
  0x00,0x00,0xee,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0x3a,0xeb,0xeb,0x3a,0xeb,
  0x00,0x00,0x00,0x03,0xd0,0xd0,0xf9,0xca,0xca,0xeb,0x3a,0x00,0x01,0xf2,0xf2,0xf2,
  0xf2,0x6b,0x04,0xcc,0x77,0xcc,0x0d,0x0d,0x0d,0xcc,0xcc,0x6b,0x6b,0x02,0x02,0x02,
  0x01,0x01,0x02,0x02,0x6b,0x6b,0x6b,0xf2,0x6b,0x02,0x02,0x02,0xdd,0x00,0x00,0x01,
  0x00,0x00,0x00,0xfb,0x03,0x3a,0x3a,0x00,0x3a,0x00,0x00,0x00,0x00,0x00,0x3b,0x3b,
  0x3b,0x3b,0x3b,0x3a,0x3b,0x3b,0x3a,0xd9,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x01,0x3b,
  0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x00,0x00,0x3b,0x3b,0x3a,0x3a,0xd9,0xd9,
  0x3a,0x03,0xca,0xca,0x03,0x3a,0x3a,0x00,0x00,0x00,0x00,0x01,0x01,0x3a,0x3a,0x3a,
  0x3a,0x00,0x00,0x00,0x3a,0xeb,0x03,0xeb,0xeb,0xca,0xca,0xeb,0xeb,0xca,0xca,0x03,
  0x3a,0x03,0xca,0xd0,0x9d,0x9d,0xd0,0xca,0x03,0x00,0xee,0x01,0xf2,0xf2,0x6b,0x6b,
  0xcc,0x25,0x25,0x25,0xcc,0x25,0xcc,0xcc,0x6b,0x6b,0x6b,0xdd,0x02,0x01,0x01,0x01,
  0x01,0x00,0x01,0xdd,0x6b,0x6b,0x6b,0x6b,0x02,0x02,0xdd,0x02,0x01,0x01,0x00,0x00,
  0x01,0x00,0x00,0x00,0xd9,0x3a,0x3a,0x00,0x3a,0x00,0x3a,0x00,0x01,0x00,0x3b,0x00,
  0x00,0x3a,0x3a,0x3a,0x3a,0x3a,0x00,0x3b,0x3b,0x01,0x3b,0x3a,0x00,0x68,0x01,0x3b,
  0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x00,0x3b,0x3a,0x3a,0x01,0x01,0x00,0x3a,0x3a,0x00,
  0x3a,0x03,0xca,0x88,0x03,0x88,0x03,0x3a,0x01,0x00,0x00,0x00,0x00,0x3a,0x3a,0x3a,
  0x3a,0x3a,0x3a,0x3a,0xeb,0x03,0xca,0xca,0xca,0xca,0xca,0xca,0x9d,0x9d,0xd0,0x03,
  0xca,0xd0,0x9d,0xd0,0x9d,0x9d,0xd0,0x03,0x00,0xee,0x01,0xdd,0xdd,0x6b,0xcc,0x0d,
  0x0d,0x25,0x77,0xcc,0xcc,0x6b,0x02,0xdd,0x01,0x02,0x02,0xdd,0x01,0x00,0x3a,0x00,
  0x00,0xee,0x00,0x00,0xdd,0x02,0x02,0x01,0x68,0x01,0x00,0x00,0xdd,0xdd,0x00,0xee,
  // 10: sky 1, part C
  0x5a,0xbe,0xdc,0xc6,0xfa,0xad,0x3b,0xa6,0x00,0x00,0x3a,0xb6,0x08,0x00,0x10,0x00,
  0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,
  0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,
  0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,
  0x18,0x06,0xb9,0xa5,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,
  0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,
  0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,
  0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,
  0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,
  0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x5c,0xb6,0x00,0x42,
  0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,
  0x08,0x44,0x10,0x44,0x99,0x95,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,
  0x00,0x46,0x08,0x46,0x7b,0xc6,0x9c,0xbe,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,
  0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,0x39,0x8d,0x18,0x60,0x00,0x61,
  0x9b,0xc6,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,
  0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,
  0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,
  0x00,0x67,0x08,0x67,0x10,0x67,0xbc,0xce,0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,
  0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,
  0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,
  0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x5e,0xdf,0x00,0x86,0x08,0x86,0x1e,0xdf,
  0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,
  0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,0x08,0xa2,
  0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x00,0xa4,0x08,0xa4,
  0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,0x18,0xa5,0x00,0xa6,0x08,0xa6,
  0x10,0xa6,0x18,0xa6,0xfc,0xd6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,0x7c,0xae,
  0x00,0xc0,0x08,0xc0,0xba,0x9d,0x3d,0xd7,0x10,0xc0,0x18,0xc0,0x00,0xc1,0x08,0xc1,
  0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,
  0x10,0xc3,0xbf,0xe7,0xdb,0xce,0xf9,0xad,0x18,0xc3,0x00,0xc4,0x08,0xc4,0x7c,0xc6,
  0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,0xfa,0xb5,0xd9,0x9d,0x10,0xc5,0x37,0x95,
  0x19,0xbe,0x18,0xc5,0x00,0xc6,0x36,0x8d,0x35,0x95,0x10,0xc6,0x35,0x8d,0xfb,0x9d,
  0x18,0xc6,0x7f,0xe7,0x3c,0xae,0x00,0xe0,0x08,0xe0,0xdc,0x8d,0x77,0x9d,0x9b,0xbe,
  0x4e,0x03,0x03,0x03,0x03,0xfa,0x02,0x03,0xfa,0x02,0x03,0x02,0x02,0x03,0x02,0x03,
  0x02,0x21,0x03,0x02,0x03,0xcf,0x63,0x8b,0x4e,0x03,0xfa,0x4e,0x4e,0x4e,0x4e,0x03,
  0x03,0x03,0xfa,0xfa,0x03,0x03,0x02,0x03,0x03,0x03,0xfa,0x4e,0x4e,0x03,0x02,0x03,
  0x02,0xd2,0x5a,0x6d,0x5a,0x5a,0xd2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0xd2,
  0x03,0x03,0x03,0x02,0xd2,0x6d,0x6d,0x6d,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,0xd2,0x5a,
  0x6d,0x6d,0x6d,0x6d,0x6d,0x6d,0x5a,0x03,0x02,0x4e,0xff,0x4e,0x63,0x8b,0xd3,0xf9,
  0xd3,0xa7,0xa7,0x01,0x8b,0x01,0xff,0x02,0x02,0x4e,0xe7,0x4e,0x05,0x02,0x02,0x21,
  0x02,0x05,0x05,0x02,0x03,0x03,0x02,0x02,0x02,0xfa,0x03,0x03,0x4e,0xff,0x03,0x03,
  0x4e,0x4e,0x03,0x02,0x03,0x21,0x02,0x4e,0x03,0x02,0x02,0x02,0x21,0x21,0x03,0xfa,
  0x03,0x02,0x02,0x03,0xcf,0x4e,0x4e,0x01,0x01,0x01,0x4e,0x03,0xcf,0x4e,0xcf,0x4e,
  0xfa,0x03,0x03,0xf7,0x03,0xfa,0xfa,0x03,0x03,0x02,0x03,0x03,0x4e,0xcf,0x4e,0x03,
  0x03,0x02,0xd2,0x6d,0x6d,0x5a,0x02,0x02,0x02,0x21,0x02,0x02,0x02,0x02,0xd2,0x03,
  0xf7,0xf7,0x03,0xd2,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,0x6d,0x5a,0x5a,0x5a,0x02,0xd2,
  0x5a,0xd2,0xd2,0x03,0x5a,0x5a,0xd2,0x03,0x02,0x4e,0x4e,0x4e,0xcf,0x01,0xa4,0xa4,
  0xa7,0xa7,0xa7,0xa7,0x01,0x4e,0x05,0x02,0x02,0x4e,0x05,0x05,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x21,0x21,0xd2,0x02,0x02,0x02,0x03,0xff,0x63,0xff,
  0xcf,0x02,0x02,0x02,0x03,0x02,0x03,0x4e,0x4e,0x03,0x03,0x02,0x02,0x02,0x02,0x03,
  0xd2,0xf7,0x03,0xfa,0x4e,0x02,0xd2,0x03,0x03,0x63,0x63,0x4e,0x4e,0xe7,0x4e,0x03,
  0x03,0x03,0xfa,0xcf,0x4e,0x4e,0xcf,0x8b,0x01,0xe7,0x4e,0x4e,0x4e,0x4e,0x4e,0x02,
  0x02,0x02,0x02,0xd2,0x5a,0xd2,0x21,0x03,0x02,0x21,0x02,0x02,0xd2,0xd2,0xd2,0x03,
  0x03,0x03,0xd2,0x5a,0x5a,0x6d,0x6d,0x6d,0x5a,0x5a,0x21,0x21,0x02,0x02,0x02,0x03,
  0x5a,0xd2,0x03,0x03,0x03,0x02,0x02,0x03,0x4e,0x4e,0x03,0x4e,0xcf,0x01,0xf9,0xe1,
  0xa7,0xa7,0x01,0x01,0x63,0x4e,0x03,0x02,0x05,0x05,0x05,0x05,0x02,0x02,0x03,0x02,
  0x02,0x02,0x21,0x21,0xed,0x21,0xed,0xed,0x02,0x02,0x03,0x03,0x03,0x4e,0xff,0x4e,
  0x03,0x03,0x03,0x4e,0x03,0xcf,0xcf,0xcf,0xff,0x4e,0xcf,0x4e,0x03,0xfa,0x03,0x03,
  0x03,0x02,0x03,0xcf,0xcf,0xd2,0x5a,0xd2,0x02,0xcf,0xa7,0xa7,0xa7,0x01,0x01,0x63,
  0x4e,0xcf,0x4e,0x4e,0xfa,0x03,0x03,0x03,0x4e,0x63,0x01,0x4e,0x4e,0xcf,0xfa,0x02,
  0x21,0x21,0x21,0x21,0xd2,0xd2,0xd2,0x5a,0x5a,0x5a,0x6d,0x6d,0x5a,0xd2,0xd2,0x03,
  0x02,0x21,0x21,0x5a,0x6d,0x6d,0x5a,0x21,0x21,0x21,0x21,0xec,0x02,0x02,0x02,0x03,
  0xd2,0xd2,0xf7,0xd2,0xd2,0x03,0x05,0x03,0x4e,0x03,0x4e,0xe7,0xe7,0xd3,0xa4,0xa4,
  0x8b,0x8b,0x01,0xe7,0xff,0x4e,0x4e,0x05,0x02,0x05,0x05,0x05,0x05,0x05,0x05,0x02,
  0x02,0x21,0xed,0x5a,0x5a,0x5a,0x21,0x02,0x02,0x02,0x03,0x05,0x03,0x4e,0x03,0x03,
  0x4e,0x63,0x4e,0x4e,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x4e,0x4e,0x4e,0xcf,0x4e,
  0xcf,0xcf,0x63,0x01,0x4e,0x5a,0x5a,0x21,0x03,0x4e,0x63,0xd3,0x01,0x63,0x63,0x63,
  0x63,0xcf,0x63,0x03,0x03,0x02,0x02,0x03,0x03,0x63,0x63,0xcf,0x4e,0x02,0x03,0xd2,
  0x5a,0x5a,0x5a,0xd2,0x21,0xd2,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,0x6d,0x5a,0x5a,0x21,
  0x02,0x02,0x21,0x5a,0x6d,0x5a,0x21,0x21,0x21,0x02,0x02,0x02,0xec,0x02,0x02,0x03,
  0xd2,0x5a,0x5a,0xf7,0xd2,0xd2,0x03,0x03,0xff,0x63,0x63,0xe7,0x01,0x8b,0xca,0x8b,
  0x8b,0x01,0x63,0x63,0xff,0x4e,0x05,0x02,0x02,0x02,0x05,0x02,0x05,0xff,0x03,0x02,
  0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0xd2,0x21,0x02,0x02,0x02,0x02,0x02,0x03,0x4e,0x4e,
  0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0xcf,0xcf,0x4e,0x4e,0xcf,
  0x4e,0x4e,0x4e,0x4e,0xfa,0x5a,0x5a,0x02,0x02,0x4e,0xff,0x01,0x63,0xe7,0x4e,0x03,
  0x63,0xa7,0xa7,0x4e,0x4e,0x03,0x02,0x02,0x02,0x03,0x4e,0xcf,0x03,0x03,0x03,0x21,
  0x5a,0x5a,0xd2,0x5a,0x5a,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,0x5a,0x5a,0x6d,0x5a,0x21,
  0x02,0xec,0x02,0x5a,0x21,0x21,0x21,0x02,0x02,0x02,0x02,0x02,0x5a,0x6d,0x6d,0x5a,
  0xf7,0xf7,0x03,0x03,0xd2,0xf7,0x03,0x63,0x8b,0x8b,0xe7,0xe7,0x01,0x01,0x01,0x8b,
  0x8b,0x8b,0x63,0xe7,0xff,0x00,0xff,0x05,0x02,0x02,0x05,0x05,0xff,0x70,0xff,0x02,
  0x21,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x21,0x21,0x03,0x02,0x02,0x03,0x4e,0x4e,0x4e,
  0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x4e,0xcf,0x4e,0xcf,
  0x03,0x4e,0x02,0x02,0x02,0xed,0x21,0xed,0x21,0x03,0x4e,0x4e,0x01,0x01,0xa7,0x01,
  0x8b,0x63,0x63,0x4e,0x4e,0x03,0x05,0x02,0x02,0x03,0x4e,0x4e,0x03,0x4e,0x03,0x02,
  0x5a,0x5a,0x21,0x6d,0x5a,0x5a,0x5a,0x6d,0x6d,0x6d,0x5a,0x21,0x21,0x21,0x21,0x21,
  0x21,0x21,0x02,0xec,0x02,0x02,0x21,0x21,0x02,0x02,0x21,0x21,0x5a,0x5a,0x03,0x02,
  0xd2,0x03,0x03,0x4e,0x4e,0x4e,0x4e,0x8b,0xca,0x8b,0x70,0x70,0x01,0x8b,0x8b,0x01,
  0x63,0x63,0xff,0xff,0x63,0x00,0x4e,0x4e,0x4e,0x05,0x05,0x4e,0x01,0x8b,0x8b,0x4e,
  0x05,0xed,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0xd2,0xd2,0x02,0x03,0x03,0x4e,0x4e,0x63,
  0x63,0x63,0x63,0x63,0xff,0x4e,0xff,0x4e,0x4e,0x4e,0x63,0x63,0x63,0x4e,0x4e,0x4e,
  0x4e,0x03,0xed,0xed,0x21,0x5a,0x5a,0x5a,0x5a,0x21,0x02,0x4e,0x4e,0x4e,0x4e,0x01,
  0x63,0x63,0x4e,0x63,0xe7,0x02,0x02,0x03,0x02,0xf7,0x4e,0xe7,0xcf,0x4e,0xcf,0xd2,
  0x5a,0x5a,0x5a,0x6d,0x6d,0x5a,0x21,0x5a,0x21,0x21,0x21,0x21,0x02,0xec,0xec,0xec,
  0x02,0x02,0x02,0x21,0x21,0x21,0x21,0x02,0xff,0xe7,0xff,0x4e,0x05,0x05,0x4e,0x4e,
  0x03,0x03,0x03,0x4e,0x63,0x63,0xe7,0x8b,0xca,0x8b,0x01,0x01,0x8b,0x8b,0x01,0x70,
  0x63,0xff,0x00,0xff,0xff,0xff,0xff,0x00,0xff,0x4e,0xff,0x01,0xa7,0xd3,0xd3,0x70,
  0x05,0x21,0x21,0x5a,0x5a,0x5a,0x5a,0x5a,0x21,0x21,0x02,0x03,0x03,0xcf,0xff,0x63,
  0x4e,0x4e,0x4e,0x4e,0x4e,0xcf,0x4e,0x4e,0xff,0xff,0x4e,0x63,0xcf,0xcf,0xcf,0x4e,
  0x03,0x02,0x21,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x21,0x02,0xff,0xff,0x02,0x05,0x4e,
  0x4e,0x01,0x01,0x63,0x01,0x05,0x00,0x03,0xfd,0xcf,0x4e,0x4e,0x4e,0x4e,0x03,0xfd,
  0xfd,0x6d,0x5a,0x5a,0x6d,0x6d,0x5a,0x21,0x5a,0x21,0xec,0x02,0x05,0x02,0x21,0x21,
  0x21,0x21,0x5a,0x6d,0x5a,0x05,0x00,0x4e,0x00,0x05,0x02,0x02,0x21,0x02,0x02,0x03,
  0xd2,0x03,0x03,0x03,0x03,0x4e,0x63,0x01,0x01,0x63,0xe7,0xe7,0x70,0x01,0x70,0x70,
  0xff,0xff,0x63,0x63,0x8b,0x01,0x70,0xff,0x70,0x70,0x01,0xa7,0xd3,0xd3,0x8b,0xff,
  0x02,0x21,0x02,0x02,0xed,0x21,0x21,0xed,0xed,0x02,0x02,0x03,0xcf,0xff,0xff,0x4e,
  0x03,0x02,0x03,0x03,0x03,0xcf,0x4e,0x4e,0xff,0xff,0x4e,0xff,0x63,0x63,0x4e,0x02,
  0x03,0x21,0x5a,0x5a,0x5a,0x5a,0xed,0x02,0x05,0x4e,0x4e,0x00,0x4e,0x02,0x02,0x4e,
  0xa7,0xa7,0x01,0x4e,0x4e,0xf7,0x03,0x03,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xfd,0xfd,
  0xfd,0xf7,0x03,0x21,0x21,0x5a,0x21,0x02,0x21,0xec,0x21,0x5a,0x5a,0x6d,0x6d,0x6d,
  0x6d,0x6d,0x6d,0x6d,0x05,0x05,0x02,0x21,0x21,0x21,0x21,0x5a,0x6d,0x6d,0x5a,0x6d,
  0x6d,0x6d,0x6d,0x6d,0xf7,0x4e,0x63,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xe7,0x8b,0x8b,0x8b,0x8b,0x01,0x01,0x70,0x00,0xe7,0x8b,0xd3,0xd3,0xd3,0x8b,0x01,
  0x05,0x02,0x02,0x02,0x21,0x02,0xed,0x02,0x02,0x02,0x03,0x4e,0xff,0x4e,0x03,0x05,
  0x4e,0xff,0x4e,0x4e,0x4e,0x4e,0xff,0x4e,0x4e,0xff,0xcf,0x4e,0x4e,0x4e,0x63,0x63,
  0xcf,0x02,0x21,0x21,0x02,0x4e,0x05,0x62,0xff,0x70,0x01,0xff,0x05,0x05,0x4e,0x01,
  0xa7,0x63,0xcf,0xcf,0xcf,0x03,0xfa,0xfd,0xcf,0xcf,0xcf,0xcf,0xcf,0x63,0x63,0x03,
  0xfd,0x4e,0x4e,0x00,0xec,0x02,0x21,0x21,0xec,0x02,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,
  0x6d,0x6d,0x6d,0xd2,0x05,0x02,0x21,0x5a,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,0xfd,0xf7,
  0xf7,0xf7,0xfd,0xfd,0xf7,0x4e,0x03,0xcf,0x4e,0x4e,0x63,0xff,0xff,0x4e,0x70,0xff,
  0x8b,0xd3,0xd3,0xd3,0x8b,0x01,0x70,0xff,0x00,0x70,0x8b,0x70,0x01,0xa7,0xd3,0x01,
  0x00,0x05,0x02,0x05,0x05,0x05,0x05,0x05,0x4e,0x05,0x70,0x70,0x01,0xff,0x4e,0x05,
  0x02,0x4e,0xff,0x63,0x63,0xff,0xff,0xff,0xff,0x4e,0xcf,0x4e,0x4e,0x4e,0x63,0x63,
  0x63,0x01,0x01,0x8b,0x70,0xff,0x00,0x4e,0x70,0x8b,0x01,0x01,0x00,0xff,0x01,0x01,
  0x4e,0xcf,0xfd,0xfd,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0x63,0x63,0x4e,0x4e,
  0x01,0x4e,0x63,0x63,0x03,0x02,0x02,0x02,0x02,0x5a,0x6d,0x6d,0x6d,0x6d,0x6d,0x6d,
  0x6d,0x6d,0x6d,0x00,0x4e,0x05,0x02,0x02,0xd2,0x5a,0x6d,0x5a,0x5a,0x6d,0xf7,0xcf,
  0x63,0x03,0xfd,0xcf,0xcf,0x63,0x4e,0xcf,0xcf,0xff,0x01,0x63,0x4e,0x4e,0xff,0x01,
  0x01,0x8b,0xd3,0xca,0x01,0xff,0x05,0x05,0x70,0x01,0xd3,0x8b,0x8b,0x8b,0xca,0xff,
  0xff,0x4e,0x05,0x4e,0xff,0x63,0x70,0x70,0x01,0x01,0x01,0x01,0xff,0x70,0x05,0x02,
  0x05,0x05,0x4e,0xff,0x63,0x4e,0xff,0xcf,0x4e,0x4e,0xcf,0x4e,0x4e,0x63,0x63,0x63,
  0x63,0x63,0x00,0x00,0x05,0x05,0x05,0x62,0x01,0x01,0x70,0xff,0x62,0xff,0x4e,0xcf,
  0xfd,0x03,0xfd,0xfd,0xcf,0x63,0x01,0x01,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,0x01,0xa7,
  0xd3,0xd3,0xd3,0xa7,0xcf,0xf7,0xd2,0xd2,0x6d,0x6d,0x6d,0x6d,0x5a,0x5a,0x5a,0x02,
  0xd2,0xd2,0x21,0x02,0x5a,0x5a,0x21,0x5a,0x5a,0xd2,0x5a,0x6d,0xfd,0xf7,0xf7,0xfd,
  0x03,0xcf,0xfd,0x03,0xcf,0x4e,0x63,0x4e,0x4e,0x8b,0x01,0x63,0xff,0x00,0x02,0xff,
  0x63,0x01,0x63,0x4e,0x05,0x02,0x02,0x05,0xff,0xff,0x8b,0xca,0xca,0x01,0x01,0x8b,
  0xff,0x02,0x02,0x05,0x05,0xff,0xff,0x63,0x01,0x01,0x70,0xff,0x05,0x05,0x02,0x02,
  0x02,0x05,0x4e,0xff,0xff,0x4e,0xcf,0x4e,0x4e,0x4e,0xcf,0xff,0x4e,0x03,0x4e,0x4e,
  0x4e,0x4e,0x05,0x05,0x05,0x05,0xff,0x62,0x62,0x62,0x00,0x02,0x05,0x03,0xfd,0xfd,
  0xfd,0xfd,0x03,0xcf,0x4e,0x01,0x01,0xa7,0xd3,0xf9,0xf9,0xd3,0xd3,0xa7,0x01,0x8b,
  0x01,0xa7,0xa7,0x01,0x03,0xf7,0x5a,0x6d,0x6d,0x6d,0x5a,0x05,0x8b,0xca,0x01,0xff,
  0x05,0x05,0x03,0x05,0x02,0x02,0xd2,0x5a,0x5a,0x5a,0xf7,0xfd,0xfd,0xfd,0xfd,0xfd,
  0xfd,0xfd,0xfd,0xcf,0x4e,0x63,0x63,0x4e,0x63,0x8b,0x63,0xff,0x4e,0x02,0x02,0xcf,
  0x4e,0xff,0x00,0x02,0x21,0x21,0xec,0x05,0xff,0xff,0x01,0x8b,0x01,0x01,0x01,0xff,
  0x02,0x02,0xec,0xec,0xec,0x05,0x4e,0x4e,0xff,0xff,0x05,0x05,0x05,0x02,0x21,0x02,
  0x02,0x02,0x4e,0x4e,0x02,0x05,0x4e,0xff,0x4e,0x4e,0x4e,0x4e,0x4e,0x02,0x21,0x02,
  0x02,0x62,0x00,0x00,0x05,0x70,0x70,0xff,0x05,0x05,0x02,0x02,0x02,0xcf,0xcf,0x03,
  0x03,0x4e,0x4e,0x63,0x8b,0x01,0xa7,0xa7,0xd3,0xd3,0xa7,0x8b,0xa7,0x01,0x63,0x8b,
  0x01,0xa7,0xd3,0x8b,0x4e,0x03,0x02,0x02,0x02,0x02,0xff,0x8b,0x8b,0xca,0x70,0x4e,
  0x02,0x02,0x02,0x02,0x02,0x02,0xd2,0x5a,0x5a,0x03,0x03,0x4e,0xf7,0xfd,0xfd,0xfd,
  0xcf,0x4e,0x4e,0x4e,0x4e,0x01,0x8b,0x63,0xa7,0x8b,0x63,0x05,0x02,0x63,0x01,0x01,
  0x63,0xff,0x4e,0x02,0x21,0xfe,0x21,0x02,0x05,0xff,0x70,0xe2,0x01,0x70,0xff,0x05,
  0x05,0x05,0x02,0x02,0x02,0x02,0x02,0x05,0x05,0x05,0xe3,0x02,0x02,0x02,0xec,0x02,
  0x02,0x02,0x02,0x02,0x03,0x4e,0x4e,0x4e,0x4e,0xff,0x01,0x8b,0x01,0x05,0x02,0xe3,
  0x05,0x00,0x00,0xf0,0x05,0x05,0x05,0xec,0x02,0x4e,0x4e,0x00,0x05,0x00,0x63,0x63,
  0x01,0x01,0x01,0x01,0xa7,0xa7,0xd3,0xd3,0xa4,0xa7,0xa7,0x63,0xe7,0xe7,0xe7,0x63,
  0x63,0x01,0xa7,0x01,0xe7,0x05,0x05,0x05,0x05,0x05,0xff,0x62,0x00,0x00,0x62,0xff,
  0xff,0x4e,0x03,0xd2,0x5a,0xd2,0xd2,0xd2,0xfa,0x03,0x03,0x4e,0x03,0x4e,0xa4,0xe1,
  0xe1,0xf9,0xa4,0xf9,0xa4,0xa7,0xa7,0xd3,0xd3,0x01,0x63,0x4e,0xff,0x01,0x01,0x8b,
  0x01,0xff,0xff,0x05,0xfe,0x5a,0x5a,0x02,0x05,0xf0,0x05,0x00,0xff,0x00,0x05,0x05,
  0x05,0x02,0x02,0xec,0x21,0x02,0x02,0x02,0x02,0x02,0xe3,0x02,0xec,0x02,0x02,0x02,
  0x02,0x02,0x02,0x05,0x05,0x05,0x4e,0xff,0x8b,0x8b,0xa4,0xd3,0x8b,0x00,0x05,0xec,
  0x05,0x00,0x05,0xec,0xec,0xec,0x02,0x05,0x00,0x62,0xe7,0x70,0x70,0xff,0x63,0xe7,
  0xe7,0x01,0x8b,0x01,0xa7,0xa7,0xa4,0xa4,0xa4,0x8b,0x8b,0x62,0x00,0x00,0x00,0x62,
  0xe7,0x8b,0x8b,0x8b,0xe7,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xe3,0x02,0x21,0x00,
  0xff,0x01,0x01,0x70,0x63,0xfa,0xed,0x03,0xfa,0xd3,0xd3,0xd3,0xd3,0xa4,0xe1,0xe1,
  0xf9,0xd3,0xe1,0xe1,0xa4,0xd3,0xa7,0xa7,0xa7,0x8b,0x8b,0xff,0xff,0x8b,0x8b,0x8b,
  0xa7,0x01,0xff,0x00,0x02,0xef,0xfe,0x21,0x21,0x21,0x02,0x05,0x70,0x00,0x05,0x02,
  0xec,0x21,0x21,0x21,0x21,0x21,0x02,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x02,0x02,
  0x02,0x02,0x03,0x4e,0x4e,0x4e,0x01,0x01,0x01,0xd3,0xd3,0x8b,0x8b,0x8b,0xe7,0x05,
  0xec,0x05,0x05,0x21,0x21,0x02,0x05,0x62,0x8b,0x01,0x70,0x62,0x62,0xe7,0x62,0x8b,
  0x8b,0xd3,0xa4,0xa4,0xa4,0xa4,0xf9,0xa7,0x01,0x70,0xe7,0x00,0x00,0x00,0x05,0x05,
  0xe7,0xe7,0x63,0x62,0x62,0x00,0x05,0x05,0x05,0xf0,0x05,0x21,0x21,0xfe,0x21,0x4e,
  0x00,0xe7,0x70,0x8b,0x8b,0x01,0x01,0xd3,0xa4,0xf9,0xa4,0xf9,0xf9,0xa4,0xa4,0xa4,
  0xd3,0xa4,0xf9,0xf9,0xe1,0xe1,0xf9,0x8b,0x01,0xa7,0x8b,0x63,0x70,0xa7,0xa7,0xa7,
  0x8b,0x01,0xff,0xff,0x00,0x02,0x05,0x05,0x02,0xe3,0x05,0x00,0x62,0x00,0x05,0x02,
  0x02,0xed,0x21,0x21,0x21,0x21,0x02,0x21,0x21,0x21,0x21,0x02,0x02,0x02,0x02,0x02,
  0x05,0x4e,0xff,0x63,0x70,0xff,0x70,0x70,0x01,0x01,0x8b,0x01,0xa7,0xa7,0x8b,0x01,
  0x00,0x05,0x02,0x21,0xd2,0x02,0x4e,0xe7,0x8b,0x8b,0x8b,0x01,0x62,0x62,0x01,0x8b,
  0xca,0xca,0x8b,0xca,0xa7,0xa4,0xd3,0x01,0x62,0xff,0x4e,0x4e,0x05,0x05,0x05,0x05,
  0x05,0x4e,0x00,0x8b,0x01,0x8b,0x62,0x4e,0x05,0x05,0x02,0xfe,0x21,0x5a,0x21,0x02,
  0x02,0x02,0x05,0xff,0xff,0xff,0x8b,0xf9,0xe1,0xe1,0xe1,0xe1,0xf9,0xf9,0xf9,0xf9,
  0xf9,0xf9,0xe1,0xe1,0xf9,0xa4,0xd3,0xa7,0x01,0x01,0xa7,0xd3,0xd3,0xd3,0xe1,0xa4,
  0x8b,0x01,0x01,0x62,0x00,0xff,0xff,0x62,0x62,0x70,0x62,0x62,0x00,0x05,0x05,0xec,
  0x21,0x21,0x21,0xfe,0x21,0x21,0xe3,0xe3,0x02,0x21,0xe3,0x05,0xe3,0xec,0x05,0x05,
  0xff,0xff,0x70,0x01,0x01,0x01,0x4e,0x4e,0x4e,0x4e,0xff,0x8b,0xa7,0xa4,0x8b,0x01,
  0x62,0x05,0x02,0x02,0x05,0x05,0x00,0x70,0x62,0x62,0xff,0x62,0x62,0x70,0x62,0x8b,
  0xca,0xd3,0x63,0xe7,0x01,0x8b,0x8b,0x01,0x70,0xff,0x62,0x62,0x05,0x05,0x00,0x00,
  0x00,0xff,0x62,0x62,0x62,0x62,0x62,0x62,0x62,0x05,0x02,0xfe,0x5a,0x5a,0x5a,0x5a,
  0xed,0x02,0x03,0x02,0x4e,0x4e,0x63,0xa7,0xe1,0xe1,0xe1,0xe1,0xf9,0xf9,0xf9,0xf9,
  0xf9,0xe1,0xe1,0xf9,0xd3,0x8b,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa7,0xa4,0xf9,0xa4,
  0x8b,0x01,0x01,0x62,0x00,0x05,0x00,0x00,0xff,0x62,0x00,0x00,0x00,0xf0,0x05,0x05,
  0xe3,0x21,0x21,0xfe,0xed,0xe3,0xe3,0xec,0xe3,0x21,0xe3,0x02,0x21,0x21,0x21,0x05,
  0xff,0x63,0x8b,0xa7,0xa7,0x01,0x70,0x00,0x00,0xff,0x4e,0x70,0xca,0xe1,0xd3,0x01,
  0x01,0x70,0x70,0x62,0x62,0x70,0x70,0x70,0x00,0x62,0xff,0xff,0x00,0x62,0x62,0x8b,
  0xa4,0xf9,0xa7,0x70,0x70,0x01,0x01,0x01,0x01,0x62,0xe7,0x70,0x62,0x62,0x70,0x01,
  0x70,0x62,0x62,0x8b,0x8b,0x8b,0x01,0x01,0x01,0x70,0x4e,0x02,0x5a,0x5a,0x21,0x02,
  0xed,0x02,0x03,0x02,0x02,0x03,0x4e,0x8b,0xd3,0xa4,0xa4,0xf9,0xf9,0xf9,0xf9,0xa4,
  0xe1,0xe1,0xf9,0xd3,0x8b,0xa7,0xa4,0xa4,0xd3,0xd3,0xd3,0x8b,0x8b,0xa7,0xd3,0xd3,
  0xe2,0x62,0xff,0x00,0x00,0x05,0xf0,0x00,0x00,0x00,0x05,0x00,0x00,0xe3,0x05,0x02,
  0x21,0xed,0x21,0x21,0x21,0x02,0xec,0x02,0x02,0x02,0x02,0x02,0x21,0x02,0x02,0x05,
  0xe7,0x8b,0xd3,0xd3,0xa4,0xd3,0x01,0x8b,0xe7,0x01,0x70,0x01,0xa4,0xe1,0xa7,0x01,
  0x62,0x70,0x70,0x70,0x70,0x62,0xff,0x62,0x62,0x00,0x00,0x62,0x62,0x62,0x62,0x8b,
  0xa4,0xf9,0xe1,0xca,0x62,0x62,0x8b,0x8b,0x01,0x8b,0x8b,0xca,0x8b,0x01,0x01,0x8b,
  0xca,0xca,0xca,0xd3,0xca,0xca,0x8b,0x8b,0x01,0x01,0x01,0xff,0x03,0x02,0x03,0x03,
  0x02,0x03,0xff,0x05,0xed,0xed,0x02,0x03,0x05,0xd3,0xa7,0xa4,0xf9,0xf9,0xd3,0xd3,
  0xd3,0xa4,0xa4,0xd3,0x8b,0xa7,0xd3,0xa4,0xd3,0xd3,0xd3,0x8b,0x8b,0x8b,0x8b,0x01,
  0xff,0x00,0x62,0x00,0x05,0x62,0x01,0x70,0x70,0x62,0x62,0xff,0x00,0x05,0x02,0x02,
  0xed,0x21,0x02,0x05,0xe3,0x00,0x05,0x05,0xf0,0x05,0xe3,0x05,0x02,0x05,0x05,0x4e,
  0x63,0xca,0x8b,0x8b,0xd3,0xd3,0x8b,0x8b,0xca,0x01,0x70,0x70,0x8b,0xa4,0xca,0xe2,
  0xe2,0x01,0x62,0x62,0x62,0x00,0x05,0x00,0x00,0x00,0x62,0xff,0x70,0x62,0x62,0x8b,
  0x8b,0xa4,0xa4,0xd3,0x01,0xa7,0xe1,0xd3,0xca,0x8b,0x8b,0x8b,0x8b,0xe2,0x01,0x01,
  0x8b,0xca,0xd3,0xd3,0x8b,0x8b,0x01,0x01,0x01,0x01,0x8b,0x63,0x4e,0x4e,0xcf,0x03,
  0x03,0x03,0x4e,0x05,0x02,0xed,0x02,0x02,0x4e,0xd3,0xd3,0xd3,0xa4,0xa4,0xd3,0xa4,
  0xa4,0xd3,0x8b,0x01,0x8b,0x01,0x8b,0x8b,0xca,0x8b,0x8b,0x01,0xca,0x01,0xff,0xff,
  0x62,0x00,0xff,0x00,0x62,0x8b,0x8b,0x01,0x01,0x8b,0x01,0x01,0x62,0xff,0xff,0x02,
  0x02,0x05,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0xff,0xff,0x00,0x4e,0x05,0x63,0x01,
  0x70,0x01,0x8b,0xa7,0x8b,0x8b,0xca,0xe2,0xca,0x8b,0x01,0x01,0xe2,0x8b,0x8b,0x70,
  0x70,0x62,0x62,0x00,0x05,0x05,0x05,0x05,0x05,0x05,0xf0,0x00,0x00,0x62,0x62,0x62,
  0x01,0xd3,0xa4,0xe1,0xe1,0xf9,0xa7,0xa4,0xa4,0xca,0x62,0x70,0x8b,0xca,0xca,0xca,
  0x8b,0xa7,0xd3,0xca,0x01,0x63,0x70,0xff,0x63,0x01,0x63,0xff,0x02,0x4e,0x4e,0x63,
  0x03,0x02,0x02,0x21,0x21,0x21,0x02,0x03,0xa7,0xa4,0xd3,0xd3,0xd3,0xd3,0xd3,0xd3,
  0xd3,0x8b,0x01,0x01,0x01,0x63,0x70,0x01,0x01,0x8b,0x8b,0x8b,0x8b,0x01,0xff,0x4e,
  0x05,0x05,0x05,0x00,0xe7,0x8b,0xca,0xca,0xca,0xca,0x01,0x62,0x62,0x01,0x01,0xff,
  0x4e,0xff,0x70,0x62,0xe7,0xe7,0x62,0x63,0x70,0xff,0xff,0xff,0xe7,0xe7,0xe7,0x01,
  0x01,0x8b,0xca,0x8b,0x01,0x8b,0x01,0x01,0x8b,0x01,0x01,0xe2,0x01,0xca,0xe2,0x62,
  0xff,0x00,0x62,0x05,0x05,0x00,0x00,0x05,0x00,0xf0,0x05,0x05,0xf0,0xf0,0x00,0x62,
  0x70,0xca,0xa7,0xa7,0x8b,0x8b,0x8b,0x8b,0x8b,0xa4,0x8b,0x01,0xd3,0xa7,0xa7,0xa4,
  0xd3,0xd3,0xa7,0xca,0x70,0xe7,0x4e,0x05,0x62,0xff,0x4e,0x03,0x03,0x4e,0x4e,0x4e,
  0x02,0x02,0x21,0xed,0x5a,0x5a,0xed,0xed,0x03,0xa7,0xca,0x8b,0xd3,0xd3,0xd3,0x8b,
  0xd3,0x8b,0x8b,0x01,0x01,0x70,0xff,0xff,0x62,0x01,0x62,0x01,0x8b,0x70,0x00,0x05,
  0xe3,0xec,0xec,0x05,0x70,0x62,0xff,0x00,0x70,0x01,0x8b,0x01,0x62,0xff,0xff,0x01,
  0x70,0x01,0x01,0x70,0x01,0x01,0x70,0x01,0x70,0x62,0x62,0x62,0xff,0x70,0x70,0x70,
  0x01,0x01,0x8b,0xe7,0x01,0x01,0x62,0x70,0x62,0x62,0x62,0x62,0x8b,0xa7,0x62,0x62,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xe3,0xe3,0x21,0xe3,0x02,0xf0,0xf0,0x05,
  0x00,0x00,0x62,0x70,0x62,0x70,0x62,0x62,0x8b,0xca,0xd3,0xd3,0x8b,0xca,0xd3,0xca,
  0x8b,0xa7,0x8b,0x70,0xe7,0xff,0x05,0x05,0x4e,0x4e,0x05,0x02,0x03,0x4e,0x4e,0x03,
  0x02,0xed,0x21,0x21,0x21,0x21,0x05,0xff,0x01,0xd3,0xd3,0xd3,0xa4,0xd3,0xd3,0xd3,
  0xa4,0xca,0x01,0x62,0x00,0x4e,0x00,0xff,0xff,0x00,0xff,0x62,0x62,0x01,0xff,0x4e,
  0x05,0xf0,0xf0,0xf0,0xff,0x62,0xff,0x8b,0x01,0x8b,0xe2,0x01,0xff,0x62,0x62,0x62,
  0x01,0xe7,0xff,0xff,0xff,0xe7,0x62,0xf0,0x00,0x00,0x00,0x62,0x62,0x62,0x62,0x70,
  0xe2,0x62,0x62,0x00,0x00,0x05,0x62,0x00,0x62,0x00,0x05,0x05,0x00,0x00,0xf0,0x00,
  0xff,0x00,0x00,0x00,0x00,0x05,0x05,0x05,0xf0,0x21,0x21,0x21,0x21,0x21,0x21,0xec,
  0x02,0x05,0x05,0x00,0xff,0x01,0x01,0x70,0xff,0xe7,0x01,0xa7,0x8b,0x01,0x01,0x8b,
  0x8b,0x01,0x01,0xe7,0xff,0x00,0x02,0x05,0x05,0x05,0x02,0x03,0x03,0x03,0x02,0x02,
  0x21,0x21,0xed,0x21,0x02,0x4e,0xd3,0xd3,0xa4,0xd3,0xa7,0x8b,0xd3,0xd3,0xd3,0x8b,
  0x01,0x70,0x62,0x00,0x00,0x05,0x4e,0x00,0x05,0x00,0xff,0xff,0xa7,0x01,0x70,0xff,
  0xff,0xff,0xff,0x05,0x05,0x4e,0x8b,0xd3,0xca,0xd3,0x8b,0x70,0x00,0x00,0xff,0x4e,
  0x62,0xff,0xff,0xff,0x00,0x00,0x00,0xe2,0x70,0x70,0x70,0x00,0xff,0x00,0xff,0x01,
  0x00,0x05,0x00,0x00,0x62,0xe3,0x02,0x02,0xec,0x02,0x02,0x21,0x02,0x00,0x01,0x70,
  0x01,0x70,0x05,0x05,0xe3,0xe3,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
  0x21,0xec,0x02,0x05,0xff,0xff,0x05,0x05,0x4e,0x63,0x8b,0x8b,0x8b,0x01,0x70,0x70,
  0x70,0x8b,0x70,0xff,0x4e,0x05,0x02,0x03,0x02,0x02,0x02,0x4e,0x03,0x03,0x02,0x02,
  0x21,0x21,0x21,0x02,0x03,0x4e,0xa7,0xca,0xd3,0xd3,0xa4,0xd3,0xa7,0xca,0x01,0x01,
  0x70,0x62,0xff,0x62,0x00,0x05,0x62,0x70,0xe2,0xe2,0x62,0xff,0xca,0x01,0x62,0x00,
  0x62,0x70,0x00,0x00,0x05,0x05,0x70,0x01,0x8b,0x8b,0xff,0x4e,0xff,0x70,0xff,0x00,
  0x00,0x00,0x05,0x05,0x21,0xec,0x62,0x62,0x62,0x00,0x05,0x05,0x05,0x00,0xff,0x00,
  0x02,0xe3,0x21,0x21,0x21,0x21,0x21,0x02,0x02,0x05,0x02,0x00,0x70,0x8b,0x8b,0x62,
  0x62,0xf0,0x21,0xf0,0x21,0x21,0xfe,0xfe,0xfe,0xfe,0x21,0x21,0xfe,0xfe,0xfe,0xfe,
  0x21,0x21,0x21,0x21,0x02,0xec,0xec,0xec,0x05,0x05,0x63,0xff,0x63,0xe7,0xe7,0xff,
  0xff,0x70,0xe7,0x63,0xff,0x4e,0x4e,0x03,0x02,0xfa,0x03,0x02,0x02,0x03,0x02,0x02,
  0xed,0xfe,0x5a,0x21,0x02,0x03,0xe7,0xa7,0xd3,0xd3,0xd3,0xe1,0xd3,0xca,0xe2,0xe2,
  0x8b,0x01,0x62,0xff,0x70,0x01,0x70,0x62,0x01,0x62,0x62,0x70,0x8b,0x01,0xff,0xff,
  0xff,0x62,0x05,0x05,0xf0,0xf0,0x00,0x62,0x70,0x00,0x05,0x05,0x00,0x62,0x05,0x05,
  0x02,0x21,0x21,0x5a,0xfe,0x21,0x21,0xf0,0xf0,0xe3,0xe3,0x02,0xe3,0xe3,0xec,0x02,
  0x21,0x5a,0x5a,0x5a,0x5a,0x5a,0xfe,0x21,0xec,0x05,0x70,0xd3,0xe2,0x01,0xff,0x05,
  0x21,0xfe,0xfe,0x8b,0xe2,0x05,0xef,0xef,0xef,0xef,0xfe,0x5a,0xfe,0xfe,0xfe,0xfe,
  0x5a,0xfe,0x5a,0xfe,0xfe,0x21,0xec,0x21,0xe3,0x05,0x00,0x4e,0x05,0x00,0x4e,0x05,
  0x00,0xe7,0x63,0x63,0x63,0x63,0x4e,0x03,0xfa,0x03,0x03,0xed,0x03,0xfa,0x02,0x02,
  0xed,0x5a,0x5a,0xfe,0xed,0x4e,0xe7,0xd3,0xa4,0xca,0xca,0xca,0xd3,0xd3,0xca,0x01,
  0x70,0x62,0x70,0xff,0x62,0x70,0xff,0x70,0x63,0x00,0x01,0x01,0xe2,0x70,0x00,0xf0,
  0x00,0x05,0x02,0x02,0xf0,0x05,0x62,0xff,0x00,0x05,0x05,0x00,0x8b,0x70,0x02,0xec,
  0x21,0x5a,0xfe,0xfe,0x5a,0x5a,0x5a,0x5a,0x21,0x21,0x21,0x21,0x21,0x5a,0x5a,0x21,
  0x00,0xe3,0x21,0x02,0x05,0xff,0x70,0x62,0xff,0x00,0x62,0xca,0x62,0x00,0x05,0xe3,
  0x21,0xfe,0x21,0x70,0x62,0xe2,0xe2,0x62,0x21,0xef,0xef,0xef,0xef,0xef,0xef,0xfe,
  0xef,0xef,0xfe,0xef,0xef,0xfe,0xfe,0x21,0x21,0x21,0x02,0x02,0x05,0x00,0xff,0x4e,
  0x63,0x63,0x00,0x4e,0x4e,0x03,0x03,0x03,0x03,0xf7,0x03,0xed,0x03,0x4e,0xcf,0xcf,
  0x63,0x03,0x5a,0x5a,0xed,0x02,0x02,0x8b,0xca,0xe2,0xe2,0xca,0xd3,0xd3,0xd3,0xca,
  0x01,0xff,0x01,0x01,0xe7,0xff,0xff,0x70,0xff,0x62,0xe2,0x70,0x70,0x70,0xff,0x05,
  0x05,0x00,0x00,0x70,0x00,0x00,0x00,0xf0,0x02,0xe3,0x05,0x70,0xe2,0x62,0x00,0x21,
  0x21,0x21,0x21,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x21,0x02,0xfe,0x5a,0xed,0xe3,0x05,
  0x70,0x62,0x70,0x62,0xe2,0xe2,0x01,0x62,0x70,0x05,0x05,0x62,0x00,0xe3,0x05,0xe3,
  0xfe,0xfe,0xe3,0x00,0x00,0x70,0xca,0xca,0x70,0xf6,0xef,0xef,0xef,0xf4,0xef,0xef,
  0xef,0xef,0xef,0xef,0xf3,0xfe,0xef,0xfe,0x5a,0x21,0x21,0x5a,0xed,0x02,0x4e,0x63,
  0x4e,0x4e,0x03,0x03,0x03,0x03,0x03,0xd2,0xf7,0x03,0xfa,0x05,0x03,0x03,0x03,0x02,
  0x03,0xcf,0xcf,0x03,0x03,0xed,0x21,0x05,0x00,0x62,0x8b,0xe2,0x8b,0xca,0x8b,0x01,
  0x62,0x63,0x70,0x62,0xff,0x00,0xff,0x62,0x70,0x62,0x62,0x62,0x70,0xff,0x70,0x70,
  0x00,0x70,0xca,0xca,0x70,0xca,0xff,0x4e,0x00,0x00,0xff,0x8b,0x01,0x62,0x00,0x02,
  0x02,0x21,0x5a,0x5a,0x5a,0x5a,0x21,0x00,0xff,0x01,0xca,0xd3,0x00,0x05,0x05,0x62,
  // 11: sky 1, part D
  0x7b,0xbe,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,0x00,0x01,0xed,0x83,0x08,0x01,
  0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,
  0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,
  0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,
  0x10,0x07,0x08,0x4b,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,
  0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,
  0x08,0x23,0x10,0x6c,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,
  0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,
  0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,
  0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,
  0x00,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,
  0x08,0x45,0x10,0x45,0x99,0x95,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,
  0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,0x18,0x60,
  0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,
  0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x18,0x64,0x00,0x65,
  0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,
  0x08,0x67,0x14,0x9d,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,
  0x00,0x81,0x08,0x81,0x10,0x81,0xb9,0xa5,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,
  0x18,0x82,0x00,0x83,0x08,0x83,0x73,0x7c,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,
  0xd5,0x84,0x10,0x84,0xfa,0xa5,0x9b,0xc6,0x18,0x84,0x00,0x85,0x18,0x85,0x08,0x85,
  0x10,0x85,0x3a,0xb6,0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,
  0x10,0x87,0x18,0x87,0x00,0xa0,0x7b,0xc6,0x08,0xa0,0x0a,0x7c,0x10,0xa0,0x18,0xa0,
  0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,
  0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x00,0xa4,0x08,0xa4,0x1b,0xae,0x10,0xa4,
  0x18,0xa4,0x88,0x5b,0x00,0xa5,0x08,0xa5,0x37,0x8d,0x10,0xa5,0xfc,0xd6,0x18,0xa5,
  0xb3,0x84,0x69,0x5b,0x00,0xa6,0xeb,0x7b,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,
  0x08,0xa7,0x77,0x9d,0x10,0xa7,0x18,0xa7,0x00,0xc0,0xd3,0x94,0x08,0xc0,0x10,0xc0,
  0x18,0xc0,0x00,0xc1,0xb4,0x7c,0xc8,0x63,0x4b,0x4a,0x08,0xc1,0x35,0x8d,0x10,0xc1,
  0xe9,0x5a,0x18,0xc1,0x12,0x6c,0x00,0xc2,0x08,0xc2,0x10,0xc2,0xe7,0x42,0x2a,0x32,
  0xf8,0xad,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x26,0x74,0x10,0xc3,0x18,0xc3,0x3d,0xd7,
  0x96,0xad,0x6c,0x63,0x00,0xc4,0xe7,0x6b,0x07,0x6c,0x08,0xc4,0x10,0xc4,0x29,0x32,
  0x18,0xc4,0x69,0x63,0x89,0x62,0xc8,0x6a,0x00,0xc5,0x08,0x53,0x8b,0x5b,0x86,0x42,
  0xa1,0xab,0x9b,0x9b,0x9b,0x9b,0x9b,0x00,0xab,0x9b,0x9a,0x5a,0x8b,0x8b,0x8b,0xd1,
  0xd1,0xc4,0xd1,0xa1,0x00,0xa1,0xa1,0xa1,0xab,0x00,0x8b,0xc4,0xc4,0xc4,0xc4,0xc4,
  0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0x5a,0x8b,0xbe,0xbe,
  0xbe,0xbe,0xbe,0x9a,0x8b,0x5a,0x8b,0x8b,0x9a,0xbe,0xbe,0xbe,0xbe,0xbe,0xbe,0x9a,
  0x8b,0x8b,0xbe,0x00,0x9a,0x8b,0x8b,0x9a,0xa1,0xa1,0x00,0x9b,0x9b,0x9b,0x00,0xab,
  0x00,0x00,0xa1,0x00,0xa1,0x00,0x00,0x00,0x00,0xab,0x00,0xab,0x9b,0x00,0x9b,0x9b,
  0x9b,0xc6,0xc6,0x9b,0xab,0x9b,0x00,0x00,0x00,0x00,0x00,0xab,0x9b,0x9b,0x9b,0x00,
  0x8b,0x5a,0x8b,0x8b,0xa1,0x9a,0xa1,0xc6,0xc6,0xef,0xef,0xc6,0x9b,0x9b,0xab,0xab,
  0x00,0x9b,0xab,0x9b,0xab,0x9b,0xc6,0x9b,0x9b,0x9b,0x9b,0x9b,0xa1,0xc4,0xe8,0xc4,
  0x98,0x98,0xa1,0xa1,0x00,0x9b,0xa1,0xe8,0xe8,0xe8,0xe8,0x8b,0x98,0x98,0x98,0x98,
  0xc4,0x98,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0x00,0xef,0xef,0xef,0xc6,
  0x00,0xbe,0xbe,0x9a,0x9a,0x5a,0x5a,0x8b,0x9a,0xbe,0xbe,0xbe,0xbe,0xbe,0x9a,0x9a,
  0x8b,0x8b,0x8b,0xbe,0xbe,0x9a,0x9a,0x9a,0x9a,0xa1,0x9a,0xa1,0x00,0xab,0xa1,0xa1,
  0xa1,0xa1,0xa1,0x00,0x00,0x00,0x00,0x00,0x00,0xab,0x00,0xab,0x9b,0x9b,0xc6,0x9b,
  0x9b,0xc6,0xc6,0x9b,0x00,0xab,0x00,0x00,0xa1,0xa1,0x00,0x9b,0x9b,0xab,0xab,0x8b,
  0xc4,0xef,0xef,0xef,0xef,0xef,0x9b,0x8b,0x9b,0xc6,0xef,0xc6,0xc6,0xc6,0xab,0x00,
  0x9b,0x9b,0x00,0xab,0x00,0xab,0xab,0xab,0xc6,0xc6,0x9b,0x9b,0xc6,0x00,0xc4,0xc4,
  0x98,0xa1,0x00,0xa1,0xe8,0xa1,0xe8,0xd1,0xd1,0xd1,0x98,0x98,0x98,0x98,0x98,0x98,
  0x98,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0x9a,0xc6,0xc6,0xef,0xef,0xc6,0x9b,0xc6,
  0x00,0xa1,0xbe,0xbe,0x9a,0x9a,0xbe,0x8b,0x8b,0xbe,0xbe,0x9a,0x9a,0x9a,0x9a,0x9a,
  0x8b,0x8b,0x9a,0xbe,0x00,0x9a,0x9a,0x8b,0x9a,0x9a,0x8b,0x9a,0xa1,0x00,0xa1,0x9a,
  0xe8,0x9a,0xa1,0x00,0xa1,0x00,0xa1,0x00,0x00,0x00,0xab,0x9b,0x9b,0x9b,0x9b,0xc6,
  0xef,0xef,0xc6,0xab,0xab,0xab,0x9b,0x00,0xa1,0xa1,0x00,0xab,0xc6,0x9a,0x5a,0x8b,
  0x9b,0xef,0xef,0xef,0xef,0xef,0xc6,0x9b,0xef,0xef,0xef,0xef,0xef,0xc6,0xc6,0x9b,
  0xa1,0x00,0x9b,0xab,0x00,0xa1,0x00,0x9b,0x9b,0x9b,0x9b,0x00,0xa1,0xa1,0x5a,0x5a,
  0x8b,0x00,0xa1,0xa1,0xe8,0x00,0xe8,0xd1,0xc4,0x98,0x98,0x98,0x98,0x98,0x98,0x98,
  0x98,0x98,0x98,0xc4,0xc4,0xc4,0xc4,0xc4,0x9b,0xef,0xef,0xef,0xc6,0xc6,0xab,0x9b,
  0x00,0x9a,0xa1,0x9a,0x9a,0xbe,0xbe,0xbe,0x9a,0x9a,0x8b,0x5a,0x5a,0x8b,0x9a,0x9a,
  0x9a,0xbe,0xbe,0xbe,0xbe,0x9a,0x5a,0x5a,0x8b,0x9a,0x8b,0x8b,0x9a,0x00,0x9a,0x9a,
  0x9a,0xa1,0xa1,0x00,0x00,0x9a,0x9a,0xab,0x9b,0x9b,0x9b,0xc6,0xc6,0x9b,0xab,0xc6,
  0xef,0xc6,0xab,0x00,0x9b,0x9b,0x9b,0xab,0x00,0x9a,0xa1,0x9b,0x9b,0x00,0x9b,0xc6,
  0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xc6,0x9b,
  0x5a,0xa1,0xa1,0xe8,0x9a,0x9a,0xe8,0xa1,0xa1,0x00,0x9b,0xe8,0x5a,0xc4,0x5a,0x5a,
  0xc4,0x8b,0x9b,0xc6,0x9b,0x9b,0x00,0xa1,0xc4,0x98,0xc4,0xc4,0xd1,0xde,0x98,0x98,
  0x5a,0x9b,0x9b,0x8b,0xc4,0xc4,0xc4,0x5a,0xef,0xc6,0xc6,0x9b,0x00,0x9b,0x00,0x00,
  0xa1,0xbe,0xa1,0xbe,0x9a,0xbe,0x9a,0x9a,0x9a,0x9a,0x8b,0x5a,0x5a,0x5a,0x8b,0x9a,
  0xbe,0xbe,0xbe,0x9a,0x9a,0x8b,0x5a,0x5a,0x8b,0x8b,0x8b,0x9a,0xab,0x9b,0x9b,0x00,
  0xa1,0xa1,0xa1,0x00,0xa1,0xa1,0x00,0xab,0xc6,0xc6,0xc6,0x9b,0x9b,0x9b,0x00,0x00,
  0xa1,0x00,0xa1,0x00,0x00,0x9b,0x00,0x00,0xab,0x9a,0x9a,0x9b,0x9b,0x9b,0xef,0xef,
  0xef,0xc6,0xc6,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xc6,0xc6,0xc6,0x9b,0xab,0x00,
  0xc4,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x8b,0x00,0x00,0xe8,0x8b,0x5a,0x5a,0xc4,
  0x5a,0x9b,0xc6,0xc6,0xc6,0xab,0x9b,0x00,0x98,0xc4,0x00,0x9b,0xab,0xc6,0xa1,0x5a,
  0xef,0xef,0xc6,0xc6,0x5a,0xd1,0xa1,0xe8,0x9b,0xc6,0xab,0x00,0x9b,0x00,0xa1,0xa1,
  0xa1,0x9a,0x9a,0xbe,0x9a,0x8b,0x9a,0x9a,0x8b,0x8b,0x8b,0x5a,0x5a,0x5a,0x5a,0x8b,
  0x9a,0x9a,0x9a,0xbe,0xbe,0x9a,0x8b,0x8b,0x9a,0xa1,0x8b,0xa1,0x9b,0xab,0x9b,0x9b,
  0xa1,0x00,0xa1,0x00,0xa1,0x00,0x9b,0x9b,0x9b,0x9b,0x00,0xab,0xab,0x9b,0x9b,0xc6,
  0xef,0xef,0xc6,0x9b,0x00,0x00,0x00,0x00,0xa1,0x8b,0xab,0xc6,0x9b,0xc6,0xc6,0xc6,
  0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x9b,0xc6,0xef,0xef,0xc6,0xef,0xef,0x9b,0x00,0xa1,
  0xa1,0x5a,0x5a,0x8b,0x8b,0x5a,0x5a,0x5a,0x5a,0x00,0x00,0xab,0xa1,0x5a,0x5a,0x9a,
  0x9b,0xc6,0x9b,0x9b,0x00,0xab,0x00,0xab,0x9b,0x00,0x00,0xab,0xab,0x9b,0x9b,0x9b,
  0xc6,0xc6,0xc6,0xc6,0xab,0xab,0xc6,0xef,0xc6,0x9b,0x00,0xa1,0xa1,0xe8,0x9a,0x9a,
  0x9a,0x8b,0x8b,0x9a,0x9a,0x9a,0xbe,0x9a,0x9a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,
  0x9a,0x8b,0x8b,0xbe,0x8b,0x9a,0x8b,0x9a,0x00,0x00,0x00,0x00,0x00,0xa1,0x00,0x00,
  0xa1,0x00,0xa1,0xa1,0x00,0x00,0x9b,0x9b,0x9b,0xab,0x00,0x00,0x9b,0xef,0xef,0xef,
  0xef,0xef,0xef,0xc6,0x00,0x00,0xa1,0x00,0xa1,0x00,0xc6,0x9b,0xc6,0xef,0xef,0x9b,
  0x9b,0x9b,0x9b,0x9b,0xc6,0xef,0xef,0xef,0xef,0xef,0xc6,0xef,0xc6,0x9b,0x00,0xa1,
  0x9a,0x5a,0x8b,0x9a,0xe8,0x9a,0xe8,0x8b,0x00,0x9b,0x00,0x00,0x00,0xa1,0xe8,0xa1,
  0xa1,0xab,0xab,0x00,0xe8,0xa1,0xab,0x9b,0x00,0x00,0xab,0x00,0x9b,0xab,0x9b,0x9b,
  0xc6,0xc6,0xab,0x9b,0x9b,0xc6,0xc6,0xab,0xab,0xab,0xe8,0x00,0x00,0xe8,0xe8,0xe8,
  0x8b,0x8b,0x9a,0xe8,0x8b,0x9a,0xbe,0x9a,0x5a,0x5a,0x5a,0x8b,0x5a,0x5a,0x9a,0x8b,
  0x8b,0x5a,0x5a,0x5a,0x5a,0x5a,0x9a,0xa1,0x00,0x00,0xa1,0x00,0x00,0xa1,0x9a,0xa1,
  0x00,0x00,0xa1,0x00,0x00,0x9b,0xab,0xab,0xab,0x00,0xa1,0x00,0xc6,0xef,0xef,0xc6,
  0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00,0x00,0x00,0x9b,0x9b,0xc6,0xef,0xc6,0x9b,0x9b,
  0x9b,0x9b,0x9b,0x9b,0xc6,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0x9b,0x00,0x00,
  0xa1,0x5a,0x5a,0x8b,0x9a,0x9a,0x8b,0xa1,0xab,0xa1,0xa1,0x9a,0xa1,0x9a,0x8b,0xe8,
  0xe8,0xe8,0xe8,0x8b,0xe8,0x00,0x9b,0x00,0xab,0xc6,0xc6,0x00,0xe8,0xe8,0xa1,0x00,
  0xab,0xab,0xab,0x00,0xab,0xab,0x9b,0xab,0xab,0xa1,0xe8,0xa1,0xa1,0x8b,0x8b,0xe8,
  0xd1,0xe8,0xe8,0x8b,0x8b,0xd1,0x5a,0x8b,0x8b,0x5a,0x5a,0x8b,0x9a,0x8b,0xbe,0x5a,
  0x5a,0x5a,0x5a,0x5a,0xd1,0x8b,0xa1,0xa1,0x00,0xab,0x00,0xa1,0xa1,0xa1,0x9a,0xa1,
  0x00,0xa1,0xa1,0x00,0xa1,0x00,0x00,0x9b,0x00,0xa1,0xa1,0x00,0x9b,0x9b,0x9b,0x9b,
  0xc6,0xc6,0x9b,0x9b,0x00,0xa1,0x00,0x00,0x00,0x00,0xc6,0x00,0x9b,0xab,0x9b,0xab,
  0x9b,0xab,0x9b,0xc6,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xc6,0x9b,0x00,0x9b,
  0xa1,0x5a,0x5a,0x5a,0x5a,0x8b,0x5a,0x8b,0x8b,0x8b,0x8b,0x5a,0x8b,0x5a,0xc4,0x5a,
  0xd1,0x8b,0xe8,0x8b,0xe8,0xe8,0xe8,0xe8,0x00,0x00,0xab,0xe8,0xe8,0xe8,0xe8,0xa1,
  0xab,0x00,0xab,0xab,0xab,0x9b,0x9b,0x9b,0xa1,0xe8,0xe8,0xe8,0x8b,0xe8,0xe8,0xd1,
  0xd1,0xd1,0x8b,0x8b,0xd1,0xd1,0x5a,0x5a,0x5a,0x5a,0xc4,0x5a,0x5a,0x8b,0x5a,0x5a,
  0x5a,0x8b,0x8b,0x8b,0x8b,0x9a,0x9a,0x9a,0xa1,0x00,0x00,0x00,0x00,0xbe,0xa1,0x00,
  0x00,0x00,0xab,0x9b,0xa1,0xa1,0x00,0xa1,0x00,0xa1,0x9b,0xc6,0xc6,0x9b,0x9b,0x9b,
  0x9b,0x9b,0x00,0x00,0xa1,0x9a,0xa1,0xa1,0xa1,0x9a,0xa1,0x9b,0x00,0xc6,0x9b,0x9b,
  0x00,0x00,0x9b,0xef,0xef,0xc6,0xc6,0xc6,0xc6,0x9b,0x9b,0x9b,0x9b,0xab,0x00,0xa1,
  0x9a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0xc4,0xc4,0xc4,
  0xd1,0xc4,0xd1,0xe8,0xe8,0xe8,0x8b,0xe8,0x8b,0x8b,0x8b,0xe8,0xe8,0xe8,0x8b,0xe8,
  0xe8,0xe8,0xe8,0x00,0xe8,0xe8,0xab,0x00,0xe8,0xe8,0xe8,0x8b,0xe8,0x8b,0xd1,0xd1,
  0xd1,0xc4,0xd1,0xd1,0x5a,0xc4,0x5a,0x5a,0xd1,0x5a,0xc4,0xc4,0x5a,0x8b,0x5a,0x5a,
  0x8b,0x5a,0x5a,0x5a,0x8b,0x8b,0x9a,0x9a,0x9a,0xa1,0xa1,0xa1,0x00,0x00,0xa1,0x00,
  0x00,0x00,0x00,0x00,0xa1,0xbe,0xbe,0xbe,0xbe,0xa1,0x00,0x00,0x00,0xa1,0xa1,0x9b,
  0x9b,0x9b,0x00,0xa1,0xa1,0x00,0xa1,0xa1,0x00,0xa1,0x00,0x00,0x00,0x9b,0x9b,0x9b,
  0xa1,0x00,0x00,0x9b,0x9b,0xc6,0x9b,0x9b,0xab,0x00,0xa1,0x00,0xab,0x00,0xc6,0xab,
  0x8b,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0xc4,
  0xc4,0x8b,0xab,0xab,0x8b,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0x8b,
  0x8b,0xe8,0xe8,0xab,0xab,0xe8,0xe8,0xab,0xab,0xab,0xe8,0x8b,0xe8,0xe8,0xe8,0xe8,
  0xab,0xe8,0x8b,0xe8,0xc4,0xc4,0xc4,0xd1,0xd1,0xc4,0xc4,0x5a,0x5a,0x8b,0x8b,0x8b,
  0x8b,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x8b,0x9a,0x9a,0x9a,0xa1,0x00,0x00,0x00,
  0x9b,0x00,0xa1,0xa1,0xa1,0xbe,0xbe,0xa1,0xbe,0x9a,0xbe,0xa1,0xa1,0xa1,0x00,0x00,
  0x00,0x00,0x00,0xa1,0x00,0x00,0x00,0xa1,0x00,0xa1,0x00,0x00,0x00,0x00,0x9b,0x9b,
  0x00,0xa1,0xa1,0xa1,0xa1,0x9b,0x9b,0x9b,0x00,0xa1,0xa1,0xa1,0x9a,0x9a,0x00,0xa1,
  0x5a,0xa1,0x8b,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0xc4,
  0x98,0xd1,0xab,0x9b,0xe8,0xd1,0xd1,0x81,0xd1,0x81,0xe8,0xd1,0xd1,0xe8,0xe8,0xe8,
  0x8b,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0x8b,0x8b,0xe8,0xe8,0xe8,
  0xe8,0xe8,0xe8,0x8b,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0xde,0x81,
  0xd1,0xd1,0xd1,0xd1,0x5a,0x5a,0x5a,0xd1,0xd1,0xc4,0x5a,0x8b,0x8b,0x8b,0x9a,0x00,
  0xa1,0xa1,0x00,0xbe,0xbe,0x9a,0xbe,0x00,0xbe,0xbe,0xa1,0x00,0x00,0xa1,0xa1,0x00,
  0xbe,0x00,0xbe,0xa1,0x00,0xa1,0x00,0x00,0x00,0x00,0x00,0xbe,0x00,0x9b,0x00,0x00,
  0x00,0x9a,0x8b,0x9a,0xa1,0xa1,0x00,0x9b,0xa1,0xa1,0xa1,0xa1,0x00,0x9a,0xa1,0x9a,
  0xa1,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0xc4,
  0xc4,0xa1,0xab,0x9b,0xe8,0xd1,0xd1,0xd1,0xc4,0xd1,0xd1,0x81,0xd1,0xd1,0xd1,0xd1,
  0xe8,0xe8,0xf0,0xd1,0x8b,0xe8,0xe8,0xe8,0xe8,0xf0,0x8b,0xe8,0xe8,0xe8,0xe8,0xf0,
  0xf0,0x8b,0xd1,0xde,0x98,0xc4,0xc4,0x98,0x98,0xde,0xde,0x93,0x93,0x93,0x93,0x93,
  0xc8,0xde,0x81,0xda,0x93,0xda,0xde,0xde,0xe8,0xde,0xe8,0xde,0xde,0xde,0xde,0xde,
  0xde,0xde,0xde,0xd1,0xde,0xde,0xf0,0xe8,0xe8,0xe8,0x9a,0xa1,0x00,0xa1,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0xa1,0xa1,0xab,0xab,0x00,0x00,0xa1,0x00,0x00,0xa1,0xa1,
  0xa1,0x9a,0x8b,0x8b,0x8b,0x9a,0xa1,0xe8,0x9a,0xa1,0xa1,0xa1,0x00,0x00,0xa1,0xa1,
  0xa1,0xa1,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x5a,0x8b,0x5a,0x8b,0x5a,0xc4,
  0xde,0xd1,0xde,0xde,0x98,0xda,0xda,0xda,0xe8,0xf0,0x81,0x81,0x81,0x81,0xe8,0x81,
  0xda,0xda,0xc8,0xe2,0x93,0x93,0xda,0xde,0xd1,0xd1,0xe8,0xe8,0xe8,0x8b,0xd1,0xde,
  0xda,0x93,0xe2,0x93,0xe2,0x93,0x31,0xe2,0xe2,0x93,0xde,0xc8,0x93,0xde,0x93,0xda,
  0x93,0xda,0xde,0xde,0xde,0xda,0xde,0xde,0x98,0x98,0x98,0xde,0x98,0x8b,0xf0,0x81,
  0xf0,0xde,0xde,0x98,0xc4,0x98,0x98,0x81,0xe8,0xe8,0xe8,0x98,0x98,0xde,0xde,0xde,
  0xde,0x98,0xc4,0xde,0x98,0xc4,0xde,0xde,0xd1,0xf0,0xe8,0xe8,0xe8,0x00,0xa1,0xa1,
  0x9a,0xa1,0xa1,0x8b,0x8b,0x9a,0x8b,0xe8,0xa1,0x00,0x9a,0xc4,0xde,0xde,0xda,0x8b,
  0xda,0xde,0xda,0xda,0x98,0x98,0xde,0xde,0x93,0x98,0x98,0x93,0x93,0xe2,0xe2,0xe2,
  0xc8,0x93,0x93,0x93,0x31,0x93,0xe2,0xe2,0x31,0x98,0xde,0xde,0x98,0x98,0x98,0x93,
  0x93,0x93,0x93,0x93,0x93,0x31,0xe2,0x31,0xe2,0x31,0x93,0x93,0x93,0x31,0xe2,0xe2,
  0xe2,0x93,0x93,0xc8,0xc8,0xde,0xde,0x93,0x93,0x31,0xe2,0x31,0x93,0xe2,0xe2,0x93,
  0x93,0x93,0x93,0x93,0xda,0x98,0x93,0x98,0x93,0x98,0xda,0x98,0xda,0x98,0x98,0x98,
  0x98,0xde,0xde,0xde,0xde,0x98,0x98,0x98,0xde,0xde,0xde,0xde,0xde,0xde,0x98,0x98,
  0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,0x98,0xde,0xf0,0xa1,0xa1,0x00,
  0x00,0xe8,0x00,0xe8,0xe8,0xf0,0xde,0xde,0xde,0x93,0xda,0xf0,0xf0,0xde,0xde,0xc8,
  0x93,0x93,0x93,0xc8,0x81,0xc8,0xc8,0xc8,0x81,0xc8,0xc8,0xf0,0xe8,0xf0,0x81,0xf0,
  0x93,0x81,0x81,0x93,0xc4,0xc4,0x81,0x81,0x93,0xc4,0xc4,0x98,0x98,0xef,0xbe,0x98,
  0xe2,0x31,0x31,0x93,0x93,0x31,0x93,0xc8,0xc8,0x93,0x31,0xc8,0xc8,0x93,0xe2,0xe2,
  0x93,0x93,0xc8,0xc8,0xc8,0x93,0x81,0xd5,0x93,0x31,0x31,0x31,0x31,0x31,0xe2,0xe2,
  0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0xda,0xda,0x98,0x98,0x98,
  0x98,0x98,0xda,0x98,0xde,0xde,0xde,0x98,0x98,0xde,0x98,0xde,0xde,0xde,0x98,0xde,
  0x98,0x98,0xde,0x98,0xde,0x98,0x98,0xde,0xde,0xde,0x98,0x98,0x98,0xda,0x98,0x98,
  0xde,0xde,0xde,0x93,0xe2,0xe2,0xde,0xc8,0x81,0xd5,0x81,0xde,0xc8,0xda,0x93,0x93,
  0x31,0x93,0x93,0x93,0x81,0xf0,0xa1,0xf0,0xa1,0xf0,0xd5,0x81,0xde,0xde,0x81,0x81,
  0x31,0xd5,0xc8,0x93,0x98,0x93,0x98,0xd5,0xc8,0x98,0x93,0x93,0x98,0xef,0x00,0xe8,
  0xf0,0xd5,0x31,0x31,0x93,0xd5,0xde,0x31,0xe2,0x31,0x31,0x31,0xd5,0xd5,0xd5,0x93,
  0x93,0xd5,0xd5,0x81,0xd5,0xc8,0x93,0x93,0x93,0x31,0x31,0x31,0x31,0x93,0x93,0x93,
  0x93,0x93,0x93,0xc8,0xc8,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x98,0xda,0xda,0xda,
  0xda,0xda,0xda,0xda,0xda,0x98,0xde,0x98,0xde,0xde,0xde,0xde,0xde,0xde,0xde,0x8b,
  0xde,0x98,0xde,0xde,0x81,0xde,0xde,0xde,0xde,0x93,0x93,0xe2,0xe2,0xe2,0x93,0xe2,
  0xe2,0xe2,0xe2,0xe2,0xe2,0x31,0xc8,0xda,0xde,0x81,0x81,0xc8,0xda,0xde,0xde,0xda,
  0x81,0xc8,0xc8,0xde,0xc8,0x93,0xd5,0x81,0xd5,0xe8,0xf0,0xc8,0xc8,0xd5,0xd5,0x93,
  0x31,0x93,0x98,0xc4,0x98,0x93,0x93,0x93,0x93,0xc4,0x98,0x98,0xbe,0x9a,0xc4,0xc4,
  0x5a,0x81,0x81,0xd5,0x31,0x31,0xd5,0xd5,0x31,0x31,0x31,0x31,0x93,0x93,0x31,0x31,
  0x31,0x93,0x93,0x31,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x31,0x93,0x93,0x31,
  0x93,0x93,0x93,0xc8,0xd5,0x81,0x81,0x81,0xd5,0x81,0x81,0xde,0xc8,0xc8,0xde,0xda,
  0xda,0xda,0xda,0x98,0xde,0x98,0xda,0x98,0xde,0x98,0xde,0xde,0xde,0xde,0xd5,0xde,
  0xde,0xc8,0xda,0xe2,0x93,0xe2,0xe2,0xe2,0xe2,0x93,0x93,0xc8,0xc8,0x93,0x31,0x93,
  0x31,0xc8,0xd5,0x93,0x93,0xf0,0x81,0x81,0x81,0xde,0x81,0xe8,0x81,0x81,0xf0,0xf0,
  0x81,0xde,0xd5,0x81,0xf0,0xf0,0xe8,0x81,0x81,0xd5,0xd5,0xc8,0x93,0xc8,0xc8,0x31,
  0xc8,0xe8,0xd5,0x93,0x93,0x93,0x98,0xd1,0xd1,0x98,0x98,0x93,0x98,0x93,0x72,0x9e,
  0x9e,0x98,0x93,0x81,0x81,0xd5,0x31,0xc8,0x93,0x93,0x93,0x98,0x98,0xc4,0xc4,0x93,
  0x98,0x98,0x93,0x93,0x31,0xc8,0x81,0x81,0xd5,0xde,0xde,0x81,0x81,0x81,0xd5,0xd5,
  0x93,0xd5,0x93,0x81,0x81,0x81,0x81,0xd5,0x81,0x81,0x81,0x81,0x81,0xde,0x81,0xf0,
  0xf0,0xa1,0xe8,0xf0,0x81,0xde,0x81,0xde,0xde,0x31,0x93,0x93,0x93,0xc8,0xde,0x93,
  0xda,0xda,0xe2,0xe2,0xe2,0xe2,0xe2,0x31,0xde,0x81,0x81,0xde,0x81,0x81,0xf0,0xf0,
  0x81,0xde,0x98,0x98,0xc8,0x93,0x31,0xde,0x93,0x93,0xd5,0xc8,0x31,0xc8,0xd5,0xf0,
  0xc8,0x81,0xf0,0x93,0x81,0x81,0xd5,0xf0,0xd5,0x81,0x93,0xd5,0xc8,0xc8,0x93,0x31,
  0x31,0x93,0x98,0x93,0x31,0xe2,0x8b,0xc6,0xc6,0x93,0x93,0xe2,0x51,0x51,0x51,0xc4,
  0x98,0xc4,0xc4,0x81,0x81,0xf0,0xf0,0xc8,0xde,0xc8,0x98,0xc4,0xc4,0x9a,0xbe,0x5a,
  0x5a,0x9a,0x93,0xa1,0x8b,0xe8,0xe8,0x5a,0x98,0xc4,0xc4,0x98,0x98,0x99,0x31,0x31,
  0x99,0xf1,0xdc,0xdc,0x99,0xf1,0x93,0x81,0x81,0x93,0x93,0x72,0xf1,0xdc,0xf1,0xf1,
  0x99,0x93,0x93,0x99,0xd5,0xf1,0xdc,0xf1,0xf1,0xdc,0xdc,0xdc,0x51,0xdc,0x72,0xdc,
  0x51,0x72,0x93,0x93,0x98,0xda,0x31,0x81,0xc4,0xc4,0xc4,0xc4,0xd1,0xde,0xd5,0x81,
  0xd1,0x5a,0x5a,0xbe,0xde,0x93,0x31,0x93,0xc8,0x31,0xe2,0x93,0xc8,0xc8,0xc8,0xd5,
  0x93,0x93,0x81,0x81,0xd5,0xc8,0xd5,0xc8,0x81,0xd5,0xc8,0xd5,0x93,0x93,0x93,0x93,
  0xc8,0xc8,0xda,0x31,0xf1,0xe2,0x72,0xa1,0x00,0x98,0x93,0x93,0x51,0x51,0x51,0x72,
  0x51,0x72,0x72,0x93,0x72,0x93,0xe2,0x51,0x93,0x93,0x93,0xc4,0x98,0x9a,0x9a,0x8b,
  0x5a,0xc4,0x98,0xc4,0x98,0xc4,0xc4,0xc4,0x98,0xc4,0x98,0x98,0x98,0x72,0x93,0xf1,
  0xc8,0xc4,0xe7,0xe7,0xdc,0xdc,0xdc,0xdc,0xf1,0xdc,0xdc,0xe7,0xe7,0xe7,0xdc,0xdc,
  0xf1,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xdc,0xf1,0xdc,0xdc,0xdc,
  0xdc,0xe7,0xdc,0xe7,0x98,0x98,0xe8,0x98,0x98,0x98,0x5a,0xc4,0x98,0x98,0xc4,0xc4,
  0x5a,0x5a,0x9a,0xbe,0x81,0xd5,0x81,0x81,0xf0,0x81,0xde,0xde,0xc8,0xde,0x93,0x31,
  0xd5,0x93,0x93,0x93,0x81,0x81,0xf1,0x93,0xdc,0x31,0x72,0xf1,0xdc,0xdc,0xf1,0x72,
  0x51,0xf1,0xf1,0xe6,0xf7,0xf1,0xe7,0x31,0xde,0xf1,0xf1,0xe2,0x51,0x51,0xf1,0xe2,
  0xdc,0xdc,0xdc,0xdc,0xdc,0xdc,0x72,0x51,0x51,0x51,0xdc,0x98,0x93,0x8b,0x8b,0x8b,
  0x5a,0xc4,0x98,0x98,0x93,0x98,0x98,0x93,0x98,0xc4,0x98,0x72,0x72,0x98,0xe8,0x81,
  0xe2,0x93,0xe7,0x2c,0xdc,0xdc,0xe6,0xdc,0xdc,0xdc,0xdc,0xe7,0xe7,0xe7,0xdc,0xdc,
  0xdc,0xdc,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xe7,0xdc,0xdc,0xdc,0xdc,0xdc,0xe7,0xdc,
  0xdc,0xdc,0xdc,0xe7,0xe2,0xe2,0xf0,0x98,0xe6,0x31,0x98,0x98,0x98,0x98,0x93,0x98,
  0xc4,0x5a,0x8b,0x8b,0xde,0x81,0x81,0x81,0x81,0x99,0x99,0x99,0xd5,0xd5,0x81,0x81,
  0x81,0xd5,0xda,0xc8,0x93,0x93,0xdc,0xdc,0xe7,0xe7,0xe7,0xdc,0xdc,0xdc,0xdc,0xdc,
  0xdc,0xe7,0xf1,0xe7,0x2c,0xe7,0xe7,0xe7,0xe7,0xf1,0xe7,0xdc,0x51,0xdc,0x51,0xdc,
  0xdc,0xdc,0xdc,0xf7,0xe6,0xe6,0xdc,0xdc,0xdc,0xdc,0xdc,0x72,0xe2,0x98,0xc4,0xc4,
  0xc4,0x98,0x98,0xf7,0xdc,0x72,0x93,0x93,0x93,0x31,0xf1,0xf1,0xe7,0x98,0x93,0xf1,
  0xe6,0x72,0xe7,0xe7,0xdc,0x4d,0x4d,0xdc,0x4d,0x4d,0xdc,0xdc,0xe7,0xdc,0xe7,0xf7,
  0xdc,0xdc,0xdc,0xdc,0xe7,0xdc,0xdc,0xdc,0xdc,0xe7,0xdc,0xdc,0xf7,0xdc,0xe7,0xe7,
  0xe7,0xdc,0xdc,0xdc,0xe6,0xfd,0xe7,0xe2,0xdc,0xe6,0xfe,0x9e,0x93,0x51,0xe7,0x31,
  0xc4,0x98,0x98,0x98,0xd5,0xd5,0xd5,0xd5,0x99,0xf1,0xf1,0xf1,0xf1,0xf1,0xd5,0xf0,
  0xfd,0xfd,0xfd,0xff,0xe6,0xe6,0x4d,0xdc,0x4d,0x4d,0x4d,0x4d,0xf7,0xdc,0xdc,0xdc,
  0xdc,0xe7,0xfd,0xf7,0x2c,0xdc,0xfa,0x92,0xfa,0xe7,0xe7,0x99,0xdc,0xdc,0xdc,0xf1,
  0xdc,0xcb,0x92,0xf0,0xc6,0xc6,0x06,0xf1,0xdc,0xdc,0x51,0x98,0xe2,0x51,0xdc,0x4d,
  0x51,0x93,0x72,0xdc,0xfa,0xfa,0xf1,0x72,0xe2,0x21,0xff,0x2c,0xe7,0x72,0xf7,0x4d,
  0x4d,0xf7,0xe7,0xe7,0xdc,0x4d,0xe7,0xdc,0xe6,0xe6,0xfd,0xfd,0xe6,0xdc,0xdc,0xe6,
  0xe6,0xfd,0xf1,0xcb,0xdc,0xdc,0xdc,0xdc,0xdc,0xdc,0xe7,0xe7,0xf7,0xdc,0xe7,0xf7,
  0xe6,0xe7,0xe7,0xe7,0xe6,0xf1,0xe6,0xdc,0xf1,0xfd,0xfe,0x72,0xf1,0xf1,0xdc,0xf1,
  0x31,0xf1,0xdc,0x93,0xc1,0xfd,0x21,0x21,0x21,0xfe,0xfe,0xfe,0xfd,0xf1,0xfd,0xfd,
  0xff,0x21,0xff,0xff,0xff,0xdc,0x4d,0x4d,0x4d,0xff,0xff,0x4d,0xff,0xff,0xff,0xff,
  0xff,0xff,0x4d,0xff,0xf7,0x06,0xbf,0x92,0x92,0xfa,0xf9,0xbf,0xe0,0xff,0xe6,0xf9,
  0xcb,0xcb,0x71,0x06,0xcb,0x06,0x92,0x06,0x92,0xcb,0xf1,0xfa,0xfa,0xcb,0xcb,0xcb,
  0x71,0xcb,0xcb,0x71,0x92,0x92,0x92,0x06,0xbf,0x92,0xf9,0x06,0xf1,0xfe,0xfd,0xe0,
  0x4d,0xf7,0xff,0x21,0xff,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21,
  0x21,0xe6,0xf9,0xbf,0xe0,0xfa,0xcb,0xfa,0x99,0xdc,0x4d,0x4d,0x4d,0xe0,0x21,0x21,
  0x21,0xfd,0xfd,0xc1,0xfe,0xdc,0xdc,0xfd,0xfd,0xfd,0xc1,0xc1,0xfd,0xfd,0xfd,0xc1,
  0xc1,0xfd,0xfd,0xfd,0xfd,0xfd,0xc1,0xc1,0xe6,0xe6,0x21,0xfd,0xfd,0xc1,0xfd,0xe6,
  0xff,0xff,0xff,0xff,0xff,0xff,0x4d,0x4d,0x4d,0xff,0xff,0x4d,0x4d,0xff,0x4d,0x4d,
  0x4d,0xff,0xff,0xe0,0xff,0xf9,0xe0,0x4d,0xe0,0x71,0xe0,0xfb,0x92,0xfb,0xfb,0xfb,
  0x71,0xff,0xfd,0x92,0xcb,0x92,0x92,0xfb,0xe0,0xff,0x4d,0x4d,0x4d,0x71,0xcb,0x71,
  0x71,0xfb,0x71,0x71,0xfb,0xe0,0xf9,0xfb,0x06,0xfb,0xe0,0xe0,0xf9,0xf9,0xe0,0xe0,
  0xe0,0xfa,0xe0,0xe0,0xe0,0xe0,0xe0,0xe0,0xe0,0xe0,0xe0,0xe0,0xf9,0xfd,0xe0,0xfd,
  0xfd,0xfd,0xf9,0xfd,0xc9,0xfd,0xfd,0x21,0xff,0xe0,0xe0,0xe0,0x4d,0xf1,0xf9,0x4d,
  0xe0,0xfd,0xc1,0xe6,0xe0,0x4d,0xdc,0xfd,0xc1,0xc1,0xfd,0xc1,0xdb,0xc1,0xc1,0xc1,
  0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc9,0xc1,0xc1,0xc1,0xc1,0xc9,0xc9,0xfe,0xff,
  0xf9,0xfd,0x21,0x21,0x21,0xff,0x21,0xfd,0xff,0xe6,0xfd,0xe6,0xe6,0xe6,0xe6,0xfd,
  0xe6,0xe6,0x21,0xe6,0xe6,0x21,0xe6,0x21,0xe6,0xe6,0xfd,0xe6,0xe6,0xe6,0xe6,0xe6,
  0xe6,0x21,0x21,0xe6,0x21,0xe6,0xe6,0xe6,0xe6,0x21,0xe6,0xe6,0x21,0x21,0x21,0x21,
  0x21,0xe6,0x21,0xe6,0x21,0x21,0xe6,0xe6,0xe6,0xfd,0xfd,0xfd,0xfd,0xc9,0xc1,0xfd,
  0xc1,0xc1,0xc1,0xc9,0xc1,0xc1,0xc9,0xc1,0xc1,0xdb,0xc1,0xc1,0xc1,0xc1,0xf9,0xf9,
  0xc9,0xf9,0xf9,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xc1,0xfd,0xc9,
  0xfd,0xfd,0xfd,0x21,0xfd,0x4d,0x21,0x21,0xfd,0x21,0x21,0xc9,0xc9,0xc9,0xc1,0xc1,
  0xc1,0xdb,0xc1,0xdb,0xc1,0xc9,0xc1,0xc1,0xc1,0xc9,0xc1,0xc1,0xc1,0xc9,0xc1,0xf9,
  0xf3,0xf3,0xdb,0xdb,0xdb,0xdb,0xdb,0xc1,0xc1,0xc1,0xc1,0xc9,0xc9,0xc9,0xc9,0xc9,
  0xc9,0xc9,0xc9,0xc9,0xf9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xc9,0xf9,
  0xc9,0xf9,0xc9,0xf9,0xf9,0xc9,0xc9,0xc9,0xc9,0xf9,0xc9,0xf9,0xc1,0xc1,0xc1,0xdb,
  0xdb,0xc1,0xc1,0xc1,0xdb,0xc1,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xf3,0xf3,0xf3,0xf3,
  0xf3,0xf4,0xf4,0xf4,0xf4,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,0xec,
  0xec,0xf4,0xf4,0xf4,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xdb,0xdb,0xdb,
  0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xdb,0xf3,0xf3,0xf4,0xf4,
  0xf4,0xec,0xec,0xec,0xec,0xf4,0xec,0xec,0xf4,0xf4,0xf4,0xf4,0xf4,0xf3,0xf4,0xf3,
  0xad,0xad,0xad,0xad,0xad,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,
  0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,
  0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xcb,0xad,0xad,0xad,
  0xad,0xcb,0xcb,0xcb,0xcb,0xcb,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
  0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
  0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xcb,0xcb,
  0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
  0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,0xad,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0xcb,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0xcb,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0xcb,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0xad,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0xad,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0xad,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,0x06,
  // 12: sky 2, part A
  0xdc,0xce,0xbc,0xce,0x9c,0xbe,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,0x00,0x01,
  0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,0x00,0x03,
  0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,0x00,0x05,
  0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,
  0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,
  0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,
  0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x7b,0xbe,0x10,0x24,0x18,0x24,
  0x00,0x25,0xbb,0xc6,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,
  0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,
  0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x7c,0xbe,0x18,0x41,0x00,0x42,0x08,0x42,
  0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,
  0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,
  0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,
  0x10,0x60,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,
  0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,
  0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,
  0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,
  0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,
  0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,
  0x10,0x84,0x18,0x84,0x00,0x85,0x9b,0xbe,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,
  0x08,0x86,0x10,0x86,0x18,0x86,0xdc,0xbe,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,
  0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,
  0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,
  0x00,0xa4,0x08,0xa4,0x10,0xa4,0xfd,0xd6,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,
  0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,
  0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x00,0xc1,0x08,0xc1,0x10,0xc1,
  0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,
  0x18,0xc3,0x00,0xc4,0x3e,0xd7,0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,
  0x7c,0xc6,0x10,0xc5,0x18,0xc5,0x00,0xc6,0x08,0xc6,0x1d,0xcf,0x10,0xc6,0xfe,0xce,
  0x18,0xc6,0x00,0xc7,0x08,0xc7,0x10,0xc7,0x18,0xc7,0x00,0xe0,0x08,0xe0,0x10,0xe0,
  0x1d,0xd7,0x18,0xe0,0x08,0xe1,0xbb,0xd6,0xfe,0xd6,0x18,0xe1,0x08,0xe2,0x18,0xe2,
  0x08,0xe3,0x10,0xe3,0x00,0xe4,0x08,0xe5,0xbc,0xd6,0x10,0xe5,0x08,0xe6,0x18,0xe6,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0xa3,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x02,0x02,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x01,0x02,0x02,0x01,0x02,0x02,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x01,0x01,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x4c,0x02,0x4c,0x4c,
  0x4c,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x01,0x02,0x01,0x01,0x02,0x01,
  0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x4c,0x4c,0x02,0x4c,0x02,0x4c,0x4c,0x4c,0x9b,0x4c,0x9b,0x02,0x02,0x02,
  0x02,0x4c,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x9b,0x02,0x02,0x02,0x02,0x02,0xa3,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,
  0x02,0x02,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x01,0xa3,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x9b,
  0x02,0x02,0x9b,0x02,0x4c,0x35,0x4c,0x02,0x02,0x35,0x02,0x02,0x02,0x02,0x35,0x02,
  0x35,0x35,0x4c,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x9b,0x4c,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x01,0x01,
  0x02,0x02,0x02,0x02,0x02,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x39,0x02,0x02,0x02,
  0x4c,0x02,0x4c,0x4c,0x02,0x02,0x02,0x02,0x02,0x02,0x9b,0x02,0x02,0x02,0x02,0x02,
  0x02,0x4c,0x9b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x4c,0x4c,0x4c,0x4c,0x02,0x4c,0x02,0x02,0x02,0x4c,0x02,0x02,0x4c,0x02,0x02,
  0x02,0x35,0x02,0x02,0x35,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x02,0x01,0x02,0xa3,0x02,0x02,0x02,0x9b,0x02,0x9b,0x4c,0x4c,0x02,0x9b,0x02,
  0x9b,0x39,0x9b,0x9b,0x9b,0x4c,0x35,0x4c,0x4c,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x9b,0x9b,0x02,0x02,0x4c,
  0x02,0x02,0x02,0x4c,0x02,0x4c,0x02,0x02,0x02,0x02,0x02,0x02,0x4c,0x02,0x9b,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x9b,
  0x4c,0x4c,0x02,0x4c,0x4c,0x35,0x02,0x02,0x02,0x4c,0x4c,0x35,0x4c,0x4c,0x02,0x02,
  0x02,0x02,0x35,0x35,0x35,0x35,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x02,0x02,0x02,0x02,
  0x01,0x01,0x02,0x02,0x02,0x02,0xa3,0x39,0x02,0x39,0x9b,0x02,0x02,0xe0,0x39,0x02,
  0x39,0x4c,0x4c,0xe0,0x02,0x02,0x4c,0x9b,0x4c,0x4c,0x35,0x35,0x02,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x4c,0x4c,0x39,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x02,0x02,0x02,0x02,0x02,0x02,0xa3,0x01,0xa3,0x01,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x9b,0x9b,0x9b,
  0x9b,0x4c,0x9b,0x9b,0x9b,0x35,0x35,0x9b,0x9b,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x02,0x02,0x4c,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xa3,0x02,0x02,0x02,
  0x01,0x01,0xa3,0x02,0xa3,0x01,0xa3,0x01,0xa3,0xa3,0x01,0x01,0x01,0xa3,0x01,0x01,
  0x01,0x02,0x02,0x02,0x02,0x39,0x02,0x02,0x39,0x39,0x02,0x4c,0x9b,0x4c,0x4c,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x4c,0x4c,0x4c,0x4c,0x35,0x35,
  0x35,0x4c,0x4c,0x4c,0x4c,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x02,0x01,0x02,0x02,0x02,0x01,0x02,0x02,0xa3,0x02,0x02,0x02,0x02,0x02,0x4c,
  0x4c,0x4c,0x02,0x4c,0x4c,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x4c,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x4c,0x4c,0x9b,0x4c,0x02,0x02,0x02,0x02,0x01,0x02,0x01,
  0xa3,0xa3,0xa3,0x02,0x02,0x01,0xa3,0x02,0x02,0x02,0x02,0x02,0xa3,0x01,0x01,0x01,
  0x01,0x02,0x02,0xa3,0x01,0xa3,0xa3,0x01,0x01,0x4c,0x4c,0x35,0x4c,0x39,0x39,0x4c,
  0x4c,0x35,0x39,0x39,0x35,0x35,0x35,0x35,0x35,0x4c,0x39,0x39,0x4c,0x4c,0x4c,0x4c,
  0x4c,0x4c,0x39,0x39,0x02,0x02,0x02,0x01,0xa3,0x01,0x01,0xa3,0x01,0xa3,0xa3,0xa3,
  0xa3,0xa3,0xa3,0xa3,0x02,0x39,0x02,0xa3,0xa3,0x02,0x02,0x9b,0x02,0x02,0x9b,0x9b,
  0x4c,0x9b,0x4c,0x35,0x4c,0x9b,0x9b,0x02,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x4c,0x35,0x35,0x35,0x4c,0x39,0x9b,0x9b,0xa3,0x02,0xa3,
  0xa3,0xa3,0x02,0x02,0x01,0xe0,0x02,0xe0,0x39,0x39,0xe0,0xe0,0x02,0x01,0x01,0x02,
  0x01,0x01,0x01,0xa3,0x01,0x01,0x01,0x02,0x39,0x39,0x39,0x39,0x39,0xe0,0x39,0x39,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x39,0x39,0x39,0x39,0x39,0x39,0x4c,0x4c,0x02,
  0x02,0x9b,0x02,0x01,0xa3,0x9b,0x39,0x39,0x39,0x02,0x39,0x39,0x39,0x39,0x02,0x4c,
  0x39,0x02,0x39,0x9b,0x9b,0x39,0x39,0x02,0x02,0x9b,0x4c,0x9b,0x4c,0x4c,0x4c,0x9b,
  0x9b,0x9b,0x4c,0x9b,0x9b,0x9b,0x9b,0x9b,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,
  0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x35,0x4c,0x4c,0x9b,0x39,0x02,0x02,0x02,0x01,
  0xa3,0x02,0x02,0x39,0x39,0xe0,0xe0,0xe0,0xe0,0xe0,0x39,0x01,0x01,0x01,0x01,0x00,
  0x00,0xa3,0xa3,0x01,0xa3,0xa3,0x00,0x01,0x01,0x01,0xe0,0x01,0x01,0x01,0x02,0xe0,
  0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x4c,0xe0,0x39,0x01,0xe0,0x01,0x01,
  0x39,0x39,0x39,0x39,0xe0,0x39,0x39,0x39,0x39,0x39,0x39,0x9b,0x4c,0x4c,0x39,0x39,
  0x39,0x39,0x39,0x9b,0x9b,0x9b,0x39,0x4c,0x4c,0x39,0x9b,0x9b,0x9b,0x4c,0x9b,0x9b,
  0x4c,0x9b,0x9b,0x9b,0x9b,0x4c,0x35,0x35,0x35,0x35,0x4c,0x9b,0x9b,0x9b,0x9b,0x4c,
  0x9b,0x4c,0x9b,0x4c,0x35,0x35,0x4c,0x4c,0x35,0x4c,0x9b,0x35,0x35,0x39,0x35,0x35,
  0x4c,0x39,0x4c,0x35,0x35,0x35,0x4c,0x4c,0x39,0x39,0x39,0x01,0x01,0xa3,0xa3,0x01,
  0x01,0x01,0x00,0x01,0x01,0x02,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,
  0x00,0x00,0x00,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,
  0x02,0x01,0x9b,0xa3,0x02,0x01,0x02,0xe0,0xe0,0x39,0x4c,0x4c,0x39,0x39,0x39,0x39,
  0x39,0x4c,0x4c,0x39,0x4c,0x4c,0x39,0x4c,0x39,0x02,0x39,0x9b,0x9b,0x39,0x02,0x39,
  0x4c,0x4c,0x4c,0x4c,0x35,0x4c,0x35,0x35,0x35,0x35,0x4c,0x39,0x35,0x4c,0x39,0x9b,
  0x9b,0x9b,0x9b,0x9b,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0x4c,0xe0,0x39,0x39,0x39,0xe0,
  0xe0,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x01,0xa3,0xa3,0xa3,0x01,0x01,0x01,0x01,
  0x01,0x00,0x00,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0xe5,0xe5,0xe5,0xe5,
  0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe7,0xe5,0xe5,0x00,0x00,0x00,0xe5,0xe5,
  0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xa3,0x02,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xa3,0x01,0x01,0x39,0x02,0x02,0x39,0x02,
  0x39,0x39,0x39,0x39,0x39,0x39,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0xa3,0x01,0x02,
  0x9b,0x4c,0x4c,0x4c,0x4c,0x35,0x9b,0x35,0x35,0x9b,0x39,0x39,0x9b,0x9b,0x9b,0x9b,
  0x4c,0x39,0x9b,0x9b,0x9b,0x02,0x9b,0x39,0x02,0xa3,0xa3,0xa3,0x01,0x01,0x01,0x01,
  0x01,0xa3,0x00,0x01,0x00,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x01,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe5,0xe5,0xe5,0xe5,0xe5,
  0xe5,0xe5,0xe7,0xe5,0xe7,0xe7,0xe7,0xe7,0xe5,0xe5,0xe5,0xe5,0x00,0x00,0x00,0xe5,
  0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0xa3,0x01,0x01,0x01,0xa3,0x01,0x01,0xa3,0xa3,0x01,0x01,0x01,0x01,0x02,0x01,
  0x01,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x01,0x01,0xa3,0x01,0x01,
  0x01,0x39,0x39,0x9b,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x02,0x02,0x02,0x01,0xa3,
  0xa3,0xa3,0xa3,0xa3,0x02,0xa3,0xa3,0x01,0x01,0x01,0x01,0x01,0xa3,0x01,0xa3,0x00,
  0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0xa3,0x01,0x00,0x00,0x00,0x00,
  0x00,0xe5,0x00,0x00,0xe5,0xe5,0xe5,0xe5,0x00,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,
  0xe5,0xe5,0xe5,0xe5,0xe7,0xe7,0xe5,0xe7,0xe7,0xe5,0xe5,0xe5,0xe5,0xe5,0xe7,0x00,
  0x00,0x00,0xa3,0x00,0x00,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x02,
  0xa3,0x01,0x02,0x01,0x01,0xa3,0xa3,0x02,0x01,0xa3,0x01,0xa3,0xa3,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0xa3,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x01,0xa3,0x01,
  0x01,0x01,0x39,0x39,0x39,0x01,0x02,0x01,0x39,0x9b,0x01,0x01,0x01,0x01,0x01,0xa3,
  0x01,0x01,0x01,0x00,0xa3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe7,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,
  0xe5,0xe7,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x02,0xe0,
  0x01,0x01,0x02,0x02,0x02,0x02,0x4c,0x02,0x02,0x02,0x02,0x01,0xa3,0xa3,0xa3,0x01,
  0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0xa3,0x01,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0xa3,0x01,0x39,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xa3,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x39,0x39,0x01,0x01,0x01,0x39,0x39,0x39,0x39,
  0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe5,0x00,0xe5,0x00,
  0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe7,0xe7,0xe5,0xe5,0xe5,0xbb,0xe5,0x00,0xe5,
  0xe5,0x00,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,
  0x00,0xe5,0xe5,0xe5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,
  0x01,0x02,0x01,0x39,0x02,0x39,0x02,0x01,0x01,0x39,0x02,0x02,0x01,0x01,0x39,0x02,
  0x01,0x01,0x01,0x01,0xa3,0xa3,0x00,0x00,0x01,0x01,0xa3,0x01,0x01,0x00,0x01,0x01,
  0x00,0x00,0x00,0x00,0x01,0x01,0xa3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0xa3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x39,0x39,0x39,
  0x39,0x39,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe5,0x00,
  0xbb,0xbb,0x01,0xbb,0x01,0x00,0x00,0xe5,0xbb,0xe5,0xe5,0xbb,0xbb,0xbb,0xbb,0xbb,
  0xbb,0xe5,0xe5,0xe5,0xe5,0xe5,0xf4,0xe5,0xf4,0xe5,0xe5,0xf4,0xe5,0xe7,0xe5,0xe5,
  0xe5,0xe5,0xe5,0x00,0xe5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x39,0x01,0x01,0x39,0x39,0x39,0x39,0x39,0x39,
  0x01,0x39,0x39,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0xa3,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x00,0x00,0x01,0x01,0x00,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x39,
  0x01,0x01,0x39,0x01,0x01,0xbb,0x01,0xbb,0x01,0x01,0x01,0xbb,0x01,0xbb,0xbb,0xbb,
  0xe5,0x00,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xe5,0xbb,0xe5,0xe5,0xf4,0xe5,0xf4,
  0xda,0xda,0xda,0xe5,0xe5,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xf4,0xe5,0xe5,
  0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,
  0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x39,0x01,0x39,0x39,0x39,
  0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x01,0x39,0x01,0x39,0x39,0x39,0x39,
  0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x01,0xbb,0x01,0xbb,0xf3,0x00,0x01,
  0xbb,0x01,0xfc,0xfc,0xbb,0x00,0xbb,0xbb,0xbb,0x01,0x01,0xbb,0xbb,0xbb,0xbb,0xf0,
  0xbb,0xbb,0xbb,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xf4,0xda,0xda,0xda,
  0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xda,0xe7,
  0xf4,0xe7,0xe7,0xe7,0xe7,0xe5,0xe5,0xe5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x39,0x01,0x39,0x01,0x39,0x39,
  0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x39,0x01,0x39,0x39,0x39,0x39,0x39,
  0x39,0x39,0x39,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x39,0x01,0x01,0x00,0x00,
  0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0xbb,0xbb,0x00,0x00,0xbb,0xbb,0xbb,
  0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xf0,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xda,
  // 13: sky 2, part B
  0x9b,0xd6,0x9a,0xce,0xfc,0xce,0x1c,0xd7,0x1d,0xcf,0x3e,0xd7,0x00,0x00,0x08,0x00,
  0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,
  0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,
  0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,
  0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,
  0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,
  0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,
  0x10,0x24,0x18,0x24,0x00,0x25,0x7e,0xdf,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,
  0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,
  0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,
  0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,
  0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,
  0x08,0x46,0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,
  0x08,0x60,0x10,0x60,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,
  0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,
  0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,
  0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,
  0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,
  0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,
  0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,
  0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,
  0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,
  0x08,0xa2,0x7d,0xdf,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,
  0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x7a,0xce,0x10,0xa5,
  0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x3e,0xcf,0xbc,0xce,
  0x08,0xa7,0x10,0xa7,0xdb,0xd6,0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x18,0xc0,
  0x9e,0xdf,0x7e,0xe7,0x00,0xc1,0x08,0xc1,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,
  0xfd,0xd6,0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x18,0xc3,0x00,0xc4,
  0x08,0xc4,0xfb,0xce,0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,0x10,0xc5,0x18,0xc5,
  0xdc,0xbe,0x00,0xc6,0x08,0xc6,0x10,0xc6,0x18,0xc6,0x00,0xc7,0x08,0xc7,0x10,0xc7,
  0x18,0xc7,0x7a,0xd6,0x00,0xe0,0x08,0xe0,0x10,0xe0,0x00,0xe1,0x10,0xe1,0xbc,0xd6,
  0x08,0xe2,0x18,0xe2,0xfe,0xce,0x08,0xe3,0x79,0xce,0x18,0xe3,0x10,0xe4,0x08,0xe5,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x3b,0x3b,0x05,0x05,0x05,0x3b,0x05,0x3b,0x05,
  0x05,0x05,0x05,0xfa,0x04,0xfa,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0x04,0x02,0xe8,
  0x02,0xe8,0xe8,0xe8,0xe8,0xe8,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x00,0xc7,0xc7,
  0xc7,0xc7,0x00,0xc7,0x00,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xe8,0xc7,0xc7,
  0xc7,0xc7,0xc7,0xc7,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0x02,0xe8,0x02,0x02,0x02,
  0x04,0x02,0x02,0x04,0x04,0x02,0x02,0x02,0xc7,0x02,0x02,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x3b,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x3b,0xd0,0x05,0x3b,0x3b,0x05,0xd0,0x3b,0x05,0x05,0x05,0x3b,
  0x05,0x05,0x05,0xc6,0xc6,0xc6,0xfa,0xfa,0xfa,0xfa,0x04,0x04,0x04,0x04,0x04,0x02,
  0x04,0x04,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xc7,
  0xc7,0xc7,0xc7,0xe8,0xe8,0xe8,0xe8,0xc7,0xc7,0xc7,0xc7,0xc7,0xe8,0x02,0xe8,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0x04,0x04,
  0x02,0x04,0x02,0x02,0x04,0x04,0x04,0x04,0x02,0x04,0x02,0xd8,0xd8,0x02,0xd8,0xd8,
  0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x05,0xd8,0xd8,0x05,0xd8,0x05,0x04,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x3b,0xd1,0xd0,0xd0,0xd0,0xd0,0xd0,0x3b,0xd0,0xd0,0xd0,
  0x3b,0xd0,0x05,0x05,0xc6,0xc6,0xfa,0xfa,0xfa,0xfa,0xfa,0x04,0x04,0x02,0x04,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0xe8,0xe8,0xe8,0x02,0xc7,0x02,0xc7,0xc7,0xe8,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x02,0xe8,0xe8,0xe8,0x02,0xc7,0x02,0x02,0x02,
  0x02,0x02,0x02,0x04,0x02,0x04,0x02,0x04,0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0xd8,0x04,0x04,
  0xd8,0xd8,0xd8,0xd8,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,0x3b,0x05,0x05,
  0x3b,0x05,0x05,0x05,0x05,0x05,0x3b,0x3b,0x05,0xd1,0xd1,0xd0,0x3b,0x05,0xd0,0xd0,
  0xd0,0xd1,0x3b,0x05,0xc6,0xc6,0xc6,0xc6,0xc6,0x04,0x04,0x04,0x04,0x04,0x04,0x02,
  0x04,0x04,0x02,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x02,0x02,0x02,0x02,0x02,
  0x02,0x04,0x04,0x04,0x04,0x04,0xfa,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x05,
  0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x3b,0x3b,0x05,0x3b,0xd1,
  0xd0,0xd0,0x3b,0x3b,0x05,0x05,0xc6,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x04,0x04,0x04,
  0x02,0x02,0x04,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0x04,0x04,0x04,0x02,0x02,0x02,
  0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,
  0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd0,
  0xd0,0xd1,0xd0,0x05,0x3b,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x04,
  0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x02,0x04,0x02,0x04,0x04,0x04,0x02,0x02,0x04,
  0x02,0x02,0x02,0x02,0x04,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0x02,
  0x02,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x02,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0xc6,0x05,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0xc6,0x05,0xc6,0x05,0x05,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0xd0,0xd0,0x3b,0x3b,0x05,0x05,0xb1,0x05,0x05,0x05,0x04,0x04,0x02,0x04,
  0xe8,0xe8,0xe8,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0x04,0x02,0x02,0x02,0x02,
  0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x02,0x02,0x04,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x04,0x04,0x05,0x05,0x04,0x04,0x05,
  0x05,0x05,0x05,0x04,0x05,0xc6,0xc6,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,
  0x04,0x05,0x04,0x04,0xc6,0xc6,0x04,0x05,0xc6,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x3b,0x3b,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x02,0x04,0x02,
  0xe8,0xe8,0xc7,0xc7,0xe8,0x04,0xc7,0xe8,0xc7,0xe8,0xe8,0xc7,0xe8,0x02,0x02,0x02,
  0x02,0xe8,0xe8,0xe8,0xe8,0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x02,0x02,0x04,0x02,
  0x02,0x04,0x02,0x04,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0x05,0x05,0x04,0x05,0x05,
  0x05,0xc6,0xc6,0x04,0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0xfa,
  0x04,0xc6,0xc6,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0xd0,0x3b,0xd1,0x3b,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0xe8,0x02,0xc7,0x02,0xc7,0x04,0x02,0x02,0x02,0xe8,0x02,
  0x02,0x02,0xe8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x04,0x04,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xc6,0x04,0x04,0xc6,0x05,0x05,0x05,0x05,0xc6,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0xc6,0x05,0x05,
  0xc6,0xc6,0x05,0xc6,0xc6,0xc6,0xc6,0x05,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0xd0,0xd0,0xd0,0xd0,0xd0,0x05,0x05,0x05,0x05,0xc6,0xc6,0xc6,0xc6,0xfa,0x04,
  0x04,0x02,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0x02,0x02,0x02,
  0x02,0x04,0x04,0x02,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x02,0x02,0xe8,
  0xe8,0xc7,0x02,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x02,0x04,0x04,0x02,0x02,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x04,0x05,0x05,0x04,0x05,0x05,0x04,0x04,0x05,0x05,0x05,0x05,0x05,0xc6,
  0xc6,0xc6,0x05,0x05,0x05,0x05,0xc6,0xc6,0x05,0x04,0xc6,0x05,0x05,0x05,0x05,0x3b,
  0x05,0xd0,0xd0,0xd0,0xd0,0xd1,0xd1,0xd0,0xd1,0xd0,0x05,0x05,0xc6,0xc6,0xfa,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x02,0x04,
  0x04,0xe8,0xe8,0xe8,0xc7,0xc7,0xc7,0xe8,0xe8,0x02,0x02,0x02,0x02,0xe8,0xe8,0xe8,
  0xc7,0xc7,0xe8,0x02,0xc7,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0xd8,0xd8,0x04,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0xc6,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xc6,0x04,0x04,0x05,0xc6,0x05,0xc6,
  0x04,0x05,0xc6,0xc6,0x05,0x04,0x05,0x05,0x05,0x05,0xc6,0xc6,0x05,0x05,0x05,0x05,
  0x05,0x3b,0xd1,0xd0,0xd1,0xd1,0xd1,0xd0,0xd0,0xd0,0xd1,0x3b,0x05,0x05,0x04,0x04,
  0x04,0x04,0x04,0x04,0x02,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0xe8,0xe8,
  0xe8,0xe8,0xc7,0xc7,0xe8,0xe8,0xe8,0xc7,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xc7,
  0xe8,0x02,0x02,0x04,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0xd8,0x04,0x04,0x05,0xb1,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0xd8,
  0x05,0x05,0xc6,0x04,0xc6,0x05,0xc6,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0xd0,0xd0,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd0,0xd0,0x05,0x05,0x05,
  0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0xc7,0xe8,0xe8,
  0xe8,0xc7,0xc7,0xc7,0xc7,0xe8,0xc7,0xc7,0xe8,0xe8,0xe8,0xe8,0x02,0x02,0x02,0x02,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x04,
  0x04,0x04,0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x05,0xd8,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xd8,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,
  0x04,0x04,0xc6,0x05,0x05,0xc6,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x3b,0x05,0x05,
  0x05,0x3b,0x3b,0xd0,0xd0,0xd0,0xd1,0xd1,0xd1,0xd0,0xd0,0xd0,0xd0,0x3b,0x3b,0x05,
  0x3b,0x05,0x04,0x04,0x04,0x04,0x02,0x02,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,
  0xe8,0xe8,0xe8,0xe8,0x02,0x02,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xe8,0xc7,0xe8,0x02,
  0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0xc6,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,
  0xb1,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x05,0x04,0x05,0xd8,0xd8,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x3b,0x3b,0xd0,0xd0,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0x3b,0xd0,0x05,0x3b,0x3b,0x05,
  0xb1,0x05,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0x02,0xe8,0xc7,0xc7,0xe8,0xe8,0x02,
  0xe8,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x02,0x02,0x02,0x04,0x02,0x02,0x02,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0xd8,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,0xb1,
  0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0x3b,0x05,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0xd1,0x3b,0x3b,0x3b,0x05,0xb1,
  0x05,0xb1,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x02,0x04,
  0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x02,0xe8,0xe8,0xe8,0xe8,
  0xe8,0xe8,0xe8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x02,0x02,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xc6,0x05,0x05,0x05,0x05,0x05,0x05,
  0x04,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0xd8,0x05,0x05,0x04,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0xb1,0xb1,0x05,0x05,
  0x05,0x3b,0xd0,0xd1,0xd1,0xd1,0xd1,0x3b,0x3b,0x3b,0xd1,0xd1,0x3b,0x05,0xb1,0x05,
  0xb1,0x05,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0xe8,
  0xc7,0xe8,0x02,0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,
  0x02,0x02,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xc6,0x04,0x04,0x05,0x05,
  0x05,0x05,0x05,0x05,0xb1,0xb1,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0xb1,0x05,0x05,0x05,0x05,
  0xb1,0x05,0x05,0x3b,0x3b,0x3b,0xb1,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0x3b,0xb1,
  0x05,0x05,0x04,0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0x04,0xe8,0xe8,
  0xe8,0xe8,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0x04,0x05,0x04,0x05,
  0x04,0x04,0x04,0xd8,0x04,0x04,0x04,0x02,0x04,0x02,0x04,0x04,0x04,0x04,0x04,0x02,
  0x02,0x04,0x04,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0xc6,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0xb1,0xb1,0x05,0x3b,0x3b,0x05,0x05,0x3b,0x05,0x3b,0xd1,0xd1,0x3b,0x3b,0xb1,
  0x05,0x05,0x04,0x04,0x02,0x04,0x04,0x04,0xd8,0x02,0x04,0x04,0x04,0xd8,0x02,0x04,
  0x02,0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0xd8,0x02,0x04,0x04,0x04,0x02,0xe8,
  0xe8,0x02,0x02,0x04,0x04,0x04,0xd8,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x05,0x05,
  0x05,0x05,0xd8,0x04,0x04,0x04,0x04,0x02,0x02,0x04,0x02,0x04,0x02,0x02,0x04,0x04,
  0x02,0x02,0x02,0x02,0x04,0x04,0x04,0x04,0x04,0x02,0x04,0xd8,0xd8,0xd8,0xd8,0x05,
  0x05,0x05,0x05,0xb1,0xb1,0xb1,0xb1,0x05,0xd8,0xd8,0x04,0xd8,0xd8,0xd8,0xd8,0xd8,
  0x04,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,
  0x05,0xb1,0xb1,0xb1,0xb1,0x05,0x05,0x05,0xb1,0x3b,0x3b,0x3b,0xd1,0x3b,0x3b,0x3b,
  0x3b,0xb1,0xd8,0x04,0x02,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0xd8,0x02,0xd8,0x04,
  0x04,0x02,0x02,0x02,0x04,0x02,0x04,0x04,0x04,0x04,0x02,0x04,0x04,0x04,0x04,0x02,
  0xe8,0xe8,0xe8,0x02,0x02,0xd8,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x04,0xd8,0xd8,
  0x05,0x05,0x05,0x05,0xd8,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0xd8,0x02,0x02,0x04,0xd8,0xd8,0x05,0x05,
  0x05,0x04,0x05,0x05,0x05,0x05,0x05,0x05,0xd8,0x04,0x05,0x03,0xd8,0xd8,0x04,0x04,
  0x04,0xd8,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0xb1,0xb1,0x05,0xb1,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0x05,0x3b,0xb1,0x3b,0x3b,0xb1,0x3b,0x3b,0x05,
  0x3b,0xb1,0x05,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0x02,0x02,0xd8,0xd8,
  0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xd8,0x04,0x02,
  0xe8,0xe8,0x02,0x04,0x04,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xd8,0xd8,
  0xd8,0x05,0xd8,0x05,0x05,0x05,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x04,0x04,0x02,0x04,0x02,0x04,0x04,0x04,0x04,0xd8,0x04,0x04,
  0x04,0xd8,0x03,0xd8,0xd8,0x04,0x04,0x04,0x05,0xd8,0x05,0x03,0xd8,0x04,0xd8,0xd8,
  0x04,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0xb1,0xb1,0xb1,0x05,0x05,0x03,0x03,0x05,
  0x05,0xd8,0x05,0x05,0x05,0xb1,0x3b,0x3b,0x3b,0xb1,0x3b,0xb1,0xb1,0xb1,0xb1,0xb1,
  0xb1,0xb1,0xb1,0x05,0x04,0x02,0x02,0xe8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xd8,
  0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xd8,0x05,0xd8,
  0x02,0x02,0x02,0x02,0xd8,0x04,0x02,0x02,0x02,0x02,0x02,0x04,0x04,0xd8,0xd8,0xd8,
  0xd8,0x05,0x05,0xd8,0x04,0x04,0xd8,0xd8,0xd8,0x04,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0x04,0x04,0x02,0x04,0x04,
  0x02,0x04,0x02,0x02,0x04,0x02,0xd8,0xd8,0x04,0x04,0x02,0x02,0xd8,0x04,0x04,0xd8,
  0x02,0xd8,0xd8,0x04,0xd8,0x05,0xd8,0xd8,0x05,0x05,0x05,0x05,0x03,0x02,0xd8,0xd8,
  0xd8,0x05,0xd8,0x05,0x05,0xb1,0xb1,0x3b,0xd1,0x3b,0xb1,0xb1,0xb1,0x05,0x05,0xb1,
  0x05,0xb1,0xb1,0x05,0xd8,0x02,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0xd8,0x05,0x05,
  0x02,0x02,0x02,0x02,0x02,0x04,0x02,0xd8,0x04,0x02,0xd8,0x02,0xd8,0xd8,0x05,0xd8,
  0x05,0xd8,0xd8,0xd8,0x05,0x05,0xd8,0xd8,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x04,0x02,0x04,0x02,0x04,0x02,0x02,0x04,0x02,0x04,0x02,
  0xe1,0x02,0x04,0x02,0x02,0xe1,0xe1,0x02,0x02,0x02,0x02,0x04,0x04,0x02,0x02,0x02,
  0x02,0xd8,0x02,0x04,0x03,0xe1,0x03,0x03,0xd8,0xd8,0x05,0x05,0xd8,0x04,0xd8,0xd8,
  0xd8,0x05,0x05,0xb1,0x05,0xb1,0xb1,0x3b,0xb1,0xb1,0xb1,0x05,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0x02,0x02,0x02,0x02,0xd8,0x05,
  0xd8,0x04,0x02,0x02,0x04,0x04,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x04,0xd8,0xd8,
  0x05,0x03,0x03,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x02,0x02,0x02,0x02,
  0x02,0x04,0x02,0x02,0x02,0x04,0x04,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0xe1,0x02,0x02,0x02,0x02,0xe1,
  0x02,0x02,0x02,0x02,0x02,0xd8,0x02,0xd8,0x03,0x03,0xd8,0x03,0x03,0xd8,0x02,0x02,
  0xd8,0xb1,0x03,0x05,0xb1,0x03,0xb1,0xb1,0xb1,0xb1,0x05,0xd8,0xd8,0xd8,0xd8,0xd8,
  0xd8,0x02,0xd8,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0x02,0xc7,0x02,0x02,
  0x02,0x02,0xc7,0xc7,0x02,0x02,0x02,0xc7,0xc7,0x02,0x02,0x02,0x02,0x02,0x03,0x05,
  0x05,0x05,0xd8,0xd8,0xd8,0xd8,0xd8,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0xd8,0x02,0x02,0x02,0x02,0x02,0xd8,0xd8,0x03,0x03,0xd8,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe8,0x02,0x02,0x02,
  0x02,0xc7,0x02,0x02,0xc7,0x02,0x00,0x00,0x00,0xc7,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0xd8,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xb1,0xb1,0x03,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0xe1,0x02,0x02,0xe8,0x02,0x02,0x02,0x02,0x02,0xc7,0xc7,0xc7,0xc7,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x02,0xc7,0x02,0x02,0xe1,0xd8,0x02,
  0x02,0xd8,0xd8,0xd8,0xd8,0xd8,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0xe1,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0xe8,0x02,0xe1,0xe1,0x02,
  0x00,0x00,0x00,0x00,0x00,0x00,0xc7,0x00,0x00,0xf7,0x00,0xc7,0xf1,0xf1,0xbe,0x00,
  0x00,0x00,0x00,0xbe,0x01,0x00,0x00,0x00,0x01,0x01,0x00,0x00,0xca,0x00,0x00,0xe1,
  0x02,0x02,0x02,0x02,0x00,0x01,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x02,0x02,0x02,0x02,0x02,0x02,0xc7,0x02,0xc7,0x02,0x02,0xc7,0xc7,0xc7,0xc7,0xc7,
  0x00,0x00,0xc7,0xc7,0x00,0x00,0xc7,0xc7,0x00,0x00,0x01,0xc7,0x02,0x02,0xe1,0x02,
  0x02,0xe1,0xe1,0xe1,0xe1,0x02,0x02,0xe1,0x02,0x00,0x02,0x00,0xc7,0xc7,0xc7,0xc7,
  0xc7,0x02,0x02,0x02,0xc7,0xc7,0x02,0xe1,0x02,0x02,0xe1,0xe1,0xe1,0x02,0xe1,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe1,0xe1,0x00,0x02,0x02,0x00,0x00,0xc7,0x01,
  0xf1,0x00,0xca,0xf1,0xca,0xf1,0x00,0x00,0x00,0x00,0x00,0x00,0xf1,0x00,0x00,0x00,
  0x01,0xf1,0xbe,0xbe,0xfc,0xf1,0xbe,0xbe,0xbe,0xbe,0x00,0x00,0x00,0x00,0xbe,0x00,
  0xf1,0x00,0xf1,0x01,0xbe,0xbe,0xbe,0xbe,0xbe,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xc7,0xc7,0xc7,0x00,0xc7,0x00,0xc7,0xc7,0x02,0xc7,0xc7,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x01,0x00,0x00,0x01,0xc7,
  0x01,0x00,0x00,0x00,0x02,0xe1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x01,0x01,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x01,0x02,0xe1,0xe1,0x00,
  0xe1,0xe1,0x01,0xf1,0xf1,0xe1,0x00,0x00,0x02,0x00,0x01,0xf1,0xf1,0x00,0x00,0x01,
  0x00,0x00,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0xf1,0x00,0x00,0xf1,0x00,0x00,0xf1,
  0xf1,0x01,0x00,0xf1,0xf1,0xf1,0xf1,0xbe,0xf1,0xbe,0xbe,0xbe,0xbe,0xf1,0x01,0x01,
  0x00,0xf1,0xbe,0xbe,0xf1,0xf1,0xbe,0xbe,0xbe,0xbe,0xbe,0xf1,0xbe,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x01,0x00,0xf1,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xbe,0x00,0x00,0x00,0x00,0x01,0x01,0x00,
  0x01,0x01,0x00,0x01,0xca,0xe1,0xe1,0xe1,0x01,0x00,0x00,0xf1,0xf1,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x00,0x00,
  0x01,0x00,0x00,0x00,0x00,0x01,0x01,0xca,0xf1,0xca,0x01,0x00,0x00,0x00,0x00,0x02,
  0x01,0xf1,0xf1,0xca,0x01,0xc7,0x01,0xf1,0xf1,0xf1,0x00,0xca,0x01,0x01,0x00,0x00,
  0xf1,0xf1,0xf1,0xf1,0x00,0xf1,0xf1,0x00,0x00,0x00,0x00,0x00,0xf1,0xf1,0xf1,0x00,
  0x00,0x00,0x00,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xbe,0xbe,0xf1,0xbe,0xf1,0xf1,0x00,
  0xca,0xca,0x00,0x00,0x00,0xca,0x00,0x01,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
  0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x01,
  0x01,0x00,0x00,0x00,0x00,0xca,0xca,0x00,0xca,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xbe,0xbe,0xbe,0x00,0xf1,0xf1,0xf1,0x01,0xf1,0x00,0x00,0x00,0x00,0xbe,0xf1,0x00,
  0x00,0x00,0x00,0x00,0x01,0x00,0x00,0xf1,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0xf1,0xf1,0xf1,0xf1,0xf1,0x00,0xf1,0x01,0x01,0x00,0xf1,0xf1,0x00,0x01,0x01,0x00,
  0x01,0xf1,0x01,0xf1,0xbe,0x01,0x01,0xca,0x01,0xf1,0x00,0xca,0x00,0x00,0xca,0x00,
  0xca,0xca,0xca,0xca,0xf1,0xf1,0xca,0x00,0xf1,0xf1,0xf1,0x00,0xca,0xca,0x00,0x00,
  0x00,0xca,0xca,0xca,0xca,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0xf1,0x01,0x01,0x00,0xf1,0x01,0x01,0x01,0x01,0x01,0x01,0x00,
  0x01,0x01,0x00,0x01,0x01,0xca,0xca,0x00,0xca,0x00,0x00,0xf1,0xf1,0xbe,0xbe,0xbe,
  0xbe,0x01,0xf1,0xf1,0xf1,0xca,0xf1,0xf1,0xf1,0xf1,0xf1,0xbe,0xf1,0xf1,0xf1,0xf1,
  0xbe,0xbe,0x00,0xbe,0xbe,0x01,0x01,0xf1,0x00,0x01,0x01,0xf1,0xf1,0xf1,0xf1,0xf1,
  0xf1,0xf1,0x01,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xbe,0x01,0xf1,
  0xbe,0xbe,0xbe,0xbe,0x01,0x01,0xf1,0xf1,0xf1,0xf1,0xca,0xca,0xca,0x00,0x00,0xca,
  0xca,0xca,0xf1,0x00,0x00,0xf1,0xca,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0x00,0x00,0x00,
  0x00,0x00,0xca,0x00,0x00,0x00,0x00,0xca,0xbe,0x01,0x01,0xbe,0xf1,0xf1,0x00,0xf1,
  0xf1,0xf1,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0xf1,0x01,0xf1,0xf1,0x00,0xf1,0x01,
  0x00,0x01,0xf1,0x01,0xf1,0xf1,0xf1,0xca,0xf1,0x00,0x00,0xca,0x00,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0xf1,0xf1,0xf1,0x00,0x00,
  0xf1,0xf1,0xf1,0xbe,0xbe,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0xf1,0x00,0x00,0xf1,
  // 14: sky 2, part C
  0x58,0xde,0x99,0xd6,0x9a,0xd6,0xda,0xde,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,
  0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,
  0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,
  0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,
  0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,
  0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,
  0x00,0x23,0x08,0x23,0x10,0x23,0x18,0xc6,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,
  0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,
  0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,
  0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,
  0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,
  0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x17,0xce,0x08,0x46,
  0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,
  0x10,0x60,0x18,0x60,0x00,0x61,0x7a,0xd6,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,
  0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,
  0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,
  0x78,0xe6,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,
  0x00,0x80,0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x58,0xf6,0x10,0x81,
  0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,
  0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,
  0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,
  0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,
  0x18,0xa1,0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,
  0x18,0xa3,0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0xf7,0xc5,
  0x10,0xa5,0x79,0xce,0x18,0xa5,0x99,0xe6,0xba,0xe6,0x00,0xa6,0x08,0xa6,0x10,0xa6,
  0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,
  0x18,0xc0,0x00,0xc1,0x08,0xc1,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,
  0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0xda,0xe6,0x38,0xf6,0x18,0xc3,0x00,0xc4,
  0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,0x10,0xc5,0x18,0xc5,0x00,0xc6,
  0x08,0xc6,0x10,0xc6,0x00,0xc7,0x9a,0xf6,0x08,0xc7,0x10,0xc7,0x18,0xc7,0x00,0xe0,
  0x08,0xe0,0x10,0xe0,0x18,0xe0,0x00,0xe1,0x08,0xe1,0x10,0xe1,0x99,0xf6,0x18,0xe1,
  0x00,0xe2,0x08,0xe2,0x10,0xe2,0xb6,0xbd,0x9a,0xce,0x5a,0xe6,0x00,0xe3,0x7a,0xce,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,
  0x01,0x6b,0xff,0x02,0x6b,0x02,0x02,0x02,0xfc,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,
  0x02,0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x6b,0xff,0x6b,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xfc,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0xfc,0xff,0xff,0xff,0x02,0xfc,0x02,0x02,0x02,0x03,0x02,0xff,0xff,
  0xfc,0xfc,0xfc,0x02,0xfc,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,
  0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x6b,0x02,
  0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x01,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,
  0x03,0x02,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,0xff,0x02,0x02,0x03,0x02,0x02,
  0x02,0x02,0x02,0x03,0x03,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x6b,0x02,0xfc,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x02,
  0x03,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x03,0x03,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x03,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x6b,0x02,0x6b,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x03,0x03,0x02,
  0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x02,0x02,0x02,0x02,
  0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x02,0x02,0x02,0x6b,0x6b,0x02,0x6b,0x02,0x02,0x02,0x6b,0x6b,0x02,0x02,0x6b,
  0x6b,0x6b,0x6b,0x02,0x6b,0x02,0x6b,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,0x03,0x03,0x03,0x03,0x02,0x03,
  0x02,0x03,0x03,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,
  0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x02,0x02,0x03,0x03,0x03,
  0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,0xfc,0x02,0x02,
  0x02,0x02,0x02,0x03,0x03,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xfc,
  0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x02,0x6b,0x02,0x6b,0x02,0x02,0x02,0x6b,0x02,0x6b,0x6b,0x02,0x6b,
  0x02,0x6b,0x6b,0x03,0x03,0x03,0xc4,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x02,
  0x02,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x02,0x03,0x03,0x02,
  0x02,0x02,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,0x6b,0x02,0x02,
  0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x03,0x02,0x6b,0x6b,0x02,0x6b,0x02,0x6b,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,
  0x02,0x02,0x02,0x6b,0x02,0x02,0x6b,0x6b,0x6b,0x02,0x02,0x02,0x01,0x02,0x6b,0x02,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x6b,0x02,0x02,0x02,0x02,0x6b,0x02,0x6b,
  0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,
  0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,
  0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x6b,0x03,0x03,0x03,0x6b,0x6b,
  0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x02,0x02,0x03,0x03,0x03,0x6b,0xc4,0x03,0x03,
  0xdc,0x03,0xc4,0xc4,0x6b,0x6b,0x02,0x02,0x02,0x6b,0x03,0x03,0x03,0x03,0x03,0x03,
  0xc4,0x03,0x03,0x03,0x03,0x02,0x02,0x6b,0x02,0x6b,0x6b,0x6b,0x01,0x02,0x02,0x02,
  0x6b,0x6b,0x02,0x6b,0x02,0x03,0x6b,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0xc4,0xc4,0xc4,0xc4,0xc4,0xc4,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,
  0x03,0x03,0x03,0x03,0x03,0x03,0x6b,0x6b,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x01,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0xdc,0xdc,
  0xdc,0xdc,0x03,0x02,0x6b,0x02,0x6b,0xc4,0xc4,0x03,0xc4,0xc4,0xc4,0xc4,0xc4,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xc4,0xc4,0x03,0x03,0x03,0xc4,0xc4,0x03,
  0x03,0xc4,0x03,0x03,0x03,0x02,0x03,0x03,0x03,0x02,0x02,0x6b,0x02,0x02,0x6b,0x02,
  0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x02,0x6b,0x6b,0x6b,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x6b,0x02,0x03,0x03,0x6b,0x02,0x02,0x03,0x03,0xc4,0xc4,0x03,
  0x03,0x03,0x03,0x03,0x03,0x6b,0x02,0xc4,0x6b,0x6b,0x02,0x02,0x6b,0x02,0x02,0x03,
  0x02,0x02,0x6b,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x6b,0x6b,0x6b,0x01,0x01,0x6b,0x02,0x02,0x02,0x6b,0x03,0x03,0x03,
  0xdc,0xdc,0xdc,0x03,0x03,0xdc,0x03,0x02,0x03,0x03,0x03,0xc4,0x03,0x03,0x03,0xdc,
  0xdc,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xdc,0x03,0x03,0x03,0x03,
  0x03,0x03,0xdc,0x03,0xc4,0x03,0x03,0x03,0x03,0xc4,0x02,0x02,0x02,0x02,0x02,0x02,
  0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,
  0x02,0x02,0x02,0x6b,0x03,0x03,0x03,0x02,0x03,0x03,0x02,0x6b,0x02,0x02,0x02,0xc4,
  0x03,0x03,0x03,0x03,0x03,0x02,0x6b,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x03,0xc4,
  0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x6b,0x6b,0x6b,
  0xc4,0x03,0x03,0x03,0x03,0xc4,0xc4,0x03,0x03,0x03,0x03,0xc4,0x03,0x03,0xdc,0xdc,
  0x03,0x03,0x03,0xdc,0xdc,0x03,0x03,0xdc,0x03,0x03,0x03,0xdc,0xdc,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,
  0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x02,0x02,
  0x02,0x02,0x02,0x6b,0x02,0x02,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x00,0x02,0x6b,0x03,0x03,0x03,0x03,
  0x02,0x03,0x03,0x03,0x03,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x02,0x02,
  0x6b,0x02,0x02,0x01,0x02,0x00,0x00,0x01,0x02,0x02,0x02,0x01,0x02,0x02,0x02,0x6b,
  0x02,0x02,0xc4,0x03,0xc4,0xc4,0xc4,0xdc,0xdc,0x03,0x03,0xc4,0xc4,0x03,0x03,0xc4,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x6b,0xc1,0x6b,0xc1,0x01,0x01,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x6b,0x02,0x02,0x02,0x03,0x03,0x03,
  0x03,0x03,0x02,0x02,0x02,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0xfc,0x02,
  0x02,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x01,0x01,0x02,0x02,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0xdc,0x03,0x03,0x03,0x02,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x03,0x03,0x03,0x02,0x03,0x03,0x03,
  0x03,0x02,0x02,0xc4,0x03,0x03,0xdc,0x03,0x03,0x03,0x02,0x02,0x02,0x03,0x6b,0x02,
  0x01,0x01,0x6b,0x6b,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0xc1,0x01,0x6b,0x01,0x01,
  0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x6b,0x02,0x02,0x02,0x02,0x02,
  0x03,0x02,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x6b,0xc1,0x01,0x01,
  0x6b,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0xdc,0x03,0x03,0x03,0x02,0x02,0x02,0x03,0x02,0x02,0x02,
  0x01,0x01,0x01,0x6b,0x6b,0x01,0x01,0x6b,0x6b,0x6b,0x01,0xc1,0xc1,0xc1,0xc1,0xc1,
  0xc1,0x01,0x01,0x6b,0x6b,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,
  0x01,0x01,0x02,0x02,0x01,0x02,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x01,0x02,0x6b,0x01,0x01,0x01,0x6b,0xc1,0x01,0x01,0x01,0x6b,
  0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x6b,0x01,0x01,0x01,0x01,0x01,0x01,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x01,0x6b,0xc1,0x6b,0x6b,0x01,0x01,0xc1,0xc1,0x6b,0xc1,0xc1,0xc1,0xc1,0xc1,0x01,
  0xc1,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x01,0x01,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x02,0x01,0x6b,0x6b,0x6b,0x01,
  0xc1,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x6b,0x01,0x01,
  0x01,0x02,0x02,0x02,0x03,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x01,0x6b,0x02,0x01,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x01,0x00,0x00,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,
  0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,
  0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xc1,0x01,0xc1,
  0xc1,0xc1,0x01,0x02,0x01,0x01,0x01,0x6b,0x01,0xc1,0x01,0x01,0xc1,0xc1,0x01,0x01,
  0x01,0x01,0x02,0x02,0x02,0x01,0x03,0x02,0x03,0x03,0x03,0x02,0x02,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0xdc,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x02,0x03,0xc4,0x03,0xdc,0xdc,0x03,0x03,0x03,0x03,0x03,0xc4,0xc4,0x01,0xc4,
  0xc4,0xc4,0x01,0xc4,0xc4,0xc4,0xc4,0xc3,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x02,0x02,
  0x01,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0xc1,0x01,0x01,0x01,0x02,0x01,
  0x01,0x01,0x01,0xc1,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0xc1,
  0x01,0x01,0x02,0x01,0x02,0x01,0xc1,0xc1,0xc1,0xc1,0x01,0x01,0xc1,0xc1,0xc1,0x01,
  0x01,0x02,0x02,0x02,0x02,0x01,0x02,0x02,0x01,0x02,0x03,0x01,0x03,0x03,0x03,0x03,
  0x02,0x03,0x02,0x02,0x01,0x01,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0xdc,0xdc,0x03,
  0xdc,0xdc,0xdc,0xdc,0x03,0xc4,0xc4,0xc4,0xc4,0xc4,0xc3,0xeb,0xeb,0xeb,0xc3,0xeb,
  0xc3,0xeb,0xc4,0xc3,0xc3,0xc4,0xc3,0x00,0x00,0x01,0x01,0x00,0x01,0x01,0x01,0x01,
  0x01,0x01,0xc1,0xc1,0x01,0x01,0x01,0xc1,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,
  0x01,0x01,0x02,0x01,0x01,0x01,0x01,0xc1,0x01,0x01,0x01,0xc1,0x01,0x01,0x01,0x01,
  0x01,0x01,0xc1,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x01,0x02,0x01,
  0x01,0x01,0x01,0x01,0x01,0xc1,0xc1,0xc1,0xc1,0x01,0xc1,0x01,0x01,0x01,0x01,0x02,
  0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x01,0x03,0x01,0x03,
  0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xc3,0xc3,0xc3,0xc4,0xc3,0xc3,0xc3,
  0xc4,0xeb,0xeb,0xf6,0xf6,0xf6,0xf6,0xf6,0x8e,0xf6,0x8e,0xf6,0x8e,0xf6,0xeb,0xeb,
  0xc3,0xc3,0xc3,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0xc1,
  0x01,0x01,0x01,0xc1,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x00,0x01,0x01,0xc1,0x01,0x01,0xc1,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0xc1,0xc1,0xc1,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x02,0x02,0x02,0x01,0x01,
  0x02,0x02,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xc3,
  0xc3,0x01,0x01,0x01,0x02,0x02,0x01,0x02,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x01,
  0x02,0x02,0xfd,0xc3,0xfd,0xfd,0xfd,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xf6,0xf6,0xc3,0xeb,0xc3,0xc3,0xc3,0x01,
  0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0xc1,0xc1,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,
  0x01,0xc1,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x01,0x00,
  0xc3,0xc3,0x01,0xc3,0x8e,0xc3,0x8e,0x8e,0x8e,0x8e,0x8e,0xc3,0xc3,0xc3,0xc3,0xc3,
  0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x01,0x02,
  0xc3,0xc3,0xdd,0xdd,0x8e,0xdd,0x8e,0xdd,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x8e,0x8e,0x8e,0x8e,0x8e,0xf6,0xeb,0xeb,0xc3,0xc3,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x00,0x01,0x01,0x00,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x00,
  0x00,0x01,0x01,0x00,0x00,0x00,0x00,0xc3,0x80,0x80,0xc3,0xc3,0xc3,0xc3,0x8e,0xdd,
  0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xc3,0xc3,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xc3,0xc3,0xc3,0xfd,0xfd,
  0xdd,0x8e,0xdd,0xdd,0xdd,0xdd,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0xf6,0xf6,0xf6,0xeb,0xeb,0xc3,0xdc,0xdc,0xdc,0xdc,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x00,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x01,0x01,0x00,0x00,0xc1,0x01,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0xc3,
  0x8e,0x8e,0x8e,0x80,0xc3,0xc3,0xc3,0x8e,0xdd,0xdd,0x8e,0xdd,0xdd,0x8e,0x8e,0x8e,
  0x8e,0x8e,0x8e,0x8e,0x8e,0xc3,0xc3,0x01,0xc3,0xc3,0xc3,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xc3,0xc3,0xc3,0xc3,0x8e,0x8e,0x8e,0x8e,0x8e,
  0xdd,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xf6,0xeb,0xdc,
  0xc4,0xdc,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,0x00,0x80,0x8e,0x8e,
  0x8e,0xdd,0xdd,0x8e,0x8e,0x8e,0x8e,0xdd,0xdd,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x8e,0x8e,0xc3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xdc,0xdc,
  0x01,0x01,0x01,0x01,0x01,0x01,0xc3,0x00,0xc3,0xc3,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xeb,0xc3,0xc3,0xc3,0xdc,0xc3,0x01,0xdc,0x01,
  0x01,0xdc,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x00,0x00,0x80,0x80,0x80,0x80,0x80,0x8e,0x80,0x80,0x80,0x80,0x8e,
  0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xdd,0xdd,0xdd,0xdd,0x8e,0xc3,0x00,0xc3,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0xc3,0xc3,0xc3,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xdd,0xdd,0x8e,0x8e,
  0x8e,0x8e,0xc3,0xc3,0xc3,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xc3,
  0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,
  0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,0x8e,0x80,0x80,0x80,0x80,0x8e,0x8e,0xdd,
  0xdd,0xdd,0xdd,0xdd,0xc3,0x80,0x80,0x00,0x80,0x80,0x00,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0xc3,0x00,0x01,0x01,0x00,0x8e,0xdd,0xdd,0xc3,0xc3,0xc3,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x01,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,
  0xc3,0x01,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x00,0x00,0x00,
  0x00,0x00,0x80,0x8e,0x8e,0x80,0x80,0x8e,0x8e,0x8e,0xdd,0xdd,0xdd,0xdd,0xdd,0x8e,
  0x8e,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x01,0x01,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0x80,0xc3,0x8e,
  0x8e,0x80,0x00,0x00,0x01,0x00,0x80,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x01,0x01,
  0x01,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x80,0xc3,0x00,0x00,0x01,0x01,
  0x00,0x00,0x00,0x00,0x01,0x00,0x01,0x00,0x00,0x01,0x01,0x00,0x01,0x01,0x01,0x01,
  0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x00,0x00,
  0x00,0x00,0x80,0x8e,0x80,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,
  0x00,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x01,0x01,0x01,
  0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x00,0x01,
  0x00,0x00,0x00,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x80,0x80,0x00,0x00,0x00,0x00,0x80,0x8e,0x8e,0x80,0x00,0x00,
  0x00,0x00,0x00,0x00,0x33,0x33,0x33,0x33,0x33,0x00,0x33,0x33,0x33,0x33,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x00,
  0x33,0x00,0x00,0x00,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x00,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x33,0x00,0x33,0x33,0x33,0x33,0x33,0x00,0x00,0x00,0x00,
  0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x33,0x00,0x00,0x33,0x00,0x00,0x33,0x33,0x33,0x00,0x33,0x33,
  0x33,0x00,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x00,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x00,0x33,0x33,0x33,0x00,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x5e,0x33,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,0x33,0x33,0x33,0x33,
  0x5e,0x5e,0x5e,0x5e,0x33,0x33,0x33,0x33,0x33,0x5e,0x33,0x33,0x33,0x33,0x33,0x5e,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x5e,0x5e,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,0x33,
  0x33,0x33,0x33,0x33,0x33,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0xbf,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,0x5e,0x5e,0x33,0x5e,0x5e,0x33,0x5e,0x5e,0x5e,
  0x33,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,0x33,0x33,0x33,0x33,0x33,
  0x5e,0xbf,0x5e,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0x5e,0x5e,0x5e,0x5e,0x5e,
  0xfb,0xfb,0xfb,0xbf,0xbf,0xbf,0xfb,0xfb,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,
  0xbf,0xbf,0xbf,0xbf,0x5e,0x5e,0x33,0x5e,0x5e,0x33,0x33,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,
  0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,
  0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x5e,0x33,0x33,0x33,0x5e,0x33,0xbf,0xbf,0xbf,0xbf,
  0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,0xbf,
  // 15: sky 2, part D
  0x4d,0xac,0x52,0xb5,0xf1,0xac,0xd5,0xc5,0xd6,0xc5,0x00,0x00,0x08,0x00,0x10,0x00,
  0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,
  0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,
  0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,
  0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,
  0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,
  0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,
  0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,
  0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,
  0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,
  0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,
  0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,
  0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,
  0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,
  0x18,0x62,0x08,0x63,0x00,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,
  0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,
  0x18,0x66,0x0d,0x94,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x30,0x84,
  0x08,0x80,0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,
  0x08,0x82,0x2c,0xa4,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,
  0x00,0x84,0x12,0xad,0x08,0x84,0xb5,0xbd,0x18,0x84,0x31,0xbd,0x00,0x85,0x08,0x85,
  0x10,0x85,0x94,0xc5,0x18,0x85,0x00,0x86,0x12,0xb5,0x08,0x86,0x10,0x86,0x18,0x86,
  0xac,0x8b,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,
  0x18,0xa0,0x4e,0x9c,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,0x11,0xad,
  0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x6e,0xa4,0x08,0xa3,0x10,0xa3,0x18,0xa3,
  0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x18,0xa5,0x00,0xa6,
  0x08,0xa6,0xb0,0xa4,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0xb6,0xbd,
  0x18,0xa7,0x00,0xc0,0xf0,0xb4,0x08,0xc0,0x71,0x94,0x10,0xc0,0x18,0xc0,0x00,0xc1,
  0x54,0xbd,0x08,0xc1,0xaf,0xac,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,
  0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x18,0xc3,0xed,0x8b,0x00,0xc4,0x08,0xc4,
  0x10,0xc4,0x93,0xbd,0xb0,0x9c,0x37,0xce,0x18,0xc4,0x00,0xc5,0x4d,0x9c,0xe9,0x5a,
  0x08,0xc5,0xf2,0xac,0xed,0x93,0x93,0xc5,0x0a,0x73,0x72,0xc5,0x8b,0x83,0x2e,0x94,
  0x4a,0x7b,0x08,0xc6,0x38,0xce,0xd1,0x9c,0x88,0x6a,0x58,0xd6,0x2f,0x8c,0x98,0xde,
  0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0xeb,0xeb,0xeb,0x04,0x04,0x04,0xeb,0xeb,0xeb,0xeb,0xeb,0x04,
  0xeb,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xd8,0x03,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xeb,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0xeb,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xd8,0xcf,0xcf,0xcf,0xcf,
  0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0x04,0xcf,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x03,0xcf,0x99,0x02,0x01,0x03,0x03,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xcf,0xcf,
  0xcf,0xcf,0xcf,0xcf,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x99,0x02,0x01,0xcf,0xcf,0xcf,
  0xcf,0xcf,0xcf,0xcf,0xcf,0x04,0xcf,0x04,0xcf,0xcf,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x03,0xcf,0x03,0x03,0x02,0xc9,0x02,0x02,0x01,0x03,0x03,
  0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0xeb,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xcf,0xcf,0xcf,
  0xcf,0xcf,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0x9b,0xf1,0xc9,0x02,0x99,0xe9,0x9b,0xcf,
  0x9b,0xcf,0xcf,0x03,0xcf,0x03,0x03,0x03,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x03,0x03,0x03,0x03,0x9b,0x02,0xc9,0xc9,0x02,0x02,0xa4,0x01,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x03,0x03,0x03,0x04,0x04,0x04,0x04,
  0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xeb,0xeb,0x04,
  0x04,0x04,0xeb,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xcf,0xcf,
  0xcf,0xcf,0xcf,0x9b,0x9b,0x9b,0x9b,0x9b,0x02,0xc9,0xc9,0x02,0x02,0x01,0x01,0x9b,
  0x9b,0x9b,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x04,
  0x04,0x03,0x04,0x04,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x03,0x03,0x04,
  0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x03,0x03,0x03,0x03,0x9b,0x9b,0xf1,0xc9,0xc9,0xc9,0xc9,0x02,0x02,0xa4,0xd8,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x04,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x03,0x04,0x04,0x04,0x04,0x04,0x04,0xcf,
  0xcf,0xcf,0xcf,0x9b,0x9b,0x9b,0xa1,0x02,0xc9,0xc9,0xc9,0xc9,0x02,0x02,0xa4,0x01,
  0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x04,0xa1,0x04,0x03,0x04,0x03,0x04,0x03,
  0x03,0x03,0x03,0x03,0x9b,0x9b,0x01,0xc9,0xc9,0xea,0xc9,0x02,0x02,0x02,0xa4,0xa4,
  0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x03,0x04,0x04,0x04,0x03,
  0x03,0x03,0x04,0x03,0x03,0x04,0x03,0x04,0x03,0xa1,0x04,0x04,0x04,0x04,0x04,0x03,
  0x03,0x03,0x9b,0x9b,0x9b,0x9b,0xa4,0xc9,0xc9,0xea,0xc9,0x02,0x02,0x02,0xa4,0xa4,
  0x9d,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0xa1,0x01,0xa1,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x04,0x03,0x03,0x03,0x99,0xfb,0x01,0x03,0x03,0x04,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x01,0x02,0xc9,0xc9,0xc9,0xc9,0x02,0x02,0x02,0x02,0x9d,
  0x01,0xd8,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x04,0x03,0x03,0x03,0x99,0xfb,0x01,0x04,0x03,0x04,0x03,0x03,
  0x03,0x03,0x03,0x9b,0xa1,0x01,0x02,0xc9,0xc9,0xc9,0xc9,0x02,0x02,0x02,0x02,0x9d,
  0xa4,0x9d,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0xd8,0x01,0x03,0xeb,0xeb,0xff,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xd8,0xc9,0xfb,0xfb,0x01,0x03,0x03,0x03,0x03,
  0x03,0x9b,0x03,0x9b,0x01,0x99,0x02,0x02,0x02,0x02,0xc9,0x02,0x02,0x02,0x02,0xa4,
  0x01,0x9d,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xd8,0xc9,0xfb,0xfb,0x01,0x03,0x03,0x03,0x03,
  0x03,0xa1,0xa1,0xd8,0xa1,0x01,0x02,0x02,0x02,0x02,0xc9,0x02,0x02,0x02,0x02,0xa4,
  0xa4,0xa4,0x9d,0x01,0xa1,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xa1,0xfb,
  0xd4,0xf1,0xa1,0xa1,0x03,0xeb,0xeb,0xff,0xeb,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0xe9,0x02,0x02,0x02,0xfb,0x02,0xd8,0x03,0x9b,0x9b,
  0x9b,0xa1,0xa1,0x01,0xa4,0x99,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0xa4,0xa4,0x9d,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0xe9,0x02,0x02,0x02,0xfb,0x02,0xd8,0x03,0xa1,0x99,
  0x99,0xfb,0x01,0xa1,0xe9,0x03,0x03,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0xa4,0x9d,0x9d,0x01,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xf1,0xfe,0xd4,
  0xea,0xc9,0xd8,0xe9,0xa1,0xeb,0x03,0xa1,0xeb,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x9b,0xa1,0x02,0x02,0x02,0x02,0xfb,0x02,0xfb,0x01,0x9b,0xa1,
  0xa1,0xa1,0x01,0x01,0xa4,0x99,0x02,0xd2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xd2,
  0xa4,0x9d,0x01,0x9d,0x01,0xa1,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0xcf,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x04,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0xa1,0x02,0x02,0x02,0x02,0xc9,0xc9,0xfb,0xf1,0xd4,0xd4,
  0xfb,0xfe,0x99,0x01,0x03,0x03,0xa1,0xa1,0xa1,0x01,0x99,0x02,0x02,0x02,0xd2,0xd2,
  0x02,0xf1,0x99,0xd8,0xa1,0xd8,0x01,0xa1,0x03,0xa1,0x01,0xea,0xe5,0xf6,0xe5,0xe5,
  0xd4,0xf1,0xd8,0x01,0x99,0x03,0xd8,0x03,0xeb,0x03,0x03,0x03,0x9b,0x03,0x03,0x03,
  0x03,0x03,0x03,0x9b,0xa1,0x99,0x02,0xfb,0xfb,0x02,0xfb,0xfb,0xfb,0x02,0x01,0xa1,
  0xe9,0xe9,0xd8,0x01,0xa4,0xa4,0xa4,0x02,0xd2,0x02,0x02,0xd2,0xd2,0x02,0xd2,0x02,
  0xd2,0x01,0x9d,0x9d,0xf5,0x01,0xa1,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0xd8,0x03,0x03,0x03,0x03,0x03,0x03,0xa1,0xd4,0xf1,0xe5,0xa1,0x03,
  0x03,0x03,0x03,0x9b,0xa1,0x99,0x02,0xfb,0xfb,0x02,0xf1,0xe9,0x01,0xe9,0x01,0x99,
  0xfe,0x99,0xd8,0xa1,0x01,0x01,0xd8,0x03,0xf3,0xa1,0x03,0x01,0x01,0xf1,0x99,0x02,
  0x01,0x99,0x99,0x99,0x01,0xfb,0xfe,0xd4,0xfe,0xfe,0xe5,0xd4,0xfe,0xf8,0xf6,0xf6,
  0xf2,0xfe,0xf1,0xfb,0xa1,0xd8,0xe9,0x03,0x03,0xe9,0x03,0x03,0xe9,0x01,0xa1,0xe9,
  0x03,0x03,0x03,0xa1,0x9d,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x01,
  0xe9,0x01,0xf5,0xf5,0x01,0xa4,0xa4,0xb7,0x02,0xd2,0x02,0xd2,0xd2,0x02,0xd2,0xd2,
  0xd2,0xa4,0x9d,0x01,0x01,0xf5,0x01,0xa1,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x99,0x03,0x03,0x03,0x03,0x03,0x99,0xfb,0xf4,0xf4,0xe5,0xe5,0xfb,
  0x03,0x03,0x03,0xa1,0x01,0x02,0x02,0x02,0xfb,0xfb,0x01,0xd8,0xe9,0xa1,0xa1,0x99,
  0xb1,0x99,0x01,0x03,0x01,0x02,0xfb,0xfb,0xf7,0xea,0xe9,0xe9,0xc9,0xf1,0xfb,0x01,
  0x01,0xfb,0xfe,0xb1,0xe5,0xe5,0xd4,0xd4,0xd4,0xc9,0x02,0xe5,0xea,0xf8,0xf8,0xa8,
  0xf6,0xf2,0xd4,0xfb,0xfb,0xfb,0x01,0xf1,0x03,0x03,0x04,0x03,0xeb,0xeb,0xa1,0x9b,
  0x01,0xe9,0xa1,0x01,0x99,0x02,0xb7,0x02,0x02,0x02,0x02,0xfb,0x02,0x02,0x02,0x02,
  0x01,0x01,0xf5,0xf5,0x01,0x01,0xa4,0xa4,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0xb7,
  0xd2,0xa4,0x9d,0x9d,0xf5,0x01,0x01,0xf3,0xf3,0xa1,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0xf1,0x03,0x03,0x03,0x03,0x03,0x02,0xfe,0xf6,0xf4,0xf6,0xe5,0x99,
  0x01,0x03,0xa1,0x01,0x99,0xa4,0x9d,0xd2,0xf5,0x02,0xf1,0xf1,0xfb,0xf1,0x99,0x99,
  0xa1,0xa1,0xd8,0xa1,0x01,0xa1,0x03,0xb1,0xfe,0xd4,0xf7,0xf6,0xe5,0xf6,0xe5,0xd4,
  0x01,0xd4,0xfb,0x01,0x01,0x01,0xc9,0xa8,0xf7,0xd4,0xea,0xd4,0xf7,0xf7,0xf2,0xa8,
  0xf6,0xa8,0x81,0xea,0xfb,0x02,0xfb,0xd4,0x01,0xd8,0x03,0xa1,0xeb,0x04,0x03,0xa1,
  0xa1,0xa1,0xa1,0xd8,0x99,0x99,0xb7,0x99,0x99,0xb7,0x02,0x02,0x02,0x02,0x02,0x02,
  0x99,0xf5,0xf5,0xf5,0xf5,0x9d,0x01,0x9d,0x9d,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,0xa4,
  0xa4,0xa4,0x9d,0x9d,0x01,0x01,0xf5,0x01,0xf3,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0xf1,0xeb,0xeb,0x03,0x03,0xfb,0xd4,0xe5,0xf6,0xf6,0xe5,0xe5,0xf4,
  0xfe,0xe9,0x01,0x99,0x01,0xf3,0xa4,0x01,0x03,0xa4,0xc9,0x02,0x02,0x02,0x02,0xd8,
  0x02,0xfb,0xfb,0xfb,0xf1,0x01,0xf5,0x03,0x01,0xa1,0xa1,0x02,0xf1,0xd4,0xfe,0xf1,
  0x02,0x99,0x01,0x01,0xa1,0x01,0xf1,0x02,0xa8,0xf7,0xf7,0xf2,0xe5,0xf7,0xd4,0xf7,
  0xe5,0xf6,0xb1,0xb1,0xf7,0xd4,0xd4,0xc9,0x99,0x02,0x02,0x01,0x03,0xa1,0x03,0x03,
  0x03,0x03,0xe9,0xa1,0xa4,0x99,0xa4,0xa4,0x99,0xa4,0xa4,0x99,0xb7,0xb7,0x99,0xa4,
  0xa4,0xa4,0xf5,0xf5,0xf5,0xf5,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,0x9d,
  0x9d,0xa4,0x9d,0x9d,0x9d,0x01,0x01,0x01,0x01,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,
  0xf3,0xf3,0xf3,0xeb,0xfd,0xeb,0xeb,0x03,0x02,0xd4,0xfe,0xe5,0xfc,0xe5,0xd4,0xfb,
  0xfb,0xf3,0x01,0xf3,0x03,0x03,0xf3,0x03,0x03,0x03,0x9d,0xa4,0xd8,0x99,0x02,0x01,
  0x02,0x01,0x02,0xd4,0xf1,0x01,0x01,0x03,0x03,0xd2,0x01,0xa1,0x01,0x01,0xd8,0x99,
  0xc9,0x02,0xfb,0x02,0xfb,0x02,0xfb,0xc9,0xa8,0xe5,0xe5,0xa8,0xa8,0xf8,0xa8,0xf7,
  0xc9,0xe5,0xf7,0xb1,0xb1,0xb1,0xb1,0xea,0x02,0x99,0x01,0xa1,0xe9,0xa1,0xe9,0xa1,
  0xd8,0x9b,0xd8,0x01,0xa1,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xa4,0x01,0xa4,0xa4,
  0x01,0x01,0x01,0x9d,0x9d,0x9d,0x9d,0x9d,0xd2,0xd2,0xa4,0x01,0x9d,0x9d,0x9d,0x9d,
  0x9d,0xa4,0xa4,0x9d,0x9d,0xa4,0x9d,0x9d,0x9d,0x9d,0xf5,0xd2,0xe5,0xc9,0xe5,0xb7,
  0xf5,0x99,0xf3,0xcf,0xfb,0xd4,0x99,0x01,0x02,0xf3,0xf3,0xea,0xef,0xb7,0xf3,0xf5,
  0x01,0xf5,0xa1,0x03,0xeb,0x03,0x03,0xe9,0xa1,0xa1,0x01,0x03,0x03,0x01,0xe9,0xd8,
  0x01,0xd8,0x02,0x99,0x01,0x01,0x02,0x99,0x99,0xe9,0xe9,0x01,0xa4,0xbc,0x01,0x02,
  0x02,0xf7,0xea,0xfb,0xb1,0xb1,0xb1,0xa8,0xa8,0xa8,0xf6,0xe5,0xe5,0xf4,0xf4,0xf4,
  0xf6,0xf8,0xf8,0xa8,0xe5,0xe5,0xf7,0xb1,0xb1,0x99,0x01,0xa1,0x01,0x01,0x9b,0x01,
  0xa1,0xf1,0x01,0xa1,0x01,0xa1,0xfb,0xfb,0xa4,0xf5,0xf5,0xf5,0xf5,0x01,0x01,0x01,
  0x01,0x01,0x9d,0x9d,0x01,0x01,0xa4,0xa4,0xd2,0xd2,0xd2,0xa4,0xd2,0x99,0x99,0xfb,
  0xd4,0x02,0xf5,0xf5,0xf5,0xf5,0xf5,0xf5,0xf5,0x01,0xf5,0xc9,0xfc,0xe5,0xf4,0xbc,
  0xf3,0x02,0xd8,0x04,0xf1,0xe5,0xe5,0x99,0x02,0xf5,0x9d,0x02,0xf4,0x01,0x01,0xf5,
  0xf5,0xa1,0x03,0xeb,0x03,0x04,0xa1,0xeb,0xff,0xff,0xff,0xeb,0x03,0xa1,0x01,0xf3,
  0xf3,0xa4,0xf7,0x99,0xa4,0xb7,0xd8,0xd8,0x02,0xda,0xa4,0x02,0xd2,0xbc,0x02,0xa4,
  0xbc,0xc9,0x81,0xa8,0xa8,0xa8,0xa8,0xf6,0xf6,0xa8,0xa8,0xf6,0xf6,0xf8,0xf4,0xf4,
  0xf4,0xf8,0xa8,0xf6,0xf6,0xa8,0xe5,0xe5,0xe5,0xd4,0xf1,0x02,0x02,0x99,0xd8,0x99,
  0x01,0xd8,0x99,0xfb,0xa1,0xfb,0x87,0xfb,0xfe,0xa4,0x01,0x01,0x01,0x01,0x01,0x01,
  0xa4,0xa4,0xb7,0xd2,0xd2,0xd2,0xd2,0x9d,0x9d,0xd2,0xd2,0xa4,0xd2,0x03,0xd8,0xfb,
  0xe5,0x01,0x99,0xd4,0xf5,0x9d,0xf5,0xf5,0xf5,0xf5,0x9d,0xbc,0xef,0xf7,0xef,0x99,
  0xeb,0xfd,0xfd,0xff,0xfa,0x9b,0x9b,0x04,0xfd,0xa1,0x9d,0x02,0x71,0xa4,0x01,0x01,
  0x03,0x03,0x03,0x03,0x03,0x03,0xe9,0xe9,0xa1,0xf3,0xa1,0x03,0x03,0x03,0x03,0xa1,
  0x01,0xe9,0xc9,0xb1,0xe5,0x02,0xd8,0xe9,0xa4,0x02,0xda,0xda,0xee,0xda,0xbc,0xbc,
  0xbc,0xb1,0xbc,0x81,0xa8,0xf6,0xf6,0xf6,0xa8,0xa8,0xa8,0xf6,0xf6,0xf8,0xf8,0xf6,
  0xf8,0xf6,0xa8,0xa8,0xe5,0xe5,0xf6,0xf6,0xf6,0xe5,0xfb,0xfb,0xd4,0xb1,0xd8,0xd8,
  0x99,0x99,0x99,0xfe,0xd4,0xd4,0x01,0xd4,0xd4,0xd8,0x01,0x99,0xa4,0x99,0xa4,0xb7,
  0xa4,0xb7,0xa4,0xd2,0xd2,0xd2,0xd2,0xd2,0x9d,0x9d,0xd8,0xe5,0xd8,0xd8,0x03,0x87,
  0xb1,0x03,0x01,0x02,0xfb,0xe9,0xeb,0x03,0x03,0xf7,0x71,0x4e,0x4e,0x4e,0xfc,0xf6,
  0xd4,0xb1,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0x99,0x99,0xb1,0xfc,0xf4,0xb7,0x01,
  0xf5,0xf5,0xa4,0xa1,0xa1,0x03,0xfb,0x02,0xe9,0xe9,0xeb,0xeb,0x03,0x03,0x03,0x03,
  0x03,0xeb,0xa4,0xe9,0x01,0x01,0x02,0xc9,0x02,0xa4,0x02,0x02,0xc9,0xbc,0xb1,0xf7,
  0xb1,0xb1,0xbc,0x02,0x02,0xbc,0x81,0x81,0xa8,0xa8,0xf6,0xa8,0xf6,0xf6,0xf6,0xa8,
  0xa8,0xa8,0xf6,0xa8,0xe5,0xf6,0xf6,0xe5,0xa8,0xe5,0xd4,0xb1,0xf7,0x81,0xf7,0xf6,
  0xf6,0xfb,0x87,0xe5,0xfb,0x99,0x01,0xfb,0xfb,0xfb,0x99,0xd8,0xf1,0x99,0xf1,0x99,
  0x02,0x02,0xf1,0x02,0xd2,0xd2,0xd2,0xd2,0xd2,0xf3,0xa1,0xd8,0x02,0xe9,0xa1,0xd4,
  0xe5,0xf7,0xf7,0x01,0xa1,0xfb,0xa1,0x03,0xa1,0xd8,0xe5,0x4e,0x4e,0x4e,0x4e,0xf4,
  0x81,0xfa,0xff,0xd4,0xf4,0xa8,0xd4,0xe5,0xf4,0xf6,0xe5,0xe5,0xfc,0xfc,0x02,0x01,
  0x9d,0x01,0x01,0x02,0xfb,0x99,0xfb,0x02,0xe9,0x01,0xd8,0x03,0xf3,0x9d,0x03,0x01,
  0xf3,0xa1,0xa1,0x02,0x01,0x03,0xa1,0xe9,0xa4,0xbc,0xa4,0x02,0x02,0xbc,0xd4,0xb1,
  0xc9,0xb1,0xb1,0xf7,0xbc,0x02,0xb1,0xbc,0xbc,0xbc,0xbc,0x81,0xf2,0xa8,0xa8,0xf6,
  0xf6,0xa8,0xa8,0xa8,0xf6,0xa8,0xe5,0xa8,0xa8,0xf6,0xe5,0xa8,0xe5,0xf6,0xf6,0xd4,
  0x87,0x99,0x87,0xd4,0xd4,0xf6,0x01,0xfb,0xf1,0x87,0x87,0xfb,0x99,0x04,0x04,0xfb,
  0xb1,0x02,0xd4,0xf1,0xf3,0x03,0xfb,0xf3,0xfb,0x03,0x99,0xa1,0xfb,0xf1,0x99,0xea,
  0x02,0xf6,0xfe,0x03,0xe9,0xfb,0x87,0x01,0x99,0x99,0x99,0xfc,0xfc,0x4e,0x4e,0xef,
  0xfc,0xfc,0xe5,0xe5,0xfc,0x4e,0xfc,0xf6,0xe5,0xe5,0xf4,0xe5,0xef,0x02,0x99,0xfb,
  0x02,0xea,0xfe,0xf7,0xe5,0xfe,0x01,0x99,0x01,0xfb,0x02,0xa1,0x03,0xf3,0xe9,0x01,
  0xf5,0xd8,0x01,0x01,0xf3,0xf5,0x03,0x01,0xe9,0xd2,0xbc,0x01,0xda,0xda,0xda,0xc9,
  0xbc,0xbc,0xbc,0xda,0xbc,0xbc,0x81,0xee,0xda,0xbc,0x02,0xb1,0xb1,0x81,0x81,0x81,
  0xf2,0xf2,0xf2,0xa8,0xa8,0xe5,0xe5,0xa8,0xe5,0xd4,0xd4,0xfe,0x87,0xd4,0x01,0x99,
  0xfb,0xf6,0xd4,0x99,0x99,0xfb,0x01,0x01,0xd8,0xa1,0xa1,0x01,0xf6,0xf4,0xf4,0xf6,
  0xe5,0xef,0xef,0xe5,0xf7,0xd2,0x03,0x9d,0xf6,0xf4,0x87,0x01,0xfb,0xfb,0x01,0xfb,
  0xd4,0x99,0x99,0xd4,0xfb,0x9b,0x99,0x03,0x03,0xa1,0x03,0xfc,0x4e,0x4e,0x4e,0xef,
  0xfc,0xf4,0xa8,0xe5,0xe5,0xf4,0xfc,0xe5,0xe5,0xf7,0xf6,0xa8,0xf4,0xe5,0xd4,0xd4,
  0xfb,0xd8,0x01,0xf1,0xfb,0xfb,0x01,0xe9,0xa1,0x01,0x9b,0x03,0x03,0x03,0x01,0x02,
  0xd8,0x01,0xa4,0xf3,0x03,0xf3,0x01,0xa4,0xa4,0x02,0xb1,0xb1,0xa4,0xbc,0x91,0x91,
  0xbc,0xbc,0xbc,0xbc,0xda,0x02,0x02,0x02,0xbc,0xb1,0xbc,0x02,0x02,0xbc,0xc9,0x02,
  0xbc,0x9d,0xd2,0xa4,0xbc,0xc9,0xc9,0xf1,0xfb,0xfb,0xfb,0x87,0xe5,0xe5,0x87,0xfe,
  0xf6,0xf6,0x87,0x99,0x9b,0xa1,0xfb,0xd4,0xfb,0xe5,0x71,0xf6,0xef,0xef,0xef,0xef,
  0xef,0xef,0xef,0xfc,0xef,0xef,0xfc,0xef,0xf4,0xf4,0xe5,0xe5,0xf1,0x99,0xf1,0x02,
  0xe9,0xfb,0xa4,0x02,0xc9,0xd2,0xd2,0xa4,0xd2,0xd2,0x02,0xbc,0xf6,0xef,0xfc,0xef,
  0xf4,0xe5,0xe5,0xe5,0xe5,0xe5,0xf7,0xf7,0xe5,0xe5,0x71,0xfc,0xfc,0xf4,0xf6,0xf8,
  0xf6,0xfc,0xd8,0xeb,0x04,0xa1,0xfb,0xd4,0xd4,0xfe,0xfe,0xd4,0x99,0x02,0xd4,0xb1,
  0xf7,0xc9,0xd2,0xf5,0x9d,0x01,0xf5,0x01,0xd2,0xf5,0x9d,0x01,0xf5,0xda,0x02,0x02,
  0xc9,0xb1,0xe5,0xf6,0xb1,0xbc,0x02,0x02,0xd2,0x02,0xd2,0x9d,0x01,0xd8,0xa1,0xe9,
  0xd8,0xf5,0x01,0xa4,0xf5,0xa4,0x9b,0x03,0xd8,0x01,0xd8,0x03,0xf1,0x99,0x01,0xfb,
  0xd4,0xfb,0x02,0xfb,0xf1,0x02,0xef,0x71,0xfc,0x71,0xf6,0xf4,0xf6,0xe5,0xf6,0xef,
  0x87,0xef,0xf4,0xef,0xf4,0xf4,0xfc,0xfc,0xef,0xf6,0xd4,0xd4,0xa4,0xd2,0xd2,0x02,
  0x02,0xbc,0xbc,0xbc,0xd2,0x02,0xd2,0x9d,0x01,0x9d,0x01,0xe9,0xea,0xfe,0xe9,0xe9,
  0x03,0x03,0xa1,0x01,0x03,0x03,0xff,0xff,0x03,0xeb,0x9b,0xeb,0x01,0xe9,0xf1,0xd4,
  0xfb,0xe5,0xc9,0xfb,0x99,0x01,0xd8,0xa1,0x03,0xa1,0x01,0x9b,0xeb,0xa1,0xfb,0x99,
  0xea,0xfb,0x02,0x02,0xd2,0xbc,0xc9,0x02,0x9d,0xd8,0xa4,0xa4,0xa4,0x9d,0xa4,0x02,
  0xa4,0x9d,0x9d,0xf7,0xa8,0xe5,0xf6,0xe5,0x81,0x81,0xf2,0xb1,0x02,0x9d,0xa4,0x01,
  0xa1,0xf5,0x03,0x01,0x01,0xf1,0xa1,0xa1,0x01,0x99,0x02,0x01,0xd8,0x03,0x9b,0x03,
  0x9b,0xa1,0xfb,0xf1,0x01,0x01,0xd4,0xfb,0xa1,0x99,0xfb,0x87,0xf6,0xf4,0xe5,0xd4,
  0x87,0xe5,0x87,0xe5,0xf7,0xf6,0xf6,0xf6,0xf6,0xe5,0x02,0xfb,0xf2,0xee,0x02,0xa4,
  0xd2,0x9d,0x9d,0xf2,0xf6,0xf8,0xfc,0xf4,0xe5,0xf2,0x81,0xc9,0xf5,0x03,0xa1,0x03,
  0xff,0x03,0xeb,0x03,0x03,0xd8,0xa1,0xa1,0x01,0x01,0x01,0xd8,0x9b,0xeb,0x03,0xeb,
  0x03,0xa1,0xf1,0x01,0xa1,0xd8,0x99,0x99,0x04,0x9b,0x01,0x01,0xfb,0xeb,0xeb,0x03,
  0x03,0x01,0x99,0xe9,0x01,0xe5,0xf6,0xf4,0xf4,0xf8,0xf6,0xe5,0xc9,0xbc,0xa4,0xf5,
  0xa4,0x02,0x9d,0x9d,0xa4,0xd2,0xbc,0xa8,0xf6,0xe5,0xb1,0x02,0xc9,0xd2,0x02,0xbc,
  0xe5,0xb1,0x99,0xea,0x03,0xe9,0xe9,0xe9,0x02,0x03,0xd8,0xa1,0x03,0x03,0xe9,0x03,
  0xd8,0x01,0x03,0x9b,0xeb,0x03,0xfd,0xeb,0xeb,0xa1,0xa1,0xa1,0x9b,0xf1,0xf1,0xd8,
  0xd8,0x02,0x01,0x02,0xc9,0xb1,0xee,0x91,0xbc,0xbc,0xbc,0xa8,0xf2,0xda,0x9d,0xf5,
  0xf3,0xd2,0xf5,0xf5,0xd2,0x02,0xee,0xa8,0xf8,0xf2,0xb1,0xa4,0xa4,0x9d,0x02,0x02,
  0x81,0xb1,0x01,0xc9,0x03,0xa1,0xa1,0xa1,0x02,0xeb,0x03,0x03,0xeb,0x03,0xe9,0xeb,
  0xd8,0x01,0x03,0x03,0xfa,0xeb,0xfd,0xeb,0xeb,0xa1,0xa1,0xa1,0x03,0xf1,0x99,0xd8,
  0x04,0xa1,0x9b,0x02,0xb1,0xb1,0xbc,0xbc,0xee,0x81,0xbc,0xb1,0xf2,0x02,0xa4,0x9d,
  0xbc,0x02,0xa4,0xa4,0x02,0xda,0xbc,0x81,0xf2,0xf8,0xf2,0xee,0x91,0x81,0xf6,0xf4,
  0xf8,0xe5,0xf2,0xf8,0xf2,0xf7,0xf2,0xe5,0xe5,0xc9,0xb1,0xb1,0x02,0xb1,0xf7,0x01,
  0xd8,0xc9,0xc9,0xb1,0xfb,0xd8,0x01,0x03,0xfa,0xeb,0x01,0x99,0x02,0x99,0xa4,0x01,
  0xd4,0xfe,0xfb,0x01,0x02,0xbc,0x91,0x81,0x81,0x91,0x91,0x81,0x00,0x00,0x91,0xbc,
  0xda,0xda,0x9d,0x9d,0xd2,0xda,0xbc,0x81,0xf2,0xf4,0xa8,0x91,0x91,0x81,0xf6,0xfc,
  0xf8,0xf2,0xa8,0xf4,0xf2,0x81,0xa8,0xa8,0xe5,0x02,0xb1,0xb1,0xd2,0xbc,0xf7,0xe9,
  0xd8,0xc9,0xc9,0xb1,0xfb,0xd8,0x01,0x03,0xfa,0xeb,0x01,0x99,0x02,0x99,0x99,0x99,
  0xd4,0xfe,0xfb,0x01,0x02,0xbc,0x91,0x81,0x81,0x91,0xbc,0xee,0xbc,0xbc,0xbc,0xda,
  0x91,0xbc,0x81,0xbc,0xbc,0x91,0xbc,0xbc,0x91,0x81,0x81,0x81,0x81,0x81,0xb1,0x81,
  0xb1,0x81,0x81,0x81,0xf2,0x81,0x81,0xee,0x81,0xf2,0xf2,0x81,0xf2,0x81,0xbc,0xc9,
  0xe9,0x02,0xf7,0xbc,0x02,0x02,0xbc,0xf2,0xc9,0x01,0xa1,0xb1,0xe9,0xd8,0xf1,0x99,
  0xa4,0xc9,0x01,0x02,0x01,0xc9,0xbc,0x81,0x81,0x81,0x91,0x81,0x91,0x91,0xbc,0x81,
  0x81,0x91,0x81,0xbc,0xbc,0x91,0xbc,0xbc,0x91,0x81,0x81,0x81,0xee,0x81,0xb1,0xee,
  0x81,0x81,0x81,0xf2,0xf2,0x81,0x81,0xee,0x81,0xf2,0x81,0x81,0xf2,0x81,0xbc,0xc9,
  0xe9,0x02,0xf7,0xbc,0x02,0x02,0xbc,0xf2,0xc9,0x01,0xa1,0xb1,0xe9,0xd8,0xf1,0x02,
  0x02,0xc9,0x01,0x02,0x01,0xc9,0xbc,0x81,0x81,0x81,0x91,0x81,0x91,0x91,0xbc,0x81,
  0x91,0x81,0x91,0x81,0x81,0x81,0x81,0x91,0x81,0x91,0x91,0x91,0x91,0x81,0x81,0x91,
  0x91,0x91,0x81,0x91,0x91,0x91,0x91,0x91,0x81,0x91,0x81,0x81,0x81,0x81,0x81,0x81,
  0xee,0xbc,0xb1,0x81,0x81,0xa8,0x81,0x81,0x81,0x81,0xbc,0x81,0x81,0x81,0xf2,0x81,
  0x81,0x81,0xee,0x81,0x81,0x81,0x91,0x91,0x91,0x00,0x81,0x81,0x81,0x81,0x81,0xda,
  0x91,0x81,0x91,0x81,0x81,0x81,0x81,0x91,0x81,0x91,0x91,0x91,0x91,0x81,0x81,0x91,
  0x91,0x91,0x81,0x91,0x91,0x91,0x91,0x91,0x81,0x91,0x81,0x81,0x81,0x81,0x81,0x81,
  0xee,0xbc,0xb1,0x81,0x81,0xa8,0x81,0x81,0x81,0x81,0xbc,0x81,0x81,0x81,0xf2,0x81,
  0x81,0x81,0xee,0x81,0x81,0x81,0x91,0x91,0x91,0x00,0x81,0x81,0x81,0x81,0x81,0xda,
  0x00,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0x00,0x00,0x00,0x00,0xbc,0xbc,
  0x00,0x00,0x91,0x00,0x00,0x00,0x91,0x00,0x91,0x91,0x91,0x91,0x00,0x91,0xbc,0x00,
  0x91,0x91,0x91,0xbc,0x91,0xbc,0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x00,0xbc,
  0x00,0x00,0x00,0x00,0x00,0xbc,0x91,0x00,0x00,0x00,0x91,0x00,0x91,0x91,0x91,0x00,
  0x00,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0x00,0x91,0x00,0x00,0xbc,0x00,
  0x00,0x00,0x91,0x00,0x00,0x00,0x91,0x00,0x91,0x91,0x91,0x91,0x00,0x00,0x00,0x00,
  0x00,0x00,0x91,0xbc,0x91,0xbc,0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x91,0x00,0xbc,
  0x00,0x00,0x00,0x00,0x00,0xbc,0x00,0x00,0x00,0x00,0x91,0x00,0x91,0x91,0x91,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  // 16: sky 3, part A
  0xf9,0xad,0xfa,0xad,0xb9,0xad,0x1b,0xb6,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,
  0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,
  0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0x18,0x04,
  0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,
  0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,
  0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,
  0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,
  0x00,0x25,0x08,0x25,0xfb,0xb5,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,
  0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,
  0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,
  0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,
  0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,
  0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,
  0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,
  0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,
  0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,
  0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,0x10,0x80,
  0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,
  0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,0x08,0x84,0x5c,0xbe,
  0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,0x08,0x86,
  0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,
  0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,0x08,0xa2,
  0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x00,0xa4,0x08,0xa4,
  0x10,0xa4,0x18,0xa4,0x00,0xa5,0x5b,0xbe,0x08,0xa5,0x10,0xa5,0x18,0xa5,0x00,0xa6,
  0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x18,0xa7,0x00,0xc0,
  0x08,0xc0,0x10,0xc0,0x18,0xc0,0xd8,0xb5,0x00,0xc1,0x08,0xc1,0x10,0xc1,0x18,0xc1,
  0x00,0xc2,0x08,0xc2,0x19,0xb6,0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,
  0x18,0xc3,0x00,0xc4,0x08,0xc4,0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,0x10,0xc5,
  0x18,0xc5,0x00,0xc6,0x08,0xc6,0x10,0xc6,0x18,0xc6,0x00,0xc7,0x08,0xc7,0x7d,0xc6,
  0x10,0xc7,0x18,0xc7,0x00,0xe0,0x08,0xe0,0x10,0xe0,0x18,0xe0,0x00,0xe1,0x08,0xe1,
  0x10,0xe1,0x18,0xe1,0x1c,0xae,0x00,0xe2,0x08,0xe2,0x10,0xe2,0x3c,0xb6,0x08,0xe3,
  0x10,0xe3,0x00,0xe4,0x1a,0xae,0x18,0xe4,0x00,0xe5,0x08,0xe5,0x00,0xe6,0x08,0xe7,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0x02,0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,
  0x02,0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0xc2,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0xc2,0xc2,0xc2,0xc2,
  0x02,0x02,0x02,0xc2,0xc2,0xc2,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,
  0x02,0x02,0xc2,0xc2,0x02,0x02,0x02,0x02,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0xc2,0xc2,0x02,0x02,0xc2,0x02,0xc2,0x02,0x02,
  0x02,0x02,0x02,0xc2,0x02,0x02,0xc2,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,
  0x02,0x02,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0xc2,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,
  0x02,0x02,0x02,0x02,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,
  0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,
  0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0xc2,0x02,0xc2,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,
  0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x02,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0xc2,0xc2,0x02,0xc2,
  0xc2,0xc2,0xc2,0x00,0x00,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x00,
  0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x02,
  0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x02,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0xc2,0xc2,
  0xc2,0xc2,0x00,0x00,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,
  0x00,0x00,0x00,0x02,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,
  0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,
  0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,
  0x02,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0xc2,0xc2,
  0xc2,0xc2,0xc2,0xc2,0x02,0xc2,0x02,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x02,0x02,0x00,0x00,0x02,0x02,0x02,0x00,0x02,0x02,0x00,0x02,0x02,0xc2,
  0xc2,0xc2,0xc2,0xc2,0x02,0xc2,0x02,0x02,0x00,0x02,0x00,0x02,0x00,0x00,0x00,0x00,
  0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,
  0xc2,0xc2,0xc2,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x00,
  0x00,0x00,0x00,0xc2,0x00,0x00,0x00,0x02,0x00,0x00,0xcb,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x01,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x01,0x01,0x00,0xfa,0x00,0x00,0x00,0x01,0x01,0x01,0x00,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x02,0x02,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x00,0x00,0x02,0x00,0x00,0x00,0x02,0x02,0x00,
  0x01,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xc2,
  0xc2,0x02,0xc2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0xcb,0x00,0x02,0x00,0x00,0x00,
  0x00,0xd2,0x00,0x00,0x00,0x01,0xfa,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xfa,0x01,
  0x01,0x01,0xd2,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x3a,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x02,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0xc2,0x02,0xc2,0xc2,0xc2,0x02,0x02,0x02,0x00,0x00,0x02,0x00,0x02,0x00,0x00,0x00,
  0x00,0x02,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x02,0xc2,
  0x02,0xc2,0xc2,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xcb,0x00,0x00,0x00,0x00,0x00,0xcb,0xcb,0x00,0x02,0x02,0x02,0x00,
  0x00,0x00,0x00,0x00,0xd2,0xfa,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x03,0x01,0x01,0xd2,0x01,0x01,0x01,0x03,0x03,0x01,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x01,0x01,0x01,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x00,
  0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x02,0x02,
  0xc2,0x02,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0xcb,0x00,0x00,0x02,0x00,0x00,
  0xcb,0xd2,0x00,0x01,0x01,0xfa,0x01,0x01,0x01,0xfa,0x03,0x03,0xd2,0x01,0x01,0xd2,
  0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xbb,0xbb,0xbb,
  0xbb,0xbb,0x03,0x03,0x03,0x03,0x3a,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x00,0x02,0x02,0x02,0xc2,0x02,0x02,0x00,0x00,0x02,0x02,0x01,0x00,0x02,0x02,
  0x01,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0xc2,0x02,
  0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xcb,0x00,0x00,
  0x00,0x00,0x00,0x00,0xcb,0x02,0x02,0xc2,0x02,0xcb,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x01,0xd2,0x01,0x01,0x01,0x01,0x01,0x03,0x01,0x03,0x03,0x03,0x03,0xd2,
  0x01,0x03,0x03,0xbb,0xbb,0x03,0x03,0xbb,0xbb,0xbb,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,
  0xbb,0x97,0x03,0x03,0x03,0x03,0x03,0x03,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x00,
  0x02,0x00,0x00,0xc2,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x01,0x02,0x01,0x01,0x02,
  0x02,0x01,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x02,0x02,0x02,
  0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xcb,0x00,0x02,0x02,0xcb,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,
  0x00,0x00,0x00,0xd2,0x01,0x01,0x01,0x01,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0xbb,0xbb,0x03,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0x03,0xbb,0xbb,
  0xbb,0x97,0x97,0x97,0x97,0xbb,0xbb,0x03,0x03,0x01,0x01,0x01,0x01,0x01,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0x00,0x02,
  0x02,0x02,0xc2,0x02,0x02,0x02,0x00,0x01,0x01,0x02,0x00,0x02,0x02,0x02,0x01,0x01,
  0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xcb,
  0x00,0x00,0x00,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0xcb,0x02,0xcb,0x00,0x00,
  0x01,0x01,0xd2,0x00,0x01,0xd2,0xd2,0x03,0x03,0x03,0x01,0x01,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0x03,0xbb,0xbb,
  0x97,0x97,0xbb,0xbb,0x97,0x97,0x03,0xbb,0x03,0x03,0x03,0x03,0x3a,0x01,0x01,0x01,
  0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0x02,0x00,
  0x02,0x02,0x02,0x02,0x00,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x02,0x02,
  0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0xc2,0x02,0xc2,0x02,0xc2,0x02,0xc2,0x02,0x02,0x02,0x00,0x00,0x00,
  0x01,0x01,0x01,0x01,0xd2,0xd2,0x03,0x03,0x03,0x03,0x01,0x03,0x03,0x03,0x03,0x01,
  0x03,0x03,0x03,0x03,0x03,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0x97,
  0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xbb,0xbb,0xbb,0x03,0x03,0x3a,0x01,0x01,
  0x01,0x01,0x00,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x02,0x02,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x02,0x02,0x02,
  0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x01,0x02,0x00,0x00,0x02,
  0xc2,0x02,0x02,0x00,0x02,0x00,0x02,0x02,0x00,0x02,0x02,0x00,0x00,0x00,0x00,0x00,
  0xcb,0xcb,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0xcb,0x00,
  0xd2,0x01,0x01,0xd2,0xd2,0xd2,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x01,
  0x01,0x01,0x3a,0x03,0x03,0x03,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0x97,0x97,0xbb,
  0x97,0x97,0xe7,0xe7,0x97,0x97,0xbb,0x97,0x03,0x03,0x03,0x03,0x03,0x03,0x3a,0x01,
  0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x00,0x02,0x00,0x02,0x02,0xc2,
  0x02,0x02,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x01,
  0x02,0x01,0x02,0x01,0x01,0x01,0x02,0x02,0x01,0x01,0x00,0x01,0x00,0x02,0x00,0x02,
  0x00,0x02,0x02,0x00,0x00,0x02,0x02,0x02,0x00,0x02,0x02,0x02,0x02,0x00,0x00,0x00,
  0xcb,0x00,0xcb,0x02,0x02,0x02,0xc2,0x02,0xc2,0x02,0x02,0x02,0x02,0xcb,0x00,0xcb,
  0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0x03,0xd2,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x01,0x01,0x01,0x01,0x03,0x03,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,0xbb,0x03,0xbb,0x97,
  0x97,0xe7,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xbb,0xbb,0xbb,0x03,0x03,0x01,0x01,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x00,
  0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x02,0x02,0x02,0x01,0x01,0x02,0x00,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,
  0x02,0x02,0x02,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0x00,0x00,0x00,
  0x00,0xcb,0xcb,0x00,0xc2,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xcb,0xcb,0xcb,
  0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0x01,0x03,0xd2,0x03,0x03,0x03,0x03,0x03,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x03,0x03,0x03,0xbb,0x03,0x97,0xbb,0xbb,0xbb,0x97,0xbb,
  0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xbb,0x03,0x03,0x03,0x03,0x3a,0x01,
  0x01,0x01,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x02,0x00,0x02,0x00,0x00,0x02,0x02,
  0x02,0x02,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,
  0x02,0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x00,0x00,0x01,0x00,
  0x00,0x00,0x02,0x00,0x00,0x02,0x02,0x02,0x00,0x02,0x02,0xcb,0x00,0x02,0x02,0x02,
  0x02,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0xcb,
  0xcb,0xd2,0x00,0xd2,0x00,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0x03,0x03,0x01,0x03,0x01,
  0x3a,0x01,0x01,0x00,0x01,0x01,0x03,0xbb,0xbb,0xbb,0x03,0x03,0x03,0x03,0x97,0x97,
  0x97,0x97,0x97,0xe7,0xe7,0xe7,0x97,0x97,0x97,0x97,0x03,0x03,0xbb,0x03,0x03,0x03,
  0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x01,0x02,0x00,0x00,0x00,0x00,0x00,0x02,0x02,
  0x00,0x00,0x01,0x01,0x01,0x01,0x3a,0x03,0x01,0x3a,0x01,0x01,0x01,0x01,0x02,0x02,
  0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x00,0x00,
  0x00,0x00,0x02,0x02,0x02,0x00,0x02,0x02,0x00,0x00,0x00,0x02,0x02,0x00,0xc2,0x02,
  0x02,0x02,0xcb,0x02,0x02,0x02,0x02,0xc2,0xc2,0x02,0x02,0xc2,0x02,0xcb,0xcb,0xcb,
  0xcb,0xcb,0xd2,0xd2,0xd2,0x01,0xd2,0xd2,0xfa,0xd2,0x01,0x01,0xd2,0x03,0x03,0x01,
  0x01,0x00,0x00,0x01,0x01,0x03,0xbb,0xbb,0x03,0xbb,0x03,0xbb,0x03,0x97,0x03,0xbb,
  0x97,0x97,0xe7,0xe7,0xe7,0xe7,0x97,0x97,0x97,0x97,0x97,0x03,0x03,0x03,0x03,0x3a,
  0x03,0x01,0x01,0x00,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x01,0x00,0x00,0x02,
  0x00,0x01,0x01,0x01,0x01,0x03,0x03,0x03,0x03,0x3a,0x03,0x01,0x01,0x01,0x02,0x02,
  0x02,0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x00,0x00,0x02,0x00,0x00,0xc2,
  0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,0x02,0x02,0xc2,0xc2,0x02,0x02,
  0xcb,0xcb,0x00,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x00,0x01,0x03,0x03,0xbb,0xbb,0xbb,0xbb,0x97,0xbb,0x97,0xbb,0x97,0x97,
  0x97,0x97,0x97,0xe7,0xe7,0xe7,0x97,0x97,0x97,0x97,0x97,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x01,0x01,0x00,0x00,0x00,0x00,0x01,0x00,0x01,0x00,0x00,0x01,0x01,
  0x00,0x00,0x01,0x01,0x03,0x03,0xf6,0x03,0x03,0x03,0xf2,0x3a,0x01,0x01,0x02,0x02,
  0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x3a,0x3a,0x01,0x03,0x03,0x3a,0x01,
  0x01,0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x02,
  0x02,0x02,0x02,0xc2,0x02,0xcb,0xcb,0x02,0x00,0x02,0x02,0x02,0xc2,0xc2,0x02,0x02,
  0xcb,0xcb,0x00,0x00,0xd2,0xd2,0xd2,0xd2,0xd2,0xd2,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x3a,0x01,0x00,0x01,0x03,0xbb,0xbb,0xbb,0xbb,0x97,0x97,0xbb,0x97,0x97,0xbb,
  0x97,0x97,0x97,0xe7,0xe7,0xe7,0x97,0x97,0x97,0x97,0x03,0x03,0x03,0x03,0xbb,0x97,
  0x03,0xbb,0x03,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x01,0x00,0x01,0x00,
  0x01,0x01,0x3a,0x3a,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x3a,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x3a,0x03,0xf2,0x03,0x03,0x03,0x03,0x03,0x3a,
  0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x02,0x00,0x02,
  0x02,0x02,0x02,0x00,0xcb,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0xc2,0x02,0x02,
  0xcb,0xcb,0xcb,0x00,0x00,0xd2,0xd2,0xd2,0xd2,0xd2,0x01,0x01,0x01,0x01,0x01,0x03,
  0x01,0x01,0x01,0xfa,0x01,0x03,0xbb,0xbb,0xbb,0xbb,0xbb,0xe7,0xe7,0xe7,0xe7,0x97,
  0x97,0xe7,0x97,0xe7,0xe7,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,
  0x97,0x97,0x03,0x03,0x03,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,
  0x01,0x01,0x3a,0x03,0x03,0x97,0xf6,0xf6,0x97,0x03,0x03,0x03,0x3a,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0xf2,0xf2,0x03,0xf6,0xf6,0xf6,0xf6,0x03,0x03,0x03,0x03,
  0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xcb,0x00,0x00,0x00,0x02,
  // 17: sky 3, part B
  0x7d,0xc6,0x9d,0xc6,0x5c,0xbe,0x3b,0xbe,0xdd,0xce,0x5b,0xbe,0x00,0x00,0x08,0x00,
  0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,
  0x10,0x02,0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,
  0x10,0x04,0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x1e,0xcf,0x00,0x06,
  0x08,0x06,0x10,0x06,0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,
  0x08,0x20,0x10,0x20,0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,
  0x08,0x22,0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,
  0x08,0x24,0x9e,0xc6,0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,
  0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,
  0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,
  0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,
  0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,
  0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,0x3e,0xdf,0x00,0x47,0x08,0x47,0x10,0x47,
  0x18,0x47,0x00,0x60,0x08,0x60,0xd9,0xad,0x10,0x60,0x18,0x60,0x00,0x61,0x5e,0xd7,
  0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,0x00,0x63,
  0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,
  0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,0x18,0x66,0x00,0x67,
  0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x9e,0xbe,0x08,0x80,0x10,0x80,0x18,0x80,
  0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,0x08,0x82,0x10,0x82,0x18,0x82,
  0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x19,0xb6,0x00,0x84,0x08,0x84,0x10,0x84,
  0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x18,0x85,0x00,0x86,0x08,0x86,0x10,0x86,
  0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,0xfb,0xb5,
  0x10,0xa0,0x18,0xa0,0x00,0xa1,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x00,0xa2,0x08,0xa2,
  0x10,0xa2,0x18,0xa2,0xfa,0xad,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x00,0xa4,
  0x08,0xa4,0xfe,0xd6,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,0x9f,0xe7,
  0xfe,0xc6,0x18,0xa5,0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x00,0xa7,0x08,0xa7,
  0x10,0xa7,0xf9,0xb5,0x18,0xa7,0x1d,0xd7,0x00,0xc0,0x08,0xc0,0x7e,0xe7,0x10,0xc0,
  0x18,0xc0,0x00,0xc1,0x08,0xc1,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x08,0xc2,0x10,0xc2,
  0x18,0xc2,0x00,0xc3,0x08,0xc3,0x10,0xc3,0x3c,0xb6,0x18,0xc3,0x00,0xc4,0x08,0xc4,
  0x10,0xc4,0x18,0xc4,0x00,0xc5,0x08,0xc5,0x10,0xc5,0x18,0xc5,0x00,0xc6,0x08,0xc6,
  0x10,0xc6,0x18,0xc6,0x00,0xc7,0x08,0xc7,0x1c,0xae,0x00,0xe0,0xb9,0xad,0x08,0xe0,
  0x1a,0xae,0x10,0xe0,0xbd,0xbe,0x00,0xe1,0x08,0xe1,0x18,0xe1,0x00,0xe2,0x18,0xe3,
  0xf6,0xf6,0x6b,0xd1,0xd1,0x6b,0x6b,0x6b,0xf6,0x6b,0xf6,0xf6,0xf6,0xf6,0xf6,0xf6,
  0x6b,0xd1,0xd1,0x6b,0x6b,0xd1,0x9c,0x03,0x03,0x9c,0xaf,0x9c,0x9c,0xaf,0xaf,0xaf,
  0xaf,0x6b,0x6b,0xaf,0xaf,0x9c,0x03,0x05,0x05,0x05,0x00,0x00,0x00,0x02,0x00,0x02,
  0x02,0x02,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe4,0x02,0x02,0x02,
  0x02,0x02,0x03,0x03,0xaf,0x9c,0xaf,0xba,0xaf,0x6b,0x6b,0x6b,0x6b,0x6b,0xba,0x6b,
  0xba,0xaf,0xaf,0x03,0x02,0xe4,0x02,0xe4,0xe4,0xe4,0xe4,0xaf,0xaf,0xba,0xba,0xba,
  0xba,0xba,0xba,0xba,0xba,0xaf,0xaf,0xe4,0xe4,0xe4,0xe4,0x02,0xe4,0x02,0x03,0xaf,
  0xaf,0xba,0xba,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0xf6,
  0x6b,0x6b,0x6b,0x6b,0xd1,0x6b,0xd1,0x6b,0xf6,0xf6,0xf6,0xf6,0xf6,0xf6,0xf6,0x6b,
  0x6b,0x6b,0xd1,0x6b,0xd1,0x9c,0x9c,0x03,0x9c,0x03,0xaf,0xaf,0x9c,0xaf,0xaf,0x6b,
  0x6b,0x6b,0x6b,0xaf,0xaf,0x9c,0x03,0x05,0x05,0x00,0x00,0x01,0x01,0x00,0x02,0x00,
  0x00,0x00,0x00,0x00,0x02,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x02,
  0x00,0x02,0x02,0x02,0x03,0x03,0x9c,0xaf,0xaf,0xba,0xba,0x6b,0x6b,0x6b,0x6b,0xba,
  0xba,0xaf,0xaf,0x03,0x02,0xe4,0xe4,0xe4,0xe4,0x02,0x03,0x03,0xaf,0xaf,0xaf,0xba,
  0xba,0xba,0xba,0xba,0xba,0xaf,0xe4,0xe4,0xe4,0xe4,0x00,0x00,0x00,0x02,0x02,0x03,
  0xaf,0xaf,0xba,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,
  0xf6,0xf6,0xd1,0x6b,0xd1,0x6b,0x6b,0xd1,0x6b,0x6b,0x6b,0xf6,0xf6,0xcd,0xf6,0xf6,
  0x6b,0xd1,0xd1,0xd1,0xd1,0x9c,0x9c,0x03,0x03,0x03,0x9c,0x9c,0x9c,0x9c,0xaf,0x6b,
  0x6b,0x6b,0xaf,0xaf,0xaf,0x03,0x03,0x05,0x02,0x00,0x00,0x00,0x01,0x01,0x00,0x00,
  0x00,0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x02,0x00,0x00,0x00,0x01,0x00,
  0x00,0x00,0x02,0x02,0x02,0x03,0x03,0xaf,0x9c,0xaf,0xaf,0xba,0xba,0xba,0xba,0xaf,
  0xaf,0xaf,0x03,0xe4,0x02,0xe4,0xe4,0xe4,0xe4,0x02,0x02,0xe4,0xe4,0x03,0xf4,0xaf,
  0xba,0xba,0xba,0xba,0xaf,0x03,0xe4,0x00,0x00,0x8c,0x8c,0x8c,0x00,0x00,0x02,0x03,
  0x03,0xaf,0xaf,0xba,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,
  0x6b,0x6b,0x6b,0xd1,0x6b,0xd1,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,
  0x6b,0x6b,0x6b,0x6b,0xd1,0x9c,0x03,0x03,0x03,0x03,0x03,0x03,0x9c,0x9c,0x9c,0x9c,
  0x6b,0x6b,0x6b,0xaf,0x9c,0x9c,0x05,0x02,0x02,0x02,0x00,0x00,0x01,0x01,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x03,0x03,0xaf,0xaf,0xaf,0xba,0xba,0xba,0xba,0xaf,0xaf,
  0xaf,0x03,0x03,0x02,0x02,0x00,0xe4,0xe4,0xe4,0x02,0xe4,0xe4,0xe4,0xe4,0x02,0x03,
  0xaf,0xba,0xba,0xba,0xaf,0xe4,0x00,0x8c,0x00,0x01,0x39,0x00,0x00,0x02,0x02,0x02,
  0x05,0x03,0x9c,0xaf,0xba,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0x6b,0xba,0x6b,0x6b,
  0x6b,0x6b,0x6b,0xd1,0x6b,0x6b,0x9c,0x9c,0xf8,0xaf,0x9c,0xd1,0x6b,0x6b,0x6b,0x6b,
  0xd1,0x6b,0xd1,0xd1,0xd1,0x9c,0x9c,0x03,0x03,0x03,0x03,0x9c,0x9c,0x9c,0x9c,0x9c,
  0x9c,0xaf,0xaf,0x9c,0x9c,0x03,0x02,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x05,0xaf,0xaf,0xaf,0xaf,0xba,0xba,0xba,0xba,0xaf,0xaf,
  0xf4,0x03,0xe4,0x00,0x00,0x00,0xe4,0x02,0x02,0x02,0x00,0xe4,0xe4,0x02,0x02,0x03,
  0xaf,0xaf,0xaf,0xaf,0xaf,0x03,0x02,0x01,0xfa,0xfa,0xfa,0x01,0x8c,0x02,0x02,0x02,
  0x03,0x03,0x9c,0xaf,0x6b,0xba,0x6b,0xba,0x6b,0xba,0xba,0xba,0x6b,0x6b,0x6b,0x6b,
  0xd1,0x6b,0xaf,0xaf,0x9c,0xaf,0xaf,0x9c,0x03,0x9c,0x9c,0xaf,0xd1,0x6b,0x6b,0x6b,
  0xd1,0x6b,0xd1,0xd1,0xd1,0x9c,0x9c,0x9c,0x9c,0x03,0x9c,0x03,0x9c,0x03,0x9c,0x9c,
  0x03,0x9c,0xaf,0xaf,0x9c,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,
  0x00,0x00,0x00,0x05,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x01,0x00,0x00,0x00,0x00,
  0x02,0x02,0x00,0x00,0x02,0x03,0xe4,0xaf,0xaf,0xf4,0xba,0xba,0xba,0xba,0xba,0xaf,
  0xf4,0xaf,0xe4,0x02,0xe4,0x00,0x00,0xe4,0x02,0x00,0x02,0x02,0x00,0x00,0x02,0x02,
  0x03,0xaf,0xaf,0xaf,0x03,0x02,0x00,0x01,0x04,0x04,0x01,0x00,0x8c,0x02,0x02,0x02,
  0x03,0x03,0x9c,0x9c,0xaf,0xf8,0x6b,0x6b,0x6b,0xaf,0x6b,0x6b,0xf8,0xd1,0xaf,0xd1,
  0xaf,0x9c,0x9c,0xf8,0x03,0x9c,0x03,0x05,0x05,0x03,0x03,0x03,0x9c,0x6b,0x6b,0xd1,
  0x6b,0xd1,0x6b,0xd1,0x6b,0x9c,0xf8,0x9c,0x9c,0x03,0x03,0x9c,0x9c,0x03,0x9c,0x9c,
  0x03,0x9c,0x9c,0x9c,0x03,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x00,0x00,
  0x00,0x00,0x00,0x05,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x01,0x00,0x00,0x00,
  0x02,0x02,0x02,0x02,0x02,0x03,0xaf,0xaf,0xaf,0xaf,0xba,0xba,0xba,0xba,0xaf,0xaf,
  0xf4,0xaf,0xe4,0x02,0xe4,0x00,0x00,0x00,0x02,0x8c,0x00,0x00,0x00,0x8c,0x00,0x00,
  0xe4,0x03,0xaf,0xaf,0x02,0x00,0x8c,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0xe4,0x03,
  0x03,0x03,0x9c,0x9c,0xaf,0xaf,0xaf,0x6b,0x6b,0x6b,0x6b,0xaf,0xaf,0xaf,0xaf,0xaf,
  0x03,0x03,0x03,0x03,0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x03,0x9c,0x9c,0xd1,0x6b,
  0x6b,0xd1,0xd1,0xd1,0x6b,0x9c,0x9c,0x9c,0x9c,0x9c,0x03,0x9c,0x03,0x9c,0x9c,0x9c,
  0x9c,0x9c,0x03,0x03,0x05,0x05,0x00,0x00,0x00,0x01,0x00,0x01,0x01,0x00,0x00,0x00,
  0x00,0x00,0x00,0x05,0x02,0x02,0x02,0x02,0x02,0x00,0x00,0x00,0x01,0x01,0x00,0x00,
  0x00,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0xaf,0xaf,0xaf,0xba,0xba,0xba,0xaf,0xaf,
  0xf4,0xaf,0xaf,0xe4,0xe4,0x8c,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x02,
  0x03,0xaf,0xaf,0xaf,0x02,0x8c,0x01,0x8c,0x01,0x04,0x04,0x01,0x00,0x00,0x02,0x02,
  0x05,0x05,0x03,0x9c,0xaf,0xf8,0xaf,0x9c,0xaf,0xaf,0xaf,0xaf,0x9c,0xaf,0x9c,0x9c,
  0x03,0x03,0x05,0x05,0x05,0x05,0x00,0x05,0x05,0x05,0x05,0x05,0x03,0x9c,0x9c,0xd1,
  0xd1,0xd1,0xd1,0x6b,0x6b,0x9c,0x9c,0x9c,0x03,0x03,0x9c,0x03,0x9c,0x9c,0x9c,0x9c,
  0x9c,0x9c,0x03,0x03,0x05,0x05,0x00,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x00,0x01,0x04,0x04,0x04,
  0x01,0x00,0x02,0x02,0xe4,0x02,0x02,0x03,0x03,0xaf,0xaf,0xaf,0xaf,0xf4,0xaf,0xe4,
  0xe4,0xe4,0xe4,0xe4,0xe4,0x8c,0x00,0x00,0x8c,0x00,0x01,0x01,0x01,0x8c,0x02,0x02,
  0xaf,0xaf,0xaf,0xaf,0xe4,0x00,0x8c,0x01,0x01,0x04,0x04,0x01,0x00,0x02,0x02,0x02,
  0x05,0x03,0x03,0x9c,0x9c,0xaf,0xf8,0xf8,0x9c,0xf8,0xaf,0x9c,0xf8,0x9c,0x03,0x9c,
  0x05,0x05,0x05,0x05,0x05,0x00,0x01,0x00,0x00,0x00,0x05,0x05,0x03,0x03,0x9c,0xd1,
  0xd1,0x6b,0xd1,0xd1,0xd1,0xd1,0x9c,0xaf,0x9c,0x9c,0x03,0x03,0x03,0x03,0x9c,0x9c,
  0x9c,0x9c,0x9c,0x9c,0x03,0x05,0x00,0x00,0x01,0x01,0x00,0x00,0x05,0x05,0x02,0x00,
  0x00,0x00,0x00,0x00,0x05,0x02,0x02,0x00,0x01,0x00,0x00,0x00,0x01,0x04,0x04,0x04,
  0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x02,0x02,
  0xe4,0x03,0x03,0x03,0xe4,0x00,0x00,0x00,0x00,0x8c,0x00,0x00,0x00,0x00,0x02,0x03,
  0x9c,0xaf,0xaf,0xaf,0x03,0x02,0x8c,0x01,0x04,0x04,0x04,0xfa,0x01,0x00,0x02,0x02,
  0x02,0x05,0x03,0x03,0x9c,0xaf,0xaf,0x9c,0xf8,0xf8,0xaf,0x9c,0x9c,0xf8,0x03,0x03,
  0x03,0x05,0x05,0x05,0x01,0x01,0x01,0x01,0x01,0x00,0x05,0x03,0x05,0x03,0x9c,0x6b,
  0xd1,0x6b,0x6b,0xd1,0xd1,0xd1,0x9c,0x9c,0x9c,0x9c,0x03,0x03,0x03,0x9c,0x9c,0x9c,
  0x9c,0x9c,0x9c,0x9c,0x03,0x05,0x05,0x02,0x01,0x01,0x00,0x00,0x05,0x05,0x05,0x02,
  0x00,0x00,0x05,0x05,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x01,0x00,0x00,0x01,0x02,0x02,0x05,0x02,0x02,0x02,0xe4,0xe4,
  0x02,0x02,0x03,0x03,0xe4,0xe4,0x00,0x02,0x02,0x8c,0x00,0x00,0x00,0x02,0x02,0x03,
  0xaf,0xaf,0xaf,0xaf,0xaf,0x03,0xe4,0x01,0x04,0x04,0x04,0xfa,0x01,0x00,0x02,0x02,
  0x02,0x03,0x03,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0xaf,0xaf,0x9c,0x03,0x9c,0x03,0x03,
  0x05,0x05,0x05,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x05,0x05,0x03,0xd1,
  0x6b,0xd1,0xd1,0xd1,0xd1,0x9c,0x9c,0x03,0x03,0x05,0x03,0x03,0x05,0x9c,0x9c,0x9c,
  0x9c,0x9c,0x9c,0x03,0x03,0x05,0x05,0x02,0x00,0x01,0x00,0x05,0x05,0x05,0x05,0x00,
  0x00,0x02,0x00,0x00,0x00,0x02,0x00,0x00,0x01,0x00,0x01,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x00,0x02,0x02,0x00,0x00,0x01,0x00,0x02,
  0xe4,0xe4,0x02,0x02,0x02,0x02,0x02,0x02,0xe4,0x00,0x00,0x00,0xe4,0x03,0x03,0xaf,
  0xaf,0xaf,0xaf,0xaf,0xf4,0xaf,0xe4,0x8c,0x8c,0x01,0xfa,0x04,0x01,0x00,0x02,0x02,
  0x05,0x03,0x03,0x03,0x9c,0xf8,0xf8,0x9c,0x9c,0x9c,0xf8,0x9c,0x03,0x03,0x05,0x05,
  0x05,0x05,0x00,0x00,0x01,0x04,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x05,0x03,0x9c,
  0x9c,0xd1,0xd1,0xd1,0xd1,0x9c,0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x03,0x9c,0x03,
  0x9c,0x9c,0x03,0x03,0x03,0x05,0x05,0x05,0x01,0x00,0x00,0x05,0x05,0x05,0x00,0x02,
  0x01,0x01,0x00,0x00,0x00,0x02,0x01,0x00,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x1e,
  0xc1,0x1e,0x1e,0xc1,0xc1,0x04,0x04,0x04,0x04,0x00,0x00,0x00,0x01,0x01,0x00,0x00,
  0x02,0x02,0x02,0x02,0x02,0xe4,0x02,0x02,0x02,0xe4,0x02,0xe4,0x02,0xe4,0xaf,0xaf,
  0xaf,0xaf,0xaf,0xaf,0xaf,0xaf,0xe4,0xe4,0x00,0x8c,0x01,0xfa,0x01,0x8c,0x02,0x02,
  0x02,0x03,0x03,0x9c,0x9c,0x9c,0xaf,0x9c,0xaf,0xaf,0x9c,0x9c,0x03,0x05,0x05,0x05,
  0x00,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x05,0x9c,
  0x9c,0xd1,0xd1,0xd1,0xd1,0x9c,0x9c,0x03,0x03,0x05,0x05,0x05,0x05,0x03,0x9c,0x9c,
  0x03,0x03,0x9c,0x05,0x05,0x05,0x05,0x00,0x00,0x02,0x05,0x05,0x05,0x05,0x02,0x00,
  0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0xc1,0x1e,
  0x1e,0x1e,0x1e,0x6f,0x1e,0x04,0x04,0x04,0x04,0x01,0x00,0x01,0x01,0x01,0x01,0x00,
  0x00,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe4,0x00,0x00,0x00,0x00,0x02,0x03,0xaf,
  0xaf,0xf4,0xaf,0xf4,0xf4,0xf4,0xe4,0xe4,0xe4,0x00,0x8c,0x01,0x8c,0x00,0x02,0x02,
  0x03,0x03,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x9c,0x03,0x05,0x05,0x05,0x05,
  0x01,0x01,0x04,0x04,0x04,0x01,0x04,0x01,0x01,0x01,0x04,0x01,0x01,0x00,0x05,0x03,
  0x9c,0x9c,0xd1,0x6b,0x9c,0x9c,0x9c,0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x03,0x03,
  0x03,0x03,0x03,0x05,0x05,0x05,0x05,0x02,0x02,0x00,0x00,0x05,0x05,0x05,0x00,0x01,
  0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0xc1,0x1e,0xc1,
  0x1e,0x1e,0x1e,0x1e,0x04,0x04,0x01,0x04,0x04,0x01,0x01,0x01,0x01,0x01,0x04,0x01,
  0x00,0x02,0xe4,0x02,0x02,0x02,0xe4,0x02,0x02,0x8c,0x00,0x01,0x8c,0x00,0xe4,0xaf,
  0xf4,0xaf,0xaf,0xaf,0xf4,0xf4,0xe4,0xe4,0xe4,0xe4,0x00,0x8c,0x00,0xe4,0x02,0x02,
  0x03,0x03,0x03,0x9c,0x03,0x03,0x9c,0x9c,0x9c,0x9c,0x03,0x03,0x05,0x05,0x05,0x00,
  0x01,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x01,0x05,0x05,
  0x9c,0x9c,0x6b,0x6b,0x9c,0x9c,0x03,0x05,0x03,0x05,0x05,0x05,0x05,0x05,0x03,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x01,0x01,0x01,0x00,0x00,0x02,0x02,0x00,0x01,
  0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x01,0x04,0x04,0x04,0x1e,0x1e,0x1e,0xc1,0x1e,
  0xc1,0xc1,0x1e,0xc1,0x04,0x04,0x01,0x04,0x04,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0x02,0x02,0x02,0xe4,0x00,0x00,0x39,0x39,0x39,0x8c,0x00,0xe4,
  0xe4,0xe4,0xe4,0xaf,0x03,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x05,0x05,0x05,0x00,0x01,
  0x01,0x04,0x04,0x1e,0xc1,0x1e,0x1e,0x04,0x04,0x04,0x04,0x04,0x04,0x00,0x05,0x05,
  0x03,0x9c,0xd1,0xd1,0x9c,0x03,0x05,0x05,0x05,0x05,0x00,0x00,0x05,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x00,0x00,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x01,
  0x04,0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x04,0x04,0xc1,0x1e,0x1e,0xc1,0x04,0xc1,
  0x1e,0x6f,0x6f,0x1e,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x39,0x04,0x39,0x04,0x39,0x8c,0xe4,
  0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0xe4,0x02,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x05,0x03,0x03,0x03,0x03,0x05,0x00,0x01,0x01,
  0x04,0x04,0x04,0xd3,0x6f,0xd3,0x1e,0xd3,0xd3,0x1e,0x1e,0x04,0x04,0x01,0x01,0x00,
  0x03,0x9c,0xd1,0x9c,0x03,0x03,0x03,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x05,
  0x05,0x05,0x05,0x05,0x05,0x05,0x02,0x00,0x01,0x04,0x04,0x01,0x00,0x00,0x01,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x01,0x04,0x04,0x1e,0x1e,0x1e,0xc1,0x04,0x1e,
  0x6f,0x6f,0x1e,0x6f,0x04,0x04,0x04,0x04,0x1e,0xc1,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x01,0x04,0x01,0x00,0x8c,0x8c,0x39,0x04,0x04,0x04,0x04,0x04,0x8c,0xe4,
  0xe4,0xe4,0xe4,0x02,0xe4,0xe4,0xe4,0x00,0x00,0x00,0xe4,0xe4,0xe4,0x02,0x03,0x03,
  0xaf,0x03,0x03,0x03,0x05,0x03,0x05,0x05,0x05,0x03,0x03,0x05,0x00,0x01,0x01,0x01,
  0x04,0x04,0xc1,0xd3,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x04,0x04,0x04,0x00,
  0x03,0x03,0xf8,0x9c,0x03,0x03,0x05,0x05,0x05,0x05,0x00,0x01,0x01,0x01,0x00,0x00,
  0x05,0x02,0x05,0x05,0x05,0x05,0x00,0x01,0x04,0x04,0x04,0x01,0x01,0x01,0x04,0x04,
  0x04,0x04,0x04,0x1e,0x04,0x04,0x04,0x04,0x04,0xc1,0x1e,0x6f,0x6f,0x6f,0xc1,0x1e,
  0x6f,0x6f,0x6f,0x1e,0x04,0x04,0xc1,0x1e,0x1e,0x04,0x04,0x04,0xc1,0xc1,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x01,0x00,0x01,0x04,0x04,0xc8,0x1e,0xc8,0x04,0x8c,0xe4,
  0xe4,0xe4,0xe4,0xe4,0xe4,0x00,0x00,0x8c,0x8c,0x8c,0x8c,0x00,0xe4,0x02,0x03,0x03,
  0xaf,0x03,0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x05,0x00,0x01,0x01,0x01,
  0x04,0x04,0xc1,0x64,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x04,0x04,0x04,0x00,
  0x05,0x03,0x9c,0x03,0x03,0x03,0x03,0x05,0x05,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x00,0x05,0x05,0x05,0x00,0x00,0x01,0x04,0xc1,0x1e,0x04,0x04,0x04,0x04,0x04,
  0x04,0x1e,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0xc1,0x1e,0x6f,0x6f,0x6f,0x6f,0x6f,
  0xc7,0x6f,0x64,0x6f,0x1e,0x1e,0x6f,0x6f,0xc1,0x04,0x1e,0xc1,0xc1,0x6f,0x1e,0x1e,
  0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x04,0x04,0xc8,0x04,0x1e,0x1e,0x04,0x8c,0x00,
  0xe4,0xe4,0xe4,0xe4,0xe4,0x8c,0x8c,0x8c,0x8c,0x39,0x01,0x00,0xe4,0x02,0x02,0x03,
  0x03,0x03,0x03,0x03,0x03,0x05,0x05,0x02,0x05,0x05,0x05,0x05,0x00,0x01,0x01,0x04,
  0x04,0x04,0xd3,0xd3,0xd3,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x1e,0x04,0x04,0x01,
  0x05,0x05,0x03,0x03,0x03,0x03,0x03,0x05,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x01,0x00,0x05,0x02,0x00,0x01,0x04,0x1e,0xd3,0xd3,0x1e,0x04,0x04,0x1e,0xd3,
  0x6f,0x1e,0x04,0x1e,0xc1,0x04,0x04,0x04,0x04,0x04,0xc1,0x1e,0x6f,0xc7,0xc7,0xc7,
  0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,
  0x1e,0x04,0x04,0x04,0xc8,0x04,0xc8,0x04,0x04,0x04,0x1e,0x1e,0x1e,0x04,0x00,0xe4,
  0xe4,0xe4,0xe4,0xe4,0x00,0x8c,0x8c,0x8c,0x39,0x39,0x8c,0x00,0xe4,0xe4,0x02,0x03,
  0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x02,0x00,0x05,0x05,0x05,0x00,0x01,0x01,0x04,
  0x04,0xc1,0xd3,0xd3,0xd3,0x6f,0xd3,0x6f,0x6f,0x6f,0xd3,0x1e,0x1e,0x1e,0x04,0x01,
  0x00,0x05,0x03,0x03,0x03,0x03,0x05,0x05,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x00,0x02,0x00,0x01,0x04,0x1e,0x1e,0xd3,0xd3,0x1e,0xd3,0xd3,0x1e,
  0xc1,0x04,0x04,0x1e,0x1e,0x6f,0xc1,0x04,0x04,0x04,0xc1,0x6f,0xc7,0xc7,0xc7,0xc7,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0xc7,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,
  0x1e,0xc1,0x04,0x1e,0x1e,0x1e,0x1e,0xc1,0x1e,0x1e,0x1e,0x1e,0x1e,0x39,0x8c,0xe4,
  0xe4,0xe4,0xe4,0xe4,0x00,0x00,0x8c,0x8c,0x39,0x39,0x8c,0x00,0xe4,0xe4,0x02,0x03,
  0x03,0x03,0x03,0x05,0x05,0x05,0x05,0x05,0x02,0x00,0x05,0x05,0x00,0x01,0x04,0x04,
  0x04,0x1e,0xd3,0xd3,0xd3,0xd3,0x6f,0x6f,0x6f,0x1e,0xc1,0x1e,0xd3,0xd3,0x04,0x01,
  0x01,0x05,0x03,0x05,0x03,0x05,0x05,0x00,0x01,0x01,0x01,0x01,0x01,0x04,0x04,0x04,
  0x01,0x01,0x01,0x00,0x05,0x00,0x00,0x04,0xd3,0x6f,0x6f,0x1e,0x1e,0xd3,0x64,0x64,
  0x6f,0xc1,0x1e,0x1e,0x1e,0x1e,0x1e,0x04,0x04,0x04,0xc1,0x6f,0x6f,0xc7,0xc7,0xc7,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,
  0x6f,0x6f,0x1e,0xc1,0x1e,0x1e,0x1e,0x6f,0x1e,0x1e,0x1e,0x1e,0x1e,0x04,0x04,0x8c,
  0xe4,0x00,0xe4,0xe4,0xe4,0x00,0x8c,0x8c,0x39,0x39,0x8c,0xe4,0xe4,0xe4,0xe4,0x03,
  0x03,0xf4,0x05,0x03,0x05,0x05,0x00,0x02,0x05,0x00,0x00,0x05,0x00,0x01,0x01,0x04,
  0x1e,0x1e,0xd3,0xd3,0x1e,0xd3,0x6f,0x6f,0x6f,0x6f,0x04,0x04,0xc1,0x04,0x04,0x01,
  0x00,0x05,0x05,0x03,0x03,0x03,0x05,0x00,0x01,0x04,0x04,0x01,0x01,0x04,0x04,0x04,
  0x04,0x01,0x04,0x01,0x01,0x00,0x01,0x04,0x6f,0x6f,0x64,0x64,0x6f,0x6f,0x6f,0x64,
  0x1e,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0xc1,0x04,0x04,0x1e,0x6f,0x6f,0x6f,0x6f,0xc7,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,0x6f,
  0x6f,0x6f,0x1e,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x1e,0x1e,0xc8,0xc8,0x39,
  0x8c,0x00,0x00,0xe4,0xe4,0x00,0x8c,0x39,0x39,0x39,0x8c,0xe4,0xe4,0x02,0x02,0x03,
  0x03,0xf4,0x03,0x03,0x05,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x04,
  0x1e,0xd3,0xd3,0x1e,0xd3,0x1e,0xd3,0x6f,0xd3,0xd3,0xc1,0x04,0x04,0x04,0x01,0x01,
  0x00,0x05,0x05,0x03,0x03,0x05,0x05,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x1e,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0xc7,0x64,
  0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0x04,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0xc7,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,
  0x6f,0x1e,0x04,0x04,0x1e,0xc1,0x1e,0x6f,0x6f,0x1e,0x1e,0x1e,0xc8,0x04,0x04,0x39,
  0x8c,0x8c,0x8c,0x8c,0x00,0x8c,0x39,0x39,0x39,0x8c,0x00,0xe4,0xe4,0xe4,0xe4,0x03,
  0xaf,0xaf,0x03,0x03,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x04,
  0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x04,0x04,0x04,0x01,0x01,
  0x00,0x00,0x05,0x03,0x03,0x05,0x05,0x01,0x01,0x04,0x04,0x04,0xc1,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x6f,0x6f,0x1e,0xd3,0x6f,0xc7,0xc7,0x6f,
  0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x64,0x6f,0xc1,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0x6f,
  0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,
  0x6f,0x6f,0x1e,0xc1,0x1e,0x1e,0x1e,0x1e,0x1e,0x1e,0x04,0x04,0x04,0x39,0x01,0x8c,
  0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x39,0x39,0x8c,0x8c,0xe4,0xe4,0xe4,0xe4,0x03,0x03,
  0xaf,0xaf,0xaf,0x03,0x05,0x02,0x01,0x01,0x00,0x01,0x01,0x00,0x00,0x01,0x01,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x00,0x01,0x02,
  0x00,0x05,0x05,0x05,0x05,0x05,0x00,0x01,0x01,0x04,0x04,0x04,0xc1,0x04,0xd3,0x04,
  0xd3,0x1e,0xc1,0xd3,0x6f,0x04,0x04,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0xc7,0xc7,0x6f,
  0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0x64,
  0x6f,0x6f,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,
  0x6f,0x1e,0x6f,0x1e,0x1e,0x1e,0x1e,0xc8,0x04,0x04,0x01,0x8c,0x8c,0x00,0x00,0x00,
  0x8c,0x8c,0x8c,0x8c,0x00,0x8c,0x8c,0x8c,0x8c,0x00,0xe4,0xe4,0xe4,0xaf,0xaf,0xaf,
  0xf4,0xaf,0xaf,0xaf,0x03,0x05,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x04,0x04,0x01,0x01,0x02,0x00,0x02,
  0x02,0x05,0x02,0x05,0x05,0x05,0x05,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0x1e,
  0xd3,0xd3,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,0x6f,0xc7,0x6f,0x6f,0xc7,0xc7,0x6f,0x6f,
  0x6f,0x6f,0x6f,0x6f,0x6f,0xc7,0x6f,0x6f,0x6f,0x1e,0x1e,0x6f,0x6f,0x6f,0xc1,0x6f,
  0x6f,0x6f,0x6f,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,0x6f,0x6f,
  0x6f,0x6f,0x1e,0x1e,0x1e,0x1e,0xc8,0x04,0x39,0x39,0x8c,0x00,0x00,0xe4,0xe4,0x00,
  0x8c,0x8c,0x39,0x8c,0x8c,0x8c,0x8c,0x8c,0x00,0xe4,0xe4,0xe4,0xaf,0xaf,0xf4,0xaf,
  0xaf,0xaf,0xaf,0x9c,0x03,0x00,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
  0x04,0x04,0x01,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x02,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x02,0x00,0x00,0x01,0x01,0x04,0x04,0x04,0x04,0x04,0xc1,
  0xd3,0x64,0xc7,0xc7,0xc7,0xc7,0x6f,0xd3,0x6f,0x6f,0x6f,0x6f,0x6f,0xc7,0x6f,0x1e,
  0x6f,0x6f,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0x6f,0xc1,0xc1,0x1e,0x6f,0x6f,0x6f,0x6f,
  0x6f,0x1e,0x6f,0x6f,0x6f,0xc7,0xc7,0xc7,0xc7,0x6f,0x6f,0x6f,0x6f,0x6f,0x1e,0x1e,
  0x1e,0x1e,0x1e,0xc1,0x1e,0x04,0x04,0x04,0x39,0x8c,0x8c,0x00,0xe4,0xe4,0xe4,0x00,
  0x00,0x8c,0x8c,0x39,0x39,0x8c,0x8c,0x00,0xe4,0xe4,0xe4,0xf4,0xaf,0xaf,0xaf,0xba,
  0xba,0xba,0xba,0xaf,0xaf,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x04,
  0x04,0x04,0x01,0x00,0x00,0x02,0x02,0x00,0x02,0x05,0x05,0x02,0x02,0x05,0x05,0x05,
  0x05,0x05,0x05,0x05,0x05,0x02,0x00,0x01,0x01,0x01,0x01,0x04,0x04,0x04,0x04,0x1e,
  0x6f,0x64,0xd6,0xc7,0xd6,0xc7,0x6f,0x6f,0xd3,0x6f,0x6f,0x1e,0x1e,0x6f,0x6f,0x1e,
  0x1e,0x1e,0x1e,0x04,0x1e,0x6f,0x6f,0x6f,0xc1,0x04,0xc1,0x1e,0xd3,0x1e,0xc1,0x1e,
  0x64,0x1e,0x1e,0x6f,0x6f,0x6f,0x6f,0x6f,0x64,0x1e,0x1e,0x6f,0x1e,0x6f,0x1e,0x1e,
  0xc8,0x1e,0x04,0x04,0x04,0x01,0x39,0x01,0x39,0x8c,0x8c,0x00,0x00,0xe4,0xe4,0x00,
  0x00,0x00,0x8c,0x39,0x8c,0x8c,0xe4,0xe4,0xe4,0x03,0xe4,0xaf,0xaf,0xba,0xba,0xba,
  0xba,0xba,0xba,0xaf,0xaf,0x02,0x02,0x01,0x04,0x04,0x01,0x00,0x00,0x01,0x04,0x04,
  0x01,0x04,0x04,0x01,0x01,0x00,0x02,0x05,0x05,0x05,0x05,0x05,0x05,0x9c,0x9c,0x03,
  0x03,0x03,0x05,0x05,0x05,0x00,0x02,0x00,0x00,0x01,0x00,0x01,0x04,0x04,0x1e,0xd3,
  0x6f,0xd6,0xd6,0xd6,0xd6,0xd6,0x6f,0x6f,0xd3,0xd3,0x1e,0x04,0x04,0x1e,0x1e,0x1e,
  0x04,0x04,0x04,0x04,0x04,0x1e,0x1e,0x1e,0x1e,0x04,0xc1,0x04,0xc1,0x04,0x04,0x04,
  0x1e,0xc8,0x04,0xc1,0x1e,0x1e,0x1e,0x1e,0x1e,0x04,0x04,0x04,0x1e,0xc1,0x1e,0x1e,
  0x1e,0x04,0x04,0x04,0x39,0x8c,0x39,0x8c,0x8c,0x8c,0x8c,0x8c,0x00,0xe4,0xe4,0xe4,
  0xe4,0x00,0x8c,0x8c,0x8c,0x8c,0xe4,0xe4,0xe4,0xe4,0xaf,0xaf,0xba,0xba,0xba,0xba,
  0xba,0xba,0xba,0xaf,0xaf,0x02,0x02,0x00,0x01,0x01,0x00,0x00,0x00,0x01,0x04,0x04,
  0x00,0x01,0x04,0x01,0x02,0x05,0x03,0x05,0x03,0x05,0x05,0x03,0x9c,0xf8,0xaf,0xf8,
  0x9c,0x03,0x03,0x05,0x05,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x04,0x04,0xd3,0x6f,
  0xc7,0xd6,0xd6,0xd6,0xc7,0xc7,0x6f,0x6f,0x6f,0x64,0xd3,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x1e,0x1e,0x1e,0x04,0xc8,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
  0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x01,0x01,0x01,0x01,0x04,0x04,0xc8,
  0xc8,0x04,0x04,0x39,0x8c,0x8c,0x8c,0x8c,0x8c,0x8c,0x00,0x8c,0x00,0x00,0xe4,0xe4,
  0xe4,0xe4,0xe4,0x00,0x00,0xe4,0xe4,0xe4,0xe4,0xe4,0xaf,0xba,0xba,0xba,0xba,0xba,
  0xba,0x6b,0xaf,0x9c,0x03,0x02,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,
  // 18: sky 3, part C
  0x98,0xa5,0xd9,0xad,0x1a,0xb6,0x5c,0xbe,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,
  0x00,0x01,0x08,0x01,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x58,0x95,
  0x18,0x02,0x00,0x03,0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,
  0x18,0x04,0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,
  0x18,0x06,0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,
  0x18,0x20,0x00,0x21,0x08,0x21,0x10,0x21,0xfa,0xa5,0x18,0x21,0x00,0x22,0x08,0x22,
  0x10,0x22,0x18,0x22,0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,
  0x10,0x24,0x18,0x24,0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,
  0x10,0x26,0x18,0x26,0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,
  0x10,0x40,0x18,0x40,0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,
  0x10,0x42,0x18,0x42,0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,
  0x10,0x44,0x18,0x44,0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,
  0x10,0x46,0x18,0x46,0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,
  0x10,0x60,0x18,0x60,0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,
  0x99,0x9d,0x10,0x62,0x18,0x62,0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,
  0x08,0x64,0x10,0x64,0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,
  0x08,0x66,0xdd,0xce,0x17,0x8d,0x9c,0xc6,0x10,0x66,0x18,0x66,0x00,0x67,0x08,0x67,
  0x10,0x67,0x18,0x67,0x00,0x80,0x38,0x8d,0x7c,0xc6,0xf6,0x8c,0xb6,0x7c,0x08,0x80,
  0x10,0x80,0x18,0x80,0x7e,0xe7,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x00,0x82,
  0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x08,0x83,0x10,0x83,0x18,0x83,0x00,0x84,
  0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,0x00,0x86,0x08,0x86,
  0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,0x10,0x87,0x18,0x87,0x00,0xa0,0x08,0xa0,
  0x10,0xa0,0x18,0xa0,0x00,0xa1,0xbe,0xce,0x08,0xa1,0x10,0xa1,0x18,0xa1,0x5e,0xd7,
  0x00,0xa2,0x08,0xa2,0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x1d,0xd7,
  0x18,0xa3,0x00,0xa4,0x08,0xa4,0x10,0xa4,0x18,0xa4,0x00,0xa5,0x08,0xa5,0x10,0xa5,
  0x57,0x95,0xfa,0xb5,0x95,0x7c,0x18,0xa5,0x00,0xa6,0x08,0xa6,0x75,0x74,0x55,0x6c,
  0x10,0xa6,0x1e,0xcf,0x18,0xa6,0x00,0xa7,0x08,0xa7,0x10,0xa7,0x1b,0xae,0x18,0xa7,
  0x00,0xc0,0xf3,0x6b,0x08,0xc0,0x10,0xc0,0x18,0xc0,0x14,0x6c,0x00,0xc1,0x9e,0xef,
  0xd6,0x7c,0x08,0xc1,0x10,0xc1,0x18,0xc1,0x00,0xc2,0x5b,0xbe,0xb3,0x5b,0x08,0xc2,
  0x10,0xc2,0x18,0xc2,0x00,0xc3,0x08,0xc3,0x35,0x74,0x10,0xc3,0x18,0xc3,0x00,0xc4,
  0x37,0x95,0xf4,0x5b,0x10,0xc4,0x18,0xc4,0x00,0xc5,0xf7,0x94,0x17,0x7d,0x13,0x6c,
  0xd4,0x63,0xd8,0xb5,0x19,0xb6,0x78,0xa5,0xb5,0x84,0xbd,0xbe,0xb6,0x6c,0x18,0xe0,
  0x83,0x83,0x8c,0x8c,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0x02,0x02,0xc9,0x01,0x2c,0xc9,
  0x02,0x02,0x02,0x02,0xe5,0x8c,0xfd,0x83,0xfd,0xfd,0x81,0xfd,0x81,0xbf,0xb7,0xb7,
  0x92,0x92,0x92,0x92,0x92,0x92,0xb7,0xb7,0xb7,0x92,0xb7,0x81,0xb3,0xb3,0xb3,0x83,
  0xb3,0x81,0xb3,0xb3,0xb3,0xd1,0x81,0x81,0xb3,0x81,0xb3,0xfd,0xfd,0xb3,0xb3,0xfd,
  0x81,0xb3,0xfd,0x83,0x83,0xb3,0x81,0xb3,0xb3,0xb3,0x83,0x83,0x8c,0x83,0xfd,0xb3,
  0xb3,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0x03,0x03,
  0x03,0x03,0x03,0xfd,0x03,0x03,0x03,0x03,0xd6,0xd6,0xc9,0x2c,0x2c,0x2c,0x2c,0x2c,
  0x2c,0x2c,0xd6,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x8c,0x03,0x8c,0x8c,
  0x03,0x8c,0x8c,0x03,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0x02,0x02,0x02,0x01,0x01,0x01,
  0xc9,0xc9,0xc9,0x02,0xe5,0x03,0x8c,0x83,0xfd,0x81,0x81,0x81,0xbf,0xb7,0xd1,0xbf,
  0xb7,0x92,0x92,0x92,0x92,0xb7,0xb7,0xb7,0xb7,0xb7,0xbf,0xd1,0x81,0x81,0xfd,0x8c,
  0xb3,0xfd,0x83,0xb3,0x81,0xd1,0xd1,0xb7,0xb3,0x81,0x83,0x8c,0x8c,0x83,0x8c,0x8c,
  0x83,0x83,0x8c,0x03,0x83,0xb3,0x81,0xb3,0xb3,0xb3,0xfd,0x81,0x8c,0x8c,0xfd,0x8c,
  0xfd,0xfd,0x03,0x03,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xd6,0xd6,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,
  0x2c,0x2c,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0xe5,0xe5,0xe5,0x02,0x02,0x02,0x02,0xc9,0x01,0x01,0x01,0x01,
  0x01,0x2c,0xc9,0x02,0xe5,0xe5,0x8c,0x81,0x81,0x81,0xd1,0xbf,0xb7,0x92,0xbf,0xd1,
  0xd1,0xb7,0xb7,0x92,0x92,0xb7,0xb7,0xb7,0xd1,0xd1,0xd1,0x81,0x81,0xb3,0x83,0x83,
  0xb3,0x83,0x8c,0xb3,0x81,0xd1,0xb7,0x92,0x81,0xb3,0x8c,0x8c,0x8c,0x83,0x8c,0x8c,
  0x03,0x03,0x03,0x03,0x8c,0xb3,0xb3,0xb3,0xb3,0xb3,0xb3,0x83,0x8c,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xfd,0xfd,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0xd6,0xd6,0x2c,0x2c,0x2c,0x2c,0x2c,0x01,0x2c,0x2c,
  0x2c,0x2c,0xd6,0xd6,0xd6,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0xe5,0xe5,0x02,0xe5,0x02,0xe5,0x02,0x02,0xc9,0x02,0x02,0x2c,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0xc9,0x02,0xe5,0x03,0xfd,0x81,0x81,0xd1,0xd1,0xb7,0xbf,0xd1,0xd1,
  0xd1,0xbf,0xb7,0xb7,0x92,0xbf,0xd1,0xd1,0x81,0x81,0x81,0x81,0xb3,0x83,0x83,0x83,
  0x8c,0x83,0x8c,0x83,0xb3,0x81,0xd1,0x92,0xb7,0xb3,0x8c,0x03,0x8c,0x83,0x8c,0x03,
  0x03,0x03,0x03,0x03,0x8c,0x8c,0x8c,0x83,0x83,0xb3,0xb7,0xb7,0xb7,0xfd,0x02,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0xd6,0x2c,0x2c,0x2c,0x00,0x00,0x00,0x2c,0x2c,0x2c,
  0x2c,0x2c,0xc9,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x8c,0x03,0xe5,0x02,
  0x02,0x02,0xc9,0x01,0xc9,0x2c,0xc9,0xc9,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0xc9,0x02,0xe5,0x8c,0x81,0xd1,0xbf,0xbf,0xbf,0x81,0x81,0xb3,
  0xd1,0xd1,0xb7,0xb7,0xbf,0xd1,0x81,0x81,0xb3,0xb3,0xfd,0x83,0x83,0x8c,0x8c,0x8c,
  0x8c,0x03,0x8c,0x83,0xfd,0xb3,0x81,0x92,0x92,0x81,0x8c,0x8c,0xfd,0x8c,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xfd,0x03,0x83,0xd1,0xd1,0xbf,0xbf,0x83,0x02,
  0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xd6,0x03,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x2c,0x2c,0x00,0x00,0x00,0x00,0x2c,0x2c,0x2c,
  0x2c,0x02,0x02,0x02,0xd6,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0xe5,0xd6,
  0x02,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0xc9,0x02,0x03,0x83,0x81,0xbf,0x92,0xb7,0xbf,0xb3,0xb3,0x81,
  0x81,0x81,0xb7,0xd1,0xd1,0x81,0x81,0xb3,0x8c,0x8c,0x8c,0x03,0x03,0x03,0x03,0x03,
  0x03,0x03,0x03,0x03,0x8c,0xb3,0xd1,0x92,0x92,0xbf,0x81,0x83,0x03,0x03,0xfd,0x03,
  0x03,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0xb3,0xb3,0xd1,0xb7,0xb7,0xbf,0x03,
  0x03,0x02,0x02,0x02,0xd6,0x02,0x03,0x03,0xd6,0xd6,0xd6,0xd6,0xd6,0x03,0xd6,0x03,
  0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x2c,0x00,0x00,0x00,0x00,0x2c,0x2c,0x2c,0x2c,
  0x2c,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x02,0x02,0x02,0x02,0x02,
  0x2c,0x01,0x01,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x2c,0xc9,0x02,0x03,0x83,0x81,0xb7,0xb7,0x81,0xb3,0x83,0xfd,
  0xb3,0x81,0x81,0x81,0x81,0xb3,0x8c,0x8c,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x02,
  0xe5,0x02,0x02,0x03,0x03,0x83,0xbf,0x92,0x92,0xb7,0xd1,0xb3,0x03,0x03,0x03,0x03,
  0x03,0x03,0x02,0x03,0x03,0x03,0x03,0x03,0x02,0x03,0x83,0x81,0xd1,0xd1,0xbf,0xbf,
  0xb7,0x03,0x03,0x03,0x02,0x02,0xd6,0xd6,0xd6,0xd6,0xd6,0x2c,0x2c,0xd6,0xd6,0xd6,
  0xd6,0x02,0x02,0x03,0x03,0x03,0x02,0xc9,0x00,0x70,0x00,0x2c,0x01,0x2c,0x2c,0xc9,
  0xc9,0x03,0x83,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0xe5,0x02,0x02,0xc9,
  0x01,0x01,0x01,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x01,0x00,0x01,0x01,0x01,0x01,0xe5,0x03,0x81,0xbf,0xd1,0x81,0x83,0x83,0x8c,
  0xfd,0xfd,0xb3,0xfd,0xfd,0x8c,0x03,0x03,0x03,0x03,0xe5,0x02,0x02,0x02,0xe5,0x02,
  0x02,0x02,0x02,0x02,0x03,0x83,0xd1,0x92,0x92,0xd1,0x81,0x81,0x8c,0x03,0x03,0x03,
  0x03,0x02,0x02,0x03,0x03,0x8c,0x03,0x02,0x02,0xfd,0x83,0x81,0xb3,0xbf,0xb7,0x92,
  0x92,0x81,0x03,0x02,0xd6,0x03,0x92,0xb7,0xfd,0xd6,0x02,0xfd,0x81,0x83,0x02,0x2c,
  0xd6,0x2c,0x03,0x03,0x8c,0x03,0x02,0x02,0x00,0x70,0x00,0x2c,0x2c,0x2c,0x02,0x02,
  0x03,0x8c,0x8c,0x03,0x03,0x02,0x02,0xd6,0x02,0xd6,0xd6,0x02,0xc9,0xc9,0x01,0x2c,
  0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,
  0x00,0x01,0x00,0x00,0x01,0x01,0x01,0x02,0xe5,0x8c,0x83,0x8c,0x8c,0x8c,0x03,0x8c,
  0x8c,0x8c,0x8c,0x8c,0xfd,0x03,0x03,0xe5,0x02,0xc9,0xc9,0x02,0x2c,0x2c,0xc9,0xc9,
  0xc9,0x01,0xc9,0x02,0x03,0x8c,0x81,0xbf,0xb7,0xd1,0xb3,0xb3,0x8c,0x03,0x03,0x03,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x83,0xb3,0x81,0xbf,0xb7,0x92,
  0x92,0xb7,0xc9,0x2c,0x2c,0x81,0x92,0x92,0x92,0x83,0x81,0xb7,0xb7,0xd1,0xb3,0x8c,
  0x2c,0x02,0x8c,0xb3,0x81,0xb3,0x03,0x02,0x2c,0x70,0x70,0x70,0x2c,0xd6,0x02,0x03,
  0x03,0x03,0x03,0x03,0x03,0x02,0x02,0x02,0xd6,0xd6,0x02,0x01,0xc9,0x2c,0x01,0x01,
  0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x70,0x00,0x70,0x00,0x01,0x01,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0xc9,0xe5,0xe5,0x03,0x03,0x03,0x8c,
  0xe5,0xe5,0x03,0x03,0xe5,0x03,0x03,0x02,0xc9,0x01,0x01,0x01,0x01,0x01,0x2c,0x2c,
  0x01,0x01,0x01,0xc9,0x02,0x8c,0x81,0xb7,0x92,0xb7,0x81,0xfd,0x8c,0x03,0x03,0x03,
  0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x8c,0xb3,0xd1,0xb7,0xb7,0x92,
  0x92,0x92,0x01,0x00,0xe5,0x92,0x92,0x92,0x92,0xbf,0xb7,0xb7,0xb7,0xb7,0x81,0xb3,
  0x83,0x03,0x83,0x81,0x81,0x81,0x81,0x03,0x00,0x70,0x70,0x70,0x00,0x02,0x02,0x03,
  0x02,0x02,0x03,0x02,0x03,0x02,0x02,0x2c,0x2c,0x2c,0x2c,0x01,0x01,0x2c,0x01,0x01,
  0x00,0x01,0x01,0x01,0x01,0x00,0x70,0x00,0xfb,0xfb,0x00,0xfb,0x70,0x00,0x01,0x01,
  0x01,0x00,0x00,0xfb,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0xe5,
  0xe5,0xe5,0x02,0x02,0xe5,0xe5,0x03,0x02,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x01,
  0x01,0x01,0x01,0xc9,0x02,0x03,0xb3,0xb7,0xb7,0x81,0x83,0xfd,0x03,0x03,0x03,0x03,
  0x03,0x02,0x02,0x03,0x02,0x02,0x02,0x03,0x02,0x03,0x03,0x83,0x81,0xbf,0xb7,0x92,
  0x92,0x92,0xbf,0xfd,0xbf,0x92,0x92,0x92,0x92,0x92,0xb7,0xbf,0xbf,0xbf,0xd1,0x81,
  0xd1,0x81,0x81,0xbf,0x81,0xd1,0xbf,0xd1,0x02,0x70,0x70,0x70,0x2c,0x02,0xc9,0x02,
  0x02,0xd6,0x02,0x02,0x02,0xc9,0x2c,0x2c,0x2c,0x2c,0x01,0x01,0x01,0x00,0x00,0x00,
  0x01,0x00,0x01,0x01,0x00,0xfb,0xfb,0xfb,0xfb,0xfb,0xc8,0xc8,0x0f,0x01,0x01,0x01,
  0x01,0x01,0x01,0x70,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x01,0x01,0xc9,0xc9,0xc9,
  0xc9,0xc9,0x01,0xc9,0xc9,0x02,0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x01,0x01,0x01,0xc9,0x02,0x03,0xb3,0xd1,0xd1,0x8c,0x03,0x03,0x03,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0xe5,0x02,0x02,0x02,0x02,0x03,0xfd,0xb3,0x81,0xbf,0xb7,
  0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0xb7,0xb7,0xb7,0xb7,0xbf,
  0x92,0x92,0xbf,0xbf,0x81,0xbf,0xb7,0xb7,0x81,0x03,0x2c,0x2c,0x2c,0x02,0x02,0x2c,
  0x2c,0x02,0xd6,0x02,0x2c,0x2c,0x2c,0x2c,0x01,0x01,0x01,0x01,0x00,0x01,0x01,0x00,
  0x01,0x01,0x00,0x00,0x70,0xfb,0xfb,0xc8,0x0f,0xc8,0xc8,0xc8,0x00,0x01,0x01,0x01,
  0x01,0x00,0x00,0x70,0x01,0x01,0x2c,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x02,0xc9,0x00,0x00,0x00,0x00,0x00,0x00,0xc8,0xfb,0xfb,
  0x00,0x00,0x01,0x2c,0x02,0xe5,0x83,0xb3,0x83,0x8c,0x03,0x03,0x03,0x02,0x02,0x02,
  0x02,0x02,0x02,0x02,0x02,0x02,0x83,0x83,0x03,0x03,0x03,0x8c,0xb3,0xb3,0xd1,0xb7,
  0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0x92,0xb7,0xb7,
  0xb7,0xb7,0xbf,0x81,0x81,0xd1,0xbf,0xbf,0xbf,0x81,0x02,0x2c,0x2c,0xd6,0x02,0x02,
  0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x01,0x2c,0x01,0x00,0x00,0x70,0x00,0x01,
  0x70,0x00,0x70,0x70,0x70,0x0f,0xfb,0xc8,0x0f,0x0f,0xc8,0xfb,0x00,0x01,0xc9,0xc9,
  0x01,0x01,0x01,0x00,0x01,0xc9,0xc9,0x01,0x00,0x0f,0x00,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x02,0x01,0x00,0xfb,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x0f,
  0xfb,0x01,0x01,0x01,0xc9,0x02,0x03,0x03,0x8c,0x8c,0x03,0x03,0x02,0x02,0x02,0xd6,
  0x02,0xd6,0x02,0x03,0x03,0x03,0xb3,0xd1,0x81,0x8c,0x03,0xfd,0xfd,0xd1,0xd1,0xb7,
  0x92,0x92,0xdf,0xdf,0x92,0xdf,0x92,0x92,0x92,0xb7,0xb7,0x92,0x92,0x92,0x92,0x92,
  0xb7,0xd1,0xd1,0x81,0xb3,0x81,0xd1,0xbf,0xd1,0x83,0x02,0xd6,0xc9,0xd6,0x03,0x81,
  0xfd,0x02,0x2c,0x2c,0x2c,0x2c,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x70,0x00,0x00,
  0x00,0x70,0x70,0x0f,0x0f,0x0f,0x0f,0x00,0x00,0x0f,0xfb,0xfb,0x00,0x01,0x02,0xc9,
  0x2c,0x01,0x01,0x2c,0x01,0x01,0x01,0x01,0x01,0x00,0xfb,0x01,0x01,0x00,0x00,0x01,
  0x00,0x00,0x01,0x01,0x01,0xc9,0x01,0x00,0x00,0x00,0xfb,0x00,0x0f,0x0f,0x0f,0x0f,
  0x00,0x01,0x01,0x2c,0x2c,0x02,0x02,0x02,0x03,0x03,0x02,0x02,0x02,0x02,0x02,0x02,
  0x02,0x03,0x03,0x03,0x03,0x83,0xfd,0x81,0xd1,0xfd,0x8c,0x8c,0x83,0xb3,0x81,0xb7,
  0xb7,0xdf,0xdf,0x92,0xdf,0xdf,0xdf,0x92,0x92,0xbf,0xd1,0xb7,0x92,0xb7,0x92,0xdf,
  0x92,0xb7,0xbf,0x81,0x83,0x83,0x81,0xd1,0x83,0x02,0x2c,0x2c,0x2c,0xd6,0x03,0x8c,
  0x03,0x2c,0x2c,0x2c,0x2c,0xc9,0x01,0x01,0x00,0x00,0x00,0x00,0x70,0x70,0x00,0x00,
  0x00,0x00,0x70,0x70,0x0f,0xfb,0xfb,0x00,0x00,0x70,0xfb,0x0f,0x00,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x0f,0xc8,0xfb,0x00,0xfb,0x00,
  0x00,0x00,0x00,0x01,0xc9,0x02,0x01,0xfb,0x0f,0x0f,0x0f,0x0f,0xc8,0x0f,0x00,0x00,
  0x01,0x00,0x2c,0x2c,0x2c,0xc9,0xd6,0x02,0x02,0x02,0x02,0x02,0x02,0xd6,0x02,0x02,
  0x03,0x03,0x03,0x03,0x03,0xfd,0x83,0xb3,0xfd,0xfd,0xfd,0x8c,0x83,0xb3,0x81,0xd1,
  0xb7,0x92,0xdf,0xdf,0xdf,0xdf,0xdf,0x92,0xb7,0xb7,0xd1,0xb7,0xb7,0xbf,0xb7,0xdf,
  0xdf,0x92,0xbf,0x81,0x8c,0x8c,0x83,0xfd,0x02,0x2c,0x00,0x00,0x01,0x2c,0x02,0x03,
  0x03,0x03,0x02,0xc9,0x01,0x03,0x03,0x2c,0x2c,0x00,0x70,0x70,0x00,0x00,0x00,0x00,
  0x00,0x00,0x70,0x00,0x00,0x01,0x01,0x00,0x00,0xfb,0x70,0xfb,0x70,0xfb,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x70,0x0f,0x82,0xc8,0xc8,0xc8,0x00,
  0xfb,0x00,0x00,0x01,0xc9,0xe5,0x01,0xfb,0xc8,0xc8,0x0f,0x0f,0x0f,0xfb,0xfb,0x01,
  0x01,0x01,0x01,0x01,0x2c,0x2c,0x2c,0xd6,0xd6,0xd6,0x02,0x02,0x02,0xd6,0xd6,0x03,
  0x03,0x03,0x03,0x03,0x03,0x03,0xfd,0x8c,0xfd,0x03,0xfd,0x8c,0x83,0xb3,0x81,0xd1,
  0xb7,0xdf,0xdf,0xdf,0xdf,0x92,0x92,0xb7,0xbf,0xd1,0x81,0x81,0xd1,0x81,0x81,0xb7,
  0xb7,0xb7,0x81,0x83,0x83,0x8c,0x03,0x70,0x70,0x70,0x70,0x70,0x70,0x2c,0x2c,0x02,
  0x03,0x8c,0x83,0xfd,0x83,0xbf,0xb7,0xfd,0x02,0x02,0x00,0x70,0x2c,0x2c,0x01,0x00,
  0x70,0x70,0x00,0x70,0x01,0xc9,0x00,0x70,0x00,0x00,0x00,0x00,0xfb,0x00,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x70,0x0f,0x82,0x82,0xc8,0xc8,0xc8,
  0x00,0xfb,0x00,0x01,0x01,0x01,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,0xc8,0x0f,0xfb,0x2c,
  0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0xc9,0xd6,0xd6,0x02,0x02,0x02,0x03,
  0x03,0x03,0x8c,0x03,0xc9,0xd6,0x03,0x03,0x03,0x83,0x8c,0xfd,0xb7,0xb7,0xd1,0xd1,
  0xb7,0x92,0xdf,0xdf,0xdf,0x92,0xb7,0x81,0x81,0x81,0x81,0xb3,0x81,0x83,0x8c,0x83,
  0xd1,0xbf,0xfd,0x83,0x81,0x83,0x02,0x0f,0x0f,0x0f,0x0f,0x0f,0x70,0x70,0x00,0x2c,
  0x03,0x8c,0x83,0x81,0x81,0xbf,0xb7,0xbf,0x03,0xc9,0x00,0x00,0x02,0x2c,0x01,0x01,
  0x00,0xfb,0x00,0xfb,0x01,0x00,0x70,0x00,0x00,0x70,0xfb,0x00,0x70,0x00,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x0f,0x0f,0x0f,0x82,0x82,0xc8,0xc8,
  0xc8,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x82,0xc8,0x0f,0xc8,0x0f,0x00,0xc9,
  0x01,0x01,0xc9,0x2c,0x2c,0x2c,0xc9,0x2c,0x2c,0x2c,0xc9,0xc9,0xd6,0x02,0x03,0x03,
  0x8c,0x8c,0x8c,0x03,0x02,0x2c,0xd6,0x02,0x81,0xb7,0x81,0x81,0xb7,0xb7,0xb7,0xb7,
  0x92,0x92,0xdf,0xdf,0xdf,0x92,0xbf,0x81,0xb3,0x81,0xb3,0x8c,0x8c,0x8c,0x8c,0x03,
  0x03,0x83,0xfd,0x8c,0x81,0xd1,0xd1,0x00,0x8b,0x0f,0x8b,0x8b,0x0f,0x0f,0x70,0x2c,
  0x02,0x03,0x81,0x81,0x81,0xb7,0xbf,0x81,0x83,0xe5,0x02,0x02,0xc9,0x00,0x00,0x01,
  0x70,0x0f,0xfb,0x0f,0x00,0x00,0x00,0x00,0x00,0xfb,0x0f,0x00,0x00,0x00,0x00,0x01,
  0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x70,0x0f,0x0f,0x0f,0xf0,0x82,0xf0,0xc8,
  0xc8,0xc8,0x70,0xfb,0xfb,0x0f,0xc8,0x0f,0x82,0x82,0x82,0x0f,0xc8,0x0f,0x01,0x8c,
  0x02,0x01,0x2c,0x2c,0x2c,0xc9,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0xd6,0x02,0x03,0xfd,
  0x8c,0x8c,0xfd,0x8c,0x03,0x2c,0x8c,0xfd,0xb7,0xdf,0xdf,0x92,0x92,0x92,0xb7,0x92,
  0x92,0x92,0xb7,0xb7,0xb7,0xbf,0x81,0xfd,0x81,0x81,0x83,0x8c,0x03,0x03,0x03,0x03,
  0x02,0xd6,0xd6,0x03,0xbf,0x92,0xb7,0x03,0x8b,0x0f,0x0f,0x0f,0x0f,0x8b,0x0f,0x70,
  0x2c,0x03,0xfd,0x81,0xb7,0xbf,0xbf,0x81,0x83,0xe5,0x02,0x01,0x00,0x00,0x00,0x00,
  0x0f,0x0f,0x0f,0x82,0x0f,0xfb,0x00,0xfb,0xfb,0xfb,0x0f,0xfb,0x00,0x00,0x70,0x01,
  0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x70,0x0f,0xfb,0x0f,0x0f,0x82,0x82,0x82,0xf0,
  0x0f,0xfb,0x0f,0x0f,0x0f,0x0f,0xc8,0x00,0x82,0x82,0xc8,0x0f,0xc8,0x00,0x01,0xe5,
  0x02,0x2c,0x01,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0x2c,0xd6,0x03,0x03,
  0x03,0x8c,0x8c,0x8c,0x03,0x02,0xb7,0xb7,0xb7,0xdf,0xdf,0xdf,0x92,0xdf,0x92,0xb7,
  0x92,0x92,0xbf,0x81,0x81,0xfd,0xb3,0x83,0xb3,0xb3,0xfd,0x03,0x03,0x03,0x03,0x03,
  0x02,0x2c,0xd6,0x8c,0xb7,0x92,0x92,0x81,0x00,0x8b,0x8b,0x0f,0x0f,0x0f,0x8b,0x70,
  0x01,0x02,0x03,0xfd,0x81,0x81,0xfd,0xbf,0x83,0xe5,0x02,0x00,0x00,0x00,0x70,0x70,
  0x82,0x82,0x82,0x82,0x0f,0x0f,0x70,0x00,0x00,0x0f,0x82,0x0f,0xc8,0x00,0x0f,0xfb,
  0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x70,0x0f,0x0f,0x0f,0x0f,0x82,0xf0,0x82,0x82,
  0xf0,0x0f,0x0f,0x82,0x0f,0x0f,0x00,0x0f,0x82,0x82,0x70,0x00,0x00,0x01,0x02,0x02,
  0x02,0xe5,0x2c,0x2c,0x2c,0x2c,0x01,0x01,0x01,0x2c,0x01,0x2c,0x2c,0xd6,0x02,0x02,
  0xd6,0x02,0xd6,0xd6,0x02,0x02,0xd1,0xd1,0xb7,0x92,0xdf,0xdf,0xdf,0xdf,0xdf,0xb7,
  0xb7,0xb7,0x81,0x83,0x83,0x8c,0xfd,0xfd,0xb3,0xb3,0x81,0x81,0x81,0x81,0x8c,0x03,
  0x2c,0x2c,0xd6,0x03,0x83,0x81,0xd1,0x81,0x00,0x8b,0x8b,0x8b,0x0f,0x0f,0x8b,0x0f,
  0x00,0xe5,0x03,0x8c,0xfd,0x92,0x92,0x81,0xfd,0x8c,0x8c,0x02,0x01,0x70,0x0f,0x0f,
  0xe0,0xe0,0x82,0x82,0x82,0x82,0x0f,0x0f,0x70,0x00,0xc8,0x82,0x82,0x82,0x0f,0x0f,
  0x00,0x00,0x01,0x01,0x01,0x00,0x00,0x70,0x70,0x0f,0x0f,0x0f,0x82,0x8d,0x8d,0x8d,
  0x8d,0x8d,0x82,0x82,0x0f,0xc8,0xc8,0x0f,0x82,0xf0,0xfb,0x01,0x01,0x01,0xc9,0xc9,
  0x2c,0x2c,0x2c,0x00,0x01,0x01,0x00,0x00,0x00,0x70,0x70,0x70,0x2c,0x2c,0x01,0x2c,
  0x2c,0x2c,0x2c,0xd6,0xd6,0x03,0x03,0xfd,0xbf,0xb7,0x92,0x92,0x92,0x92,0xb7,0xb7,
  0xd1,0x81,0x83,0x8c,0xb7,0x81,0x8c,0x81,0x8c,0x81,0x92,0x92,0xbf,0x81,0x03,0x02,
  0x02,0x02,0x2c,0x02,0x03,0x8c,0x81,0x81,0x00,0x8b,0x8b,0x82,0x8b,0x8b,0x0f,0x0f,
  0x02,0x81,0x83,0x83,0x92,0xdf,0xdf,0x92,0xbf,0x83,0x83,0xe5,0xc9,0x0f,0x82,0x8d,
  0x8e,0x82,0x82,0x82,0x8d,0x8d,0x82,0x82,0x82,0x0f,0x0f,0x82,0x8d,0x82,0x82,0xc8,
  0xfb,0x70,0x00,0x70,0x70,0x70,0xfb,0x0f,0xc8,0x0f,0x0f,0x82,0x8d,0xe0,0xfc,0x8e,
  0xfc,0xfc,0xe0,0x8d,0x82,0x82,0x0f,0xc8,0xc8,0x00,0x00,0x01,0x01,0x01,0x01,0x00,
  0x01,0x00,0x70,0x70,0x70,0x00,0x70,0x70,0x0f,0x0f,0x0f,0x70,0x70,0x00,0x00,0x00,
  0x70,0x70,0x2c,0x02,0x03,0x02,0x02,0x83,0x81,0x81,0xbf,0xb7,0xb7,0x81,0x81,0x81,
  0xfd,0x02,0x03,0x81,0xdf,0x92,0x81,0xb7,0xd1,0x81,0x81,0xb7,0x81,0xfd,0x03,0x03,
  0x02,0x02,0x02,0x2c,0x02,0x03,0x83,0x03,0x0f,0x82,0x8b,0x8b,0x82,0x8b,0x8b,0x2c,
  0xbf,0x92,0x92,0x92,0x92,0x92,0x92,0xb7,0x83,0xe5,0x83,0x02,0x00,0x01,0x82,0x8e,
  0x8e,0xf0,0x82,0x82,0xe0,0xe0,0x8d,0x8d,0xe0,0x8d,0x8d,0x8e,0xe0,0x8d,0x82,0x82,
  0x82,0x0f,0x0f,0x0f,0x0f,0x82,0x0f,0x0f,0x00,0x0f,0x82,0x8d,0xe0,0x8e,0xce,0xce,
  0xce,0xce,0xce,0xfc,0x8e,0xe0,0x82,0x0f,0x00,0x2c,0x00,0x01,0x01,0x01,0x01,0x00,
  0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,0x82,0x82,0x82,0x0f,0x0f,0x0f,0x70,0x70,0x2c,
  0xc9,0xe5,0x02,0x81,0x81,0x03,0x8c,0x83,0x03,0x03,0x83,0xfd,0xfd,0x8c,0xd6,0x03,
  0x02,0x2c,0x02,0xb7,0xdf,0xdf,0x92,0xbf,0x83,0x8c,0x03,0x8c,0x8c,0x8c,0xe5,0x02,
  0x02,0xc9,0xd6,0x02,0xd6,0x02,0x03,0xe5,0x0f,0x8b,0x8b,0x8b,0x82,0x8b,0x0f,0x02,
  0x81,0xb7,0xb7,0xbf,0xbf,0x92,0xbf,0x92,0xe5,0xe5,0xe5,0xe5,0xc9,0x01,0x8e,0xce,
  0xe0,0x82,0x8d,0xe0,0x8e,0xe0,0xe0,0xe0,0x8e,0xe0,0x8e,0x8e,0x8e,0x8e,0xe0,0x8d,
  0x8d,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0xfb,0x0f,0x82,0x8e,0xca,0xce,0xce,0xce,
  0xcf,0xce,0xce,0xce,0x8e,0x8d,0x82,0x0f,0xfa,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x70,0x0f,0x0f,0x82,0x82,0xf5,0x82,0x82,0x82,0x82,0x0f,0x0f,0x70,0x2c,0xbf,
  0xb7,0xbf,0x81,0x81,0x81,0xbf,0x81,0x8c,0x03,0x03,0xe5,0x02,0x2c,0x01,0x01,0x00,
  0x00,0x00,0x2c,0xbf,0xdf,0xdf,0xbf,0x02,0x02,0x02,0x02,0x02,0x8c,0x83,0x8c,0x03,
  0x02,0x2c,0x2c,0x2c,0xd6,0x02,0xfd,0x81,0x70,0x8b,0x8b,0xf6,0x82,0x70,0x01,0x03,
  0x83,0x81,0xbf,0xbf,0xbf,0xb7,0xbf,0x92,0x92,0x8c,0xe5,0x02,0xc9,0x01,0x8e,0xce,
  0xcf,0xcf,0xce,0x8e,0x8e,0x8e,0x8e,0x8e,0xce,0xce,0xce,0xce,0xce,0xce,0xca,0xfc,
  0x8e,0xe0,0xe0,0xe0,0xe0,0x8d,0xe0,0x82,0x82,0x0f,0xe0,0x8e,0xce,0xce,0xfc,0xe0,
  0x8d,0x82,0x8d,0x82,0xf0,0xf0,0x82,0x0f,0xf0,0xfb,0xf9,0x01,0x01,0x01,0x01,0x01,
  0x01,0x0f,0x82,0x82,0x82,0x82,0xe0,0xe0,0xe0,0x82,0x82,0x0f,0x0f,0x70,0xe5,0x81,
  0xfd,0xfd,0x83,0xfd,0x81,0xbf,0x81,0x81,0x03,0x02,0x00,0x00,0x70,0x70,0xfb,0x0f,
  0x0f,0x02,0xe5,0xfd,0xbf,0xbf,0x8c,0x01,0x00,0x70,0x00,0x00,0x01,0x2c,0x2c,0x01,
  0x2c,0x2c,0x01,0x01,0x00,0x00,0x01,0x2c,0x70,0x8b,0x82,0xf6,0xe5,0x92,0x8c,0x03,
  0x03,0x83,0x83,0x83,0xfd,0x83,0xfd,0xbf,0xfd,0xe5,0x00,0x0f,0x00,0xc8,0x8e,0xec,
  0xcf,0xcf,0xcf,0xcf,0xcf,0xce,0xce,0xce,0xcf,0xec,0xdd,0xdd,0xcf,0xcf,0xcf,0xce,
  0xce,0xce,0x8e,0x8e,0x8e,0x8e,0x8e,0xe0,0x82,0x82,0x8d,0xe0,0xe0,0xe0,0x8d,0x82,
  0x82,0x82,0x8d,0x82,0x82,0x8d,0xe0,0xe0,0xe0,0x8d,0x00,0x00,0xfb,0x0f,0xc8,0x0f,
  0x0f,0x82,0xe0,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xe0,0x82,0x8b,0x82,0x0f,0x00,0x02,
  0x02,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0x01,0x0f,0x82,0x82,0x82,0x82,0x82,0x82,
  0x0f,0xe5,0x8c,0x02,0xd6,0x02,0x2c,0x70,0x0f,0x0f,0x0f,0x0f,0x82,0x0f,0x0f,0x70,
  0x00,0x01,0x2c,0x02,0x02,0xd6,0x70,0x8b,0xf6,0xe0,0xe0,0xe0,0x02,0xbf,0x8c,0x02,
  0xe5,0xe5,0xe5,0xe5,0x02,0xe5,0xe5,0x8c,0xe5,0x01,0x00,0xc8,0x00,0xc8,0xc8,0x8e,
  0x8d,0xce,0xcf,0xcf,0xdd,0xcf,0xdd,0xdd,0xd9,0xd9,0xd9,0xd9,0xd9,0xdd,0xd9,0xdd,
  0xdd,0xcf,0xce,0xcf,0xce,0xcf,0xcf,0xce,0x8e,0xe0,0xe0,0xe0,0x8d,0xe0,0x8e,0xe0,
  0xe0,0xe0,0xe0,0xe0,0x8e,0x8e,0x8e,0xce,0xce,0xce,0x8e,0x8e,0x8e,0xe0,0xe0,0x8e,
  0x8e,0xce,0x8e,0xce,0xce,0xce,0xce,0xce,0x8e,0xce,0x8e,0xe0,0xe0,0x82,0x82,0x0f,
  0x0f,0x70,0x70,0x0f,0x0f,0x0f,0x8b,0x82,0xe0,0x8e,0x8e,0x8e,0x8e,0x8e,0xe0,0x82,
  0x0f,0x0f,0x0f,0x82,0x82,0x82,0x82,0xe0,0xe0,0xe0,0xe0,0xe0,0x8d,0x82,0xf6,0x0f,
  0x70,0x70,0x2c,0x2c,0xe5,0x02,0xd6,0xf6,0xe0,0xe0,0xfe,0xe0,0x70,0xd6,0xd6,0xc9,
  0xc9,0x02,0x02,0x02,0xc9,0x83,0xc9,0x02,0x01,0x01,0x01,0xf9,0x01,0x01,0xfb,0xf0,
  0xe0,0xce,0xcf,0xd9,0xd9,0xf8,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xf8,0xf8,0xf8,0xf8,
  0xf8,0xd9,0xd9,0xdd,0xf7,0xdd,0xdd,0xcf,0xcf,0xce,0xce,0xce,0xce,0xce,0xce,0xce,
  0x8e,0xfc,0x8e,0x8e,0xce,0xce,0xcf,0xec,0xec,0xec,0xcf,0xcf,0xcf,0xce,0xce,0xcf,
  0xcf,0xcf,0xcf,0xcf,0xec,0xec,0xcf,0xcf,0xcf,0xce,0x8e,0x8e,0x8e,0x8e,0xe0,0x8e,
  0xe0,0xe0,0xe0,0xe0,0xe0,0x8e,0x8e,0xce,0xce,0xcf,0xce,0xce,0xcf,0xce,0x8e,0x8e,
  0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xce,0x8e,0xe0,0xe0,0xf6,
  0x82,0x82,0x0f,0x0f,0x70,0x70,0x8b,0x82,0x82,0xfe,0x8e,0x8e,0x82,0x0f,0x01,0x01,
  0x01,0x01,0x00,0x01,0xfa,0x8c,0xfa,0x01,0xf0,0xe0,0x82,0x00,0xc8,0xfb,0x00,0xfb,
  0xfc,0xe0,0xcf,0xd9,0xdd,0xf8,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,
  0xe6,0xe6,0xe6,0xe6,0xf8,0xe6,0xd9,0xf7,0xf7,0xdd,0xdd,0xec,0xcf,0xcf,0xcf,0xec,
  0xce,0xce,0x8e,0xce,0xcf,0xdd,0xdd,0xdd,0xd9,0xd9,0xd9,0xd9,0xd9,0xdd,0xdd,0xdd,
  0xdd,0xdd,0xdd,0xdd,0xdd,0xdd,0xf8,0xdd,0xdd,0xdd,0xec,0xcf,0xcf,0xce,0xce,0xce,
  0xce,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x8e,0x8e,0xce,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xcf,0xce,0xce,0x8e,0x8e,0x8e,
  0x8e,0x8e,0xe0,0x82,0x82,0x82,0x8e,0x8b,0x82,0x8e,0x8e,0x82,0x0f,0x70,0x00,0x00,
  0x00,0x0f,0x0f,0xc8,0x0f,0x00,0xc8,0xc8,0x8d,0xcf,0xce,0xf5,0xc8,0xfb,0xfb,0x0f,
  0x82,0x8d,0xce,0x8e,0xca,0xfc,0x8d,0xe0,0xdd,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,
  0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xe6,0xf8,0xf8,0xd9,0xd9,0xcf,0x8e,0xce,0xec,0xdd,
  0xec,0xec,0xec,0xdd,0xd9,0xd9,0xd9,0xd9,0xd9,0xf8,0xf8,0xf1,0xf8,0xd9,0xf1,0xf8,
  0xd9,0xf1,0xd9,0xd9,0xd9,0xd9,0xd9,0xf8,0xd9,0xf1,0xd9,0xd9,0xf7,0xdd,0xdd,0xcf,
  0xcf,0xcf,0xce,0x8e,0xce,0x8e,0x8e,0x8e,0xe0,0x8e,0x8e,0xe0,0x8e,0x8e,0x8e,0x8e,
  0xe0,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xce,0xcf,0xdd,0xdd,0xdd,0xce,0xce,0x8e,0x8e,
  0x8e,0xfe,0x8e,0x8e,0x82,0xf6,0xe0,0x82,0x8e,0xe0,0xe0,0x82,0x82,0x0f,0x70,0x0f,
  0x82,0x82,0x8d,0x8d,0x82,0x82,0x8e,0x8e,0xdd,0xf8,0xd9,0xca,0xf5,0x0f,0x0f,0xf0,
  // 19: sky 3, part D
  0x58,0xc6,0x38,0xc6,0x00,0x00,0x08,0x00,0x10,0x00,0x18,0x00,0x00,0x01,0x08,0x01,
  0xf7,0xbd,0x10,0x01,0x18,0x01,0x00,0x02,0x08,0x02,0x10,0x02,0x18,0x02,0x00,0x03,
  0x08,0x03,0x10,0x03,0x18,0x03,0x00,0x04,0x08,0x04,0x10,0x04,0xb7,0xbd,0x18,0x04,
  0x00,0x05,0x08,0x05,0x10,0x05,0x18,0x05,0x00,0x06,0x08,0x06,0x10,0x06,0x18,0x06,
  0x00,0x07,0x08,0x07,0x10,0x07,0x18,0x07,0x00,0x20,0x08,0x20,0x10,0x20,0x18,0x20,
  0x00,0x21,0x08,0x21,0x10,0x21,0x18,0x21,0x00,0x22,0x08,0x22,0x10,0x22,0x18,0x22,
  0x00,0x23,0x08,0x23,0x10,0x23,0x18,0x23,0x00,0x24,0x08,0x24,0x10,0x24,0x18,0x24,
  0x00,0x25,0x08,0x25,0x10,0x25,0x18,0x25,0x00,0x26,0x08,0x26,0x10,0x26,0x18,0x26,
  0x00,0x27,0x08,0x27,0x10,0x27,0x18,0x27,0x00,0x40,0x08,0x40,0x10,0x40,0x18,0x40,
  0x00,0x41,0x08,0x41,0x10,0x41,0x18,0x41,0x00,0x42,0x08,0x42,0x10,0x42,0x18,0x42,
  0x00,0x43,0x08,0x43,0x10,0x43,0x18,0x43,0x00,0x44,0x08,0x44,0x10,0x44,0x18,0x44,
  0x00,0x45,0x08,0x45,0x10,0x45,0x18,0x45,0x00,0x46,0x08,0x46,0x10,0x46,0x18,0x46,
  0x00,0x47,0x08,0x47,0x10,0x47,0x18,0x47,0x00,0x60,0x08,0x60,0x10,0x60,0x18,0x60,
  0x00,0x61,0x08,0x61,0x10,0x61,0x18,0x61,0x00,0x62,0x08,0x62,0x10,0x62,0x18,0x62,
  0x00,0x63,0x08,0x63,0x10,0x63,0x18,0x63,0x00,0x64,0x08,0x64,0xf8,0xc5,0x10,0x64,
  0x18,0x64,0x00,0x65,0x08,0x65,0x10,0x65,0x18,0x65,0x00,0x66,0x08,0x66,0x10,0x66,
  0x18,0x66,0x00,0x67,0x08,0x67,0x10,0x67,0x18,0x67,0x00,0x80,0x08,0x80,0xf6,0x8c,
  0x10,0x80,0x18,0x80,0x00,0x81,0x08,0x81,0x10,0x81,0x18,0x81,0x96,0xad,0x00,0x82,
  0x08,0x82,0x10,0x82,0x18,0x82,0x00,0x83,0x9a,0xd6,0x08,0x83,0x10,0x83,0xb6,0x7c,
  0x18,0x83,0x00,0x84,0x08,0x84,0x10,0x84,0x18,0x84,0x00,0x85,0x08,0x85,0x10,0x85,
  0x18,0x85,0x00,0x86,0x77,0x9d,0x08,0x86,0x10,0x86,0x18,0x86,0x00,0x87,0x08,0x87,
  0x10,0x87,0xb8,0xad,0x18,0x87,0x00,0xa0,0x08,0xa0,0x10,0xa0,0x18,0xa0,0x37,0x9d,
  0x00,0xa1,0x08,0xa1,0x18,0xbe,0x10,0xa1,0x18,0xa1,0xd8,0xbd,0x00,0xa2,0x08,0xa2,
  0x10,0xa2,0x18,0xa2,0x00,0xa3,0x08,0xa3,0x10,0xa3,0x18,0xa3,0x75,0xad,0x00,0xa4,
  0x08,0xa4,0x10,0xa4,0x18,0xa4,0xdb,0xd6,0x00,0xa5,0x08,0xa5,0x10,0xa5,0x18,0xa5,
  0x00,0xa6,0x08,0xa6,0x10,0xa6,0x18,0xa6,0x34,0xa5,0xb3,0x5b,0xd8,0xad,0x00,0xa7,
  0x08,0xa7,0x10,0xa7,0x18,0xa7,0x00,0xc0,0x08,0xc0,0x10,0xc0,0x95,0x7c,0x18,0xc0,
  0x50,0x84,0x76,0xa5,0x00,0xc1,0x08,0xc1,0x7a,0xd6,0x10,0xc1,0x18,0xc1,0x00,0xc2,
  0x08,0xc2,0x17,0x8d,0x10,0xc2,0x17,0xc6,0x18,0xc2,0xf3,0x6b,0x00,0xc3,0x08,0xc3,
  0x3d,0xe7,0xd3,0x5b,0x10,0xc3,0xef,0x7b,0x18,0xc3,0x00,0xc4,0x5a,0xc6,0xd6,0x7c,
  0x2c,0x63,0x08,0xc4,0x9f,0xef,0x99,0xce,0x9a,0xce,0xf3,0x9c,0x94,0xbd,0xf5,0xcd,
  0x79,0xd6,0xf7,0xcd,0xae,0x73,0xf4,0x9c,0xb2,0x94,0x48,0x42,0x24,0x21,0xaa,0x52,
  0x87,0xef,0x87,0x87,0x87,0x87,0xa9,0xa9,0xa2,0xd6,0xd6,0xe5,0xcd,0xcd,0xcd,0xcd,
  0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xe9,0xe5,0xe5,0xe5,0xd6,0x97,0xd6,0xe5,
  0xe5,0xe5,0xe5,0xe5,0xe5,0xe9,0xe9,0xe9,0xe9,0xe9,0xcd,0xe9,0xcd,0xe9,0xcd,0xe9,
  0xe9,0xe5,0xe5,0xcd,0xe9,0xcd,0xcd,0xcd,0xe9,0xcd,0xe5,0xcd,0xe9,0xe9,0xe5,0xe5,
  0xe5,0xe5,0xd6,0xd6,0xd6,0xef,0xe1,0x87,0x87,0xef,0xef,0xef,0x97,0x97,0x97,0xef,
  0xef,0xef,0xef,0x97,0xef,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xef,0x87,0xa0,0xef,
  0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xef,0xef,0xef,0xef,0x97,0xa0,0xa2,0xa2,0xe1,
  0x87,0xd6,0xef,0x97,0x97,0xef,0xe5,0xe5,0xe5,0xcd,0xe5,0xd6,0xd6,0x87,0x87,0xe1,
  0x97,0xef,0x87,0xe1,0x87,0xc7,0xa9,0xa9,0xa9,0xa2,0xc7,0xd6,0xe9,0xcd,0xcd,0xe9,
  0xcd,0xe9,0xe9,0xe9,0xe9,0xcd,0xcd,0xe9,0xcd,0xe9,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,
  0xe5,0xe9,0xe9,0xe9,0xe9,0xe9,0xe9,0xcd,0xe9,0xe9,0xcd,0xe9,0xe9,0xcd,0xcd,0xe9,
  0xcd,0xe9,0xe9,0xe9,0xe9,0xcd,0xe9,0xcd,0xcd,0xe9,0xcd,0xcd,0xcd,0xcd,0xe9,0xe5,
  0xe9,0xe5,0xd6,0xd6,0x97,0x97,0xa0,0xe1,0xe1,0xef,0x97,0x97,0x97,0x97,0x97,0x97,
  0xe1,0xe1,0xe1,0xa0,0x87,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0xef,0x87,0xef,
  0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xef,0xef,0x97,0xef,0xa0,0xe1,0x97,
  0xd6,0xd6,0xd6,0xd6,0xd6,0x97,0xd6,0x97,0xd6,0xe5,0xe5,0xd6,0xaf,0xe1,0xef,0xef,
  0x87,0xe1,0x87,0xe1,0x87,0xa2,0xb5,0xb2,0xee,0xb5,0xce,0xa2,0xa2,0xa2,0xaf,0x87,
  0x87,0xa2,0xc7,0x8e,0xa2,0xaf,0x87,0x87,0xd6,0xd6,0xd6,0x87,0x87,0xd6,0xd6,0xd6,
  0xd6,0xd6,0x87,0xd6,0xef,0xe5,0xe9,0xe9,0xe9,0xe9,0xcd,0xe9,0xe9,0xcd,0xe9,0xe9,
  0xe9,0xe9,0xe9,0xe9,0xcd,0xe9,0xe9,0xe9,0xcd,0xe9,0xe9,0xe9,0xcd,0xcd,0xcd,0xcd,
  0xe9,0xe9,0xe5,0xd6,0xd6,0xd6,0xd6,0x97,0xef,0x97,0xd6,0xd6,0x97,0x97,0xd6,0x97,
  0xe1,0xe1,0xa2,0x87,0xe1,0xa0,0x87,0xef,0xef,0xa0,0xef,0xef,0x87,0xef,0xef,0xef,
  0xef,0xef,0xef,0xef,0xef,0xef,0xef,0x97,0x97,0xa0,0xef,0x97,0x97,0x97,0xd6,0xd6,
  0xd6,0xe5,0xd6,0xe5,0xd6,0x97,0xd6,0xd6,0xd6,0xe5,0x87,0xef,0xaf,0x8e,0xaf,0xa2,
  0x87,0xe1,0xef,0x87,0xe1,0xaf,0xa9,0xa9,0xce,0xee,0xb2,0xb2,0xb2,0xb2,0xb2,0xb2,
  0xa9,0x8e,0xa9,0xc7,0xa9,0xa9,0xa9,0xa9,0xa9,0x8e,0xa2,0x8e,0xa9,0xb5,0xa9,0xa9,
  0x8e,0xa9,0xa9,0xa9,0xa9,0xe1,0xe5,0xe5,0xd6,0xe1,0xd6,0xd6,0xe5,0xe5,0xcd,0xe9,
  0xcd,0xe9,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xcd,0xe5,0xd6,
  0xd6,0xd6,0xe5,0xe5,0xe5,0xd6,0x97,0x97,0x97,0x97,0x97,0x97,0x97,0xd6,0x97,0x97,
  0x87,0xa2,0xa2,0xa2,0xe1,0x87,0xef,0x87,0xe1,0x87,0x87,0xa0,0xa0,0x87,0xef,0xef,
  0x87,0xa2,0xa2,0x87,0xef,0x97,0x97,0x97,0x97,0xef,0xef,0x97,0x97,0xd6,0xd6,0xd6,
  0xe5,0xe9,0xe5,0xe5,0xe5,0xe5,0xe5,0xd6,0xd6,0xd6,0x87,0xc7,0x87,0xaf,0x87,0x87,
  0x87,0xe1,0xaf,0xa2,0xa2,0xc7,0xa2,0xa2,0xa2,0xb5,0xb2,0xb5,0xa9,0xa9,0xb5,0xb5,
  0xb5,0xb2,0xa9,0xa9,0xa9,0xce,0xce,0xb5,0xa2,0xaf,0xc7,0xa9,0xce,0xb2,0xb2,0xb2,
  0xb5,0xb2,0xce,0xb5,0xce,0xa9,0xa2,0xa9,0xa9,0xce,0xce,0xa9,0xa2,0xa2,0xe1,0xd6,
  0x87,0x87,0x87,0xd6,0xe5,0xe9,0xe5,0xe5,0xe5,0xe5,0xe5,0xd6,0xd6,0x87,0xa2,0xa9,
  0xa9,0xa9,0xc7,0xa2,0xa2,0xc7,0xa2,0xa2,0xa2,0xe1,0xef,0xef,0x97,0xd6,0xd6,0x87,
  0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xe1,0xe1,0xe1,0xe1,0xe1,0xe1,0xa0,0xa0,
  0xa2,0xa2,0xe1,0xa0,0x87,0x97,0x97,0x97,0x97,0x97,0xef,0xef,0x97,0xd6,0xd6,0xe5,
  0xe5,0xe9,0xcd,0xcd,0xe9,0xe5,0xe5,0xd6,0xd6,0xd6,0xef,0x87,0x87,0xe1,0xef,0x87,
  0xe1,0xa2,0xc7,0xc7,0xc7,0xa9,0xc7,0xa2,0xa2,0x8e,0x8e,0xa2,0xa2,0xc7,0xc7,0x8e,
  0xa9,0xb5,0xa9,0xa2,0xc7,0xc7,0xc7,0xa2,0xaf,0x87,0x87,0x87,0x8e,0xa9,0xb2,0xb2,
  0xb5,0xb2,0xb2,0xb5,0xce,0xce,0xce,0xce,0xce,0xce,0xce,0xce,0xce,0xce,0xb2,0xce,
  0xce,0xa9,0xa9,0xb2,0xb2,0xa2,0xa2,0x8e,0xa2,0xa9,0xa9,0xa9,0xa9,0xa9,0xce,0xa9,
  0xa9,0xb5,0xa9,0xa9,0xb5,0xa9,0xa9,0xa2,0xa2,0xe1,0xef,0x97,0x97,0xd6,0x97,0x87,
  0xa2,0xc7,0xa2,0xa2,0xa2,0xc7,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xe1,0xe1,
  0xa2,0xe1,0xe1,0xe1,0xef,0x97,0x87,0xe1,0x97,0xd6,0xd6,0xa0,0xd6,0x97,0xef,0xd6,
  0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xe5,0xd6,0xd6,0xd6,0xef,0x87,0xe1,0xe1,0xef,0x97,
  0x87,0xa2,0xa2,0xc7,0xa2,0xa9,0x8e,0x8e,0xa9,0xa9,0xa9,0xc7,0xc7,0xc7,0xa2,0xa2,
  0x8e,0xa9,0x8e,0xa2,0xa2,0x87,0xa2,0xaf,0x87,0x87,0xaf,0xc7,0xa9,0xce,0xb5,0xce,
  0xce,0xb2,0xb5,0xb5,0x00,0xee,0xb2,0xb2,0xce,0xce,0xb5,0xb2,0xce,0xce,0xce,0xce,
  0xce,0xce,0xa9,0xb2,0x00,0xb2,0xce,0xce,0xb5,0xb5,0xce,0xce,0xa9,0xb5,0xb5,0xce,
  0xb5,0xce,0xce,0xce,0xb5,0xce,0xa2,0xe1,0xef,0x87,0xef,0x97,0xd6,0x97,0xe1,0x87,
  0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
  0x87,0xe1,0xe1,0xe1,0x97,0xd6,0xef,0xe1,0xef,0x97,0xd6,0x97,0x97,0xef,0xaf,0xef,
  0xd6,0x87,0x87,0xef,0xef,0xd6,0xef,0xd6,0x97,0x97,0xd6,0xef,0xd6,0xd6,0xd6,0x87,
  0x87,0xa2,0xc7,0xc7,0xa2,0xa2,0xa2,0x8e,0xa9,0xa9,0xb5,0xa9,0x8e,0x8e,0x8e,0xa2,
  0xa2,0xa2,0x8e,0xfc,0xce,0xa9,0xa9,0xa9,0xa2,0xa2,0xa2,0xa2,0xa9,0xb5,0xce,0xb5,
  0xb2,0xb2,0xb2,0xb2,0x94,0xc3,0x00,0xc3,0x00,0xe8,0xc3,0x94,0xb5,0xb2,0xb2,0xb2,
  0xce,0xb2,0xb2,0xb2,0x01,0x01,0xce,0xb2,0xb5,0xb5,0xce,0xce,0xce,0xce,0xce,0xb5,
  0xb5,0xa9,0xa9,0xce,0xce,0xb5,0xa2,0xef,0xd6,0x97,0x97,0xd6,0xd6,0xef,0xe1,0x87,
  0xa2,0xa2,0x87,0xa2,0xc7,0xa2,0xa2,0xa2,0xa2,0xa2,0xe1,0xef,0x87,0xa2,0xa2,0xa2,
  0xe1,0x87,0xe1,0xe1,0xe1,0xa2,0x87,0xa2,0xa2,0xd6,0xd6,0xd6,0xd6,0x97,0x97,0xd6,
  0x97,0xd6,0xd6,0xd6,0xd6,0x97,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0x87,
  0x87,0xa2,0xaf,0xaf,0xa2,0xa2,0xa2,0x8e,0x8e,0xa2,0xc7,0xa2,0xc7,0xa2,0xa2,0xaf,
  0xc7,0xa9,0xd6,0xff,0x8e,0xa2,0xa2,0xa9,0xa2,0xa2,0xa2,0xa2,0xa2,0xc7,0xa2,0xa9,
  0xce,0xb5,0xce,0xb5,0xb2,0x94,0xf3,0xf2,0xe8,0xc3,0xc3,0xc3,0xf3,0xee,0xb2,0xb2,
  0xb2,0xb2,0xce,0xb2,0xb2,0xee,0xb2,0xb2,0xb2,0xb2,0x8e,0xa9,0xce,0xb5,0xce,0xb5,
  0xce,0xb5,0xb5,0xb5,0xb5,0xa9,0x87,0xef,0xd6,0x97,0xd6,0xd6,0xd6,0xe1,0x87,0xa2,
  0x87,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,0xc7,0xa2,0xa2,0xaf,0xa2,0xa2,0xa2,0xa2,0xa2,
  0x87,0xe1,0x87,0x87,0x87,0xa2,0xb2,0x8e,0x87,0xef,0xd6,0xd6,0x97,0xef,0x97,0x97,
  0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0x97,0xd6,0xd6,0x97,0x97,0xd6,0xd6,0xd6,0xd6,0x97,
  0xaf,0xe1,0x87,0xef,0x87,0x87,0xe1,0xd9,0xd9,0x87,0xef,0x97,0xef,0x87,0x87,0xd6,
  0x87,0xa2,0x9b,0xfd,0xfc,0xd8,0xa9,0xc7,0xa2,0xa2,0xa2,0xa2,0xa9,0xa2,0xa9,0xa9,
  0xa9,0xa9,0xa9,0x00,0xf3,0xf3,0xf3,0xf3,0xc3,0x94,0xe3,0x8e,0x08,0x08,0xb2,0xce,
  0xb2,0xce,0xb5,0xb2,0xb2,0xb2,0xb5,0xce,0xb5,0xa2,0xa2,0x8e,0xa9,0x8e,0xa9,0xa9,
  0x8e,0x8e,0xa9,0xa2,0xa2,0xa2,0x87,0x97,0xd6,0xd6,0xd6,0xd6,0xd6,0x87,0xa2,0xa2,
  0xa2,0xa2,0xc7,0xa2,0xa2,0xa2,0xa2,0xa2,0xb5,0xb2,0xa2,0xa2,0xa2,0xa2,0xa2,0xa2,
  0x87,0x87,0xe1,0xe1,0x87,0x8e,0xb5,0xb2,0x8e,0xe1,0xe1,0x87,0xef,0xa2,0xaf,0x87,
  0x87,0x87,0x8e,0xa2,0xa2,0xa2,0x87,0xd6,0xd6,0xd6,0xd6,0xd6,0xd6,0x87,0xfc,0x87,
  0xcc,0xaf,0x87,0x87,0x87,0xa2,0xd6,0x8e,0xe3,0x8e,0xaf,0xa2,0xaf,0x87,0xd6,0xd8,
  0xd6,0x87,0xff,0xff,0x9b,0xf0,0xa2,0x8e,0xa9,0x8e,0xce,0xa9,0xce,0xb5,0xb5,0xb5,
  0xb5,0xd9,0xc3,0xc3,0x94,0x01,0xc3,0xdc,0x94,0x00,0xbe,0xcc,0xf5,0xcc,0x8e,0x00,
  0xb2,0xb2,0xb2,0xb5,0x00,0x01,0xa9,0xb2,0xce,0x8e,0xa2,0xa2,0xa2,0x8e,0x8e,0xa2,
  0xa2,0xa9,0x8e,0xa9,0x8e,0xa9,0x87,0xef,0xd6,0xe5,0xe5,0xe5,0xe5,0x87,0xa2,0xa2,
  0xa2,0xa2,0x87,0xe5,0xa2,0xa2,0xc7,0x8e,0x8e,0xb5,0x8e,0xa2,0xa2,0xa2,0xa2,0xa2,
  0xaf,0x87,0x87,0xe1,0xa2,0xaf,0xa2,0xaf,0xa2,0xa2,0x87,0x87,0xa2,0x8e,0xa2,0xa2,
  0xa2,0xa2,0x08,0x08,0xa2,0xaf,0xa2,0xa2,0xa2,0xa2,0xa2,0xb5,0xb2,0xb2,0x8e,0xcc,
  0xd8,0xaf,0xa2,0xa2,0xc7,0xa2,0xfb,0xf0,0x01,0x08,0xcc,0x8e,0x8e,0x8e,0xfc,0x9b,
  0x9b,0xd8,0xf0,0xfd,0xff,0xff,0xfb,0xce,0xa9,0xa9,0xa2,0xa9,0xa2,0x8e,0xa2,0x00,
  0xf3,0xe8,0xc3,0xc3,0xc3,0x08,0x94,0xc3,0x16,0xcc,0x8e,0xf5,0x8e,0x08,0xbe,0x8e,
  0xf3,0xb2,0xb5,0xb2,0xb2,0xb2,0x01,0x94,0xf3,0xb5,0xce,0xce,0xa9,0xb5,0xce,0xce,
  0xb5,0xce,0xb5,0x8e,0xaf,0xa2,0x87,0xd6,0xd6,0xe5,0xd6,0xd6,0xd6,0xa2,0xa2,0xfc,
  0xa2,0x01,0x87,0xe5,0x87,0xa2,0xa2,0xa2,0x08,0xb2,0xb5,0xa2,0xa2,0x87,0x87,0x87,
  0x87,0xef,0xe1,0xa2,0x8e,0xa2,0xa2,0xa2,0xa2,0xa2,0x8e,0xaf,0xaf,0x08,0x76,0x08,
  0x8e,0xa2,0xaf,0xa2,0xb2,0x94,0x94,0xf4,0xa2,0xa2,0x8e,0xb5,0x08,0x08,0xcc,0xfc,
  0xfc,0xbe,0xaf,0xa2,0xf4,0xc3,0xbe,0xeb,0xcc,0xc3,0x00,0xcc,0xbe,0x08,0xd8,0xf0,
  0xfa,0xf0,0xfd,0xff,0xfd,0xfd,0xd8,0xa2,0xa2,0x8e,0x8e,0x8e,0xa2,0x00,0x94,0x94,
  0xf2,0xe8,0xe8,0xc3,0x01,0xf3,0x00,0xf3,0xfb,0xfb,0xcc,0x08,0x08,0x01,0x8e,0xbe,
  0xbe,0xb2,0x8e,0xc7,0xa9,0xf3,0xf3,0xf3,0x00,0xa9,0xa2,0xaf,0x8e,0xaf,0xa9,0x8e,
  0x8e,0xb2,0x00,0xa2,0x87,0xd6,0xd6,0xd6,0xe5,0xe5,0xe5,0xd6,0xd6,0xa2,0xa2,0xf0,
  0xfc,0x76,0xaf,0xd6,0x87,0xb5,0xb5,0x08,0x8e,0xbe,0xf5,0xfb,0xcc,0xa2,0xaf,0xaf,
  0xaf,0xef,0x87,0xa2,0xb5,0x08,0x8e,0xa2,0x87,0x87,0x87,0x87,0xaf,0x08,0x08,0xd9,
  0xaf,0xa2,0xaf,0xb2,0x8e,0x16,0xf3,0xdc,0xe3,0x08,0x08,0x8e,0xbe,0xbe,0xd8,0xd8,
  0xf0,0xfc,0x08,0xe8,0xe8,0xe8,0x00,0xff,0xfb,0x08,0xe8,0xf3,0xbe,0xcc,0xeb,0xf0,
  0xfa,0xff,0xfd,0x4d,0xfd,0xff,0xfa,0xa2,0xa2,0xb5,0xb2,0xb2,0x94,0xe8,0xc3,0xe8,
  0xe8,0xdc,0xe8,0xfb,0xe3,0x01,0x08,0x76,0xcc,0xf5,0xcc,0x8e,0x8e,0x08,0xbe,0x8e,
  0x01,0xb2,0x01,0xb2,0xf3,0xf3,0xb2,0xb5,0xb5,0x87,0xfc,0xfc,0xfb,0xaf,0x8e,0x8e,
  0xf3,0xf3,0xb2,0xd6,0xd6,0xe5,0xe5,0xe5,0xe5,0xd6,0xe5,0x87,0xaf,0xd8,0xfc,0xfd,
  0xfb,0xe8,0xe8,0xc3,0xc3,0x01,0x08,0x16,0xbe,0xcc,0xfb,0xfc,0xf5,0x87,0x87,0x87,
  0x87,0xd6,0xd6,0x87,0x87,0xb2,0x8e,0x87,0x87,0x87,0x87,0x87,0xaf,0x8e,0xa2,0x87,
  0x8e,0x8e,0x8e,0xf5,0xfc,0xbe,0x8e,0xf3,0xf3,0xe3,0x01,0x76,0xcc,0xfa,0xfd,0xf0,
  0x94,0xe8,0xe8,0xe8,0xe8,0xe8,0xf2,0x8e,0xcc,0xf5,0xf3,0xc3,0xf3,0x01,0xe8,0x08,
  0xcc,0x8e,0xfd,0xfd,0x4d,0x2d,0xcc,0xee,0xf4,0xc3,0xc3,0xe8,0xe8,0xc3,0xc3,0x00,
  0xc3,0xc3,0xf3,0x16,0x08,0x9b,0xfa,0xf3,0x8e,0x01,0x8e,0xf5,0x01,0xe3,0xdc,0x00,
  0x94,0xf3,0x00,0x01,0x76,0x76,0x8e,0xa2,0xb2,0xd9,0xcc,0xd8,0xd8,0xfb,0xf9,0x01,
  0x76,0xb2,0xf3,0xaf,0xe5,0xfc,0xfb,0xa2,0x00,0xc3,0x94,0xb5,0x8e,0xfb,0xb2,0xf5,
  0xe8,0xc3,0x94,0xdc,0xf3,0x76,0x08,0xbe,0xcc,0xcc,0xf5,0xf5,0xd8,0xd8,0xd6,0xd6,
  0xd6,0xd6,0xd6,0x87,0x87,0xe8,0xf2,0xf2,0xe8,0xee,0xce,0xa2,0xa2,0xa9,0x01,0x94,
  0xc3,0x94,0x76,0x08,0xbe,0xfb,0xfc,0x08,0xb2,0x76,0xc3,0xc3,0xe8,0xe8,0xc3,0xdc,
  0xc3,0xc3,0x94,0xc3,0xe8,0xe8,0xc3,0x94,0xdc,0xdc,0x76,0x76,0xf4,0xe8,0xe8,0xe8,
  0xc3,0xe8,0xe8,0xf3,0x01,0xb2,0xe8,0xe8,0xe8,0xc3,0xe8,0xc3,0xe8,0xe8,0xc3,0xe8,
  0x16,0xc3,0x8e,0x08,0xfc,0xf0,0xff,0xbe,0xcc,0x8e,0xcc,0xbe,0xe3,0xe3,0x16,0xdc,
  0xe3,0xdc,0xf3,0x01,0x08,0xe3,0xb2,0x8e,0xaf,0xe3,0x8e,0xd8,0xd8,0xcc,0xbe,0x08,
  0xe3,0xe3,0x08,0xfc,0xfb,0xf5,0xfc,0xc3,0xc3,0xc3,0xc3,0xc3,0xf3,0x94,0x94,0xc3,
  0xc3,0xdc,0xc3,0x94,0xdc,0x00,0x16,0xbe,0xcc,0xfb,0xf5,0xfc,0xd8,0x9b,0xd8,0x8e,
  0x16,0x08,0xaf,0x87,0x87,0xfb,0xc3,0xc3,0xe8,0xe8,0xc3,0x94,0x01,0xf3,0xdc,0x94,
  0x94,0x94,0xdc,0xdc,0x94,0x94,0x94,0x94,0x94,0x94,0x94,0x94,0xc3,0xe8,0xe8,0xe8,
  0xe8,0xc3,0xdc,0xdc,0xc3,0xc3,0xc3,0xc3,0x94,0x94,0x94,0xc3,0xc3,0x94,0x94,0xc3,
  0xc3,0xc3,0xe8,0xe8,0xc3,0xc3,0xc3,0xc3,0xe8,0xe8,0xe8,0xf2,0xe8,0xdc,0x94,0x08,
  0x94,0xdc,0xe3,0xd8,0xeb,0xf0,0xfd,0xd8,0xfc,0x8e,0x8e,0xf3,0xf3,0x94,0x8e,0x01,
  0x01,0xcc,0xf3,0x00,0xb2,0x08,0x8e,0x76,0xf3,0xdc,0xe3,0xcc,0xff,0xff,0xeb,0xf5,
  0xf5,0xfb,0xfc,0xd8,0xd8,0xf0,0xd8,0xf5,0xdc,0xc3,0xe8,0xe8,0x94,0xf3,0x08,0xf3,
  0x08,0xbe,0x94,0xdc,0x00,0xf3,0xf3,0x76,0x08,0x08,0xbe,0xbe,0xcc,0xbe,0x16,0x08,
  0x16,0x08,0x76,0x76,0x76,0xf5,0xeb,0xeb,0xd8,0xe3,0x01,0x00,0xf3,0x08,0x00,0x00,
  0xf3,0xf3,0xdc,0xdc,0x94,0xdc,0xf4,0x94,0x94,0x94,0xdc,0xf3,0xdc,0xdc,0xc3,0xe8,
  0xf3,0x94,0xc3,0xc3,0xc3,0xe8,0xe8,0xc3,0xc3,0x94,0xf3,0xdc,0xf3,0xdc,0x94,0xc3,
  0xc3,0xe8,0xe8,0xe8,0xe8,0xe8,0xc3,0xc3,0xf3,0xc3,0xe8,0xe8,0xe3,0xe8,0xf3,0x8e,
  0x76,0x8e,0x08,0x9b,0xfd,0x2d,0xfd,0xf0,0xf0,0xd8,0xf5,0xd8,0xcc,0x8e,0xf3,0xbe,
  0xfc,0xeb,0xdc,0xf3,0x00,0x00,0xb2,0x01,0xf3,0xf3,0xdc,0x94,0xc3,0x94,0x01,0x08,
  0x8e,0x08,0x01,0x00,0x8e,0xf0,0xf0,0xf5,0xc3,0x94,0x94,0xdc,0x01,0xbe,0xcc,0x08,
  0x76,0x08,0x94,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x76,0x76,0x08,0x16,0x08,
  0x16,0x08,0x76,0x08,0x16,0x8e,0xcc,0xcc,0xcc,0xfc,0xbe,0x8e,0x16,0x08,0x08,0x76,
  0x76,0xf3,0x94,0x94,0x94,0x00,0xf3,0xf3,0x94,0x94,0x08,0xb2,0xd9,0xc3,0x94,0x01,
  0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0x00,0xf3,0xf3,0x94,0xc3,0xe8,
  0xe8,0xe8,0xe8,0xe8,0xe8,0xbe,0xc3,0xf3,0x94,0xf3,0x94,0x94,0xf3,0x16,0x9b,0xd8,
  0xfc,0xf5,0x9b,0xff,0xfe,0x2d,0xfd,0x2d,0xf0,0xf0,0xfa,0xf5,0xd8,0xfc,0xf0,0xf0,
  0xf0,0xcc,0xdc,0xdc,0xdc,0xdc,0xf3,0x01,0x01,0x01,0xf3,0xf3,0x00,0xf3,0x08,0xf3,
  0xf8,0xf3,0xdc,0x94,0x8e,0xfc,0x8e,0x94,0xdc,0xdc,0xf3,0x00,0xdc,0x94,0x94,0x01,
  0x16,0x01,0x94,0xdc,0xf3,0xf3,0xdc,0xdc,0xf3,0xf3,0xf3,0x76,0x76,0x76,0x76,0x76,
  0x08,0x08,0x08,0x08,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x16,0x16,0xe3,0x00,0x00,0x01,0xf3,0x76,0x01,0x8e,0xee,0xee,0xee,0xb2,0x01,0x01,
  0xc3,0xe3,0x01,0x00,0x01,0x00,0xf3,0xf3,0x01,0xf3,0xf3,0xf3,0xdc,0x94,0x94,0x94,
  0xc3,0xc3,0xc3,0xc3,0x08,0xbe,0x16,0xf3,0xf3,0x76,0xc3,0xf3,0xf3,0x8e,0xeb,0xfa,
  0xd8,0xd8,0xfa,0xfd,0xfd,0x2d,0x2d,0xfd,0xf0,0xeb,0xd8,0xfc,0xf5,0xf5,0xf0,0xd8,
  0xfa,0x08,0xdc,0xf3,0xf3,0xf3,0x94,0xf3,0x01,0xb2,0xb2,0x01,0x01,0xcc,0xf3,0x01,
  0xc3,0x00,0x01,0x01,0xf3,0xf4,0xf3,0xdc,0xdc,0xf3,0xf3,0x01,0xf3,0xf3,0x00,0xf3,
  0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,0x01,0x01,0x76,0xb2,0x08,0x76,0x76,0x08,0x16,
  0x16,0x16,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xbe,0x8e,0x8e,0x8e,
  0x8e,0x08,0x08,0x01,0x01,0x76,0x01,0xc3,0xc3,0xe8,0xc3,0xc3,0xc3,0xb2,0x8e,0xee,
  0xbe,0x76,0x01,0xe3,0xf3,0x01,0x08,0x08,0xe3,0x00,0xf3,0xf3,0xf3,0xf3,0xf3,0xf3,
  0xdc,0xf3,0xf3,0x94,0xf5,0x08,0xe3,0x08,0xe3,0x01,0x08,0x01,0x94,0xc3,0xbe,0xcc,
  0xcc,0xcc,0xcc,0xfe,0xfe,0x2d,0xfe,0xfe,0xf5,0xbe,0x8e,0xbe,0xeb,0xeb,0xeb,0xd8,
  0x8e,0x94,0xf3,0xf3,0x00,0x76,0x01,0xf3,0xf3,0xf3,0x01,0x76,0xbe,0xbe,0xcc,0x08,
  0x01,0x08,0xe3,0xe8,0x01,0xc3,0x01,0x00,0xf3,0xf3,0x76,0x01,0x76,0x76,0x01,0xf3,
  0xf3,0xf3,0x00,0x01,0x76,0x76,0x08,0x76,0x76,0xb2,0xb2,0x76,0x16,0xe3,0xf3,0x94,
  0xc3,0xc3,0xe8,0xf3,0x16,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x16,0x08,0x08,
  0x8e,0x8e,0x08,0x08,0xf3,0xb2,0xc3,0xc3,0xee,0xf4,0xe8,0xee,0xe8,0x00,0x01,0x00,
  0x16,0x08,0x01,0x00,0xe8,0xc3,0xc3,0xf8,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,
  0xf3,0xf3,0x00,0x00,0xcc,0xf3,0x8e,0xe3,0x94,0x94,0x08,0xf3,0x00,0x01,0xf3,0xf3,
  0xf3,0x00,0x00,0x8e,0xeb,0xbe,0xbe,0x16,0xf3,0x01,0x16,0x08,0x08,0x01,0xf3,0x01,
  0xdc,0x00,0xf3,0xf3,0x01,0xf3,0x00,0xf3,0xf3,0x01,0x01,0xf5,0xd8,0xd8,0xf5,0x8e,
  0x76,0x01,0x8e,0xbe,0x08,0x08,0x94,0xf3,0xf3,0x00,0x01,0x08,0x76,0x76,0x76,0x76,
  0x01,0x76,0x08,0x76,0x76,0x76,0x08,0x01,0x01,0x76,0x8e,0xbe,0xbe,0xbe,0xbe,0x8e,
  0x8e,0x01,0xf3,0xc3,0xe8,0x16,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x08,0x08,0x08,0x08,
  0x08,0x08,0xbe,0xcc,0xcc,0xbe,0xf4,0xee,0xee,0xee,0xb2,0xc3,0x00,0xc3,0xc3,0xb2,
  0xc3,0xf3,0x08,0x01,0xf3,0xf3,0x00,0xf8,0xe3,0xe3,0x01,0xe3,0x01,0xe3,0x76,0x01,
  0xe3,0x08,0x8e,0x08,0xf5,0xf3,0x08,0xf3,0x16,0x08,0x01,0x00,0x00,0x08,0xf8,0x00,
  0x01,0xf3,0xf3,0xf3,0xf3,0xf3,0x94,0xf3,0x00,0x08,0xb2,0x01,0xf3,0xf3,0xf3,0x94,
  0x94,0xdc,0xf3,0xf3,0x76,0x01,0x00,0xf3,0xf3,0xf3,0x01,0xfc,0xbe,0xf5,0xfc,0x94,
  0xbe,0xbe,0x01,0xf3,0x08,0x08,0xf9,0x01,0xf3,0xdc,0x94,0x01,0x01,0x76,0x76,0x08,
  0x76,0x08,0x76,0x76,0x16,0x08,0x01,0x01,0x76,0x76,0x08,0x08,0x08,0x8e,0x8e,0x8e,
  0x8e,0x8e,0x76,0x08,0xbe,0x08,0x08,0x16,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x16,0xf5,0xf5,0xfc,0x76,0x94,0xc3,0xf4,0xb2,0xee,0xee,0xee,0x01,0xf4,0xe8,0x08,
  0xe8,0xc3,0x00,0x01,0x01,0x01,0x01,0x01,0xcc,0x08,0x08,0x08,0xe3,0x08,0x08,0x08,
  0x8e,0x8e,0x8e,0x8e,0x8e,0xf5,0xd8,0xd8,0xd8,0xd8,0xeb,0xf5,0xf5,0x08,0x01,0xf3,
  0xe3,0xf3,0xf3,0xf3,0xf3,0x01,0x01,0xe3,0x76,0x08,0x8e,0x16,0x01,0x00,0x00,0xf3,
  0xf3,0x00,0xf3,0xf3,0x08,0xb2,0x76,0x01,0x01,0xf3,0x00,0xcc,0x8e,0x8e,0x8e,0x8e,
  0x08,0x00,0xbe,0x08,0x08,0x16,0x08,0xf3,0xf3,0xf3,0x94,0xf3,0x01,0xf8,0x8e,0x8e,
  0x08,0x08,0x08,0x16,0x08,0x01,0x01,0x76,0x08,0x08,0x16,0x08,0x08,0x16,0x08,0x8e,
  0x8e,0x8e,0x8e,0xbe,0x8e,0x08,0x76,0x08,0x76,0x08,0x16,0x8e,0x8e,0x8e,0x08,0x8e,
  0x8e,0xf5,0xd8,0xd8,0xfb,0xfb,0xee,0x94,0xdc,0x8e,0xbe,0xe3,0xf4,0xf4,0xc3,0xc3,
  0xc3,0x08,0xb2,0x94,0xf3,0xf3,0xf4,0x16,0xcc,0x8e,0x08,0x08,0x16,0x08,0x76,0x01,
  0x08,0x08,0x08,0x16,0x8e,0x8e,0x8e,0x08,0x08,0x16,0x08,0x08,0x16,0x76,0x76,0x01,
  0x01,0x00,0xf3,0xf3,0xf3,0x00,0x08,0x8e,0x08,0x08,0x8e,0x8e,0x01,0x01,0xe3,0x76,
  0x76,0x76,0x76,0x16,0x08,0x08,0xb2,0x76,0x76,0xf3,0xf3,0x8e,0xcc,0xf5,0xf5,0x01,
  0x08,0x8e,0xcc,0x76,0x01,0xe3,0x08,0xe3,0xf3,0xf3,0xf3,0xf8,0xf9,0xbe,0x16,0x08,
  0x08,0x16,0x16,0x08,0x08,0xf3,0x01,0x08,0x16,0x16,0x08,0x16,0x16,0x16,0x16,0x8e,
  0x08,0x16,0x08,0x16,0x16,0x16,0x16,0x08,0x08,0x08,0x16,0x8e,0x8e,0x8e,0x8e,0x8e,
  0x16,0x16,0xd8,0xbe,0xcc,0xd8,0xfb,0xee,0x8e,0xbe,0xb2,0x8e,0x8e,0xc3,0xf4,0xf2,
  0xfe,0xfc,0x08,0xb2,0xf3,0x00,0xf3,0xbe,0xcc,0xcc,0xe3,0x08,0x01,0x01,0x01,0xe3,
  0x08,0x01,0x01,0xe3,0x76,0x76,0x08,0xe3,0xe3,0x08,0x76,0x76,0x01,0xe3,0x76,0x08,
  0xe3,0x76,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0xe3,0x01,0x01,0x16,0x08,0x08,
  0x08,0x08,0x08,0x76,0xf3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x16,0xcc,0xeb,0xf0,
  0xeb,0xd8,0xfc,0xd8,0xfc,0xf5,0xcc,0xcc,0xf3,0xf3,0xf7,0xf6,0x01,0xf3,0x16,0x16,
  0x08,0x16,0x16,0x8e,0x76,0x08,0x8e,0x76,0x16,0x08,0x16,0x16,0x8e,0x16,0x16,0x16,
  0x08,0x08,0x76,0x76,0x08,0x08,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0x16,0x16,
  0x08,0x08,0xd8,0xcc,0x8e,0xbe,0xfa,0xd8,0xcc,0x8e,0x8e,0xb2,0x8e,0xfc,0xfb,0xfc,
  0xeb,0xff,0xfc,0x8e,0x8e,0xf3,0x16,0xcc,0xf5,0x8e,0xe3,0x01,0x01,0x01,0x01,0x01,
  0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xe3,0x01,0x01,0x01,0xe3,0x08,
  0x08,0x08,0x08,0x76,0x08,0x76,0xe3,0x76,0x76,0x01,0x08,0x08,0x08,0x16,0x8e,0x16,
  0x08,0x16,0x8e,0x8e,0x08,0x08,0x08,0x76,0x76,0x08,0x01,0x01,0xf3,0x76,0x08,0x16,
  0x08,0x8e,0xbe,0xbe,0xbe,0xcc,0xcc,0x16,0xf3,0x94,0x01,0xf3,0xf3,0xf3,0x01,0x08,
  0x08,0x16,0x16,0x76,0x08,0x08,0x01,0x01,0x76,0x76,0x08,0x08,0x08,0x08,0x16,0x08,
  0x16,0x08,0x76,0x76,0x08,0x08,0x08,0x08,0x08,0x76,0x08,0x16,0x16,0x16,0x08,0x16,
  0x08,0x08,0xcc,0x9b,0xfc,0x08,0xb2,0x8e,0xf5,0xcc,0xf5,0xd8,0xfc,0xfc,0xd8,0xd8,
  0xcc,0xd8,0xfe,0xfd,0xf0,0xf5,0xcc,0xf5,0xf5,0x08,0x01,0x01,0x00,0xf3,0xf3,0xf3,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x76,0xe3,0x01,0x01,0x01,0x08,
  0x08,0x08,0x08,0xe3,0x01,0x01,0x01,0x76,0x08,0x76,0x16,0x16,0x8e,0x8e,0xbe,0x8e,
  0x16,0x76,0x01,0x08,0x01,0xe3,0x76,0x76,0xb2,0x08,0xb2,0x76,0x01,0x01,0x01,0x01,
  0x00,0x00,0x76,0x08,0x08,0x16,0x08,0xf8,0xbe,0x08,0xf3,0x00,0x00,0x01,0x01,0x01,
  0x76,0x76,0x08,0x8e,0xbe,0x01,0x01,0x01,0x01,0x76,0xe3,0x08,0x16,0x16,0x16,0x16,
  0x08,0x08,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,0x76,
  0x76,0x08,0x8e,0xbe,0xcc,0xfb,0xcc,0xf5,0xcc,0x8e,0x8e,0x8e,0x8e,0x8e,0x8e,0xbe,
  0x08,0x08,0xf5,0xf0,0xf0,0xfa,0xd8,0xfc,0xbe,0x76,0x01,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0xe3,
  0x08,0xe3,0xe3,0x01,0x00,0x00,0x01,0x01,0x01,0x01,0x76,0x76,0x08,0x08,0x08,0x76,
  0x01,0x76,0x76,0x08,0x08,0x76,0x76,0x76,0x76,0xe3,0x01,0x01,0x01,0x01,0xe3,0x01,
  0x01,0x01,0x01,0x01,0x76,0x8e,0xf9,0x8e,0xf3,0x00,0x01,0x01,0x01,0x00,0x00,0x00,
  0x00,0x01,0x8e,0xcc,0x00,0x01,0x01,0x01,0x01,0x01,0xe3,0xe3,0x08,0x08,0x76,0x76,
  0x76,0x76,0x01,0x01,0x01,0x00,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x00,0x01,0x01,0x01,0xe3,0xe3,0x01,0x01,0x01,0x01,0x01,0xe3,0x76,0x76,0x76,
  0x00,0x00,0x01,0x08,0x08,0x16,0x16,0x08,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x76,0xe3,0x76,0x01,
  0x01,0x01,0x01,0x01,0xe3,0xe3,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01,0x01,0x01,0x00,0xf3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0xe3,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
  0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x00,0x00,
  0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

void LCR_loadImage(unsigned int index)
{
  _LCR_currentImage = LCR_images + index * LCR_IMAGE_STORE_SIZE;

  for (int i = 0; i < 256; ++i)
  {
    _LCR_currentImagePalette[i] = *_LCR_currentImage;
    _LCR_currentImage++;
    _LCR_currentImagePalette[i] |= ((uint16_t) (*_LCR_currentImage)) << 8;
    _LCR_currentImage++;
  }

  _LCR_currentImagePixel = _LCR_currentImage;
}

/**
  Changes currently loaded image's brightness either up or down by modifying
  its palette. This can be used for shading.
*/
void LCR_imageChangeBrightness(int up)
{
  if (up)
    for (int i = 0; i < 256; ++i)
      _LCR_currentImagePalette[i] |= 0x1042;
  else
    for (int i = 0; i < 256; ++i)
      _LCR_currentImagePalette[i] =
        ((_LCR_currentImagePalette[i] >> 1) & 0x7bef);
}

/**
  Samples currently loaded image at given pixels coordinates (with wrapping).
  This is slower than reading the image pixel by pixel.
*/
uint16_t LCR_sampleImage(int x, int y)
{
  // TODO: bottleneck, later on optimize here
  x = (y % LCR_IMAGE_SIZE) * LCR_IMAGE_SIZE + (x % LCR_IMAGE_SIZE);
  x += (x < 0) * (LCR_IMAGE_SIZE * LCR_IMAGE_SIZE);
  return _LCR_currentImagePalette[_LCR_currentImage[x]];
}

/**
  Gets the next pixel of currently loaded image. This is faster than sampling
  the image by pixel coordinates.
*/
uint16_t LCR_getNextImagePixel(void)
{
  uint16_t r = _LCR_currentImagePalette[*_LCR_currentImagePixel];
  _LCR_currentImagePixel++;
  return r;
}

#endif
